@@ -52,6 +52,13 @@ ICOMMAND(0, connectedip, "", (),
     result(address && enet_address_get_host_ip(address, hostname, sizeof(hostname)) >= 0 ? hostname : "");
 });
 
+ICOMMAND(0, connectedhost, "", (),
+{
+    const ENetAddress *address = connectedpeer();
+    string hostname;
+    result(address && enet_address_get_host(address, hostname, sizeof(hostname)) >= 0 ? hostname : (address && enet_address_get_host_ip(address, hostname, sizeof(hostname)) >= 0 ? hostname : ""));
+});
+
 ICOMMAND(0, connectedport, "", (),
 {
     const ENetAddress *address = connectedpeer();
@@ -89,6 +96,8 @@ void trydisconnect()
 
 SVAR(0, connectname, "");
 VAR(0, connectport, 0, 0, VAR_MAX);
+VAR(IDF_PERSIST, connectguidelines, 0, 0, 1);
+SVAR(0, guidelinesaction, "");
 
 void connectserv(const char *name, int port, const char *password)
 {
@@ -102,6 +111,13 @@ void connectserv(const char *name, int port, const char *password)
     setvar("connectport", 0);
     if(name && *name)
     {
+        if(!connectguidelines)
+        {
+            defformatstring(s)("connect %s %d %s", name, port, password && *password ? password : "");
+            setsvar("guidelinesaction", s);
+            showgui("guidelines");
+            return;
+        }
         addserver(name, port);
         conoutft(CON_MESG, "\faattempting to connect to %s:[%d]", name, port);
         if(!resolverwait(name, &address))
@@ -120,22 +136,23 @@ void connectserv(const char *name, int port, const char *password)
     }
 
     if(!clienthost)
+    {
         clienthost = enet_host_create(NULL, 2, server::numchannels(), rate*1024, rate*1024);
+        if(!clienthost)
+        {
+            conoutft(CON_MESG, "\frfailed creating client socket");
+            connectfail();
+            return;
+        }
+        clienthost->duplicatePeers = 0;
+    }
 
-    if(clienthost)
-    {
-        connpeer = enet_host_connect(clienthost, &address, server::numchannels(), 0);
-        enet_host_flush(clienthost);
-        connmillis = totalmillis;
-        connattempts = 0;
-        client::connectattempt(name ? name : "", port, password ? password : "", address);
-        conoutft(CON_MESG, "\fgconnecting to %s:[%d]", name != NULL ? name : "local server", port);
-    }
-    else
-    {
-        conoutft(CON_MESG, "\frfailed creating client socket");
-        connectfail();
-    }
+    connpeer = enet_host_connect(clienthost, &address, server::numchannels(), 0);
+    enet_host_flush(clienthost);
+    connmillis = totalmillis;
+    connattempts = 0;
+    client::connectattempt(name ? name : "", port, password ? password : "", address);
+    conoutft(CON_MESG, "\fgconnecting to %s:[%d]", name != NULL ? name : "local server", port);
 }
 
 void disconnect(int onlyclean, int async)
@@ -187,7 +204,7 @@ ICOMMAND(0, isconnected, "ii", (int *a, int *b), intret(connected(*a==0, *b==0)
 void reconnect(const char *pass)
 {
     int port = 0;
-    mkstring(addr);
+    string addr = "";
     if(*connectname) copystring(addr, connectname);
     if(connectport) port = connectport;
     disconnect(1);
@@ -271,7 +288,7 @@ void gets2c()           // get updates from the server
             }
             else
             {
-                if(!discmillis || event.data) conoutft(CON_MESG, "\frserver network error, disconnecting (%s) ...", disc_reasons[event.data]);
+                if(!discmillis || event.data) conoutft(CON_MESG, "\frserver disconnected (%s) ...", disc_reasons[event.data]);
                 disconnect();
             }
             return;
