@@ -2,46 +2,38 @@
 
 #include "engine.h"
 
-vector<cline> conlines;
+reversequeue<cline, MAXCONLINES> conlines;
+
 int commandmillis = -1;
-string commandbuf;
+bigstring commandbuf;
 char *commandaction = NULL, *commandicon = NULL;
 enum { CF_COMPLETE = 1<<0, CF_EXECUTE = 1<<1, CF_MESSAGE = 1<<2 };
 int commandflags = 0, commandpos = -1, commandcolour = 0;
 
 void conline(int type, const char *sf, int n)
 {
-    cline cl;
+    char *buf = conlines.length() >= MAXCONLINES ? conlines.remove().cref : newstring("", BIGSTRLEN-1);
+    cline &cl = conlines.add();
     cl.type = type;
-    cl.cref = conlines.length() > MAXSTRLEN ? conlines.pop().cref : newstringbuf("");
+    cl.cref = buf;
     cl.reftime = cl.outtime = totalmillis;
-    conlines.insert(n, cl);
-
-    int c = 0;
-    #define addcref(d) { cl.cref[c] = d; c++; }
-    if(n)
-    {
-        addcref(' ');
-        addcref(' ');
-        concatstring(cl.cref, sf);
-    }
-    addcref(0);
-    concatstring(cl.cref, sf);
+    cl.realtime = clocktime;
 
     if(n)
     {
-        string sd;
+        copybigstring(cl.cref, "  ");
+        concatbigstring(cl.cref, sf);
         loopj(2)
         {
             int off = n+j;
             if(conlines.inrange(off))
             {
-                if(j) formatstring(sd)("%s\fs", conlines[off].cref);
-                else formatstring(sd)("\fS%s", conlines[off].cref);
-                copystring(conlines[off].cref, sd);
+                if(j) concatbigstring(conlines[off].cref, "\fs");
+                else prependbigstring(conlines[off].cref, "\fS");
             }
         }
     }
+    else copybigstring(cl.cref, sf);
 }
 
 // keymap is defined externally in keymap.cfg
@@ -87,32 +79,14 @@ const char *getkeyname(int code)
     return km ? km->name : NULL;
 }
 
-#define ADDSEP(comma, conj) do { \
-    if(pretty && *pretty) \
-    { \
-        names.put("\fs", 2); \
-        names.put(pretty, strlen(pretty)); \
-        if(comma) names.add(comma); \
-        names.add(' '); \
-        if((conj)[0]) \
-        { \
-            names.put(conj, strlen(conj)); \
-            names.add(' '); \
-        } \
-        names.put("\fS", 2); \
-    } \
-    else if(sep && *sep) names.put(sep, strlen(sep)); \
-    else names.add(' '); \
-} while(0)
-
-void searchbindlist(const char *action, int type, int limit, const char *sep, const char *pretty, vector<char> &names)
+void searchbindlist(const char *action, int type, int limit, const char *s1, const char *s2, const char *sep1, const char *sep2, vector<char> &names, bool force)
 {
     const char *name1 = NULL, *name2 = NULL, *lastname = NULL;
     int found = 0;
     enumerate(keyms, keym, km,
     {
-        char *act = ((!km.actions[type] || !*km.actions[type]) && type ? km.actions[keym::ACTION_DEFAULT] : km.actions[type]);
-        if(!strcmp(act, action))
+        char *act = type && force && (!km.actions[type] || !*km.actions[type]) ? km.actions[keym::ACTION_DEFAULT] : km.actions[type];
+        if(act && !strcmp(act, action))
         {
             if(!name1) name1 = km.name;
             else if(!name2) name2 = km.name;
@@ -120,14 +94,20 @@ void searchbindlist(const char *action, int type, int limit, const char *sep, co
             {
                 if(lastname)
                 {
-                    ADDSEP(',', "");
+                    if(sep1 && *sep1) names.put(sep1, strlen(sep1));
+                    if(s1 && *s1) names.put(s1, strlen(s1));
                     names.put(lastname, strlen(lastname));
+                    if(s2 && *s2) names.put(s2, strlen(s2));
                 }
                 else
                 {
+                    if(s1 && *s1) names.put(s1, strlen(s1));
                     names.put(name1, strlen(name1));
-                    ADDSEP(',', "");
+                    if(s2 && *s2) names.put(s2, strlen(s2));
+                    if(sep1 && *sep1) names.put(sep1, strlen(sep1));
+                    if(s1 && *s1) names.put(s1, strlen(s1));
                     names.put(name2, strlen(name2));
+                    if(s2 && *s2) names.put(s2, strlen(s2));
                 }
                 lastname = km.name;
             }
@@ -137,16 +117,27 @@ void searchbindlist(const char *action, int type, int limit, const char *sep, co
     });
     if(lastname)
     {
-        ADDSEP(',', sep);
+        if(sep2 && *sep2) names.put(sep2, strlen(sep2));
+        else if(sep1 && *sep1) names.put(sep1, strlen(sep1));
+        if(s1 && *s1) names.put(s1, strlen(s1));
         names.put(lastname, strlen(lastname));
+        if(s2 && *s2) names.put(s2, strlen(s2));
     }
     else
     {
-        if(name1) names.put(name1, strlen(name1));
+        if(name1)
+        {
+            if(s1 && *s1) names.put(s1, strlen(s1));
+            names.put(name1, strlen(name1));
+            if(s2 && *s2) names.put(s2, strlen(s2));
+        }
         if(name2)
         {
-            ADDSEP('\0', sep);
+            if(sep2 && *sep2) names.put(sep2, strlen(sep2));
+            else if(sep1 && *sep1) names.put(sep1, strlen(sep1));
+            if(s1 && *s1) names.put(s1, strlen(s1));
             names.put(name2, strlen(name2));
+            if(s2 && *s2) names.put(s2, strlen(s2));
         }
     }
     names.add('\0');
@@ -162,7 +153,7 @@ const char *searchbind(const char *action, int type)
     return NULL;
 }
 
-void getkeypressed(int limit, const char *sep, const char *pretty, vector<char> &names)
+void getkeypressed(int limit, const char *s1, const char *s2, const char *sep1, const char *sep2, vector<char> &names)
 {
     const char *name1 = NULL, *name2 = NULL, *lastname = NULL;
     int found = 0;
@@ -176,14 +167,20 @@ void getkeypressed(int limit, const char *sep, const char *pretty, vector<char>
             {
                 if(lastname)
                 {
-                    ADDSEP(',', "");
+                    if(sep1 && *sep1) names.put(sep1, strlen(sep1));
+                    if(s1 && *s1) names.put(s1, strlen(s1));
                     names.put(lastname, strlen(lastname));
+                    if(s2 && *s2) names.put(s2, strlen(s2));
                 }
                 else
                 {
+                    if(s1 && *s1) names.put(s1, strlen(s1));
                     names.put(name1, strlen(name1));
-                    ADDSEP(',', "");
+                    if(s2 && *s2) names.put(s2, strlen(s2));
+                    if(sep1 && *sep1) names.put(sep1, strlen(sep1));
+                    if(s1 && *s1) names.put(s1, strlen(s1));
                     names.put(name2, strlen(name2));
+                    if(s2 && *s2) names.put(s2, strlen(s2));
                 }
                 lastname = km.name;
             }
@@ -193,16 +190,27 @@ void getkeypressed(int limit, const char *sep, const char *pretty, vector<char>
     });
     if(lastname)
     {
-        ADDSEP(',', sep);
+        if(sep2 && *sep2) names.put(sep2, strlen(sep2));
+        else if(sep1 && *sep1) names.put(sep1, strlen(sep1));
+        if(s1 && *s1) names.put(s1, strlen(s1));
         names.put(lastname, strlen(lastname));
+        if(s2 && *s2) names.put(s2, strlen(s2));
     }
     else
     {
-        if(name1) names.put(name1, strlen(name1));
+        if(name1)
+        {
+            if(s1 && *s1) names.put(s1, strlen(s1));
+            names.put(name1, strlen(name1));
+            if(s2 && *s2) names.put(s2, strlen(s2));
+        }
         if(name2)
         {
-            ADDSEP('\0', sep);
+            if(sep2 && *sep2) names.put(sep2, strlen(sep2));
+            else if(sep1 && *sep1) names.put(sep1, strlen(sep1));
+            if(s1 && *s1) names.put(s1, strlen(s1));
             names.put(name2, strlen(name2));
+            if(s2 && *s2) names.put(s2, strlen(s2));
         }
     }
     names.add('\0');
@@ -259,19 +267,19 @@ ICOMMAND(0, getbind,     "s", (char *key), getbind(key, keym::ACTION_DEFAULT));
 ICOMMAND(0, getspecbind, "s", (char *key), getbind(key, keym::ACTION_SPECTATOR));
 ICOMMAND(0, geteditbind, "s", (char *key), getbind(key, keym::ACTION_EDITING));
 ICOMMAND(0, getwaitbind, "s", (char *key), getbind(key, keym::ACTION_WAITING));
-ICOMMAND(0, searchbinds,     "siss", (char *action, int *limit, char *sep, char *pretty), { vector<char> list; searchbindlist(action, keym::ACTION_DEFAULT, max(*limit, 0), sep, pretty, list); result(list.getbuf()); });
-ICOMMAND(0, searchspecbinds, "siss", (char *action, int *limit, char *sep, char *pretty), { vector<char> list; searchbindlist(action, keym::ACTION_SPECTATOR, max(*limit, 0), sep, pretty, list); result(list.getbuf()); });
-ICOMMAND(0, searcheditbinds, "siss", (char *action, int *limit, char *sep, char *pretty), { vector<char> list; searchbindlist(action, keym::ACTION_EDITING, max(*limit, 0), sep, pretty, list); result(list.getbuf()); });
-ICOMMAND(0, searchwaitbinds, "siss", (char *action, int *limit, char *sep, char *pretty), { vector<char> list; searchbindlist(action, keym::ACTION_WAITING, max(*limit, 0), sep, pretty, list); result(list.getbuf()); });
+ICOMMAND(0, searchbinds,     "sissssb", (char *action, int *limit, char *s1, char *s2, char *sep1, char *sep2, int *force), { vector<char> list; searchbindlist(action, keym::ACTION_DEFAULT, max(*limit, 0), s1, s2, sep1, sep2, list, *force!=0); result(list.getbuf()); });
+ICOMMAND(0, searchspecbinds, "sissssb", (char *action, int *limit, char *s1, char *s2, char *sep1, char *sep2, int *force), { vector<char> list; searchbindlist(action, keym::ACTION_SPECTATOR, max(*limit, 0), s1, s2, sep1, sep2, list, *force!=0); result(list.getbuf()); });
+ICOMMAND(0, searcheditbinds, "sissssb", (char *action, int *limit, char *s1, char *s2, char *sep1, char *sep2, int *force), { vector<char> list; searchbindlist(action, keym::ACTION_EDITING, max(*limit, 0), s1, s2, sep1, sep2, list, *force!=0); result(list.getbuf()); });
+ICOMMAND(0, searchwaitbinds, "sissssb", (char *action, int *limit, char *s1, char *s2, char *sep1, char *sep2, int *force), { vector<char> list; searchbindlist(action, keym::ACTION_WAITING, max(*limit, 0), s1, s2, sep1, sep2, list, *force!=0); result(list.getbuf()); });
 
-ICOMMAND(0, keyspressed, "iss", (int *limit, char *sep, char *pretty), { vector<char> list; getkeypressed(max(*limit, 0), sep, pretty, list); result(list.getbuf()); });
+ICOMMAND(0, keyspressed, "issss", (int *limit, char *s1, char *s2, char *sep1, char *sep2), { vector<char> list; getkeypressed(max(*limit, 0), s1, s2, sep1, sep2, list); result(list.getbuf()); });
 
 void inputcommand(char *init, char *action = NULL, char *icon = NULL, int colour = 0, char *flags = NULL) // turns input to the command line on or off
 {
     commandmillis = init ? totalmillis : -totalmillis;
     SDL_EnableUNICODE(commandmillis > 0 ? 1 : 0);
     keyrepeat(commandmillis > 0);
-    copystring(commandbuf, init ? init : "");
+    copybigstring(commandbuf, init ? init : "");
     DELETEA(commandaction);
     DELETEA(commandicon);
     commandpos = -1;
@@ -295,7 +303,6 @@ ICOMMAND(0, inputcommand, "sssis", (char *init, char *action, char *icon, int *c
 #if !defined(WIN32) && !defined(__APPLE__)
 #include <X11/Xlib.h>
 #endif
-
 void pasteconsole()
 {
 #ifdef WIN32
@@ -307,17 +314,16 @@ void pasteconsole()
     }
     if(!OpenClipboard(NULL)) return;
     HANDLE h = GetClipboardData(fmt);
-    size_t commandlen = strlen(commandbuf);
-    int cblen = int(GlobalSize(h)), decoded = 0;
+    size_t commandlen = strlen(commandbuf), cblen = GlobalSize(h), decoded = 0;
     ushort *cb = (ushort *)GlobalLock(h);
     switch(fmt)
     {
         case CF_UNICODETEXT:
-            decoded = min(int(sizeof(commandbuf)-1-commandlen), cblen/2);
+            decoded = min(sizeof(commandbuf)-1-commandlen, cblen/2);
             loopi(decoded) commandbuf[commandlen++] = uni2cube(cb[i]);
             break;
         case CF_TEXT:
-            decoded = min(int(sizeof(commandbuf)-1-commandlen), cblen);
+            decoded = min(sizeof(commandbuf)-1-commandlen, cblen);
             memcpy(&commandbuf[commandlen], cb, decoded);
             break;
     }
@@ -325,28 +331,28 @@ void pasteconsole()
     GlobalUnlock(cb);
     CloseClipboard();
 #elif defined(__APPLE__)
-    extern char *mac_pasteconsole(int *cblen);
-    int cblen = 0;
+    extern char *mac_pasteconsole(size_t *cblen);
+    size_t cblen = 0;
     uchar *cb = (uchar *)mac_pasteconsole(&cblen);
     if(!cb) return;
     size_t commandlen = strlen(commandbuf);
-    int decoded = decodeutf8((uchar *)&commandbuf[commandlen], int(sizeof(commandbuf)-1-commandlen), cb, cblen);
+           decoded = decodeutf8((uchar *)&commandbuf[commandlen], sizeof(commandbuf)-1-commandlen, cb, cblen);
     commandbuf[commandlen + decoded] = '\0';
     free(cb);
-    #else
+#else
     SDL_SysWMinfo wminfo;
     SDL_VERSION(&wminfo.version);
     wminfo.subsystem = SDL_SYSWM_X11;
     if(!SDL_GetWMInfo(&wminfo)) return;
     int cbsize;
     uchar *cb = (uchar *)XFetchBytes(wminfo.info.x11.display, &cbsize);
-    if(!cb || !cbsize) return;
+    if(!cb || cbsize <= 0) return;
     size_t commandlen = strlen(commandbuf);
     for(uchar *cbline = cb, *cbend; commandlen + 1 < sizeof(commandbuf) && cbline < &cb[cbsize]; cbline = cbend + 1)
     {
         cbend = (uchar *)memchr(cbline, '\0', &cb[cbsize] - cbline);
         if(!cbend) cbend = &cb[cbsize];
-        int cblen = int(cbend-cbline), commandmax = int(sizeof(commandbuf)-1-commandlen);
+        size_t cblen = cbend-cbline, commandmax = sizeof(commandbuf)-1-commandlen;
         loopi(cblen) if((cbline[i]&0xC0) == 0x80)
         {
             commandlen += decodeutf8((uchar *)&commandbuf[commandlen], commandmax, cbline, cblen);
@@ -380,7 +386,7 @@ struct hline
 
     void restore()
     {
-        copystring(commandbuf, buf);
+        copybigstring(commandbuf, buf);
         if(commandpos >= (int)strlen(commandbuf)) commandpos = -1;
         DELETEA(commandaction);
         DELETEA(commandicon);
@@ -446,7 +452,7 @@ vector<releaseaction> releaseactions;
 
 const char *addreleaseaction(char *s)
 {
-    if(!keypressed) return NULL;
+    if(!keypressed) { delete[] s; return NULL; }
     releaseaction &ra = releaseactions.add();
     ra.key = keypressed;
     ra.action = s;
@@ -661,6 +667,12 @@ void keypress(int code, bool isdown, int cooked)
         case SDLK_TAB:
             keyintercept(iconify, SDL_WM_IconifyWindow());
             break;
+        case SDLK_CAPSLOCK:
+            if(!isdown) capslockon = capslocked();
+            break;
+        case SDLK_NUMLOCK:
+            if(!isdown) numlockon = numlocked();
+            break;
         default: break;
     }
     keym *haskey = keyms.access(code);
@@ -686,7 +698,7 @@ static inline bool sortbinds(keym *x, keym *y)
 
 void writebinds(stream *f)
 {
-    static const char *cmds[4] = { "bind", "specbind", "editbind", "waitbind" };
+    static const char * const cmds[4] = { "bind", "specbind", "editbind", "waitbind" };
     vector<keym *> binds;
     enumerate(keyms, keym, km, binds.add(&km));
     binds.sort(sortbinds);
@@ -758,7 +770,7 @@ static hashtable<fileskey, filesval *> completefiles;
 static hashtable<char *, filesval *> completions;
 
 int completeoffset = -1, completesize = 0;
-string lastcomplete;
+bigstring lastcomplete;
 
 void resetcomplete() { completesize = 0; }
 
@@ -820,8 +832,8 @@ void complete(char *s, const char *cmdprefix)
         int cmdlen = strlen(cmdprefix);
         if(strncmp(s, cmdprefix, cmdlen))
         {
-            defformatstring(cmd)("%s%s", cmdprefix, start);
-            copystring(s, cmd);
+            defformatbigstring(cmd)("%s%s", cmdprefix, start);
+            copybigstring(s, cmd);
         }
         start = &s[cmdlen];
     }
@@ -850,18 +862,18 @@ void complete(char *s, const char *cmdprefix)
         char *end = strchr(start, ' ');
         if(end)
         {
-            string command;
-            copystring(command, start, min(size_t(end-start+1), sizeof(command)));
+            bigstring command;
+            copybigstring(command, start, min(size_t(end-start+1), sizeof(command)));
             filesval **hasfiles = completions.access(command);
             if(hasfiles) f = *hasfiles;
         }
     }
     const char *nextcomplete = NULL;
-    string prefix;
+    bigstring prefix;
     if(f) // complete using filenames
     {
         int commandsize = strchr(start, ' ')+1-start;
-        copystring(prefix, s, min(size_t(commandsize+1+(start-s)), sizeof(prefix)));
+        copybigstring(prefix, s, min(size_t(commandsize+1+(start-s)), sizeof(prefix)));
         f->update();
         loopv(f->files)
         {
@@ -872,7 +884,7 @@ void complete(char *s, const char *cmdprefix)
     }
     else // complete using command names
     {
-        copystring(prefix, s, min(size_t(1+(start-s)), sizeof(prefix)));
+        copybigstring(prefix, s, min(size_t(1+(start-s)), sizeof(prefix)));
         enumerate(idents, ident, id,
             if((variable ? id.type == ID_VAR || id.type == ID_SVAR || id.type == ID_FVAR || id.type == ID_ALIAS: id.flags&IDF_COMPLETE) && strncmp(id.name, start, completesize)==0 &&
                strcmp(id.name, lastcomplete) > 0 && (!nextcomplete || strcmp(id.name, nextcomplete) < 0))
@@ -881,8 +893,8 @@ void complete(char *s, const char *cmdprefix)
     }
     if(nextcomplete)
     {
-        formatstring(s)("%s%s", prefix, nextcomplete);
-        copystring(lastcomplete, nextcomplete);
+        formatbigstring(s)("%s%s", prefix, nextcomplete);
+        copybigstring(lastcomplete, nextcomplete);
     }
     else
     {
@@ -950,3 +962,46 @@ void writecompletions(stream *f)
     }
 }
 
+bool capslockon = false, numlockon = false;
+#if !defined(WIN32) && !defined(__APPLE__)
+#include <X11/XKBlib.h>
+#endif
+bool capslocked()
+{
+    #ifdef WIN32
+    if(GetKeyState(VK_CAPITAL)) return true;
+    #elif defined(__APPLE__)
+    if(mac_capslock()) return true;
+    #else
+    Display *d = XOpenDisplay((char*)0);
+    if(d)
+    {
+        uint n = 0;
+        XkbGetIndicatorState(d, XkbUseCoreKbd, &n);
+        XCloseDisplay(d);
+        return (n&0x01)!=0;
+    }
+    #endif
+    return false;
+}
+ICOMMAND(0, getcapslock, "", (), intret(capslockon ? 1 : 0));
+
+bool numlocked()
+{
+    #ifdef WIN32
+    if(GetKeyState(VK_NUMLOCK)) return true;
+    #elif defined(__APPLE__)
+    if(mac_numlock()) return true;
+    #else
+    Display *d = XOpenDisplay((char*)0);
+    if(d)
+    {
+        uint n = 0;
+        XkbGetIndicatorState(d, XkbUseCoreKbd, &n);
+        XCloseDisplay(d);
+        return (n&0x02)!=0;
+    }
+    #endif
+    return false;
+}
+ICOMMAND(0, getnumlock, "", (), intret(numlockon ? 1 : 0));
