@@ -1,27 +1,38 @@
 #ifndef __ENGINE_H__
 #define __ENGINE_H__
 
+#include "version.h"
 #include "cube.h"
 
-#define LAN_PORT         28799
-#define MASTER_PORT      28800
-#define SERVER_PORT      28801
+#define LAN_PORT 28799
+#define MASTER_PORT 28800
+#define SERVER_PORT 28801
 
-extern bool versioning;
-extern int versionmajor, versionminor, versionpatch;
-extern char *versionstring, *versionname, *versionuname, *versionrelease, *versionurl, *versionmaster;
-#define CUR_VER_MAKE(a,b,c)  (((a)<<16) | ((b)<<8) | (c))
-#define CUR_VER              CUR_VER_MAKE(versionmajor, versionminor, versionpatch)
-#define CUR_VERSION          (versionmajor*100)+(versionminor*10)+versionpatch
+extern int versioning, versionmajor, versionminor, versionpatch, versionplatform, versionarch, versionisserver;
+extern char *versionstring, *versionname, *versionuname, *versionrelease, *versionurl, *versionmaster, *versionplatname, *versionplatlongname;
+extern uint versioncrc;
+#define CUR_VER_MAKE(a,b,c) (((a)<<16) | ((b)<<8) | (c))
+#define CUR_VER CUR_VER_MAKE(VERSION_MAJOR, VERSION_MINOR, VERSION_PATCH)
+#define CUR_VERSION (VERSION_MAJOR*100)+(VERSION_MINOR*10)+VERSION_PATCH
 
 #ifdef WIN32
-#define CUR_PLATFORM         "win"
+#define CUR_PLATFORM 0
+#define CUR_PLATID
 #elif defined(__APPLE__)
-#define CUR_PLATFORM         "mac"
+#define CUR_PLATFORM 1
 #else
-#define CUR_PLATFORM         "nix"
+#define CUR_PLATFORM 2
 #endif
-#define CUR_ARCH             (int(8*sizeof(void *)))
+#define CUR_ARCH (int(8*sizeof(void *)))
+
+#define MAX_PLATFORMS 3
+
+#define sup_platform(a) (a >= 0 && a < MAX_PLATFORMS)
+#define sup_arch(a) (a == 32 || a == 64)
+
+extern const char *platnames[MAX_PLATFORMS], *platlongnames[MAX_PLATFORMS];
+#define plat_name(a) (sup_platform(a) ? platnames[a] : "unk")
+#define plat_longname(a) (sup_platform(a) ? platlongnames[a] : "unknown")
 
 #ifdef STANDALONE
 extern void setupmaster();
@@ -33,6 +44,8 @@ extern char *masterip;
 extern int nextcontrolversion();
 #endif
 
+extern void setcrc(const char *bin);
+
 #include "irc.h"
 #include "sound.h"
 
@@ -42,128 +55,34 @@ enum { PACKAGEDIR_OCTA = 1<<0 };
 extern const char * const disc_reasons[];
 struct ipinfo
 {
-    enum { ALLOW = 0, BAN, MUTE, LIMIT, MAXTYPES };
+    enum { ALLOW = 0, BAN, MUTE, LIMIT, EXCEPT, MAXTYPES };
     enum { TEMPORARY = 0, LOCAL, GLOBAL };
     enet_uint32 ip, mask;
     int type, flag, time, version;
+    char *reason;
 
-    ipinfo() : ip(0), mask(0), type(-1), flag(TEMPORARY), time(-1), version(-1) {}
-    ~ipinfo() {}
+    ipinfo() : ip(0), mask(0), type(-1), flag(TEMPORARY), time(-1), version(-1), reason(NULL) {}
+    ~ipinfo()
+    {
+        if(reason) delete[] reason;
+    }
 };
 extern vector<ipinfo> control;
 extern const char *ipinfotypes[ipinfo::MAXTYPES];
-extern void addipinfo(vector<ipinfo> &info, int type, const char *name);
+extern void addipinfo(vector<ipinfo> &info, int type, const char *name, const char *reason = NULL);
 extern char *printipinfo(const ipinfo &info, char *buf = NULL);
-extern bool checkipinfo(vector<ipinfo> &info, int type, enet_uint32 ip);
+extern ipinfo *checkipinfo(vector<ipinfo> &info, int type, enet_uint32 ip);
 extern void writecfg();
 extern void rehash(bool reload = true);
 
-#define NZT(t) (t != 0 ? t : 1)
-
 #ifndef STANDALONE
 #include "world.h"
+#include "glexts.h"
 #include "octa.h"
 #include "lightmap.h"
 #include "bih.h"
 #include "texture.h"
 #include "model.h"
-
-// GL_ARB_multitexture
-extern PFNGLACTIVETEXTUREARBPROC        glActiveTexture_;
-extern PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTexture_;
-extern PFNGLMULTITEXCOORD2FARBPROC   glMultiTexCoord2f_;
-extern PFNGLMULTITEXCOORD3FARBPROC   glMultiTexCoord3f_;
-extern PFNGLMULTITEXCOORD4FARBPROC   glMultiTexCoord4f_;
-
-// GL_ARB_vertex_buffer_object
-extern PFNGLGENBUFFERSARBPROC       glGenBuffers_;
-extern PFNGLBINDBUFFERARBPROC       glBindBuffer_;
-extern PFNGLMAPBUFFERARBPROC        glMapBuffer_;
-extern PFNGLUNMAPBUFFERARBPROC      glUnmapBuffer_;
-extern PFNGLBUFFERDATAARBPROC       glBufferData_;
-extern PFNGLBUFFERSUBDATAARBPROC    glBufferSubData_;
-extern PFNGLDELETEBUFFERSARBPROC    glDeleteBuffers_;
-extern PFNGLGETBUFFERSUBDATAARBPROC glGetBufferSubData_;
-
-// GL_ARB_occlusion_query
-extern PFNGLGENQUERIESARBPROC       glGenQueries_;
-extern PFNGLDELETEQUERIESARBPROC     glDeleteQueries_;
-extern PFNGLBEGINQUERYARBPROC       glBeginQuery_;
-extern PFNGLENDQUERYARBPROC       glEndQuery_;
-extern PFNGLGETQUERYIVARBPROC       glGetQueryiv_;
-extern PFNGLGETQUERYOBJECTIVARBPROC  glGetQueryObjectiv_;
-extern PFNGLGETQUERYOBJECTUIVARBPROC glGetQueryObjectuiv_;
-
-// GL_EXT_framebuffer_object
-extern PFNGLBINDRENDERBUFFEREXTPROC     glBindRenderbuffer_;
-extern PFNGLDELETERENDERBUFFERSEXTPROC   glDeleteRenderbuffers_;
-extern PFNGLGENFRAMEBUFFERSEXTPROC       glGenRenderbuffers_;
-extern PFNGLRENDERBUFFERSTORAGEEXTPROC   glRenderbufferStorage_;
-extern PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC  glCheckFramebufferStatus_;
-extern PFNGLBINDFRAMEBUFFEREXTPROC       glBindFramebuffer_;
-extern PFNGLDELETEFRAMEBUFFERSEXTPROC     glDeleteFramebuffers_;
-extern PFNGLGENFRAMEBUFFERSEXTPROC       glGenFramebuffers_;
-extern PFNGLFRAMEBUFFERTEXTURE2DEXTPROC glFramebufferTexture2D_;
-extern PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC glFramebufferRenderbuffer_;
-extern PFNGLGENERATEMIPMAPEXTPROC         glGenerateMipmap_;
-
-// GL_EXT_framebuffer_blit
-#ifndef GL_EXT_framebuffer_blit
-#define GL_READ_FRAMEBUFFER_EXT           0x8CA8
-#define GL_DRAW_FRAMEBUFFER_EXT           0x8CA9
-#define GL_DRAW_FRAMEBUFFER_BINDING_EXT   0x8CA6
-#define GL_READ_FRAMEBUFFER_BINDING_EXT   0x8CAA
-typedef void (APIENTRYP PFNGLBLITFRAMEBUFFEREXTPROC) (GLint srcX0, GLint srcY0, GLint srcX1, GLint srcY1, GLint dstX0, GLint dstY0, GLint dstX1, GLint dstY1, GLbitfield mask, GLenum filter);
-#endif
-extern PFNGLBLITFRAMEBUFFEREXTPROC         glBlitFramebuffer_;
-
-// GL_EXT_draw_range_elements
-extern PFNGLDRAWRANGEELEMENTSEXTPROC glDrawRangeElements_;
-
-// GL_EXT_blend_minmax
-extern PFNGLBLENDEQUATIONEXTPROC glBlendEquation_;
-
-// GL_EXT_blend_color
-extern PFNGLBLENDCOLOREXTPROC glBlendColor_;
-
-// GL_EXT_multi_draw_arrays
-extern PFNGLMULTIDRAWARRAYSEXTPROC   glMultiDrawArrays_;
-extern PFNGLMULTIDRAWELEMENTSEXTPROC glMultiDrawElements_;
-
-// GL_EXT_packed_depth_stencil
-#ifndef GL_DEPTH_STENCIL_EXT
-#define GL_DEPTH_STENCIL_EXT 0x84F9
-#endif
-#ifndef GL_DEPTH24_STENCIL8_EXT
-#define GL_DEPTH24_STENCIL8_EXT 0x88F0
-#endif
-
-// GL_ARB_texture_compression
-extern PFNGLCOMPRESSEDTEXIMAGE3DARBPROC    glCompressedTexImage3D_;
-extern PFNGLCOMPRESSEDTEXIMAGE2DARBPROC    glCompressedTexImage2D_;
-extern PFNGLCOMPRESSEDTEXIMAGE1DARBPROC    glCompressedTexImage1D_;
-extern PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC glCompressedTexSubImage3D_;
-extern PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC glCompressedTexSubImage2D_;
-extern PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC glCompressedTexSubImage1D_;
-extern PFNGLGETCOMPRESSEDTEXIMAGEARBPROC   glGetCompressedTexImage_;
-
-// GL_EXT_fog_coord
-extern PFNGLFOGCOORDPOINTEREXTPROC glFogCoordPointer_;
-
-// GL_ARB_map_buffer_range
-#ifndef GL_ARB_map_buffer_range
-#define GL_MAP_READ_BIT                   0x0001
-#define GL_MAP_WRITE_BIT                  0x0002
-#define GL_MAP_INVALIDATE_RANGE_BIT       0x0004
-#define GL_MAP_INVALIDATE_BUFFER_BIT      0x0008
-#define GL_MAP_FLUSH_EXPLICIT_BIT         0x0010
-#define GL_MAP_UNSYNCHRONIZED_BIT         0x0020
-typedef GLvoid* (APIENTRYP PFNGLMAPBUFFERRANGEPROC) (GLenum target, GLintptr offset, GLsizeiptr length, GLbitfield access);
-typedef void (APIENTRYP PFNGLFLUSHMAPPEDBUFFERRANGEPROC) (GLenum target, GLintptr offset, GLsizeiptr length);
-#endif
-extern PFNGLMAPBUFFERRANGEPROC         glMapBufferRange_;
-extern PFNGLFLUSHMAPPEDBUFFERRANGEPROC glFlushMappedBufferRange_;
-
 #include "varray.h"
 
 extern physent *camera1, camera;
@@ -206,7 +125,7 @@ extern float textscale;
 extern font *curfont;
 
 // texture
-extern int hwtexsize, hwcubetexsize, hwmaxaniso, maxtexsize, aniso, envmapradius;
+extern int hwtexsize, hwcubetexsize, hwmaxanisotropy, maxtexsize, anisotropy, envmapradius;
 
 extern Texture *textureload(const char *name, int clamp = 0, bool mipit = true, bool msg = true);
 extern int texalign(void *data, int w, int bpp);
@@ -258,7 +177,8 @@ extern void guessshadowdir();
 
 // pvs
 extern void clearpvs();
-extern bool pvsoccluded(const ivec &bborigin, const ivec &bbsize);
+extern bool pvsoccluded(const ivec &bbmin, const ivec &bbmax);
+extern bool pvsoccludedsphere(const vec &center, float radius);
 extern bool waterpvsoccluded(int height);
 extern void setviewcell(const vec &p);
 extern void savepvs(stream *f);
@@ -267,13 +187,14 @@ extern int getnumviewcells();
 
 static inline bool pvsoccluded(const ivec &bborigin, int size)
 {
-    return pvsoccluded(bborigin, ivec(size, size, size));
+    return pvsoccluded(bborigin, ivec(bborigin).add(size));
 }
 
 // rendergl
 extern bool hasVBO, hasDRE, hasOQ, hasTR, hasFBO, hasDS, hasTF, hasBE, hasBC, hasCM, hasNP2, hasTC, hasS3TC, hasFXT1, hasTE, hasMT, hasD3, hasAF, hasVP2, hasVP3, hasPP, hasMDA, hasTE3, hasTE4, hasVP, hasFP, hasGLSL, hasGM, hasNVFB, hasSGIDT, hasSGISH, hasDT, hasSH, hasNVPCF, hasRN, hasPBO, hasFBB, hasUBO, hasBUE, hasMBR, hasFC, hasTEX;
 extern int hasstencil;
 extern int glversion, glslversion;
+extern char *gfxvendor, *gfxexts, *gfxrenderer, *gfxversion;
 
 extern bool envmapping, minimapping, renderedgame, modelpreviewing;
 extern const glmatrixf viewmatrix;
@@ -380,28 +301,14 @@ extern void calcmerges();
 extern int mergefaces(int orient, facebounds *m, int sz);
 extern void mincubeface(const cube &cu, int orient, const ivec &o, int size, const facebounds &orig, facebounds &cf, ushort nmat = MAT_AIR, ushort matmask = MATF_VOLUME);
 
-static inline uchar octantrectangleoverlap(const ivec &c, int size, const ivec &o, const ivec &s)
+static inline bool insideworld(const vec &o, bool zup = true)
 {
-    uchar p = 0xFF; // bitmask of possible collisions with octants. 0 bit = 0 octant, etc
-    ivec v(c);
-    v.add(size);
-    if(v.z <= o.z)     p &= 0xF0; // not in a -ve Z octant
-    else if(v.z >= o.z+s.z) p &= 0x0F; // not in a +ve Z octant
-    if(v.y <= o.y)     p &= 0xCC; // not in a -ve Y octant
-    else if(v.y >= o.y+s.y) p &= 0x33; // etc..
-    if(v.x <= o.x)     p &= 0xAA;
-    else if(v.x >= o.x+s.x) p &= 0x55;
-    return p;
+    return o.x>=0 && o.x<hdr.worldsize && o.y>=0 && o.y<hdr.worldsize && o.z>=0 && (!zup || o.z<hdr.worldsize);
 }
 
-static inline bool insideworld(const vec &o)
+static inline bool insideworld(const ivec &o, bool zup = true)
 {
-    return o.x>=0 && o.x<hdr.worldsize && o.y>=0 && o.y<hdr.worldsize && o.z>=0 && o.z<hdr.worldsize;
-}
-
-static inline bool insideworld(const ivec &o)
-{
-    return uint(o.x)<uint(hdr.worldsize) && uint(o.y)<uint(hdr.worldsize) && uint(o.z)<uint(hdr.worldsize);
+    return uint(o.x)<uint(hdr.worldsize) && uint(o.y)<uint(hdr.worldsize) && (!zup || uint(o.z)<uint(hdr.worldsize));
 }
 
 static inline cubeext &ext(cube &c)
@@ -555,7 +462,7 @@ extern void localconnect(bool force = true);
 extern void localdisconnect();
 
 // serverbrowser
-extern void addserver(const char *name, int port, const char *desc = NULL);
+extern void addserver(const char *name, int port, int priority = 0, const char *desc = NULL);
 
 // client
 extern char *connectname;
@@ -586,7 +493,7 @@ extern const char *getkeyname(int code);
 extern int findkeycode(char *key);
 
 extern int uimillis, commandmillis,  commandpos, commandcolour, completeoffset, completesize;
-extern string commandbuf;
+extern bigstring commandbuf;
 extern char *commandaction, *commandicon;
 extern bool fullconsole;
 // main
@@ -600,7 +507,7 @@ enum
 };
 extern int initing, fullscreen, numcpus;
 void setfullscreen(bool enable, bool force = false);
-extern bool progressing;
+extern bool progressing, pixeling;
 extern float loadprogress, progresspart, progressamt;
 extern char *progresstitle, *progresstext;
 extern void progress(float bar1 = 0, const char *text1 = NULL, float bar2 = 0, const char *text2 = NULL);
@@ -612,13 +519,16 @@ enum
     CHANGE_SOUND = 1<<1
 };
 extern bool initwarning(const char *desc, int level = INIT_RESET, int type = CHANGE_GFX, bool force = false);
+
+extern bool minimized;
+
 extern void resetcursor(bool warp = true, bool reset = true);
 extern int compresslevel, imageformat;
 
 extern void pushevent(const SDL_Event &e);
 extern bool interceptkey(int sym, int mod = 0);
 extern void getfps(int &fps, int &bestdiff, int &worstdiff);
-extern void swapbuffers();
+extern void swapbuffers(bool overlay = true);
 extern int getclockmillis();
 
 // menu
@@ -653,7 +563,7 @@ extern void entcancel();
 extern void entitiesinoctanodes();
 extern void attachentities();
 extern void freeoctaentities(cube &c);
-extern bool pointinsel(selinfo &sel, vec &o);
+extern bool pointinsel(const selinfo &sel, const vec &o);
 
 extern void clearworldvars(bool msg = false);
 extern void resetmap(bool empty);
@@ -675,11 +585,21 @@ extern void preloadmodel(const char *name);
 extern void flushpreloadedmodels(bool msg = true);
 extern void preloadusedmapmodels(bool msg = false, bool bih = false);
 
+static inline model *loadmapmodel(int n)
+{
+    extern vector<mapmodelinfo> mapmodels;
+    if(mapmodels.inrange(n))
+    {
+        model *m = mapmodels[n].m;
+        return m ? m : loadmodel(NULL, n);
+    }
+    return NULL;
+}
+
 // renderparticles
 extern void particleinit();
 extern void clearparticles();
 extern void makeparticle(const vec &o, attrvector &attr);
-extern void makeparticles(extentity &e);
 extern void updateparticles();
 extern void renderparticles(bool mainpass = false);
 
@@ -712,13 +632,19 @@ extern void drawskybox(int farplane, bool limited);
 extern bool limitsky();
 
 // gui
-extern int mouseaction[2], guibound[2];
+extern int mouseaction[2];
+extern bool guiactionon;
+
+extern int guishadow, guiclicktab, guitextblend, guitextfade, guisepsize, guiscaletime,
+    guibgcolour, guibordercolour, guihovercolour, guitooltips, guitooltiptime,
+    guitooltipcolour, guitooltipbordercolour, guifieldbgcolour, guifieldbordercolour, guifieldactivecolour, guiactivecolour;
+extern float guibgblend, guiborderblend, guihoverscale, guihoverblend, guitooltipblend, guitooltipborderblend, guifieldbgblend, guifieldborderblend, guifieldactiveblend, guiactiveblend;
 
 extern void progressmenu();
 extern void mainmenu();
 extern void texturemenu();
 extern bool menuactive();
-extern int cleargui(int n = 0);
+extern int cleargui(int n = 0, bool skip = true);
 
 #define uipad(parent,count,body) { (parent).space(count); body; (parent).space(count); }
 #define uifont(parent,font,body) { (parent).pushfont(font); body; (parent).popfont(); }
@@ -740,7 +666,7 @@ extern void replacetexcube(cube &c, int oldtex, int newtex);
 extern void loadsky(char *basename);
 
 // main
-extern void setcaption(const char *text);
+extern void setcaption(const char *text = "", const char *text2 = "");
 extern int grabinput, colorpos, curfps, bestfps, worstfps, bestfpsdiff, worstfpsdiff, maxfps;
 
 // editing
@@ -748,15 +674,16 @@ extern int getmatvec(vec v);
 extern int fullbright, fullbrightlevel;
 extern vector<int> entgroup;
 
-extern void newentity(int type, const attrvector &attrs);
-extern void newentity(const vec &v, int type, const attrvector &attrs);
+extern int newentity(int type, const attrvector &attrs);
+extern int newentity(const vec &v, int type, const attrvector &attrs);
 
 // menu
 enum { MN_BACK = 0, MN_INPUT, MN_MAX };
 
 // console
-struct cline { char *cref; int type, reftime, outtime; };
-extern vector<cline> conlines;
+#define MAXCONLINES 1000
+struct cline { char *cref; int type, reftime, outtime, realtime; };
+extern reversequeue<cline, MAXCONLINES> conlines;
 extern void conline(int type, const char *sf, int n);
 
 // rendergl
@@ -808,7 +735,7 @@ extern uchar shouldsaveblendmap();
 namespace recorder
 {
     extern void stop();
-    extern void capture();
+    extern void capture(bool overlay = true);
     extern void cleanup();
 }
 #endif // STANDALONE
