@@ -1,5 +1,8 @@
 #include "engine.h"
 
+VAR(0, ircfilter, 0, 2, 2);
+VAR(0, ircverbose, 0, 0, 2);
+
 vector<ircnet *> ircnets;
 
 ircnet *ircfind(const char *name)
@@ -13,8 +16,8 @@ ircnet *ircfind(const char *name)
 
 void ircprintf(ircnet *n, int relay, const char *target, const char *msg, ...)
 {
-    defvformatstring(str, msg, msg);
-    string s;
+    defvformatbigstring(str, msg, msg);
+    string s = "";
     if(target && *target && strcasecmp(target, n->nick))
     {
         ircchan *c = ircfindchan(n, target);
@@ -45,13 +48,14 @@ void ircprintf(ircnet *n, int relay, const char *target, const char *msg, ...)
         n->updated |= IRCUP_MSG;
         #endif
     }
-    console(0, "%s %s", s, str); // console is not used to relay
+    if(ircverbose) console(0, "%s %s", s, str); // console is not used to relay
 }
 
 void ircestablish(ircnet *n)
 {
     if(!n) return;
-    n->lastattempt = clocktime;
+    n->lastattempt = n->lastactivity = clocktime;
+    n->lastping = 0;
     if(n->address.host == ENET_HOST_ANY)
     {
         ircprintf(n, 4, NULL, "looking up %s:[%d]...", n->serv, n->port);
@@ -71,7 +75,8 @@ void ircestablish(ircnet *n)
         ircprintf(n, 4, NULL, "failed to bind connection socket: %s", n->ip);
         address.host = ENET_HOST_ANY;
     }
-    if(n->sock == ENET_SOCKET_NULL || connectwithtimeout(n->sock, n->serv, n->address) < 0)
+    if(n->sock != ENET_SOCKET_NULL) enet_socket_set_option(n->sock, ENET_SOCKOPT_NONBLOCK, 1);
+    if(n->sock == ENET_SOCKET_NULL || enet_socket_connect(n->sock, &n->address) < 0)
     {
         ircprintf(n, 4, NULL, n->sock == ENET_SOCKET_NULL ? "could not open socket to %s:[%d]" : "could not connect to %s:[%d]", n->serv, n->port);
         if(n->sock != ENET_SOCKET_NULL)
@@ -82,7 +87,7 @@ void ircestablish(ircnet *n)
         n->state = IRC_DISC;
         return;
     }
-    n->state = IRC_ATTEMPT;
+    n->state = IRC_WAIT;
     ircprintf(n, 4, NULL, "connecting to %s:[%d]...", n->serv, n->port);
 }
 
@@ -90,20 +95,21 @@ void ircsend(ircnet *n, const char *msg, ...)
 {
     if(!n) return;
     defvformatstring(str, msg, msg);
-    if(n->sock == ENET_SOCKET_NULL) return;
-    if(verbose >= 2) console(0, "[%s] >>> %s", n->name, str);
+    if(n->sock == ENET_SOCKET_NULL || !*msg) return; // don't spew \n
+    if(ircverbose >= 2) console(0, "[%s] >>> %s", n->name, str);
     concatstring(str, "\n");
     ENetBuffer buf;
     uchar ubuf[512];
-    int len = strlen(str), carry = 0;
+    size_t len = strlen(str), carry = 0;
     while(carry < len)
     {
-        int numu = encodeutf8(ubuf, sizeof(ubuf)-1, &((uchar *)str)[carry], len - carry, &carry);
+        size_t numu = encodeutf8(ubuf, sizeof(ubuf)-1, &((uchar *)str)[carry], len - carry, &carry);
         if(carry >= len) ubuf[numu++] = '\n';
         loopi(numu) switch(ubuf[i])
         {
             case '\v': ubuf[i] = '\x01'; break;
-            case '\f': ubuf[i] = '\x03'; break;
+            case '\f': ubuf[i] = '\x03'; break; // color code
+            case '\r': ubuf[i] = '\x0F'; break; // end color formatting
         }
         buf.data = ubuf;
         buf.dataLength = numu;
@@ -111,9 +117,7 @@ void ircsend(ircnet *n, const char *msg, ...)
     }
 }
 
-VAR(0, ircfilter, 0, 2, 2);
-
-void converttext(char *dst, const char *src)
+void cube2irc(char *dst, const char *src)
 {
     int colorpos = 0; char colorstack[10];
     memset(colorstack, 'u', sizeof(colorstack)); //indicate user color
@@ -127,9 +131,9 @@ void converttext(char *dst, const char *src)
                 c = *++src;
                 if(c) ++src;
             }
-            else if(c == '[' || c == '(')
+            else if(c == '[' || c == '(' || c == '{')
             {
-                const char *end = strchr(src, c == '[' ? ']' : ')');
+                const char *end = strchr(src, c == '[' ? ']' : (c == '(' ? ')' : '}'));
                 src += end ? end-src : strlen(src);
             }
             else if(c == 's') { if(colorpos < (int)sizeof(colorstack)-1) colorpos++; continue; }
@@ -137,18 +141,21 @@ void converttext(char *dst, const char *src)
             int oldcolor = colorstack[colorpos]; colorstack[colorpos] = c;
             switch(c)
             {
-                case 'g': case '0': case 'G': *dst++ = '\f'; *dst++ = '0'; *dst++ = '3'; break; // green
-                case 'b': case '1': case 'B': *dst++ = '\f'; *dst++ = '1'; *dst++ = '2'; break; // blue
-                case 'y': case '2': case 'Y': *dst++ = '\f'; *dst++ = '0'; *dst++ = '3'; break; // yellow
-                case 'r': case '3': case 'R': *dst++ = '\f'; *dst++ = '0'; *dst++ = '4'; break; // red
-                case 'a': case '4': *dst++ = '\f'; *dst++ = '1'; *dst++ = '4'; break; // grey
-                case 'm': case '5': case 'M': *dst++ = '\f'; *dst++ = '1'; *dst++ = '3'; break; // magenta
-                case 'o': case '6': case 'O': *dst++ = '\f'; *dst++ = '0'; *dst++ = '7'; break; // orange
-                case 'c': case '9': case 'C': *dst++ = '\f'; *dst++ = '1'; *dst++ = '0'; break; // cyan
-                case 'v': *dst++ = '\f'; *dst++ = '0'; *dst++ = '6'; break; // violet
-                case 'p': *dst++ = '\f'; *dst++ = '0'; *dst++ = '6'; break; // purple
+                case 'B': *dst++ = '\f'; *dst++ = '0'; *dst++ = '2'; break; // dark blue
+                case 'G': *dst++ = '\f'; *dst++ = '0'; *dst++ = '3'; break; // dark green
+                case 'r': case '3': *dst++ = '\f'; *dst++ = '0'; *dst++ = '4'; break; // red
                 case 'n': *dst++ = '\f'; *dst++ = '0'; *dst++ = '5'; break; // brown
-                case 'u': case 'w': case '7': case 'k': case '8': case 'd': case 'A': *dst++ = '\f'; *dst++ = '0'; *dst++ = '1'; break; // dark grey
+                case 'p': case 'v': *dst++ = '\f'; *dst++ = '0'; *dst++ = '6'; break; // purple
+                case 'o': case '6': case 'O': *dst++ = '\f'; *dst++ = '0'; *dst++ = '7'; break; // orange
+                case 'y': case '2': case 'Y': *dst++ = '\f'; *dst++ = '0'; *dst++ = '8'; break; // yellow
+                case 'g': case '0': *dst++ = '\f'; *dst++ = '0'; *dst++ = '9'; break; // green
+                case 'C': *dst++ = '\f'; *dst++ = '1'; *dst++ = '0'; break; // dark cyan
+                case 'c': case '9': *dst++ = '\f'; *dst++ = '1'; *dst++ = '1'; break; // cyan
+                case 'b': case '1': *dst++ = '\f'; *dst++ = '1'; *dst++ = '2'; break; // blue
+                case 'm': case '5': case 'M': *dst++ = '\f'; *dst++ = '1'; *dst++ = '3'; break; // magenta
+                case 'k': case '8': case 'd': case 'A': *dst++ = '\f'; *dst++ = '1'; *dst++ = '4'; break; // dark grey
+                case 'a': case '4': *dst++ = '\f'; *dst++ = '1'; *dst++ = '5'; break; // grey
+                case 'u': case 'w': case '7': *dst++ = '\r'; break; // no color
                 default: colorstack[colorpos] = oldcolor; break;
             }
             continue;
@@ -158,21 +165,77 @@ void converttext(char *dst, const char *src)
     *dst = '\0';
 }
 
+void irc2cube(char *dst, const char *src)
+{
+    for(int c = *src; c; c = *++src)
+    {
+        if(c == '\f')
+        {
+            c = *++src;
+            switch(c)
+            {
+                case '0':
+                    c = *++src;
+                    switch(c)
+                    {
+                        case '0': *dst++ = '\f'; *dst++ = 'w'; break; // white
+                        case '1': *dst++ = '\f'; *dst++ = 'A'; break; // dark grey (black too hard to see)
+                        case '2': *dst++ = '\f'; *dst++ = 'B'; break; // dark blue
+                        case '3': *dst++ = '\f'; *dst++ = 'G'; break; // dark green
+                        case '4': *dst++ = '\f'; *dst++ = 'r'; break; // red
+                        case '5': *dst++ = '\f'; *dst++ = 'n'; break; // brown
+                        case '6': *dst++ = '\f'; *dst++ = 'p'; break; // purple
+                        case '7': *dst++ = '\f'; *dst++ = 'o'; break; // orange
+                        case '8': *dst++ = '\f'; *dst++ = 'y'; break; // yellow
+                        case '9': *dst++ = '\f'; *dst++ = 'g'; break; // green
+                        default: *dst++ = '\f'; *dst++ = 'w'; c = *--src; break;
+                    }
+                    break;
+                case '1':
+                    c = *++src;
+                    switch(c)
+                    {
+                        case '0': *dst++ = '\f'; *dst++ = 'C'; break; // dark cyan
+                        case '1': *dst++ = '\f'; *dst++ = 'c'; break; // cyan
+                        case '2': *dst++ = '\f'; *dst++ = 'b'; break; // blue
+                        case '3': *dst++ = '\f'; *dst++ = 'm'; break; // magenta
+                        case '4': *dst++ = '\f'; *dst++ = 'A'; break; // dark grey
+                        case '5': *dst++ = '\f'; *dst++ = 'a'; break; // grey
+                        default: *dst++ = '\f'; *dst++ = 'A'; c = *--src; break;
+                    }
+                    break;
+                case '2': *dst++ = '\f'; *dst++ = 'B'; break; // dark blue
+                case '3': *dst++ = '\f'; *dst++ = 'G'; break; // dark green
+                case '4': *dst++ = '\f'; *dst++ = 'r'; break; // red
+                case '5': *dst++ = '\f'; *dst++ = 'n'; break; // brown
+                case '6': *dst++ = '\f'; *dst++ = 'p'; break; // purple
+                case '7': *dst++ = '\f'; *dst++ = 'o'; break; // orange
+                case '8': *dst++ = '\f'; *dst++ = 'y'; break; // yellow
+                case '9': *dst++ = '\f'; *dst++ = 'g'; break; // green
+                default: *dst++ = '\f'; *dst++ = 'w'; c = *--src; break;
+            }
+            continue;
+        }
+        if(iscubeprint(c) || iscubespace(c)) *dst++ = c;
+    }
+    *dst = '\0';
+}
+
 void ircoutf(int relay, const char *msg, ...)
 {
     defvformatstring(src, msg, msg);
-    mkstring(str);
+    string str = "";
     switch(ircfilter)
     {
-        case 2: filtertext(str, src); break;
-        case 1: converttext(str, src); break;
+        case 2: filterstring(str, src); break;
+        case 1: cube2irc(str, src); break;
         case 0: default: copystring(str, src); break;
     }
     loopv(ircnets) if(ircnets[i]->sock != ENET_SOCKET_NULL && ircnets[i]->type == IRCT_RELAY && ircnets[i]->state == IRC_ONLINE)
     {
         ircnet *n = ircnets[i];
 #if 0 // workaround for freenode's crappy dropping all but the first target of multi-target messages even though they don't state MAXTARGETS=1 in 005 string..
-        mkstring(s);
+        string s = "";
         loopvj(n->channels) if(n->channels[j].state == IRCC_JOINED && n->channels[j].relay >= relay)
         {
             ircchan *c = &n->channels[j];
@@ -192,8 +255,7 @@ int ircrecv(ircnet *n)
     if(!n) return -1;
     if(n->sock == ENET_SOCKET_NULL) return -2;
     int total = 0;
-    enet_uint32 events = ENET_SOCKET_WAIT_RECEIVE;
-    while(enet_socket_wait(n->sock, &events, 0) >= 0 && events)
+    for(;;)
     {
         ENetBuffer buf;
         buf.data = n->input + n->inputlen;
@@ -208,7 +270,7 @@ int ircrecv(ircnet *n)
             case '\v': case '\f': n->input[n->inputlen+i] = ' '; break;
         }
         n->inputlen += len;
-        int carry = 0, decoded = decodeutf8(&n->input[n->inputcarry], n->inputlen - n->inputcarry, &n->input[n->inputcarry], n->inputlen - n->inputcarry, &carry);
+        size_t carry = 0, decoded = decodeutf8(&n->input[n->inputcarry], n->inputlen - n->inputcarry, &n->input[n->inputcarry], n->inputlen - n->inputcarry, &carry);
         if(carry > decoded)
         {
             memmove(&n->input[n->inputcarry + decoded], &n->input[n->inputcarry + carry], n->inputlen - (n->inputcarry + carry));
@@ -226,19 +288,12 @@ void ircnewnet(int type, const char *name, const char *serv, int port, const cha
     ircnet *m = ircfind(name);
     if(m)
     {
-        if(m->state != IRC_DISC) conoutf("ircnet %s already exists", m->name);
-        else ircestablish(m);
+        conoutf("ircnet %s already exists", m->name);
         return;
     }
     ircnet &n = *ircnets.add(new ircnet);
     n.type = type;
-    n.state = IRC_NEW;
-    n.sock = ENET_SOCKET_NULL;
     n.port = port;
-    n.lastattempt = 0;
-    #ifndef STANDALONE
-    n.updated = IRCUP_NEW;
-    #endif
     copystring(n.name, name);
     copystring(n.serv, serv);
     copystring(n.nick, nick);
@@ -289,14 +344,14 @@ ICOMMAND(0, ircpass, "ss", (const char *name, const char *s), {
 ICOMMAND(0, ircauth, "sss", (const char *name, const char *s, const char *t), {
     ircnet *n = ircfind(name);
     if(!n) { conoutf("no such ircnet: %s", name); return; }
-    if(!s || !*s || !t || !*t) { ircprintf(n, 4, NULL, "current authority is: %s (%s)", n->authname, n->authpass && *n->authpass ? "<set>" : "<not set>"); return; }
+    if(!s || !*s || !t || !*t) { ircprintf(n, 4, NULL, "current auth details are: %s (%s)", n->authname, n->authpass && *n->authpass ? "<set>" : "<not set>"); return; }
     copystring(n->authname, s);
     copystring(n->authpass, t);
 });
 ICOMMAND(0, ircconnect, "s", (const char *name), {
     ircnet *n = ircfind(name);
     if(!n) { conoutf("no such ircnet: %s", name); return; }
-    if(n->state != IRC_DISC) { ircprintf(n, 4, NULL, "network already already active"); return; }
+    if(n->state > IRC_DISC) { ircprintf(n, 4, NULL, "network already already active"); return; }
     ircestablish(n);
 });
 
@@ -421,13 +476,17 @@ void ircprocess(ircnet *n, char *user[3], int g, int numargs, char *w[])
                     if(ismsg)
                     {
                         if(!strcasecmp(q, "ACTION"))
-                            ircprintf(n, 1, g ? w[g+1] : NULL, "\fv* %s %s", user[0], r);
+                        {
+                            string str = "";
+                            irc2cube(str, r);
+                            ircprintf(n, 1, g ? w[g+1] : NULL, "\fv* %s %s", user[0], str);
+                        }
                         else
                         {
                             ircprintf(n, 4, g ? w[g+1] : NULL, "\fr%s requests: %s %s", user[0], q, r);
 
                             if(!strcasecmp(q, "VERSION"))
-                                ircsend(n, "NOTICE %s :\vVERSION %s v%s-%s %d bit (%s), %s\v", user[0], versionname, versionstring, CUR_PLATFORM, CUR_ARCH, versionrelease, versionurl);
+                                ircsend(n, "NOTICE %s :\vVERSION %s v%s-%s%d (%s)%s%s\v", user[0], VERSION_NAME, VERSION_STRING, versionplatname, versionarch, VERSION_RELEASE, *VERSION_URL ? ", " : "", VERSION_URL);
                             else if(!strcasecmp(q, "PING")) // eh, echo back
                                 ircsend(n, "NOTICE %s :\vPING %s\v", user[0], r);
                         }
@@ -438,13 +497,22 @@ void ircprocess(ircnet *n, char *user[3], int g, int numargs, char *w[])
             }
             else if(ismsg)
             {
+                string str = "";
                 if(n->type == IRCT_RELAY && g && strcasecmp(w[g+1], n->nick) && !strncasecmp(w[g+2], n->nick, strlen(n->nick)))
                 {
                     const char *p = &w[g+2][strlen(n->nick)];
                     while(p && (*p == ':' || *p == ';' || *p == ',' || *p == '.' || *p == ' ' || *p == '\t')) p++;
-                    if(p && *p) ircprintf(n, 0, w[g+1], "\fa<\fw%s\fa>\fw %s", user[0], p);
+                    if(p && *p)
+                    {
+                        irc2cube(str, p);
+                        ircprintf(n, 0, w[g+1], "\fa<\fw%s\fa>\fw %s", user[0], str);
+                    }
+                }
+                else
+                {
+                    irc2cube(str, w[g+2]);
+                    ircprintf(n, 1, g ? w[g+1] : NULL, "\fa<\fw%s\fa>\fw %s", user[0], str);
                 }
-                else ircprintf(n, 1, g ? w[g+1] : NULL, "\fa<\fw%s\fa>\fw %s", user[0], w[g+2]);
             }
             else ircprintf(n, 2, g ? w[g+1] : NULL, "\fo-%s- %s", user[0], w[g+2]);
         }
@@ -510,7 +578,7 @@ void ircprocess(ircnet *n, char *user[3], int g, int numargs, char *w[])
     {
         if(numargs > g+2)
         {
-            mkstring(modestr);
+            string modestr = "";
             loopi(numargs-g-2)
             {
                 if(i) concatstring(modestr, " ");
@@ -534,10 +602,16 @@ void ircprocess(ircnet *n, char *user[3], int g, int numargs, char *w[])
             ircsend(n, "PONG %d", clocktime);
         }
     }
+    else if(!strcasecmp(w[g], "ERROR"))
+    {
+        if(numargs > g+1) ircprintf(n, 4, NULL, "%s ERROR %s", user[0], w[g+1]);
+        else ircprintf(n, 4, NULL, "%s ERROR", user[0]);
+        n->state = IRC_QUIT;
+    }
     else
     {
         int numeric = *w[g] && *w[g] >= '0' && *w[g] <= '9' ? atoi(w[g]) : 0, off = 0;
-        mkstring(s);
+        string s = "";
         #define irctarget(a) (!strcasecmp(n->nick, a) || *a == '#' || ircfindchan(n, a))
         char *targ = numargs > g+1 && irctarget(w[g+1]) ? w[g+1] : NULL;
         if(numeric)
@@ -663,7 +737,7 @@ void ircparse(ircnet *n)
             loopi(3) DELETEA(user[i]);
         }
     cleanup:
-        loopi(MAXWORDS) DELETEA(w[i]);
+        loopi(numargs) DELETEA(w[i]);
     }
     int parsed = start - (char *)n->input;
     if(parsed > 0)
@@ -677,12 +751,14 @@ void ircparse(ircnet *n)
 void ircdiscon(ircnet *n, const char *msg = NULL)
 {
     if(!n) return;
-    if(msg) ircprintf(n, 4, NULL, "disconnected from %s (%s:[%d]): %s", n->name, n->serv, n->port, msg);
+    if(n->state == IRC_WAIT) ircprintf(n, 4, NULL, "could not connect to %s:[%d]", n->serv, n->port);
+    else if(msg) ircprintf(n, 4, NULL, "disconnected from %s (%s:[%d]): %s", n->name, n->serv, n->port, msg);
     else ircprintf(n, 4, NULL, "disconnected from %s (%s:[%d])", n->name, n->serv, n->port);
     enet_socket_destroy(n->sock);
     n->state = IRC_DISC;
     n->sock = ENET_SOCKET_NULL;
     n->lastattempt = clocktime;
+    n->lastactivity = n->lastping = 0;
 }
 
 void irccleanup()
@@ -690,18 +766,74 @@ void irccleanup()
     loopv(ircnets) if(ircnets[i]->sock != ENET_SOCKET_NULL)
     {
         ircnet *n = ircnets[i];
-        ircsend(n, "QUIT :%s, %s", versionname, versionurl);
+        ircsend(n, "QUIT :%s%s%s", VERSION_NAME, *VERSION_URL ? ", " : "", VERSION_URL);
         ircdiscon(n, "shutdown");
     }
 }
 
+bool ircaddsockets(ENetSocket &maxsock, ENetSocketSet &readset, ENetSocketSet &writeset)
+{
+    int numsocks = 0;
+    loopv(ircnets)
+    {
+        ircnet *n = ircnets[i];
+        if(n->sock != ENET_SOCKET_NULL && n->state > IRC_DISC) switch(n->state)
+        {
+            case IRC_WAIT:
+                ENET_SOCKETSET_ADD(writeset, n->sock);
+                // fall-through
+            case IRC_ONLINE: case IRC_CONN: case IRC_QUIT:
+                maxsock = maxsock == ENET_SOCKET_NULL ? n->sock : max(maxsock, n->sock);
+                ENET_SOCKETSET_ADD(readset, n->sock);
+                numsocks++;
+                break;
+        }
+    }
+    return numsocks > 0;
+}
+
+void ircchecksockets(ENetSocketSet &readset, ENetSocketSet &writeset)
+{
+    loopv(ircnets)
+    {
+        ircnet *n = ircnets[i];
+        if(n->sock != ENET_SOCKET_NULL && n->state > IRC_DISC) switch(n->state)
+        {
+            case IRC_WAIT:
+                if(ENET_SOCKETSET_CHECK(readset, n->sock) || ENET_SOCKETSET_CHECK(writeset, n->sock))
+                {
+                    int error = 0;
+                    if(enet_socket_get_option(n->sock, ENET_SOCKOPT_ERROR, &error) < 0 || error) ircdiscon(n);
+                    else n->state = IRC_ATTEMPT;
+                }
+                break;
+            case IRC_ONLINE: case IRC_CONN: case IRC_QUIT:
+                if(ENET_SOCKETSET_CHECK(readset, n->sock)) switch(ircrecv(n))
+                {
+                    case -3: ircdiscon(n, "read error"); break;
+                    case -2: ircdiscon(n, "connection reset"); break;
+                    case -1: ircdiscon(n, "invalid connection"); break;
+                    case 0: break;
+                    default:
+                    {
+                        ircparse(n);
+                        n->lastactivity = clocktime;
+                        n->lastping = 0;
+                        break;
+                    }
+                }
+                break;
+         }
+    }
+}
+
 void ircslice()
 {
     #ifndef STANDALONE
     loopvrev(ircnets)
     {
         ircnet *n = ircnets[i];
-        if((n->sock == ENET_SOCKET_NULL || n->state == IRC_DISC) && n->updated&IRCUP_LEAVE)
+        if((n->sock == ENET_SOCKET_NULL || n->state <= IRC_DISC) && n->updated&IRCUP_LEAVE)
         {
             delete n;
             ircnets.remove(i);
@@ -722,11 +854,16 @@ void ircslice()
         {
             switch(n->state)
             {
+                case IRC_WAIT:
+                {
+                    if(!n->lastattempt || clocktime-n->lastattempt >= 60) ircdiscon(n);
+                    break;
+                }
                 case IRC_ATTEMPT:
                 {
                     if(*n->passkey) ircsend(n, "PASS %s", n->passkey);
                     ircsend(n, "NICK %s", n->nick);
-                    ircsend(n, "USER %s +iw %s :%s v%s-%s %d bit (%s)", versionuname, versionuname, versionname, versionstring, CUR_PLATFORM, CUR_ARCH, versionrelease);
+                    ircsend(n, "USER %s +iw %s :%s v%s-%s%d (%s)", VERSION_UNAME, VERSION_UNAME, VERSION_NAME, VERSION_STRING, versionplatname, versionarch, VERSION_RELEASE);
                     n->state = IRC_CONN;
                     loopvj(n->channels)
                     {
@@ -749,22 +886,26 @@ void ircslice()
                 }
                 case IRC_CONN:
                 {
-                    if(n->state == IRC_CONN && clocktime-n->lastattempt >= 60) ircdiscon(n, "connection attempt timed out");
-                    else switch(ircrecv(n))
+                    if(n->state == IRC_CONN && (!n->lastattempt || clocktime-n->lastattempt >= 60))
+                        ircdiscon(n, "connection attempt timed out");
+                    if(!n->lastactivity)
                     {
-                        case -3: ircdiscon(n, "read error"); break;
-                        case -2: ircdiscon(n, "connection reset"); break;
-                        case -1: ircdiscon(n, "invalid connection"); break;
-                        case 0: break;
-                        default: ircparse(n); break;
+                        n->lastactivity = clocktime;
+                        n->lastping = 0;
+                    }
+                    else if(clocktime-n->lastactivity >= 120)
+                    {
+                        if(!n->lastping)
+                        {
+                            ircsend(n, "PING %d", clocktime);
+                            n->lastping = clocktime;
+                        }
+                        else if(clocktime-n->lastping >= 120) ircdiscon(n, "connection timed out");
                     }
                     break;
                 }
-                default:
-                {
-                    ircdiscon(n, "encountered unknown connection state");
-                    break;
-                }
+                case IRC_QUIT: ircdiscon(n, "closing link"); break;
+                default: ircdiscon(n, "encountered unknown connection state"); break;
             }
         }
         else if(!n->lastattempt || clocktime-n->lastattempt >= 60) ircestablish(n);
@@ -894,7 +1035,7 @@ bool ircnetgui(guient *g, ircnet *n, bool tab)
     return true;
 }
 
-static const char *ircstates[IRC_MAX] = { "\froffline", "\foconnecting", "\fynegotiating", "\fgonline" };
+static const char * const ircstates[IRC_MAX] = { "\fowaiting", "\froffline", "\foconnecting", "\fynegotiating", "\fgonline", "\foquitting" };
 bool ircgui(guient *g, const char *s)
 {
     g->strut(94);
@@ -905,7 +1046,7 @@ bool ircgui(guient *g, const char *s)
         {
             if(!ircnetgui(g, n, false)) return false;
         }
-        else g->textf("not currently connected to %s", 0xFFFFFF, NULL, 0, s);
+        else g->textf("not currently connected to %s", 0xFFFFFF, NULL, 0, -1, s);
     }
     else
     {
@@ -914,9 +1055,9 @@ bool ircgui(guient *g, const char *s)
         {
             ircnet *n = ircnets[i];
             uilist(*g, {
-                g->buttonf("%s via %s:[%d]", 0xFFFFFF, NULL, 0, true, n->name, n->serv, n->port);
+                g->buttonf("%s via %s:[%d]", 0xFFFFFF, NULL, 0, -1, true, n->name, n->serv, n->port);
                 g->space(1);
-                g->buttonf("\fs%s\fS as %s", 0xFFFFFF, NULL, 0, true, ircstates[n->state], n->nick);
+                g->buttonf("\fs%s\fS as %s", 0xFFFFFF, NULL, 0, -1, true, ircstates[n->state], n->nick);
             });
             nets++;
         }
