@@ -1,5 +1,4 @@
 // main.cpp: initialisation & main loop
-// main.cpp: initialisation & main loop
 
 #include "engine.h"
 #include <signal.h>
@@ -36,13 +35,14 @@ void showcursor(bool show)
 #endif
 }
 
-void setcaption(const char *text)
+void setcaption(const char *text, const char *text2)
 {
-    static string caption = "";
-    defformatstring(newcaption)("%s v%s-%s %d bit (%s)%s%s", versionname, versionstring, CUR_PLATFORM, CUR_ARCH, versionrelease, text ? ": " : "", text ? text : "");
-    if(strcmp(caption, newcaption))
+    static string prevtext = "", prevtext2 = "";
+    if(strcmp(text, prevtext) || strcmp(text2, prevtext2))
     {
-        copystring(caption, newcaption);
+        copystring(prevtext, text);
+        copystring(prevtext2, text2);
+        defformatstring(caption)("%s v%s-%s%d (%s)%s%s%s%s", VERSION_NAME, VERSION_STRING, versionplatname, versionarch, VERSION_RELEASE, text[0] ? ": " : "", text, text2[0] ? " - " : "", text2);
         SDL_WM_SetCaption(caption, NULL);
     }
 }
@@ -65,22 +65,12 @@ void inputgrab(bool on)
 VARF(0, grabinput, 0, 0, 1, inputgrab(grabinput!=0));
 VAR(IDF_PERSIST, autograbinput, 0, 1, 1);
 
-void setscreensaver(bool active)
-{
-#ifdef WIN32
-    SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, active ? 1 : 0, NULL, 0);
-    SystemParametersInfo(SPI_SETLOWPOWERACTIVE, active ? 1 : 0, NULL, 0);
-    SystemParametersInfo(SPI_SETPOWEROFFACTIVE, active ? 1 : 0, NULL, 0);
-#endif
-}
-
 extern void cleargamma();
 
 void cleanup()
 {
     recorder::stop();
     cleanupserver();
-    setscreensaver(true);
     showcursor(true);
 //#ifdef FAKESHOWCURSOR
 //    if(scursor) SDL_FreeCursor(scursor);
@@ -116,14 +106,13 @@ void fatal(const char *s, ...)    // failure exit
 {
     if(++errors <= 2) // print up to one extra recursive error
     {
-        defvformatstring(msg, s, s);
+        defvformatbigstring(msg, s, s);
         if(logfile) logoutf("%s", msg);
         #ifndef WIN32
         fprintf(stderr, "%s\n", msg);
         #endif
         if(errors <= 1) // avoid recursion
         {
-            setscreensaver(true);
             if(SDL_WasInit(SDL_INIT_VIDEO))
             {
                 showcursor(true);
@@ -131,7 +120,7 @@ void fatal(const char *s, ...)    // failure exit
                 cleargamma();
             }
             #ifdef WIN32
-            defformatstring(cap)("%s: Error", versionname);
+            defformatstring(cap)("%s: Error", VERSION_NAME);
             MessageBox(NULL, msg, cap, MB_OK|MB_SYSTEMMODAL);
             #endif
             SDL_Quit();
@@ -217,10 +206,10 @@ void writeinitcfg()
     f->printf("shaders %d\n", useshaders);
     f->printf("shaderprecision %d\n", shaderprecision);
     f->printf("forceglsl %d\n", forceglsl);
-    extern int soundmono, soundchans, soundbufferlen, soundfreq;
+    extern int soundmono, soundmixchans, soundbuflen, soundfreq;
     f->printf("soundmono %d\n", soundmono);
-    f->printf("soundchans %d\n", soundchans);
-    f->printf("soundbufferlen %d\n", soundbufferlen);
+    f->printf("soundmixchans %d\n", soundmixchans);
+    f->printf("soundbuflen %d\n", soundbuflen);
     f->printf("soundfreq %d\n", soundfreq);
     f->printf("verbose %d\n", verbose);
     delete f;
@@ -236,12 +225,12 @@ void screenshot(char *sname)
     glReadPixels(0, 0, screen->w, screen->h, GL_RGB, GL_UNSIGNED_BYTE, image.data);
     string fname;
     if(sname && *sname) copystring(fname, sname);
-    else formatstring(fname)("screenshots/%s", *filetimeformat ? gettime(clocktime, filetimeformat) : (*mapname ? mapname : "screen"));
+    else formatstring(fname)("screenshots/%s", *filetimeformat ? gettime(filetimelocal ? currenttime : clocktime, filetimeformat) : (*mapname ? mapname : "screen"));
     saveimage(fname, image, imageformat, compresslevel, true);
 }
 
-COMMAND(0, screenshot, "s");
-COMMAND(0, quit, "");
+ICOMMAND(0, screenshot, "s", (char *s), if(!(identflags&IDF_WORLD)) screenshot(s));
+ICOMMAND(0, quit, "", (void), if(!(identflags&IDF_WORLD)) quit());
 
 void setfullscreen(bool enable, bool force)
 {
@@ -258,7 +247,7 @@ void setfullscreen(bool enable, bool force)
 #endif
 }
 
-VARF(0, fullscreen, 0, 1, 1, setfullscreen(fullscreen!=0));
+VARF(0, fullscreen, 0, 1, 1, if(!(identflags&IDF_WORLD)) setfullscreen(fullscreen!=0));
 
 void screenres(int *w, int *h)
 {
@@ -476,7 +465,7 @@ void resetgl()
     extern void reloadshaders();
 
     inbetweenframes = false;
-    if(!reloadtexture("textures/notexture") || !reloadtexture("textures/blank") || !reloadtexture("textures/logo") || !reloadtexture("textures/cube2badge"))
+    if(!reloadtexture(notexturetex) || !reloadtexture(blanktex) || !reloadtexture(logotex) || !reloadtexture(badgetex))
         fatal("failed to reload core textures");
     reloadfonts();
     inbetweenframes = true;
@@ -488,7 +477,7 @@ void resetgl()
     allchanged(true);
 }
 
-COMMAND(0, resetgl, "");
+ICOMMAND(0, resetgl, "", (void), if(!(identflags&IDF_WORLD)) resetgl());
 
 bool activewindow = true, warping = false, minimized = false;
 
@@ -655,9 +644,9 @@ void checkinput()
     }
 }
 
-void swapbuffers()
+void swapbuffers(bool overlay)
 {
-    recorder::capture();
+    recorder::capture(overlay);
     SDL_GL_SwapBuffers();
 }
 
@@ -693,14 +682,14 @@ void stackdumper(unsigned int type, EXCEPTION_POINTERS *ep)
     if(!ep) fatal("unknown type");
     EXCEPTION_RECORD *er = ep->ExceptionRecord;
     CONTEXT *context = ep->ContextRecord;
-    string out, t;
-    formatstring(out)("%s Win32 Exception: 0x%x [0x%x]\n\n", versionname, er->ExceptionCode, er->ExceptionCode==EXCEPTION_ACCESS_VIOLATION ? er->ExceptionInformation[1] : -1);
+    bigstring out, t;
+    formatbigstring(out)("%s Win32 Exception: 0x%x [0x%x]\n\n", VERSION_NAME, er->ExceptionCode, er->ExceptionCode==EXCEPTION_ACCESS_VIOLATION ? er->ExceptionInformation[1] : -1);
     SymInitialize(GetCurrentProcess(), NULL, TRUE);
 #ifdef _AMD64_
     STACKFRAME64 sf = {{context->Rip, 0, AddrModeFlat}, {}, {context->Rbp, 0, AddrModeFlat}, {context->Rsp, 0, AddrModeFlat}, 0};
     while(::StackWalk64(IMAGE_FILE_MACHINE_AMD64, GetCurrentProcess(), GetCurrentThread(), &sf, context, NULL, ::SymFunctionTableAccess, ::SymGetModuleBase, NULL))
     {
-        union { IMAGEHLP_SYMBOL64 sym; char symext[sizeof(IMAGEHLP_SYMBOL64) + sizeof(string)]; };
+        union { IMAGEHLP_SYMBOL64 sym; char symext[sizeof(IMAGEHLP_SYMBOL64) + sizeof(bigstring)]; };
         sym.SizeOfStruct = sizeof(sym);
         sym.MaxNameLength = sizeof(symext) - sizeof(sym);
         IMAGEHLP_LINE64 line;
@@ -712,7 +701,7 @@ void stackdumper(unsigned int type, EXCEPTION_POINTERS *ep)
     STACKFRAME sf = {{context->Eip, 0, AddrModeFlat}, {}, {context->Ebp, 0, AddrModeFlat}, {context->Esp, 0, AddrModeFlat}, 0};
     while(::StackWalk(IMAGE_FILE_MACHINE_I386, GetCurrentProcess(), GetCurrentThread(), &sf, context, NULL, ::SymFunctionTableAccess, ::SymGetModuleBase, NULL))
     {
-        union { IMAGEHLP_SYMBOL sym; char symext[sizeof(IMAGEHLP_SYMBOL) + sizeof(string)]; };
+        union { IMAGEHLP_SYMBOL sym; char symext[sizeof(IMAGEHLP_SYMBOL) + sizeof(bigstring)]; };
         sym.SizeOfStruct = sizeof(sym);
         sym.MaxNameLength = sizeof(symext) - sizeof(sym);
         IMAGEHLP_LINE line;
@@ -722,8 +711,8 @@ void stackdumper(unsigned int type, EXCEPTION_POINTERS *ep)
 #endif
         {
             char *del = strrchr(line.FileName, '\\');
-            formatstring(t)("%s - %s [%d]\n", sym.Name, del ? del + 1 : line.FileName, line.LineNumber);
-            concatstring(out, t);
+            formatbigstring(t)("%s - %s [%d]\n", sym.Name, del ? del + 1 : line.FileName, line.LineNumber);
+            concatbigstring(out, t);
         }
     }
     fatal(out);
@@ -819,7 +808,7 @@ void progress(float bar1, const char *text1, float bar2, const char *text2)
     interceptkey(SDLK_UNKNOWN); // keep the event queue awake to avoid 'beachball' cursor
     #endif
 
-    setsvar("progresstitle", text1 ? text1 : "loading..");
+    setsvar("progresstitle", text1 ? text1 : "please wait..");
     setfvar("progressamt", bar1);
     setsvar("progresstext", text2 ? text2 : "");
     setfvar("progresspart", bar2);
@@ -831,10 +820,36 @@ void progress(float bar1, const char *text1, float bar2, const char *text2)
     }
 
     progressing = true;
-    loopi(2) { drawnoview(); swapbuffers(); }
+    loopi(2) { drawnoview(); swapbuffers(false); }
     progressing = false;
 }
 
+
+bool pixeling = false;
+bvec pixel(0, 0, 0);
+char *pixelact = NULL;
+
+ICOMMAND(0, printpixel, "", (void), conoutft(CON_SELF, "pixel = 0x%.6X (%d, %d, %d)", pixel.tohexcolor(), pixel.r, pixel.g, pixel.b));
+ICOMMAND(0, getpixel, "i", (int *n), {
+    switch(*n)
+    {
+        case 1: intret(pixel.r); break;
+        case 2: intret(pixel.g); break;
+        case 3: intret(pixel.b); break;
+        case 0: default: intret(pixel.tohexcolor()); break;
+    }
+});
+
+void readpixel(char *act)
+{
+    if(pixeling) return;
+    if(!editmode) { conoutf("\froperation only allowed in edit mode"); return; }
+    if(pixelact) delete[] pixelact;
+    pixelact = act && *act ? newstring(act) : NULL;
+    pixeling = true;
+}
+ICOMMAND(0, readpixel, "s", (char *act), readpixel(act));
+
 VAR(0, numcpus, 1, 1, 16);
 
 int main(int argc, char **argv)
@@ -848,7 +863,9 @@ int main(int argc, char **argv)
     #endif
     #endif
 
-    clocktime = time(NULL); // initialise
+    currenttime = time(NULL); // initialise
+    clocktime = mktime(gmtime(&currenttime));
+    clockoffset = currenttime-clocktime;
 
     setlogfile(NULL);
     setlocations(true);
@@ -863,6 +880,7 @@ int main(int argc, char **argv)
         }
     }
     setlogfile("log.txt");
+    setcrc(argv[0]);
     execfile("init.cfg", false);
     for(int i = 1; i<argc; i++)
     {
@@ -923,6 +941,13 @@ int main(int argc, char **argv)
     initgame();
 
     conoutf("loading sdl..");
+    #ifdef WIN32
+    SetEnvironmentVariable("SDL_DISABLE_LOCK_KEYS", "1");
+    SetEnvironmentVariable("SDL_VIDEO_ALLOW_SCREENSAVER", "0");
+    #else
+    setenv("SDL_DISABLE_LOCK_KEYS", "1", 1);
+    setenv("SDL_VIDEO_ALLOW_SCREENSAVER", "0", 1);
+    #endif
     int par = 0;
     #ifdef _DEBUG
     par = SDL_INIT_NOPARACHUTE;
@@ -936,9 +961,9 @@ int main(int argc, char **argv)
     //#endif
 
     par |= SDL_INIT_TIMER|SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE;
-    if(SDL_Init(par) < 0) fatal("Unable to initialize SDL: %s", SDL_GetError());
+    if(SDL_Init(par) < 0) fatal("error initialising SDL: %s", SDL_GetError());
 
-    conoutf("loading video mode..");
+    conoutf("loading video..");
     const SDL_VideoInfo *video = SDL_GetVideoInfo();
     if(video)
     {
@@ -948,9 +973,7 @@ int main(int argc, char **argv)
     int usedcolorbits = 0, useddepthbits = 0, usedfsaa = 0;
     setupscreen(usedcolorbits, useddepthbits, usedfsaa);
 
-    conoutf("loading video misc..");
     showcursor(false);
-    setscreensaver(false);
     keyrepeat(false);
     setcaption("please wait..");
 
@@ -971,15 +994,15 @@ int main(int argc, char **argv)
     conoutf("loading gl..");
     gl_checkextensions();
     gl_init(scr_w, scr_h, usedcolorbits, useddepthbits, usedfsaa);
-    if(!(notexture = textureload("textures/notexture")) ||
-        !(blanktexture = textureload("textures/blank")))
+    if(!(notexture = textureload(notexturetex)) || !(blanktexture = textureload(blanktex)))
         fatal("could not find core textures");
 
     conoutf("loading sound..");
     initsound();
 
-    conoutf("loading defaults..");
+    game::start();
 
+    conoutf("loading defaults..");
     if(!execfile("config/stdlib.cfg", false)) fatal("cannot find data files");
     if(!setfont("default")) fatal("no default font specified");
     inbetweenframes = true;
@@ -1015,11 +1038,12 @@ int main(int argc, char **argv)
     if(SDL_GetWMInfo(&wminfo)) ShowWindow(wminfo.window, SW_SHOW);
 #endif
     if(autograbinput) setvar("grabinput", 1, true);
+    capslockon = capslocked();
+    numlockon = numlocked();
     ignoremousemotion();
 
     localconnect(false);
     resetfps();
-    UI::setup();
 
     for(int frameloops = 0; ; frameloops = frameloops >= INT_MAX-1 ? MAXFPSHISTORY+1 : frameloops+1)
     {
@@ -1054,8 +1078,25 @@ int main(int argc, char **argv)
                 swapbuffers();
                 inbetweenframes = true;
             }
-            defformatstring(cap)("%s - %s", game::gametitle(), game::gametext());
-            setcaption(cap);
+            if(pixeling)
+            {
+                if(editmode)
+                {
+                    glReadPixels(screen->w/2, screen->h/2, 1, 1, GL_RGB, GL_UNSIGNED_BYTE, &pixel.v[0]);
+                    if(pixelact) execute(pixelact);
+                }
+                if(pixelact) delete[] pixelact;
+                pixelact = NULL;
+                pixeling = false;
+            }
+            if(*progresstitle || progressamt > 0)
+            {
+                setsvar("progresstitle", "");
+                setsvar("progresstext", "");
+                setfvar("progressamt", 0.f);
+                setfvar("progresspart", 0.f);
+            }
+            setcaption(game::gametitle(), game::gametext());
         }
     }
 
