@@ -1,3 +1,11 @@
+#ifdef WIN32
+#define FD_SETSIZE 4096
+#else
+#include <sys/types.h>
+#undef __FD_SETSIZE
+#define __FD_SETSIZE 4096
+#endif
+
 #include "engine.h"
 #include <enet/time.h>
 
@@ -6,8 +14,6 @@
 #define SERVER_TIME (35*60*1000)
 #define AUTH_TIME (30*1000)
 #define DUP_LIMIT 16
-#define PING_TIME 3000
-#define PING_RETRY 5
 
 VAR(0, masterserver, 0, 0, 1);
 VAR(0, masterport, 1, MASTER_PORT, VAR_MAX);
@@ -15,9 +21,12 @@ SVAR(0, masterip, "");
 SVAR(0, masterscriptclient, "");
 SVAR(0, masterscriptserver, "");
 
+VAR(0, masterpingdelay, 1000, 3000, VAR_MAX);
+VAR(0, masterpingtries, 1, 5, VAR_MAX);
+
 struct authuser
 {
-    char *name, *flags;
+    char *name, *flags, *email;
     void *pubkey;
 };
 
@@ -27,6 +36,7 @@ struct authreq
     uint id;
     void *answer;
     authuser *user;
+    string hostname;
 };
 
 struct masterclient
@@ -62,7 +72,7 @@ void setupmaster()
 {
     if(masterserver)
     {
-        conoutf("init: master (%s:%d)", *masterip ? masterip : "*", masterport);
+        conoutf("loading master (%s:%d)..", *masterip ? masterip : "*", masterport);
         ENetAddress address = { ENET_HOST_ANY, enet_uint16(masterport) };
         if(*masterip && enet_address_set_host(&address, masterip) < 0) fatal("failed to resolve master address: %s", masterip);
         if((mastersocket = enet_socket_create(ENET_SOCKET_TYPE_STREAM)) == ENET_SOCKET_NULL) fatal("failed to create master server socket");
@@ -72,8 +82,6 @@ void setupmaster()
         if(enet_socket_set_option(mastersocket, ENET_SOCKOPT_NONBLOCK, 1) < 0) fatal("failed to make master server socket non-blocking");
         if(!setuppingsocket(&address)) fatal("failed to create ping socket");
         starttime = clocktime;
-        char *ct = ctime(&starttime);
-        if(strchr(ct, '\n')) *strchr(ct, '\n') = '\0';
         conoutf("master server started on %s:[%d]", *masterip ? masterip : "localhost", masterport);
     }
 }
@@ -86,29 +94,37 @@ void masterout(masterclient &c, const char *msg, int len = 0)
 
 void masteroutf(masterclient &c, const char *fmt, ...)
 {
-    string msg;
+    bigstring msg;
     va_list args;
     va_start(args, fmt);
-    vformatstring(msg, fmt, args);
+    vformatbigstring(msg, fmt, args);
     va_end(args);
     masterout(c, msg);
 }
 
 static hashtable<char *, authuser> authusers;
 
-void addauth(char *name, char *flags, char *pubkey)
+void addauth(char *name, char *flags, char *pubkey, char *email)
 {
+    string authname;
+    if(filterstring(authname, name, true, true, true, true, 100)) name = authname;
+    if(authusers.access(name))
+    {
+        conoutf("auth handle '%s' already exists, skipping (%s)", name, email);
+        return;
+    }
     name = newstring(name);
     authuser &u = authusers[name];
     u.name = name;
     u.flags = newstring(flags);
     u.pubkey = parsepubkey(pubkey);
+    u.email = newstring(email);
 }
-COMMAND(0, addauth, "sss");
+COMMAND(0, addauth, "ssss");
 
 void clearauth()
 {
-    enumerate(authusers, authuser, u, { delete[] u.name; delete[] u.flags; freepubkey(u.pubkey); });
+    enumerate(authusers, authuser, u, { delete[] u.name; delete[] u.flags; delete[] u.email; freepubkey(u.pubkey); });
     authusers.clear();
 }
 COMMAND(0, clearauth, "");
@@ -129,31 +145,28 @@ void purgeauths(masterclient &c)
     if(expired > 0) c.authreqs.remove(0, expired);
 }
 
-void reqauth(masterclient &c, uint id, char *name)
+void reqauth(masterclient &c, uint id, char *name, char *hostname)
 {
     purgeauths(c);
 
-    char *ct = ctime(&clocktime);
-    if(ct)
-    {
-        char *newline = strchr(ct, '\n');
-        if(newline) *newline = '\0';
-    }
-    string ip;
+    string ip, host;
     if(enet_address_get_host_ip(&c.address, ip, sizeof(ip)) < 0) copystring(ip, "-");
-    conoutf("%s: attempting \"%s\" as %u from %s\n", ct ? ct : "-", name, id, ip);
+    copystring(host, hostname && *hostname ? hostname : "-");
 
     authuser *u = authusers.access(name);
     if(!u)
     {
         masteroutf(c, "failauth %u\n", id);
+        conoutf("failed '%s' (%u) from %s on server %s (NOTFOUND)\n", name, id, host, ip);
         return;
     }
+    conoutf("attempting '%s' (%u) from %s on server %s\n", name, id, host, ip);
 
     authreq &a = c.authreqs.add();
     a.user = u;
     a.reqtime = totalmillis;
     a.id = id;
+    copystring(a.hostname, host);
     uint seed[3] = { uint(starttime), uint(totalmillis), randomMT() };
     static vector<char> buf;
     buf.setsize(0);
@@ -165,20 +178,19 @@ void reqauth(masterclient &c, uint id, char *name)
 void confauth(masterclient &c, uint id, const char *val)
 {
     purgeauths(c);
-
+    string ip;
+    if(enet_address_get_host_ip(&c.address, ip, sizeof(ip)) < 0) copystring(ip, "-");
     loopv(c.authreqs) if(c.authreqs[i].id == id)
     {
-        string ip;
-        if(enet_address_get_host_ip(&c.address, ip, sizeof(ip)) < 0) copystring(ip, "-");
         if(checkchallenge(val, c.authreqs[i].answer))
         {
             masteroutf(c, "succauth %u \"%s\" \"%s\"\n", id, c.authreqs[i].user->name, c.authreqs[i].user->flags);
-            conoutf("succeeded %u (%s [%s]) from %s\n", id, c.authreqs[i].user->name, c.authreqs[i].user->flags, ip);
+            conoutf("succeeded '%s' [%s] (%u) from %s on server %s\n", c.authreqs[i].user->name, c.authreqs[i].user->flags, id, c.authreqs[i].hostname, ip);
         }
         else
         {
             masteroutf(c, "failauth %u\n", id);
-            conoutf("failed %u (%s) from %s\n", id, c.authreqs[i].user->name, ip);
+            conoutf("failed '%s' (%u) from %s on server %s (BADKEY)\n", c.authreqs[i].user->name, id, c.authreqs[i].hostname, ip);
         }
         freechallenge(c.authreqs[i].answer);
         c.authreqs.remove(i--);
@@ -191,7 +203,7 @@ void purgemasterclient(int n)
 {
     masterclient &c = *masterclients[n];
     enet_socket_destroy(c.socket);
-    conoutf("master peer %s disconnected", c.name);
+    if(verbose) conoutf("master peer %s disconnected", c.name);
     delete masterclients[n];
     masterclients.remove(n);
 }
@@ -215,6 +227,8 @@ void checkmasterpongs()
                 c.lastpong = totalmillis ? totalmillis : 1;
                 c.listserver = true;
                 c.shouldping = false;
+                masteroutf(c, "echo \"ping reply confirmed (on port %d), server is now listed\"\n", addr.port);
+                conoutf("master peer %s responded to ping request on port %d successfully",  c.name, addr.port);
                 break;
             }
         }
@@ -285,7 +299,7 @@ bool checkmasterclientinput(masterclient &c)
         bool found = false, server = !strcmp(w[0], "server");
         if((server || !strcmp(w[0], "quick")) && !c.ishttp)
         {
-            c.port = MASTER_PORT;
+            c.port = SERVER_PORT;
             c.lastactivity = totalmillis ? totalmillis : 1;
             if(!server)
             {
@@ -295,24 +309,33 @@ bool checkmasterclientinput(masterclient &c)
             }
             else
             {
-                if(w[1]) c.port = clamp(atoi(w[1]), 1, VAR_MAX);
-                c.shouldping = true;
-                c.numpings = 0;
-                c.lastcontrol = controlversion;
-                loopv(control) if(control[i].flag == ipinfo::LOCAL)
-                    masteroutf(c, "%s %u %u\n", ipinfotypes[control[i].type], control[i].ip, control[i].mask);
-                if(c.isserver)
+                if(w[1] && *w[1]) c.port = clamp(atoi(w[1]), 1, VAR_MAX);
+                ENetAddress address = { ENET_HOST_ANY, enet_uint16(c.port) };
+                if(w[2] && *w[2] && strcmp(w[2], "*") && (enet_address_set_host(&address, w[2]) < 0 || address.host != c.address.host))
                 {
-                    masteroutf(c, "echo \"server updated, sending ping request\"\n");
-                    conoutf("master peer %s updated server info",  c.name);
+                    c.listserver = c.shouldping = false;
+                    masteroutf(c, "echo \"server IP '%s' does not match origin '%s', server will not be listed\n", w[2], c.name);
                 }
                 else
                 {
-                    if(*masterscriptserver) masteroutf(c, "%s\n", masterscriptserver);
-                    masteroutf(c, "echo \"server registered, sending ping request\"\n");
-                    conoutf("master peer %s registered as a server",  c.name);
+                    c.shouldping = true;
+                    c.numpings = 0;
+                    c.lastcontrol = controlversion;
+                    loopv(control) if(control[i].flag == ipinfo::LOCAL)
+                        masteroutf(c, "%s %u %u \"%s\"\n", ipinfotypes[control[i].type], control[i].ip, control[i].mask, control[i].reason);
+                    if(c.isserver)
+                    {
+                        masteroutf(c, "echo \"server updated (port %d), sending ping request (on port %d)\"\n", c.port, c.port+1);
+                        conoutf("master peer %s updated server info (%d)",  c.name, c.port);
+                    }
+                    else
+                    {
+                        if(*masterscriptserver) masteroutf(c, "%s\n", masterscriptserver);
+                        masteroutf(c, "echo \"server registered (port %d), sending ping request (on port %d)\"\n", c.port, c.port+1);
+                        conoutf("master peer %s registered as a server (%d)", c.name, c.port);
+                    }
+                    c.isserver = true;
                 }
-                c.isserver = true;
             }
             found = true;
         }
@@ -340,15 +363,15 @@ bool checkmasterclientinput(masterclient &c)
         }
         if(c.isserver || c.isquick)
         {
-            if(!strcmp(w[0], "reqauth")) { reqauth(c, uint(atoi(w[1])), w[2]); found = true; }
+            if(!strcmp(w[0], "reqauth")) { reqauth(c, uint(atoi(w[1])), w[2], w[3]); found = true; }
             if(!strcmp(w[0], "confauth")) { confauth(c, uint(atoi(w[1])), w[2]); found = true; }
         }
-        if(w[0] && !found)
+        if(w[0] && *w[0] && !found)
         {
             masteroutf(c, "error \"unknown command %s\"\n", w[0]);
             conoutf("master peer %s (client) sent unknown command: %s",  c.name, w[0]);
         }
-        loopj(numargs) if(w[j]) delete[] w[j];
+        loopi(numargs) DELETEA(w[i]);
     }
     c.inputpos = &c.input[c.inputpos] - p;
     memmove(c.input, p, c.inputpos);
@@ -369,9 +392,9 @@ void checkmaster()
     {
         masterclient &c = *masterclients[i];
         if(c.authreqs.length()) purgeauths(c);
-        if(c.shouldping && (!c.lastping || ((!c.lastpong || ENET_TIME_GREATER(c.lastping, c.lastpong)) && ENET_TIME_DIFFERENCE(totalmillis, c.lastping) > PING_TIME)))
+        if(c.shouldping && (!c.lastping || ((!c.lastpong || ENET_TIME_GREATER(c.lastping, c.lastpong)) && ENET_TIME_DIFFERENCE(totalmillis, c.lastping) > uint(masterpingdelay))))
         {
-            if(c.numpings < PING_RETRY)
+            if(c.numpings < masterpingtries)
             {
                 static const uchar ping[] = { 1 };
                 ENetBuffer buf;
@@ -384,15 +407,14 @@ void checkmaster()
             }
             else
             {
-                c.listserver = false;
-                c.shouldping = false;
-                masteroutf(c, "echo \"ping attempts failed, your server will not be listed\n");
+                c.listserver = c.shouldping = false;
+                masteroutf(c, "error \"ping attempts failed (tried %d times on port %d), server will not be listed\"\n", c.numpings, c.port+1);
             }
         }
         if(c.isserver && c.lastcontrol < controlversion)
         {
             loopv(control) if(control[i].flag == ipinfo::LOCAL && control[i].version > c.lastcontrol)
-                masteroutf(c, "%s %u %u\n", ipinfotypes[control[i].type], control[i].ip, control[i].mask);
+                masteroutf(c, "%s %u %u %s\n", ipinfotypes[control[i].type], control[i].ip, control[i].mask, control[i].reason);
             c.lastcontrol = controlversion;
         }
         if(c.outputpos < c.output.length()) ENET_SOCKETSET_ADD(writeset, c.socket);
@@ -407,7 +429,7 @@ void checkmaster()
     {
         ENetAddress address;
         ENetSocket masterclientsocket = enet_socket_accept(mastersocket, &address);
-        if(masterclients.length() >= MASTER_LIMIT || (checkipinfo(control, ipinfo::BAN, address.host) && !checkipinfo(control, ipinfo::ALLOW, address.host)))
+        if(masterclients.length() >= MASTER_LIMIT || (checkipinfo(control, ipinfo::BAN, address.host) && !checkipinfo(control, ipinfo::EXCEPT, address.host)))
             enet_socket_destroy(masterclientsocket);
         else if(masterclientsocket!=ENET_SOCKET_NULL)
         {
@@ -423,8 +445,8 @@ void checkmaster()
             c->socket = masterclientsocket;
             c->lastactivity = totalmillis ? totalmillis : 1;
             masterclients.add(c);
-            enet_address_get_host_ip(&c->address, c->name, sizeof(c->name));
-            conoutf("master peer %s connected", c->name);
+            if(enet_address_get_host_ip(&c->address, c->name, sizeof(c->name)) < 0) copystring(c->name, "unknown");
+            if(verbose) conoutf("master peer %s connected", c->name);
         }
     }
 
@@ -464,7 +486,7 @@ void checkmaster()
             else { purgemasterclient(i--); continue; }
         }
         /* if(c.output.length() > OUTPUT_LIMIT) { purgemasterclient(i--); continue; } */
-        if(ENET_TIME_DIFFERENCE(totalmillis, c.lastactivity) >= (c.isserver ? SERVER_TIME : CLIENT_TIME) || (checkipinfo(control, ipinfo::BAN, c.address.host) && !checkipinfo(control, ipinfo::ALLOW, c.address.host)))
+        if(ENET_TIME_DIFFERENCE(totalmillis, c.lastactivity) >= (c.isserver ? SERVER_TIME : CLIENT_TIME) || (checkipinfo(control, ipinfo::BAN, c.address.host) && !checkipinfo(control, ipinfo::EXCEPT, c.address.host)))
         {
             purgemasterclient(i--);
             continue;
@@ -481,4 +503,3 @@ void reloadmaster()
 {
     clearauth();
 }
-
