@@ -15,33 +15,28 @@ struct menu : guicb
     char *name, *header;
     uint *contents, *initscript;
     int passes, menutab, menustart;
-    bool world, useinput, usetitle;
+    bool world, useinput, usetitle, usebgfx, builtin, *keep;
 
-    menu() : name(NULL), header(NULL), contents(NULL), initscript(NULL), passes(0), menutab(0), menustart(0), world(false), useinput(true), usetitle(true) {}
+    menu() : name(NULL), header(NULL), contents(NULL), initscript(NULL), passes(0), menutab(0), menustart(0), world(false), useinput(true), usetitle(true), usebgfx(true), builtin(false), keep(NULL) {}
 
     void gui(guient &g, bool firstpass)
     {
         cgui = &g;
         cmenu = this;
         guipasses = passes;
-        if(!passes) world = (identflags&IDF_WORLD)!=0;
-        if(initscript)
-        {
-            if(world && passes) { WITHWORLD(execute(initscript)); }
-            else execute(initscript);
-        }
-        cgui->start(menustart, menuscale, &menutab, useinput, usetitle);
+        if(!passes) guiactionon = false;
+        int oldflags = identflags;
+        if(world && !builtin) identflags |= IDF_WORLD;
+        if(initscript) execute(initscript);
+        cgui->start(menustart, menuscale, &menutab, useinput, usetitle, usebgfx);
         cgui->tab(header ? header : name);
-        if(contents)
-        {
-            if(world && passes) { WITHWORLD(execute(contents)); }
-            else execute(contents);
-        }
+        if(contents) execute(contents);
         cgui->end();
+        identflags = oldflags;
         guipasses = -1;
         cmenu = NULL;
         cgui = NULL;
-        passes++;
+        if((++passes) <= 0) passes = 1;
     }
 
     virtual void clear() {}
@@ -63,13 +58,14 @@ struct delayedupdate
         float f;
         char *s;
     } val;
+    bool world;
     delayedupdate() : type(ACTION), id(NULL) { val.s = NULL; }
     ~delayedupdate() { if(type == STRING || type == ACTION) DELETEA(val.s); }
 
-    void schedule(const char *s) { type = ACTION; val.s = newstring(s); }
-    void schedule(ident *var, int i) { type = INT; id = var; val.i = i; }
-    void schedule(ident *var, float f) { type = FLOAT; id = var; val.f = f; }
-    void schedule(ident *var, char *s) { type = STRING; id = var; val.s = newstring(s); }
+    void schedule(const char *s) { type = ACTION; val.s = newstring(s); world = (identflags&IDF_WORLD)!=0; }
+    void schedule(ident *var, int i) { type = INT; id = var; val.i = i; world = (identflags&IDF_WORLD)!=0; }
+    void schedule(ident *var, float f) { type = FLOAT; id = var; val.f = f; world = (identflags&IDF_WORLD)!=0; }
+    void schedule(ident *var, char *s) { type = STRING; id = var; val.s = newstring(s); world = (identflags&IDF_WORLD)!=0; }
 
     int getint() const
     {
@@ -106,6 +102,8 @@ struct delayedupdate
 
     void run()
     {
+        int _oldflags = identflags;
+        if(world) identflags |= IDF_WORLD;
         if(type == ACTION) { if(val.s) execute(val.s); }
         else if(id) switch(id->type)
         {
@@ -114,6 +112,7 @@ struct delayedupdate
             case ID_SVAR: setsvarchecked(id, getstring()); break;
             case ID_ALIAS: alias(id->name, getstring()); break;
         }
+        identflags = _oldflags;
     }
 };
 
@@ -122,25 +121,30 @@ static vector<menu *> menustack;
 static vector<delayedupdate> updatelater;
 static bool shouldclearmenu = true, clearlater = false;
 
-void popgui()
+bool popgui(bool skip = true)
 {
-    menu *m = menustack.pop();
+    menu *m = menustack.last();
+    if(skip && m->keep && *m->keep) return false;
+    menustack.pop();
     m->passes = 0;
+    if(m->keep) *m->keep = false;
     m->clear();
+    return true;
 }
 
 void removegui(menu *m)
 {
-    loopv(menustack) if(menustack[i]==m)
+    loopv(menustack) if(menustack[i] == m)
     {
         menustack.remove(i);
         m->passes = 0;
+        if(m->keep) *m->keep = false;
         m->clear();
         return;
     }
 }
 
-void pushgui(menu *m, int pos = -1, int tab = 0)
+void pushgui(menu *m, int pos = -1, int tab = 0, bool *keep = NULL)
 {
     if(menustack.empty()) resetcursor();
     if(pos < 0) menustack.add(m);
@@ -151,10 +155,12 @@ void pushgui(menu *m, int pos = -1, int tab = 0)
         m->menustart = totalmillis;
         if(tab > 0) m->menutab = tab;
         m->usetitle = tab >= 0 ? true : false;
+        m->world = (identflags&IDF_WORLD)!=0;
+        m->keep = keep;
     }
 }
 
-void restoregui(int pos, int tab = 0)
+void restoregui(int pos, int tab = 0, bool *keep = NULL)
 {
     int clear = menustack.length()-pos-1;
     loopi(clear) popgui();
@@ -164,33 +170,42 @@ void restoregui(int pos, int tab = 0)
         m->passes = 0;
         m->menustart = totalmillis;
         if(tab > 0) m->menutab = tab;
+        m->world = (identflags&IDF_WORLD)!=0;
+        m->keep = keep;
     }
 }
 
-void showgui(const char *name, int tab)
+void showgui(const char *name, int tab, bool *keep)
 {
     menu *m = menus.access(name);
     if(!m) return;
     int pos = menustack.find(m);
-    if(pos<0) pushgui(m, -1, tab);
-    else restoregui(pos, tab);
+    if(pos<0) pushgui(m, -1, tab, keep);
+    else if(pos < menustack.length()-1) restoregui(pos, tab, keep);
+    else
+    {
+        m->world = (identflags&IDF_WORLD)!=0;
+        m->menutab = tab;
+        m->keep = keep;
+        return;
+    }
     playsound(S_GUIPRESS, camera1->o, camera1, SND_FORCED);
 }
 
 extern bool closetexgui();
-int cleargui(int n)
+int cleargui(int n, bool skip)
 {
     if(closetexgui()) n--;
     int clear = menustack.length();
     if(n>0) clear = min(clear, n);
-    loopi(clear) popgui();
+    loopi(clear) if(!popgui(skip)) break;
     if(!menustack.empty()) restoregui(menustack.length()-1);
     return clear;
 }
 
 void cleargui_(int *n)
 {
-    intret(cleargui(*n));
+    intret(cleargui(*n, false));
 }
 
 void guishowtitle(int *n)
@@ -199,6 +214,12 @@ void guishowtitle(int *n)
     cmenu->usetitle = *n ? true : false;
 }
 
+void guishowbgfx(int *n)
+{
+    if(!cmenu) return;
+    cmenu->usebgfx = *n ? true : false;
+}
+
 void guistayopen(uint *contents)
 {
     bool oldclearmenu = shouldclearmenu;
@@ -215,14 +236,14 @@ void guinohitfx(uint *contents)
     cgui->allowhitfx(true);
 }
 
-//@DOC name and icon are optional
 SVAR(0, guirollovername, "");
 SVAR(0, guirolloveraction, "");
+SVAR(0, guirollovertype, "");
 
-void guibutton(char *name, char *action, char *altact, char *icon, int *colour)
+void guibutton(char *name, char *action, char *altact, char *icon, int *colour, int *icolour, int *wrap)
 {
     if(!cgui) return;
-    int ret = cgui->button(name, *colour ? *colour : 0xFFFFFF, *icon ? icon : NULL);
+    int ret = cgui->button(name, *colour >= 0 ? *colour : 0xFFFFFF, *icon ? icon : NULL, *icolour >= 0 ? *icolour : 0xFFFFFF, *wrap > 0 ? *wrap : -1);
     if(ret&GUI_UP)
     {
         char *act = NULL;
@@ -238,13 +259,11 @@ void guibutton(char *name, char *action, char *altact, char *icon, int *colour)
     {
         setsvar("guirollovername", name, true);
         setsvar("guirolloveraction", action, true);
+        setsvar("guirollovertype", "button", true);
     }
 }
 
-SVAR(0, guirolloverimgpath, "");
-SVAR(0, guirolloverimgaction, "");
-
-void guiimage(char *path, char *action, float *scale, int *overlaid, char *altpath, char *altact)
+void guiimage(char *path, char *action, float *scale, int *overlaid, char *altpath, char *altact, int *colour)
 {
     if(!cgui) return;
     Texture *t = path[0] ? textureload(path, 0, true, false) : NULL;
@@ -253,7 +272,7 @@ void guiimage(char *path, char *action, float *scale, int *overlaid, char *altpa
         if(*altpath) t = textureload(altpath, 0, true, false);
         if(t == notexture) return;
     }
-    int ret = cgui->image(t, *scale, *overlaid!=0);
+    int ret = cgui->image(t, *scale, *overlaid!=0, *colour >= 0 ? *colour : 0xFFFFFF);
     if(ret&GUI_UP)
     {
         char *act = NULL;
@@ -267,8 +286,9 @@ void guiimage(char *path, char *action, float *scale, int *overlaid, char *altpa
     }
     else if(ret&GUI_ROLLOVER)
     {
-        setsvar("guirolloverimgpath", path, true);
-        setsvar("guirolloverimgaction", action, true);
+        setsvar("guirollovername", path, true);
+        setsvar("guirolloveraction", action, true);
+        setsvar("guirollovertype", "image", true);
     }
 }
 
@@ -295,19 +315,15 @@ void guislice(char *path, char *action, float *scale, float *start, float *end,
     }
     else if(ret&GUI_ROLLOVER)
     {
-        setsvar("guirolloverimgpath", path, true);
-        setsvar("guirolloverimgaction", action, true);
+        setsvar("guirollovername", path, true);
+        setsvar("guirolloveraction", action, true);
+        setsvar("guirollovertype", "image", true);
     }
 }
 
-void guitext(char *name, char *icon, int *colour, int *icolour)
+void guitext(char *name, char *icon, int *colour, int *icolour, int *wrap)
 {
-    if(cgui) cgui->text(name, *colour ? *colour : 0xFFFFFF, icon[0] ? icon : NULL, *icolour ? *icolour : 0xFFFFFF);
-}
-
-void guititle(char *name)
-{
-    if(cgui) cgui->title(name);
+    if(cgui) cgui->text(name, *colour >= 0 ? *colour : 0xFFFFFF, icon[0] ? icon : NULL, *icolour >= 0 ? *icolour : 0xFFFFFF, *wrap > 0 ? *wrap : -1);
 }
 
 void guitab(char *name)
@@ -315,21 +331,42 @@ void guitab(char *name)
     if(cgui) cgui->tab(name);
 }
 
+void guistatus(char *str, int *width)
+{
+    if(cgui) cgui->setstatus("%s", *width, str);
+}
+
+void guitooltip(char *str, int *width)
+{
+    if(cgui) cgui->settooltip("%s", *width, str);
+}
+
 void guibar()
 {
     if(cgui) cgui->separator();
 }
 
-void guibackground(int *colour, int *levels)
+void guifill(int *colour, int *levels)
 {
-    if(cgui) cgui->background(*colour, *levels);
+    if(cgui) cgui->fill(*colour, *levels);
+}
+
+void guioutline(int *colour, int *levels, int *offsetx, int *offsety)
+{
+    if(cgui) cgui->outline(*colour, *levels, 0, *offsetx, *offsety);
+}
+
+void guibackground(int *colour1, float *blend1, int *colour2, float *blend2, int *skinborder, int *levels)
+{
+    if(cgui) cgui->background(*colour1, *blend1, *colour2, *blend2, *skinborder!=0, *levels);
 }
 
 void guistrut(float *strut, int *alt)
 {
     if(cgui)
     {
-        if(*alt) cgui->strut(*strut); else cgui->space(*strut);
+        if(*alt) cgui->strut(*strut);
+        else cgui->space(*strut);
     }
 }
 
@@ -338,6 +375,16 @@ void guispring(int *weight)
     if(cgui) cgui->spring(max(*weight, 1));
 }
 
+void guivisible(uint *body)
+{
+    if(cgui && cgui->visible()) execute(body);
+}
+
+void guivisibletab(uint *body)
+{
+    if(cgui && cgui->visibletab()) execute(body);
+}
+
 void guifont(char *font, uint *body)
 {
     if(cgui)
@@ -348,6 +395,40 @@ void guifont(char *font, uint *body)
     }
 }
 
+int guifontwidth(char *font)
+{
+    if(font && *font) pushfont(font);
+    int width = FONTW;
+    if(font && *font) popfont();
+    return width;
+}
+
+int guifontheight(char *font)
+{
+    if(font && *font) pushfont(font);
+    int height = FONTH;
+    if(font && *font) popfont();
+    return height;
+}
+
+int guitextwidth(char *text, char *font, int wrap)
+{
+    if(font && *font) pushfont(font);
+    int width = 0, height = 0;
+    text_bounds(text, width, height, wrap > 0 ? wrap : -1, TEXT_NO_INDENT);
+    if(font && *font) popfont();
+    return width;
+}
+
+int guitextheight(char *text, char *font, int wrap)
+{
+    if(font && *font) pushfont(font);
+    int width = 0, height = 0;
+    text_bounds(text, width, height, wrap > 0 ? wrap : -1, TEXT_NO_INDENT);
+    if(font && *font) popfont();
+    return height;
+}
+
 template<class T> static void updateval(char *var, T val, char *onchange)
 {
     ident *id = writeident(var);
@@ -403,7 +484,7 @@ void guiprogress(float *percent, float *scale)
     cgui->progress(*percent, *scale);
 }
 
-void guislider(char *var, int *min, int *max, char *onchange, int *reverse, int *scroll, int *colour)
+void guislider(char *var, int *min, int *max, char *onchange, int *reverse, int *scroll, int *colour, int *style, int *scolour)
 {
     if(!cgui || !var || !*var) return;
     int oldval = getval(var), val = oldval, vmin = *max > INT_MIN ? *min : getvarmin(var), vmax = *max > INT_MIN ? *max : getvarmax(var);
@@ -412,11 +493,11 @@ void guislider(char *var, int *min, int *max, char *onchange, int *reverse, int
         int vdef = getvardef(var);
         vmax = vdef > vmin ? vdef*3 : vmin*4;
     }
-    cgui->slider(val, vmin, vmax, *colour ? *colour : 0xFFFFFF, NULL, *reverse ? true : false, *scroll ? true : false);
+    cgui->slider(val, vmin, vmax, *colour >= 0 ? *colour : 0xFFFFFF, NULL, *reverse ? true : false, *scroll ? true : false, *style, *scolour);
     if(val != oldval) updateval(var, val, onchange);
 }
 
-void guilistslider(char *var, char *list, char *onchange, int *reverse, int *scroll, int *colour)
+void guilistslider(char *var, char *list, char *onchange, int *reverse, int *scroll, int *colour, int *style, int *scolour)
 {
     if(!cgui) return;
     vector<int> vals;
@@ -430,11 +511,11 @@ void guilistslider(char *var, char *list, char *onchange, int *reverse, int *scr
     if(vals.empty()) return;
     int val = getval(var), oldoffset = vals.length()-1, offset = oldoffset;
     loopv(vals) if(val <= vals[i]) { oldoffset = offset = i; break; }
-    cgui->slider(offset, 0, vals.length()-1, *colour ? *colour : 0xFFFFFF, intstr(val), *reverse ? true : false, *scroll ? true : false);
+    cgui->slider(offset, 0, vals.length()-1, *colour >= 0 ? *colour : 0xFFFFFF, intstr(val), *reverse ? true : false, *scroll ? true : false, *style, *scolour);
     if(offset != oldoffset) updateval(var, vals[offset], onchange);
 }
 
-void guinameslider(char *var, char *names, char *list, char *onchange, int *reverse, int *scroll, int *colour)
+void guinameslider(char *var, char *names, char *list, char *onchange, int *reverse, int *scroll, int *colour, int *style, int *scolour)
 {
     if(!cgui) return;
     vector<int> vals;
@@ -449,68 +530,86 @@ void guinameslider(char *var, char *names, char *list, char *onchange, int *reve
     int val = getval(var), oldoffset = vals.length()-1, offset = oldoffset;
     loopv(vals) if(val <= vals[i]) { oldoffset = offset = i; break; }
     char *label = indexlist(names, offset);
-    cgui->slider(offset, 0, vals.length()-1, *colour ? *colour : 0xFFFFFF, label, *reverse ? true : false, *scroll ? true : false);
+    cgui->slider(offset, 0, vals.length()-1, *colour >= 0 ? *colour : 0xFFFFFF, label, *reverse ? true : false, *scroll ? true : false, *style, *scolour);
     if(offset != oldoffset) updateval(var, vals[offset], onchange);
     delete[] label;
 }
 
 void guicheckbox(char *name, char *var, float *on, float *off, char *onchange, int *colour)
 {
+    if(!cgui) return;
     bool enabled = getfval(var) != *off, two = getfvarmax(var) == 2, next = two && getfval(var) == 1.0f;
-    if(cgui && cgui->button(name, *colour ? *colour : 0xFFFFFF, enabled ? (two && !next ? "checkboxtwo" : "checkboxon") : "checkbox", 0xFFFFFF, enabled ? false : true)&GUI_UP)
+    int ret = cgui->button(name, *colour >= 0 ? *colour : 0xFFFFFF, enabled ? (two && !next ? "checkboxtwo" : "checkboxon") : "checkbox", 0xFFFFFF);
+    if(ret&GUI_UP) updateval(var, enabled ? (two && next ? 2.0f : *off) : (*on || *off ? *on : 1.0f), onchange);
+    else if(ret&GUI_ROLLOVER)
     {
-        updateval(var, enabled ? (two && next ? 2.0f : *off) : (*on || *off ? *on : 1.0f), onchange);
+        setsvar("guirollovername", name, true);
+        setsvar("guirolloveraction", var, true);
+        setsvar("guirollovertype", "checkbox", true);
     }
 }
 
 void guiradio(char *name, char *var, float *n, char *onchange, int *colour)
 {
+    if(!cgui) return;
     bool enabled = getfval(var)==*n;
-    if(cgui && cgui->button(name, *colour ? *colour : 0xFFFFFF, enabled ? "radioboxon" : "radiobox", *colour ? *colour : 0xFFFFFF, enabled ? false : true)&GUI_UP)
+    int ret = cgui->button(name, *colour >= 0 ? *colour : 0xFFFFFF, enabled ? "radioboxon" : "radiobox", *colour >= 0 ? *colour : 0xFFFFFF);
+    if(ret&GUI_UP)
     {
         if(!enabled) updateval(var, *n, onchange);
     }
+    else if(ret&GUI_ROLLOVER)
+    {
+        setsvar("guirollovername", name, true);
+        setsvar("guirolloveraction", var, true);
+        setsvar("guirollovertype", "radio", true);
+    }
 }
 
 void guibitfield(char *name, char *var, int *mask, char *onchange, int *colour)
 {
+    if(!cgui) return;
     int val = getval(var);
     bool enabled = (val & *mask) != 0;
-    if(cgui && cgui->button(name, *colour ? *colour : 0xFFFFFF, enabled ? "checkboxon" : "checkbox", *colour ? *colour : 0xFFFFFF, enabled ? false : true)&GUI_UP)
+    int ret = cgui->button(name, *colour >= 0 ? *colour : 0xFFFFFF, enabled ? "checkboxon" : "checkbox", *colour >= 0 ? *colour : 0xFFFFFF);
+    if(ret&GUI_UP) updateval(var, enabled ? val & ~*mask : val | *mask, onchange);
+    else if(ret&GUI_ROLLOVER)
     {
-        updateval(var, enabled ? val & ~*mask : val | *mask, onchange);
+        setsvar("guirollovername", name, true);
+        setsvar("guirolloveraction", var, true);
+        setsvar("guirollovertype", "bitfield", true);
     }
 }
 
 //-ve length indicates a wrapped text field of any (approx 260 chars) length, |length| is the field width
-void guifield(char *var, int *maxlength, char *onchange, int *colour, int *focus, char *parent)
+void guifield(char *var, int *maxlength, char *onchange, int *colour, int *focus, char *parent, int *height, char *prompt, int *immediate)
 {
     if(!cgui) return;
     const char *initval = getsval(var);
-    char *result = cgui->field(var, *colour ? *colour : 0xFFFFFF, *maxlength ? *maxlength : 12, 0, initval, EDITORFOCUSED, *focus!=0, parent);
-    if(result) updateval(var, result, onchange);
+    char *result = cgui->field(var, *colour >= 0 ? *colour : 0xFFFFFF, *maxlength ? *maxlength : 12, *height, initval, EDITORFOCUSED, *focus!=0, parent, prompt, *immediate!=0);
+    if(result && (!*immediate || strcmp(initval, result))) updateval(var, result, onchange);
 }
 
 //-ve maxlength indicates a wrapped text field of any (approx 260 chars) length, |maxlength| is the field width
-void guieditor(char *name, int *maxlength, int *height, int *mode, int *colour, int *focus, char *parent)
+void guieditor(char *name, int *maxlength, int *height, int *mode, int *colour, int *focus, char *parent, char *str, char *prompt)
 {
     if(!cgui) return;
-    cgui->field(name, *colour ? *colour : 0xFFFFFF, *maxlength ? *maxlength : 12, *height, NULL, *mode<=0 ? EDITORFOREVER : *mode, *focus!=0, parent);
+    cgui->field(name, *colour >= 0 ? *colour : 0xFFFFFF, *maxlength ? *maxlength : 12, *height, str && *str ? str : NULL, *mode<=0 ? EDITORFOREVER : *mode, *focus!=0, parent, prompt);
     //returns a non-NULL pointer (the currentline) when the user commits, could then manipulate via text* commands
 }
 
 //-ve length indicates a wrapped text field of any (approx 260 chars) length, |length| is the field width
-void guikeyfield(char *var, int *maxlength, char *onchange, int *colour, int *focus, char *parent)
+void guikeyfield(char *var, int *maxlength, char *onchange, int *colour, int *focus, char *parent, char *prompt, int *immediate)
 {
     if(!cgui) return;
     const char *initval = getsval(var);
-    char *result = cgui->keyfield(var, *colour ? *colour : 0xFFFFFF, *maxlength ? *maxlength : -8, 0, initval, EDITORFOCUSED, *focus!=0, parent);
-    if(result) updateval(var, result, onchange);
+    char *result = cgui->keyfield(var, *colour >= 0 ? *colour : 0xFFFFFF, *maxlength ? *maxlength : -8, 0, initval, EDITORFOCUSED, *focus!=0, parent, prompt, *immediate!=0);
+    if(result && (!*immediate || strcmp(initval, result))) updateval(var, result, onchange);
 }
 
 //use text<action> to do more...
 
-void guibody(uint *contents, char *action, char *altact)
+void guibody(uint *contents, char *action, char *altact, uint *onhover)
 {
     if(!cgui) return;
     cgui->pushlist(action[0] ? true : false);
@@ -527,6 +626,7 @@ void guibody(uint *contents, char *action, char *altact)
             if(shouldclearmenu) clearlater = true;
         }
     }
+    else if(ret&GUI_ROLLOVER) execute(onhover);
 }
 
 void guilist(uint *contents)
@@ -574,42 +674,52 @@ void guimodify(char *name, char *contents)
 COMMAND(0, newgui, "sss");
 COMMAND(0, guiheader, "s");
 COMMAND(0, guimodify, "ss");
-COMMAND(0, guibutton, "ssssi");
-COMMAND(0, guitext, "ssii");
+COMMAND(0, guibutton, "ssssbbb");
+COMMAND(0, guitext, "ssbbb");
 COMMANDN(0, cleargui, cleargui_, "i");
 ICOMMAND(0, showgui, "si", (const char *s, int *n), showgui(s, *n));
 COMMAND(0, guishowtitle, "i");
+COMMAND(0, guishowbgfx, "i");
 COMMAND(0, guistayopen, "e");
 COMMAND(0, guinohitfx, "e");
 
-ICOMMAND(0, guicount, "", (), intret(menustack.length()));
-
 COMMAND(0, guilist, "e");
-COMMAND(0, guibody, "ess");
-COMMAND(0, guititle, "s");
+COMMAND(0, guibody, "esse");
 COMMAND(0, guibar, "");
-COMMAND(0, guibackground, "ii");
+COMMAND(0, guifill, "ii");
+COMMAND(0, guioutline, "iiii");
+COMMAND(0, guibackground, "bgbgii");
 COMMAND(0, guistrut,"fi");
 COMMAND(0, guispring, "i");
+COMMAND(0, guivisible, "e");
+COMMAND(0, guivisibletab, "e");
 COMMAND(0, guifont,"se");
-COMMAND(0, guiimage,"ssfiss");
+COMMAND(0, guiimage,"ssfissb");
 COMMAND(0, guislice,"ssfffsss");
 COMMAND(0, guiprogress,"ff");
-COMMAND(0, guislider,"sbbsiii");
-COMMAND(0, guilistslider, "sssiii");
-COMMAND(0, guinameslider, "ssssiii");
-COMMAND(0, guiradio,"ssfsi");
-COMMAND(0, guibitfield, "ssisi");
-COMMAND(0, guicheckbox, "ssffsi");
+COMMAND(0, guislider,"sbbsiibib");
+COMMAND(0, guilistslider, "sssiibib");
+COMMAND(0, guinameslider, "ssssiibib");
+COMMAND(0, guiradio,"ssfsb");
+COMMAND(0, guibitfield, "ssisb");
+COMMAND(0, guicheckbox, "ssffsb");
 COMMAND(0, guitab, "s");
-COMMAND(0, guifield, "sisiis");
-COMMAND(0, guikeyfield, "sisiis");
-COMMAND(0, guieditor, "siiiiis");
+COMMAND(0, guistatus, "si");
+COMMAND(0, guitooltip, "si");
+COMMAND(0, guifield, "sisbisisi");
+COMMAND(0, guikeyfield, "sisbissi");
+COMMAND(0, guieditor, "siiibisss");
+
+ICOMMAND(0, guicount, "", (), intret(menustack.length()));
+ICOMMAND(0, guifontwidth, "s", (char *font), intret(guifontwidth(font)));
+ICOMMAND(0, guifontheight, "s", (char *font), intret(guifontheight(font)));
+ICOMMAND(0, guitextwidth, "ssb", (char *text, char *font, int *wrap), intret(guitextwidth(text, font, *wrap)));
+ICOMMAND(0, guitextheight, "ssb", (char *text, char *font, int *wrap), intret(guitextheight(text, font, *wrap)));
 
 void guiplayerpreview(int *model, int *color, int *team, int *weap, char *vanity, char *action, float *scale, int *overlaid, float *size, float *blend, char *altact)
 {
     if(!cgui) return;
-    int ret = cgui->playerpreview(*model, *color, *team, *weap, vanity, *scale, *overlaid!=0, *size!=0 ? *size : 1.f, *blend!=0 ? *blend : 1.f);
+    int ret = cgui->playerpreview(*model, *color, *team, *weap, vanity, *scale, *overlaid!=0, *size!=0 ? *size : 1.f, *blend >= 0 ? *blend : 1.f);
     if(ret&GUI_UP)
     {
         char *act = NULL;
@@ -621,8 +731,15 @@ void guiplayerpreview(int *model, int *color, int *team, int *weap, char *vanity
             if(shouldclearmenu) clearlater = true;
         }
     }
+    else if(ret&GUI_ROLLOVER)
+    {
+        defformatstring(str)("%d %d %d %d %s", *model, *color, *team, *weap, vanity);
+        setsvar("guirollovername", str, true);
+        setsvar("guirolloveraction", action, true);
+        setsvar("guirollovertype", "player", true);
+    }
 }
-COMMAND(0, guiplayerpreview, "iiiissfiffs");
+COMMAND(0, guiplayerpreview, "iiiissfifgs");
 
 void guimodelpreview(char *model, char *animspec, char *action, float *scale, int *overlaid, float *size, float *blend, char *altact)
 {
@@ -655,6 +772,13 @@ void guimodelpreview(char *model, char *animspec, char *action, float *scale, in
             if(shouldclearmenu) clearlater = true;
         }
     }
+    else if(ret&GUI_ROLLOVER)
+    {
+        defformatstring(str)("%d %s", *model, animspec);
+        setsvar("guirollovername", str, true);
+        setsvar("guirolloveraction", action, true);
+        setsvar("guirollovertype", "model", true);
+    }
 }
 COMMAND(0, guimodelpreview, "sssfiffs");
 
@@ -752,11 +876,11 @@ void progressmenu()
 {
     menu *m = menus.access("loading");
     if(m)
-    {
-        m->usetitle = m->useinput = false;
+    { // it doesn't need to exist
+        m->usetitle = m->useinput = m->world = false;
+        m->builtin = true;
         UI::addcb(m);
     }
-    else conoutf("cannot find menu 'loading'");
 }
 
 void mainmenu()
