@@ -659,6 +659,7 @@ static inline void gencubevert(const cube &c, int i, T &v)
 {
     switch(i)
     {
+        default:
 #define GENCUBEVERT(n, x, y, z) \
         case n: \
             v = T(edgeget(cubeedge(c, 0, y, z), x), \
@@ -672,9 +673,9 @@ static inline void gencubevert(const cube &c, int i, T &v)
 
 void genfaceverts(const cube &c, int orient, ivec v[4])
 {
-
     switch(orient)
     {
+        default:
 #define GENFACEORIENT(o, v0, v1, v2, v3) \
         case o: v0 v1 v2 v3 break;
 #define GENFACEVERT(o, n, x,y,z, xv,yv,zv) \
@@ -1114,7 +1115,7 @@ int visibletris(const cube &c, int orient, int x, int y, int z, int size, ushort
     int convex = (e3 = v[0]).sub(v[3]).dot(n);
     if(!convex)
     {
-        if(ivec().cross(e3, e2).iszero()) { if(n.iszero()) return 0; vis = 1; touching = 0xF&~(1<<3); }
+        if(ivec().cross(e3, e2).iszero() || v[1] == v[3]) { if(n.iszero()) return 0; vis = 1; touching = 0xF&~(1<<3); }
         else if(n.iszero()) { vis = 2; touching = 0xF&~(1<<1); }
     }
 
