@@ -6,13 +6,13 @@ VAR(0, showpastegrid, 0, 0, 1);
 VAR(0, showcursorgrid, 0, 0, 1);
 VAR(0, showselgrid, 0, 0, 1);
 
+bool boxoutline = false;
+
 void boxs(int orient, vec o, const vec &s)
 {
-    int d = dimension(orient),
-          dc= dimcoord(orient);
-
-    float f = !outline && !(fullbright && blankgeom) ? 0 : (dc>0 ? 0.2f : -0.2f);
-    o[D[d]] += float(dc) * s[D[d]] + f,
+    int d = dimension(orient), dc = dimcoord(orient);
+    float f = boxoutline ? (dc>0 ? 0.2f : -0.2f) : 0;
+    o[D[d]] += dc * s[D[d]] + f;
 
     glBegin(GL_LINE_LOOP);
 
@@ -34,13 +34,12 @@ void boxs3D(const vec &o, vec s, int g)
 
 void boxsgrid(int orient, vec o, vec s, int g)
 {
-    int d = dimension(orient),
-          dc= dimcoord(orient);
+    int d = dimension(orient), dc = dimcoord(orient);
     float ox = o[R[d]],
           oy = o[C[d]],
           xs = s[R[d]],
           ys = s[C[d]],
-          f = !outline && !(fullbright && blankgeom) ? 0 : (dc>0 ? 0.2f : -0.2f);
+          f = boxoutline ? (dc>0 ? 0.2f : -0.2f) : 0;
 
     o[D[d]] += dc * s[D[d]]*g + f;
 
@@ -63,7 +62,7 @@ void boxsgrid(int orient, vec o, vec s, int g)
     xtraverts += 2*int(xs+ys);
 }
 
-selinfo sel, lastsel;
+selinfo sel, lastsel, savedsel;
 
 int orient = 0;
 int gridsize = 8;
@@ -86,12 +85,16 @@ VARF(0, dragging, 0, 0, 1,
     sel.orient = orient;
 );
 
-VARF(0, moving, 0, 0, 1,
-    if(!moving) return;
-    vec v(cur.v); v.add(1);
-    moving = pointinsel(sel, v);
-    if(moving) havesel = false; // tell cursorupdate to create handle
-);
+int moving = 0;
+ICOMMAND(0, moving, "b", (int *n),
+{
+    if(*n >= 0)
+    {
+        if(!*n || (moving<=1 && !pointinsel(sel, cur.tovec().add(1)))) moving = 0;
+        else if(!moving) moving = 1;
+    }
+    intret(moving);
+});
 
 VARF(0, gridpower, 0, 3, 12,
 {
@@ -108,11 +111,14 @@ VARF(0, hmapedit, 0, 0, 1, horient = sel.orient);
 
 void forcenextundo() { lastsel.orient = -1; }
 
+extern void hmapcancel();
+
 void cubecancel()
 {
     havesel = false;
     moving = dragging = hmapedit = passthroughsel = 0;
     forcenextundo();
+    hmapcancel();
 }
 
 void cancelsel()
@@ -179,6 +185,11 @@ COMMAND(0, cancelsel, "");
 COMMAND(0, reorient, "");
 COMMAND(0, selextend, "");
 
+ICOMMAND(0, selmoved, "", (), { if(noedit(true)) return; intret(sel.o != savedsel.o ? 1 : 0); });
+ICOMMAND(0, selsave, "", (), { if(noedit(true)) return; savedsel = sel; });
+ICOMMAND(0, selrestore, "", (), { if(noedit(true)) return; sel = savedsel; });
+ICOMMAND(0, selswap, "", (), { if(noedit(true)) return; swap(sel, savedsel); });
+
 ///////// selection support /////////////
 
 cube &blockcube(int x, int y, int z, const block3 &b, int rgrid) // looks up a world cube, based on coordinates mapped by the block
@@ -203,9 +214,8 @@ ICOMMAND(0, havesel, "", (), intret(havesel ? selchildcount : 0));
 
 void countselchild(cube *c, const ivec &cor, int size)
 {
-    ivec ss(sel.s);
-    ss.mul(sel.grid);
-    loopoctabox(cor, size, sel.o, ss)
+    ivec ss = ivec(sel.s).mul(sel.grid);
+    loopoctaboxsize(cor, size, sel.o, ss)
     {
         ivec o(i, cor.x, cor.y, cor.z, size);
         if(c[i].children) countselchild(c[i].children, o, size/2);
@@ -248,25 +258,19 @@ void updateselection()
     sel.s.z = abs(lastcur.z-cur.z)/sel.grid+1;
 }
 
-void editmoveplane(const vec &o, const vec &ray, int d, float off, vec &handle, vec &dest, bool first)
+bool editmoveplane(const vec &o, const vec &ray, int d, float off, vec &handle, vec &dest, bool first)
 {
     plane pl(d, off);
     float dist = 0.0f;
-
     physent *player = (physent *)game::focusedent(true);
     if(!player) player = camera1;
-    if(pl.rayintersect(player->o, ray, dist))
-    {
-        dest = ray;
-        dest.mul(dist);
-        dest.add(player->o);
-        if(first)
-        {
-            handle = dest;
-            handle.sub(o);
-        }
-        dest.sub(handle);
-    }
+    if(!pl.rayintersect(player->o, ray, dist))
+        return false;
+
+    dest = vec(ray).mul(dist).add(player->o);
+    if(first) handle = vec(dest).sub(o);
+    dest.sub(handle);
+    return true;
 }
 
 inline bool isheightmap(int orient, int d, bool empty, cube *c);
@@ -296,19 +300,20 @@ void rendereditcursor()
 
     if(moving)
     {
-        ivec e;
-        static vec v, handle;
-        editmoveplane(sel.o.tovec(), ray, od, sel.o[D[od]]+odc*sel.grid*sel.s[D[od]], handle, v, !havesel);
-        if(!havesel)
+        static vec dest, handle;
+        if(editmoveplane(sel.o.tovec(), camdir, od, sel.o[D[od]]+odc*sel.grid*sel.s[D[od]], handle, dest, moving==1))
         {
-            v.add(handle);
-            (e = handle).mask(~(sel.grid-1));
-            v.sub(handle = e.tovec());
-            havesel = true;
+            if(moving==1)
+            {
+                dest.add(handle);
+                handle = ivec(handle).mask(~(sel.grid-1)).tovec();
+                dest.sub(handle);
+                moving = 2;
+            }
+            ivec o = ivec(dest).mask(~(sel.grid-1));
+            sel.o[R[od]] = o[R[od]];
+            sel.o[C[od]] = o[C[od]];
         }
-        (e = v).mask(~(sel.grid-1));
-        sel.o[R[od]] = e[R[od]];
-        sel.o[C[od]] = e[C[od]];
     }
     else if(entmoving)
     {
@@ -327,7 +332,7 @@ void rendereditcursor()
                                             | (passthroughcube==1 ? RAY_PASS : 0), gridsize, entorient, ent);
 
         if((havesel || dragging) && !passthroughsel && !hmapedit)     // now try selecting the selection
-            if(rayrectintersect(sel.o.tovec(), vec(sel.s.tovec()).mul(sel.grid), player->o, ray, sdist, orient))
+            if(rayboxintersect(sel.o.tovec(), vec(sel.s.tovec()).mul(sel.grid), player->o, ray, sdist, orient))
             {   // and choose the nearest of the two
                 if(sdist < wdist)
                 {
@@ -362,7 +367,7 @@ void rendereditcursor()
             if(gridlookup && !dragging && !moving && !havesel && hmapedit!=1) gridsize = lusize;
             int mag = gridsize && lusize ? lusize / gridsize : 0;
             normalizelookupcube(int(w.x), int(w.y), int(w.z));
-            if(sdist == 0 || sdist > wdist) rayrectintersect(lu.tovec(), vec(gridsize), player->o, ray, t=0, orient); // just getting orient
+            if(sdist == 0 || sdist > wdist) rayboxintersect(lu.tovec(), vec(gridsize), player->o, ray, t=0, orient); // just getting orient
             cur = lu;
             cor = vec(w).mul(2).div(gridsize);
             od = dimension(orient);
@@ -434,6 +439,8 @@ void rendereditcursor()
 
     renderentselection(player->o, ray, entmoving!=0);
 
+    boxoutline = outline || (fullbright && blankgeom);
+
     enablepolygonoffset(GL_POLYGON_OFFSET_LINE);
 
     #define planargrid(q,r,s) \
@@ -460,7 +467,7 @@ void rendereditcursor()
     }
 
     // selections
-    if(havesel)
+    if(havesel || moving)
     {
         d = dimension(sel.orient);
         glColor3ub(120, 120, 120);  // grid
@@ -487,12 +494,14 @@ void rendereditcursor()
 
     if(showpastegrid && localedit && localedit->copy)
     {
-        glColor3ub(30, 0, 30);
+        glColor3ub(0, 192, 192);
         boxs3D(havesel ? sel.o.tovec() : lu.tovec(), localedit->copy->s.tovec(), havesel ? sel.grid : gridsize);
     }
 
     disablepolygonoffset(GL_POLYGON_OFFSET_LINE);
 
+    boxoutline = false;
+
     notextureshader->set();
 
     glDisable(GL_BLEND);
@@ -502,9 +511,9 @@ void rendereditcursor()
 
 static bool haschanged = false;
 
-void readychanges(block3 &b, cube *c, const ivec &cor, int size)
+void readychanges(const ivec &bbmin, const ivec &bbmax, cube *c, const ivec &cor, int size)
 {
-    loopoctabox(cor, size, b.o, b.s)
+    loopoctabox(cor, size, bbmin, bbmax)
     {
         ivec o(i, cor.x, cor.y, cor.z, size);
         if(c[i].ext)
@@ -527,7 +536,7 @@ void readychanges(block3 &b, cube *c, const ivec &cor, int size)
                 discardchildren(c[i], true);
                 brightencube(c[i]);
             }
-            else readychanges(b, c[i].children, o, size/2);
+            else readychanges(bbmin, bbmax, c[i].children, o, size/2);
         }
         else brightencube(c[i]);
     }
@@ -554,17 +563,7 @@ void commitchanges(bool force)
 void changed(const block3 &sel, bool commit = true)
 {
     if(sel.s.iszero()) return;
-    block3 b = sel;
-    loopi(3) b.s[i] *= b.grid;
-    b.grid = 1;
-    loopi(3)                    // the changed blocks are the selected cubes
-    {
-        b.o[i] -= 1;
-        b.s[i] += 2;
-        readychanges(b, worldroot, ivec(0, 0, 0), hdr.worldsize/2);
-        b.o[i] += 1;
-        b.s[i] -= 2;
-    }
+    readychanges(ivec(sel.o).sub(1), ivec(sel.s).mul(sel.grid).add(sel.o).add(1), worldroot, ivec(0, 0, 0), hdr.worldsize/2);
     haschanged = true;
 
     if(commit) commitchanges();
@@ -696,7 +695,7 @@ struct undolist
 
 undolist undos, redos;
 VAR(IDF_PERSIST, undomegs, 0, 8, 100);                              // bounded by n megs
-int totalundos = 0;
+VAR(IDF_READONLY, totalundos, 1, 0, -1);
 
 void pruneundos(int maxremain)                          // bound memory
 {
@@ -937,58 +936,58 @@ void freeeditinfo(editinfo *&e)
     e = NULL;
 }
 
-struct octabrushheader
+struct prefabheader
 {
     char magic[4];
     int version;
 };
 
-struct octabrush : editinfo
+struct prefab : editinfo
 {
     char *name;
 
-    octabrush() : name(NULL) {}
-    ~octabrush() { DELETEA(name); if(copy) freeblock(copy); }
+    prefab() : name(NULL) {}
+    ~prefab() { DELETEA(name); if(copy) freeblock(copy); }
 };
 
-static inline bool htcmp(const char *key, const octabrush &b) { return !strcmp(key, b.name); }
+static inline bool htcmp(const char *key, const prefab &b) { return !strcmp(key, b.name); }
 
-static hashset<octabrush> octabrushes;
+static hashset<prefab> prefabs;
 
-void delbrush(char *name)
+void delprefab(char *name)
 {
-    if(octabrushes.remove(name))
-        conoutf("deleted brush %s", name);
+    if(prefabs.remove(name))
+        conoutf("deleted prefab %s", name);
 }
-COMMAND(0, delbrush, "s");
+COMMAND(0, delprefab, "s");
 
-void savebrush(char *name)
+void saveprefab(char *name)
 {
     if(!name[0] || noedit(true) || multiplayer()) return;
-    octabrush *b = octabrushes.access(name);
+    prefab *b = prefabs.access(name);
     if(!b)
     {
-        b = &octabrushes[name];
+        b = &prefabs[name];
         b->name = newstring(name);
     }
     if(b->copy) freeblock(b->copy);
     protectsel(b->copy = blockcopy(block3(sel), sel.grid));
     changed(sel);
-    defformatstring(filename)(strpbrk(name, "/\\") ? "%s.obr" : "brush/%s.obr", name);
+    defformatstring(filename)(strpbrk(name, "/\\") ? "%s.obr" : "prefab/%s.obr", name);
     path(filename);
     stream *f = opengzfile(filename, "wb");
-    if(!f) { conoutf("\frcould not write brush to %s", filename); return; }
-    octabrushheader hdr;
+    if(!f) { conoutf("\frcould not write prefab to %s", filename); return; }
+    prefabheader hdr;
     memcpy(hdr.magic, "OEBR", 4);
     hdr.version = 0;
     lilswap(&hdr.version, 1);
     f->write(&hdr, sizeof(hdr));
     streambuf<uchar> s(f);
-    if(!packblock(*b->copy, s)) { delete f; conoutf("\frcould not pack brush %s", filename); return; }
+    if(!packblock(*b->copy, s)) { delete f; conoutf("\frcould not pack prefab %s", filename); return; }
     delete f;
-    conoutf("wrote brush file %s", filename);
+    conoutf("wrote prefab file %s", filename);
 }
-COMMAND(0, savebrush, "s");
+ICOMMAND(0, saveprefab, "s", (char *s), if(!(identflags&IDF_WORLD)) saveprefab(s));
 
 void pasteblock(block3 &b, selinfo &sel, bool local)
 {
@@ -1000,31 +999,31 @@ void pasteblock(block3 &b, selinfo &sel, bool local)
     sel.orient = o;
 }
 
-void pastebrush(char *name)
+void pasteprefab(char *name)
 {
     if(!name[0] || noedit() || multiplayer()) return;
-    octabrush *b = octabrushes.access(name);
+    prefab *b = prefabs.access(name);
     if(!b)
     {
-        defformatstring(filename)(strpbrk(name, "/\\") ? "%s.obr" : "brush/%s.obr", name);
+        defformatstring(filename)(strpbrk(name, "/\\") ? "%s.obr" : "prefab/%s.obr", name);
         path(filename);
         stream *f = opengzfile(filename, "rb");
-        if(!f) { conoutf("\frcould not read brush %s", filename); return; }
-        octabrushheader hdr;
-        if(f->read(&hdr, sizeof(hdr)) != sizeof(octabrushheader) || memcmp(hdr.magic, "OEBR", 4)) { delete f; conoutf("\frbrush %s has malformatted header", filename); return; }
+        if(!f) { conoutf("\frcould not read prefab %s", filename); return; }
+        prefabheader hdr;
+        if(f->read(&hdr, sizeof(hdr)) != sizeof(prefabheader) || memcmp(hdr.magic, "OEBR", 4)) { delete f; conoutf("\frprefab %s has malformatted header", filename); return; }
         lilswap(&hdr.version, 1);
-        if(hdr.version != 0) { delete f; conoutf("\frbrush %s uses unsupported version", filename); return; }
+        if(hdr.version != 0) { delete f; conoutf("\frprefab %s uses unsupported version", filename); return; }
         streambuf<uchar> s(f);
         block3 *copy = NULL;
-        if(!unpackblock(copy, s)) { delete f; conoutf("\frcould not unpack brush %s", filename); return; }
+        if(!unpackblock(copy, s)) { delete f; conoutf("\frcould not unpack prefab %s", filename); return; }
         delete f;
-        b = &octabrushes[name];
+        b = &prefabs[name];
         b->name = newstring(name);
         b->copy = copy;
     }
     pasteblock(*b->copy, sel, true);
 }
-COMMAND(0, pastebrush, "s");
+COMMAND(0, pasteprefab, "s");
 
 void mpcopy(editinfo *&e, selinfo &sel, bool local)
 {
@@ -1052,8 +1051,7 @@ void copy()
 void pasteclear()
 {
     if(!localedit) return;
-    if(localedit->copy) freeblock(localedit->copy);
-    localedit->copy = NULL;
+    freeeditinfo(localedit);
 }
 
 void pastehilight()
@@ -1066,7 +1064,7 @@ void pastehilight()
 
 void paste()
 {
-    if(noedit()) return;
+    if(noedit() || !localedit) return;
     mppaste(localedit, sel, true);
 }
 
@@ -1171,7 +1169,8 @@ vector<int> htextures;
 
 COMMAND(0, clearbrush, "");
 COMMAND(0, brushvert, "iii");
-ICOMMAND(0, hmapcancel, "", (), htextures.setsize(0); );
+void hmapcancel() { htextures.setsize(0); }
+COMMAND(0, hmapcancel, "");
 ICOMMAND(0, hmapselect, "", (),
     int t = lookupcube(cur.x, cur.y, cur.z).texture[orient];
     int i = htextures.find(t);
@@ -1181,21 +1180,12 @@ ICOMMAND(0, hmapselect, "", (),
         htextures.remove(i);
 );
 
-inline bool ishtexture(int t)
-{
-    loopv(htextures)
-        if(t == htextures[i])
-            return false;
-    return true;
-}
-
-VAR(0, bypassheightmapcheck, 0, 0, 1);    // temp
-
 inline bool isheightmap(int o, int d, bool empty, cube *c)
 {
     return havesel ||
            (empty && isempty(*c)) ||
-           ishtexture(c->texture[o]);
+           htextures.empty() ||
+           htextures.find(c->texture[o]) >= 0;
 }
 
 namespace hmap
@@ -1663,7 +1653,7 @@ vector<ushort> texmru;
 void tofronttex()                                       // maintain most recently used of the texture lists when applying texture
 {
     int c = curtexindex;
-    if(c>=0)
+    if(texmru.inrange(c))
     {
         texmru.insert(0, texmru.remove(c));
         curtexindex = -1;
@@ -1854,7 +1844,7 @@ void valpha(float *front, float *back)
 }
 COMMAND(0, valpha, "ff");
 
-void vcolor(float *r, float *g, float *b)
+void vcolour(float *r, float *g, float *b)
 {
     if(noedit() || multiplayer()) return;
     VSlot ds;
@@ -1862,7 +1852,7 @@ void vcolor(float *r, float *g, float *b)
     ds.colorscale = vec(max(*r, 0.0f), max(*g, 0.0f), max(*b, 0.0f));
     mpeditvslot(ds, allfaces, sel, true);
 }
-COMMAND(0, vcolor, "fff");
+COMMAND(0, vcolour, "fff");
 
 void vpalette(int *p, int *x)
 {
@@ -1883,7 +1873,7 @@ void vcoastscale(float *value)
     ds.coastscale = clamp(*value, 0.f, 1000.f);
     mpeditvslot(ds, allfaces, sel, true);
 }
-COMMAND(0, vcoastscale, "fff");
+COMMAND(0, vcoastscale, "f");
 
 void vreset()
 {
@@ -1977,6 +1967,7 @@ void edittex_(int *dir)
 {
     if(noedit()) return;
     filltexlist();
+    if(texmru.empty()) return;
     texpaneltimer = texpaneltime;
     if(!(lastsel==sel)) tofronttex();
     curtexindex = clamp(curtexindex<0 ? 0 : curtexindex+*dir, 0, texmru.length()-1);
@@ -2157,15 +2148,12 @@ void flip()
 
 void mprotate(int cw, selinfo &sel, bool local)
 {
-    if(local)
-    {
-        client::edittrigger(sel, EDIT_ROTATE, cw);
-        makeundo();
-    }
+    if(local) client::edittrigger(sel, EDIT_ROTATE, cw);
     int d = dimension(sel.orient);
     if(!dimcoord(sel.orient)) cw = -cw;
     int m = sel.s[C[d]] < sel.s[R[d]] ? C[d] : R[d];
     int ss = sel.s[m] = max(sel.s[R[d]], sel.s[C[d]]);
+    if(local) makeundo();
     loop(z,sel.s[D[d]]) loopi(cw>0 ? 1 : 3)
     {
         loopxy(sel) rotatecube(selcube(x,y,z), d);
@@ -2286,7 +2274,6 @@ VAR(IDF_PERSIST, thumbtime, 0, 25, 1000);
 FVAR(IDF_PERSIST, thumbsize, 0, 2, 25);
 
 static int lastthumbnail = 0;
-extern bool layoutpass;
 struct texturegui : guicb
 {
     bool menuon;
@@ -2299,19 +2286,18 @@ struct texturegui : guicb
         extern VSlot dummyvslot;
         int curtex = menutex, numpages = max((texmru.length() + thumbwidth*thumbheight - 1)/(thumbwidth*thumbheight), 1)-1;
         if(autopreviewtexgui && texmru.inrange(rolltex)) curtex = rolltex;
-        if(!layoutpass) rolltex = -1;
         if(menupage > numpages) menupage = numpages;
         else if(menupage < 0) menupage = 0;
         g.start(menustart, menuscale, NULL, true);
         uilist(g, {
             g.space(2);
-            if(g.button("\fgauto apply", 0xFFFFFF, autoapplytexgui ? "checkboxon" : "checkbox", 0xFFFFFF, autoapplytexgui ? false : true)&GUI_UP)
+            if(g.button("\fgauto apply", 0xFFFFFF, autoapplytexgui ? "checkboxon" : "checkbox", 0xFFFFFF)&GUI_UP)
                 autoapplytexgui = autoapplytexgui ? 0 : 1;
             g.space(2);
-            if(g.button("\fgauto preview", 0xFFFFFF, autopreviewtexgui ? "checkboxon" : "checkbox", 0xFFFFFF, autopreviewtexgui ? false : true)&GUI_UP)
+            if(g.button("\fgauto preview", 0xFFFFFF, autopreviewtexgui ? "checkboxon" : "checkbox", 0xFFFFFF)&GUI_UP)
                 autopreviewtexgui = autopreviewtexgui ? 0 : 1;
             g.space(2);
-            if(g.button("\fgauto close", 0xFFFFFF, autoclosetexgui ? (autoclosetexgui > 1 ? "checkboxtwo" : "checkboxon") : "checkbox", 0xFFFFFF, autoclosetexgui ? false : true)&GUI_UP)
+            if(g.button("\fgauto close", 0xFFFFFF, autoclosetexgui ? (autoclosetexgui > 1 ? "checkboxtwo" : "checkboxon") : "checkbox", 0xFFFFFF)&GUI_UP)
                 autoclosetexgui = autoclosetexgui ? (autoclosetexgui > 1 ? 0 : 2) : 1;
             g.space(2);
             if(g.button("\foreset order", 0xFFFFFF, NULL)&GUI_UP)
@@ -2341,7 +2327,7 @@ struct texturegui : guicb
                     if(autoclosetexgui) menuon = false;
                 }
             }
-            else g.image(textureload("textures/nothumb", 3), thumbheight*thumbsize, true);
+            else g.image(textureload(nothumbtex, 3), thumbheight*thumbsize, true);
             g.space(1);
             uilistv(g, 2, {
                 uilist(g, loop(h, thumbheight)
@@ -2387,7 +2373,7 @@ struct texturegui : guicb
             if(texmru.inrange(curtex))
             {
                 VSlot &v = lookupvslot(texmru[curtex]);
-                g.textf("#%-3d \fa%s", 0xFFFFFF, NULL, 0, texmru[curtex], v.slot->sts.empty() ? "<unknown texture>" : v.slot->sts[0].name);
+                g.textf("#%-3d \fa%s", 0xFFFFFF, NULL, 0, -1, texmru[curtex], v.slot->sts.empty() ? "<unknown texture>" : v.slot->sts[0].name);
             }
             else g.textf("no texture selected", 0x888888);
         });
@@ -2445,7 +2431,7 @@ void rendertexturepanel(int w, int h)
         loopi(7)
         {
             int s = (i == 3 ? 285 : 220), ti = curtexindex+i-3;
-            if(ti>=0 && ti<texmru.length())
+            if(texmru.inrange(ti))
             {
                 VSlot &vslot = lookupvslot(texmru[ti]), *layer = NULL;
                 Slot &slot = *vslot.slot;
