@@ -1,3 +1,8 @@
+VAR(0, ragdolltimestepmin, 1, 5, 50);
+VAR(0, ragdolltimestepmax, 1, 10, 50);
+FVAR(0, ragdollrotfric, 0, 0.85f, 1);
+FVAR(0, ragdollrotfricstop, 0, 0.1f, 1);
+
 struct ragdollskel
 {
     struct vert
@@ -215,7 +220,6 @@ struct ragdolldata
 
     void init(dynent *d)
     {
-        extern int ragdolltimestepmin;
         float ts = ragdolltimestepmin/1000.0f;
         loopv(skel->verts) (verts[i].oldpos = verts[i].pos).sub(vec(d->vel).add(d->falling).mul(ts));
         timestep = ts;
@@ -237,6 +241,7 @@ struct ragdolldata
     void applyrotfriction(float ts);
     void tryunstick(float speed);
     void warppos(const vec &vel, const vec &offset);
+    void twitch(float vel);
 
     static inline bool collidevert(const vec &pos, const vec &dir, float radius)
     {
@@ -245,7 +250,6 @@ struct ragdolldata
             vertent()
             {
                 type = ENT_RAGDOLL;
-                collidetype = COLLIDE_AABB;
                 radius = xradius = yradius = height = aboveeye = 1;
             }
         } v;
@@ -334,11 +338,6 @@ void ragdolldata::constrainrot()
     }
 }
 
-VAR(0, ragdolltimestepmin, 1, 5, 50);
-VAR(0, ragdolltimestepmax, 1, 10, 50);
-FVAR(0, ragdollrotfric, 0, 0.85f, 1);
-FVAR(0, ragdollrotfricstop, 0, 0.1f, 1);
-
 void ragdolldata::calcrotfriction()
 {
     loopv(skel->rotfrictions)
@@ -385,7 +384,7 @@ void ragdolldata::tryunstick(float speed)
         vert &v = verts[i];
         if(v.stuck)
         {
-            if(!collidevert(v.pos, vec(0, 0, 0), skel->verts[i].radius)) { stuck++; continue; }
+            if(collidevert(v.pos, vec(0, 0, 0), skel->verts[i].radius)) { stuck++; continue; }
             v.stuck = false;
         }
         unstuck.add(v.pos);
@@ -406,7 +405,6 @@ void ragdolldata::tryunstick(float speed)
 
 void ragdolldata::warppos(const vec &vel, const vec &offset)
 {
-    extern int ragdolltimestepmin;
     float ts = ragdolltimestepmin/1000.0f;
     vec frame = vec(vel).mul(ts);
     loopv(skel->verts)
@@ -418,7 +416,16 @@ void ragdolldata::warppos(const vec &vel, const vec &offset)
     collidemillis = 0;
 }
 
-extern vec wall;
+void ragdolldata::twitch(float vel)
+{
+    float ts = ragdolltimestepmin/1000.0f;
+    loopv(skel->verts)
+    {
+        vert &v = verts[i];
+        v.oldpos.add(vec(rnd(201)-100, rnd(201)-100, rnd(201)-100).div(100.f).normalize().mul(vel).mul(ts));
+    }
+    collidemillis = 0;
+}
 
 void ragdolldata::updatepos()
 {
@@ -428,11 +435,11 @@ void ragdolldata::updatepos()
         if(v.weight)
         {
             v.newpos.div(v.weight);
-            if(collidevert(v.newpos, vec(v.newpos).sub(v.pos), skel->verts[i].radius)) v.pos = v.newpos;
+            if(!collidevert(v.newpos, vec(v.newpos).sub(v.pos), skel->verts[i].radius)) v.pos = v.newpos;
             else
             {
                 vec dir = vec(v.newpos).sub(v.oldpos);
-                if(dir.dot(wall) < 0) v.oldpos = vec(v.pos).sub(dir.reflect(wall));
+                if(dir.dot(collidewall) < 0) v.oldpos = vec(v.pos).sub(dir.reflect(collidewall));
                 v.collided = true;
             }
         }
@@ -495,11 +502,11 @@ void ragdolldata::move(dynent *pl, float ts)
         vert &v = verts[i];
         //if(v.pos.z < 0) { v.pos.z = 0; v.oldpos = v.pos; collisions++; }
         vec dir = vec(v.pos).sub(v.oldpos);
-        v.collided = !collidevert(v.pos, dir, skel->verts[i].radius);
+        v.collided = collidevert(v.pos, dir, skel->verts[i].radius);
         if(v.collided)
         {
             v.pos = v.oldpos;
-            v.oldpos.sub(dir.reflect(wall).mul(ragdollelasticity));
+            v.oldpos.sub(dir.reflect(collidewall).mul(ragdollelasticity));
             collisions++;
         }
     }
@@ -529,7 +536,7 @@ bool validragdoll(dynent *d, int millis)
 
 void moveragdoll(dynent *d, bool smooth)
 {
-    if(!curtime || !d->ragdoll) return;
+    if(!curtime || !d->ragdoll || (d->state != CS_DEAD && d->state != CS_WAITING)) return;
 
     if(!d->ragdoll->collidemillis || lastmillis < d->ragdoll->collidemillis)
     {
@@ -563,10 +570,16 @@ void cleanragdoll(dynent *d)
 
 void warpragdoll(dynent *d, const vec &vel, const vec &offset)
 {
-    if(!d->ragdoll) return;
+    if(!d->ragdoll || (d->state != CS_DEAD && d->state != CS_WAITING)) return;
     d->ragdoll->warppos(vel, offset);
 }
 
+void twitchragdoll(dynent *d, float vel)
+{
+    if(!d->ragdoll || (d->state != CS_DEAD && d->state != CS_WAITING)) return;
+    d->ragdoll->twitch(vel);
+}
+
 vec rdabove(dynent *d, float offset)
 {
     if((d->type != ENT_PLAYER && d->type != ENT_AI) || !d->ragdoll || (d->state != CS_DEAD && d->state != CS_WAITING)) return d->physent::abovehead(offset);
