@@ -1,7 +1,7 @@
 #include "engine.h"
 
-VAR(IDF_PERSIST, oqdynent, 0, 1, 1);
-VAR(IDF_PERSIST, animationinterpolationtime, 0, 150, 1000);
+VAR(0, oqdynent, 0, 1, 1);
+VAR(0, animationinterpolationtime, 0, 200, 1000);
 
 model *loadingmodel = NULL;
 
@@ -23,7 +23,7 @@ static model *__loadmodel__##modelclass(const char *filename) \
 { \
     return new modelclass(filename); \
 } \
-static int __dummy__##modelclass = addmodeltype((modeltype), __loadmodel__##modelclass);
+UNUSED static int __dummy__##modelclass = addmodeltype((modeltype), __loadmodel__##modelclass);
 
 #include "md2.h"
 #include "md3.h"
@@ -184,9 +184,8 @@ COMMAND(0, mdlspin, "fff");
 void mdlscale(int *percent)
 {
     checkmdl;
-    float scale = 0.3f;
+    float scale = 1.0f;
     if(*percent>0) scale = *percent/100.0f;
-    else if(*percent<0) scale = 0.0f;
     loadingmodel->scale = scale;
 }
 
@@ -243,7 +242,7 @@ COMMAND(0, mdlshadow, "i");
 void mdlbb(float *rad, float *h, float *height)
 {
     checkmdl;
-    loadingmodel->collideradius = *rad;
+    loadingmodel->collidexyradius = *rad;
     loadingmodel->collideheight = *h;
     loadingmodel->height = *height;
 }
@@ -269,6 +268,7 @@ COMMAND(0, mdlname, "");
 #define checkragdoll \
     if(!loadingmodel->skeletal()) { conoutf("\frnot loading a skeletal model"); return; } \
     skelmodel *m = (skelmodel *)loadingmodel; \
+    if(m->parts.empty()) return; \
     skelmodel::skelmeshgroup *meshes = (skelmodel::skelmeshgroup *)m->parts.last()->meshes; \
     if(!meshes) return; \
     skelmodel::skeleton *skel = meshes->skel; \
@@ -501,9 +501,8 @@ COMMAND(0, clearmodel, "s");
 
 bool modeloccluded(const vec &center, float radius)
 {
-    int br = int(radius*2)+1;
-    return pvsoccluded(ivec(int(center.x-radius), int(center.y-radius), int(center.z-radius)), ivec(br, br, br)) ||
-           bboccluded(ivec(int(center.x-radius), int(center.y-radius), int(center.z-radius)), ivec(br, br, br));
+    ivec bbmin = vec(center).sub(radius), bbmax = vec(center).add(radius+1);
+    return pvsoccluded(bbmin, bbmax) || bboccluded(bbmin, bbmax);
 }
 
 VAR(0, showboundingbox, 0, 0, 2);
@@ -637,7 +636,7 @@ void endmodelbatches()
         if(b.flags&(MDL_SHADOW|MDL_DYNSHADOW))
         {
             vec center, bbradius;
-            b.m->boundbox(0/*frame*/, center, bbradius); // FIXME
+            b.m->boundbox(center, bbradius);
             loopvj(b.batched)
             {
                 batchedmodel &bm = b.batched[j];
@@ -741,7 +740,7 @@ void endmodelquery()
     modelattached.setsize(minattached);
 }
 
-VAR(IDF_PERSIST, maxmodelradiusdistance, 10, 200, 1000);
+VAR(0, maxmodelradiusdistance, 10, 200, 1000);
 
 void rendermodelquery(model *m, dynent *d, const vec &center, float radius)
 {
@@ -805,7 +804,7 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
          doOQ = flags&MDL_CULL_QUERY && hasOQ && oqfrags && oqdynent;
     if(flags&(MDL_CULL_VFC|MDL_CULL_DIST|MDL_CULL_OCCLUDED|MDL_CULL_QUERY|MDL_SHADOW|MDL_DYNSHADOW))
     {
-        m->boundbox(0/*frame*/, center, bbradius); // FIXME
+        m->boundbox(center, bbradius);
         radius = bbradius.magnitude();
         if(d && d->ragdoll)
         {
@@ -878,7 +877,7 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
         glEnable(GL_BLEND);
         glBlendFunc(GL_ONE, GL_ONE);
 
-        if(d && showboundingbox==1)
+        if(d && showboundingbox==1 && (d->type == ENT_PLAYER || d->type == ENT_AI))
         {
             if(d->ragdoll && (d->state == CS_DEAD || d->state == CS_WAITING))
             {
@@ -897,8 +896,8 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
         else
         {
             vec center, radius;
-            if(showboundingbox==1) m->collisionbox(0, center, radius);
-            else m->boundbox(0, center, radius);
+            if(showboundingbox==1) m->collisionbox(center, radius);
+            else m->boundbox(center, radius);
             center.mul(size);
             radius.mul(size);
             glTranslatef(o.x, o.y, o.z);
@@ -1031,7 +1030,7 @@ void abovemodel(vec &o, const char *mdl)
 {
     model *m = loadmodel(mdl);
     if(!m) return;
-    o.z += m->above(0/*frame*/);
+    o.z += m->above();
 }
 
 bool matchanim(const char *name, const char *pattern)
@@ -1093,13 +1092,9 @@ void setbbfrommodel(dynent *d, const char *mdl, float size)
     model *m = loadmodel(mdl);
     if(!m) return;
     vec center, radius;
-    m->collisionbox(0, center, radius);
+    m->collisionbox(center, radius);
     if(d->type==ENT_INANIMATE && !m->ellipsecollide)
-    {
         d->collidetype = COLLIDE_OBB;
-        //d->collidetype = COLLIDE_AABB;
-        //rotatebb(center, radius, int(d->yaw));
-    }
     d->xradius  = (radius.x + fabs(center.x))*size;
     d->yradius  = (radius.y + fabs(center.y))*size;
     d->radius   = d->collidetype==COLLIDE_OBB ? sqrtf(d->xradius*d->xradius + d->yradius*d->yradius) : max(d->xradius, d->yradius);
