@@ -14,13 +14,13 @@ VAR(IDF_PERSIST, particletext, 0, 1, 1);
 VAR(IDF_PERSIST, particleglare, 0, 1, 100);
 VAR(0, debugparticles, 0, 0, 1);
 
-// Check emit_particles() to limit the rate that paricles can be emitted for models/sparklies
+// Check canemitparticles() to limit the rate that paricles can be emitted for models/sparklies
 // Automatically stops particles being emitted when paused or in reflective drawing
 VAR(IDF_PERSIST, emitmillis, 1, 15, VAR_MAX);
 static int lastemitframe = 0;
 static bool emit = false;
 
-static bool emit_particles()
+static bool canemitparticles()
 {
     if(reflecting || refracting) return false;
     return emit;
@@ -224,7 +224,7 @@ struct listrenderer : partrenderer
         p->blend = blend;
         p->grav = grav;
         p->collide = collide;
-        if((p->owner = pl) != NULL && (p->owner->type == ENT_PLAYER || p->owner->type == ENT_AI)) switch(type&0xFF)
+        if((p->owner = pl) != NULL && (p->owner->type == ENT_PLAYER || p->owner->type == ENT_AI)) switch(type&PT_TYPE)
         {
             case PT_TEXT: case PT_ICON: p->m.add(vec(p->o).sub(p->owner->abovehead())); break;
             default: break;
@@ -322,7 +322,7 @@ struct textrenderer : sharedlistrenderer
         {
             const char *start = text;
             while(*text && *text != '>') text++;
-            if(*text) { int len = text-(start+1); strncpy(font, start+1, len); font[len] = 0; text++; }
+            if(*text) { int len = text-(start+1); memcpy(font, start+1, len); font[len] = 0; text++; }
             else text = start;
         }
         float xoff = -text_width(text)/2;
@@ -1004,7 +1004,7 @@ static coneprimitiverenderer coneprimitives(PT_CONE|PT_LERP), coneontopprimitive
 
 static partrenderer *parts[] =
 {
-    new portalrenderer("<grey>textures/teleport"), &icons,
+    new portalrenderer("<grey>particles/teleport"), &icons,
     &lineprimitives, &lineontopprimitives, &trisprimitives, &trisontopprimitives,
     &loopprimitives, &loopontopprimitives, &coneprimitives, &coneontopprimitives,
     new softquadrenderer("<grey>particles/fire", PT_PART|PT_GLARE|PT_RND4|PT_FLIP|PT_LERP|PT_SHRINK),
@@ -1033,14 +1033,17 @@ static partrenderer *parts[] =
     new quadrenderer("<grey>particles/plasma", PT_PART|PT_GLARE|PT_FLIP|PT_SHRINK),
     new softquadrenderer("<grey>particles/electric", PT_PART|PT_GLARE|PT_RND4|PT_FLIP|PT_SHRINK),
     new quadrenderer("<grey>particles/electric", PT_PART|PT_GLARE|PT_RND4|PT_FLIP|PT_SHRINK),
+    new softquadrenderer("<grey>particles/eleczap", PT_PART|PT_GLARE|PT_RND4|PT_FLIP|PT_SHRINK),
+    new quadrenderer("<grey>particles/eleczap", PT_PART|PT_GLARE|PT_RND4|PT_FLIP|PT_SHRINK),
     new quadrenderer("<grey>particles/fire", PT_PART|PT_GLARE|PT_FLIP|PT_RND4|PT_GLARE|PT_SHRINK),
     new taperenderer("<grey>particles/sflare", PT_TAPE|PT_GLARE),
     new taperenderer("<grey>particles/mflare", PT_TAPE|PT_GLARE|PT_RND4|PT_VFLIP|PT_GLARE),
     new taperenderer("<grey>particles/lightning", PT_TAPE|PT_GLARE|PT_HFLIP|PT_VFLIP, 2), // uses same clamp setting as normal lightning to avoid conflict
+    new taperenderer("<grey>particles/lightzap", PT_TAPE|PT_GLARE|PT_HFLIP|PT_VFLIP, 2),
     new quadrenderer("<grey>particles/muzzle", PT_PART|PT_GLARE|PT_RND4|PT_FLIP),
     new quadrenderer("<grey>particles/snow", PT_PART|PT_GLARE|PT_FLIP),
     &texts, &textontop,
-    &explosions, &shockwaves, &shockballs, &lightnings,
+    &explosions, &shockwaves, &shockballs, &lightnings, &lightzaps,
     &flares // must be done last!
 };
 
@@ -1121,7 +1124,7 @@ void renderparticles(bool mainpass)
         {
             int type = parts[i]->type;
             const char *title = parts[i]->texname ? strrchr(parts[i]->texname, '/')+1 : NULL;
-            mkstring(info);
+            string info = "";
             if(type&PT_GLARE) concatstring(info, "g,");
             if(type&PT_SOFT) concatstring(info, "s,");
             if(type&PT_LERP) concatstring(info, "l,");
@@ -1129,7 +1132,7 @@ void renderparticles(bool mainpass)
             if(type&PT_RND4) concatstring(info, "r,");
             if(type&PT_FLIP) concatstring(info, "f,");
             if(type&PT_ONTOP) concatstring(info, "o,");
-            defformatstring(ds)("%d\t%s: %s %s", parts[i]->count(), partnames[type&0xFF], info, (title?title:""));
+            defformatstring(ds)("%d\t%s: %s %s", parts[i]->count(), partnames[type&PT_TYPE], info, (title?title:""));
             draw_text(ds, FONTH, (i+n/2)*FONTH);
         }
         glDisable(GL_BLEND);
@@ -1264,7 +1267,7 @@ void create(int type, int color, int fade, const vec &p, float size, float blend
 
 void regularcreate(int type, int color, int fade, const vec &p, float size, float blend, int grav, int collide, physent *pl, int delay)
 {
-    if(!emit_particles() || (delay > 0 && rnd(delay) != 0)) return;
+    if(!canemitparticles() || (delay > 0 && rnd(delay) != 0)) return;
     create(type, color, fade, p, size, blend, grav, collide, pl);
 }
 
@@ -1287,13 +1290,13 @@ void splash(int type, int color, float radius, int num, int fade, const vec &p,
 
 void regularsplash(int type, int color, float radius, int num, int fade, const vec &p, float size, float blend, int grav, int collide, float vel, int delay)
 {
-    if(!emit_particles() || (delay > 0 && rnd(delay) != 0)) return;
+    if(!canemitparticles() || (delay > 0 && rnd(delay) != 0)) return;
     splash(type, color, radius, num, fade, p, size, blend, grav, collide, vel);
 }
 
 bool canaddparticles()
 {
-    return !renderedgame && !shadowmapping;
+    return !renderedgame && !shadowmapping && !minimized;
 }
 
 void regular_part_create(int type, int fade, const vec &p, int color, float size, float blend, int grav, int collide, physent *pl, int delay)
@@ -1369,7 +1372,7 @@ void part_explosion(const vec &dest, float maxsize, int type, int fade, int colo
 
 void regular_part_explosion(const vec &dest, float maxsize, int type, int fade, int color, float size, float blend, int grav, int collide)
 {
-    if(!canaddparticles() || !emit_particles()) return;
+    if(!canaddparticles() || !canemitparticles()) return;
     part_explosion(dest, maxsize, type, fade, color, size, blend, grav, collide);
 }
 
@@ -1431,7 +1434,7 @@ void part_dir(const vec &o, float yaw, float pitch, float length, float size, fl
 {
     if(!canaddparticles()) return;
 
-    vec v; vecfromyawpitch(yaw, pitch, 1, 0, v); v.normalize();
+    vec v(yaw*RAD, pitch*RAD);
     part_line(o, vec(v).mul(length).add(o), size, blend, fade, color);
     if(interval)
     {
@@ -1509,9 +1512,9 @@ static inline vec offsetvec(vec o, int dir, int dist)
  */
 void regularshape(int type, float radius, int color, int dir, int num, int fade, const vec &p, float size, float blend, int grav, int collide, float vel)
 {
-    if(!emit_particles()) return;
+    if(!canemitparticles()) return;
 
-    int basetype = parts[type]->type&0xFF;
+    int basetype = parts[type]->type&PT_TYPE;
     bool flare = (basetype == PT_TAPE) || (basetype == PT_LIGHTNING),
          inv = (dir&0x20)!=0, taper = (dir&0x40)!=0;
     dir &= 0x1F;
@@ -1607,7 +1610,7 @@ void regularshape(int type, float radius, int color, int dir, int num, int fade,
 
 void regularflame(int type, const vec &p, float radius, float height, int color, int density, int fade, float size, float blend, int grav, int collide, float vel)
 {
-    if(!emit_particles()) return;
+    if(!canemitparticles()) return;
 
     float s = size*min(radius, height);
     vec v(0, 0, min(1.0f, height)*vel);
@@ -1632,6 +1635,8 @@ static int partcolour(int c, int p, int x)
 }
 void makeparticle(const vec &o, attrvector &attr)
 {
+    bool oldemit = emit;
+    if(attr[11]) emit = true;
     switch(attr[0])
     {
         case 0: //fire
@@ -1709,10 +1714,9 @@ void makeparticle(const vec &o, attrvector &attr)
         case 5:
         {
             float length = clamp(attr[1], 0, 100)/100.f;
-            Texture *t = textureload(hud::progresstex, 3);
             int colour = partcolour(attr[2], attr[4], attr[5]);
-            part_icon(o, t, 2, 1, 0, 0, 1, colour, 0, length);
-            part_icon(o, t, 3, 1, 0, 0, 1, colour, (totalmillis%1000)/1000.f, 0.1f);
+            part_icon(o, textureload(hud::progringtex, 3), 3, 1, 0, 0, 1, colour, (totalmillis%1000)/1000.f, 0.1f);
+            part_icon(o, textureload(hud::progresstex, 3), 3, 1, 0, 0, 1, colour, 0, length);
             break;
         }
         case 32: //lens flares - plain/sparkle/sun/sparklesun <red> <green> <blue>
@@ -1726,11 +1730,13 @@ void makeparticle(const vec &o, attrvector &attr)
             part_textcopy(o, ds);
             break;
     }
+    emit = oldemit;
 }
-void makeparticles(extentity &e) { makeparticle(e.o, e.attrs); }
 
 void updateparticles()
 {
+    if(minimized) { emit = false; return; }
+
     if(lastmillis-lastemitframe >= emitmillis)
     {
         emit = true;
