@@ -1,9 +1,16 @@
 #include "engine.h"
 
-VAR(IDF_PERSIST, textblinking, 0, 350, VAR_MAX);
-FVARF(IDF_PERSIST, textscale, FVAR_NONZERO, 1, FVAR_MAX, UI::setup());
+VAR(IDF_PERSIST, textblinking, 0, 250, VAR_MAX);
+FVAR(IDF_PERSIST, textscale, FVAR_NONZERO, 1, FVAR_MAX);
 VAR(IDF_PERSIST, textfaded, 0, 1, 1);
 VAR(IDF_PERSIST, textminintensity, 0, 32, 255);
+VARF(IDF_PERSIST, textkeybg, 0, 1, 1, changedkeys = totalmillis);
+VARF(IDF_PERSIST, textkeyseps, 0, 1, 1, changedkeys = totalmillis);
+VAR(IDF_PERSIST|IDF_HEX, textkeybgcolour, 0x000000, 0xFFFFFF, 0xFFFFFF);
+VAR(IDF_PERSIST|IDF_HEX, textkeyfgcolour, 0x000000, 0x00FFFF, 0xFFFFFF);
+FVAR(IDF_PERSIST, textkeybgblend, 0, 0.25f, 1);
+FVAR(IDF_PERSIST, textkeyfgblend, 0, 1, 1);
+TVAR(IDF_PERSIST|IDF_PRELOAD, textkeybgtex, "textures/textkeybg", 3);
 
 static inline bool htcmp(const char *key, const font &f) { return !strcmp(key, f.name); }
 
@@ -21,6 +28,7 @@ void newfont(char *name, char *tex, int *defaultw, int *defaulth)
     f->texs.shrink(0);
     f->texs.add(textureload(tex));
     f->chars.shrink(0);
+
     f->charoffset = '!';
     f->maxw = f->defaultw = *defaultw;
     f->maxh = f->defaulth = f->scale = *defaulth;
@@ -92,7 +100,7 @@ font *loadfont(const char *name)
     font *f = fonts.access(name);
     if(!f)
     {
-        defformatstring(n)("fonts/%s.cfg", name);
+        defformatstring(n)("config/fonts/%s.cfg", name);
         if(execfile(n, false)) f = fonts.access(name);
     }
     return f;
@@ -171,7 +179,7 @@ COMMAND(0, tabify, "si");
 
 int draw_textf(const char *fstr, int left, int top, ...)
 {
-    defvformatstring(str, top, fstr);
+    defvformatbigstring(str, top, fstr);
     return draw_text(str, left, top);
 }
 
@@ -241,7 +249,8 @@ static void text_color(char c, cvec *stack, int size, int &sp, cvec &color, int
         }
         case 'S': // restore
         {
-            color = stack[sp > 0 ? --sp : sp];
+            if(sp > 0) --sp;
+            color = stack[sp];
             break;
         }
         default: color = stack[sp]; break; // everything else
@@ -250,7 +259,6 @@ static void text_color(char c, cvec *stack, int size, int &sp, cvec &color, int
     glColor4ub((uchar)color.r, (uchar)color.g, (uchar)color.b, (uchar)color.a);
 }
 
-#define FONTX int(curfont->maxh*textscale)
 static const char *gettexvar(const char *var)
 {
     ident *id = getident(var);
@@ -269,8 +277,8 @@ static float draw_icon(Texture *&tex, const char *name, float x, float y, float
     if(!name && !*name) return 0;
     const char *file = name;
     if(*file == '$') file = gettexvar(++file);
-    if(!*file) { conoutf("invalid texture: %s (%s)", name, file); return 0; }
-    Texture *t = textureload(file, 3);
+    if(!*file) return 0;
+    Texture *t = textureload(file, 3, true, false);
     if(!t) return 0;
     if(tex != t)
     {
@@ -278,7 +286,7 @@ static float draw_icon(Texture *&tex, const char *name, float x, float y, float
         tex = t;
         glBindTexture(GL_TEXTURE_2D, tex->id);
     }
-    float h = scale*FONTX, w = (t->w*h)/float(t->h);
+    float h = curfont->maxh*scale, w = (t->w*h)/float(t->h);
     varray::attrib<float>(x,     y    ); varray::attrib<float>(0, 0);
     varray::attrib<float>(x + w, y    ); varray::attrib<float>(1, 0);
     varray::attrib<float>(x + w, y + h); varray::attrib<float>(1, 1);
@@ -291,10 +299,10 @@ static float icon_width(const char *name, float scale)
     if(!name && !*name) return 0;
     const char *file = name;
     if(*file == '$') file = gettexvar(++file);
-    if(!*file) { conoutf("invalid texture: %s (%s)", name, file); return 0; }
-    Texture *t = textureload(file, 3);
+    if(!*file) return 0;
+    Texture *t = textureload(file, 3, true, false);
     if(!t) return 0;
-    return (t->w*scale*FONTX)/t->h;
+    return (t->w*curfont->maxh*scale)/t->h;
 }
 
 #define TEXTCOLORIZE(h,s) \
@@ -331,14 +339,31 @@ static float icon_width(const char *name, float scale)
         { \
             if(s && end > start) \
             { \
-                string value; \
-                copystring(value, start, min(size_t(end - start + 1), sizeof(value))); \
+                bigstring value; \
+                copybigstring(value, start, min(size_t(end - start + 1), sizeof(value))); \
                 TEXTICON(value); \
             } \
             h += end-start; \
         } \
         else break; \
     } \
+    else if(str[h] == '{') \
+    { \
+        h++; \
+        const char *start = &str[h]; \
+        const char *end = strchr(start, '}'); \
+        if(end) \
+        { \
+            if(s && end > start) \
+            { \
+                bigstring value; \
+                copybigstring(value, start, min(size_t(end - start + 1), sizeof(value))); \
+                TEXTKEY(value); \
+            } \
+            h += end-start; \
+        } \
+        else break; \
+    } \
     else if(s) TEXTCOLOR(h); \
 }
 #define TEXTALIGN \
@@ -355,7 +380,7 @@ static float icon_width(const char *name, float scale)
         int c = uchar(str[i]);\
         TEXTINDEX(i)\
         if(c == '\t')      { x = TEXTTAB(x); TEXTWHITE(i) }\
-        else if(c == ' ')  { x += scale*curfont->defaultw*textscale; TEXTWHITE(i) }\
+        else if(c == ' ')  { x += scale*curfont->defaultw; TEXTWHITE(i) }\
         else if(c == '\n') { TEXTLINE(i) TEXTALIGN }\
         else if(c == '\f') { if(str[i+1]) { i++; TEXTCOLORIZE(i, true); } }\
         else if(curfont->chars.inrange(c-curfont->charoffset))\
@@ -403,6 +428,7 @@ int text_visible(const char *str, float hitx, float hity, int maxwidth, int flag
     #define TEXTCOLOR(idx)
     #define TEXTHEXCOLOR(ret)
     #define TEXTICON(ret) x += icon_width(ret, scale);
+    #define TEXTKEY(ret) x += (textkeybg ? icon_width(textkeybgtex, scale)*0.6f : 0.f)+text_widthf(ret, flags);
     #define TEXTCHAR(idx) x += cw; TEXTWHITE(idx)
     #define TEXTWORD TEXTWORDSKELETON
     TEXTSKELETON
@@ -412,6 +438,7 @@ int text_visible(const char *str, float hitx, float hity, int maxwidth, int flag
     #undef TEXTCOLOR
     #undef TEXTHEXCOLOR
     #undef TEXTICON
+    #undef TEXTKEY
     #undef TEXTCHAR
     #undef TEXTWORD
     return i;
@@ -425,7 +452,8 @@ void text_posf(const char *str, int cursor, float &cx, float &cy, int maxwidth,
     #define TEXTLINE(idx)
     #define TEXTCOLOR(idx)
     #define TEXTHEXCOLOR(ret)
-    #define TEXTICON(ret) x += icon_width(ret, scale)+2;
+    #define TEXTICON(ret) x += icon_width(ret, scale);
+    #define TEXTKEY(ret) x += (textkeybg ? icon_width(textkeybgtex, scale)*0.6f : 0.f)+text_widthf(ret, flags);
     #define TEXTCHAR(idx) x += cw;
     #define TEXTWORD TEXTWORDSKELETON if(i >= cursor) break;
     cx = cy = 0;
@@ -437,6 +465,7 @@ void text_posf(const char *str, int cursor, float &cx, float &cy, int maxwidth,
     #undef TEXTCOLOR
     #undef TEXTHEXCOLOR
     #undef TEXTICON
+    #undef TEXTKEY
     #undef TEXTCHAR
     #undef TEXTWORD
 }
@@ -448,7 +477,8 @@ void text_boundsf(const char *str, float &width, float &height, int maxwidth, in
     #define TEXTLINE(idx) if(x > width) width = x;
     #define TEXTCOLOR(idx)
     #define TEXTHEXCOLOR(ret)
-    #define TEXTICON(ret) x += icon_width(ret, scale)+2;
+    #define TEXTICON(ret) x += icon_width(ret, scale);
+    #define TEXTKEY(ret) x += (textkeybg ? icon_width(textkeybgtex, scale)*0.6f : 0.f)+text_widthf(ret, flags);
     #define TEXTCHAR(idx) x += cw;
     #define TEXTWORD TEXTWORDSKELETON
     width = 0;
@@ -461,10 +491,90 @@ void text_boundsf(const char *str, float &width, float &height, int maxwidth, in
     #undef TEXTCOLOR
     #undef TEXTHEXCOLOR
     #undef TEXTICON
+    #undef TEXTKEY
     #undef TEXTCHAR
     #undef TEXTWORD
 }
 
+int draw_key(Texture *&tex, const char *str, float sx, float sy, float sc, cvec &cl, int flags)
+{
+    float swidth = text_widthf(str, flags), ss = 0, sp = 0;
+    if(textkeybg)
+    {
+        Texture *t = textureload(textkeybgtex, 3, true, false);
+        if(tex != t)
+        {
+            xtraverts += varray::end();
+            tex = t;
+            glBindTexture(GL_TEXTURE_2D, tex->id);
+        }
+
+        glColor4ub(uchar((textkeybgcolour>>16)&0xFF), uchar((textkeybgcolour>>8)&0xFF), uchar(textkeybgcolour&0xFF), uchar(textkeybgblend*cl.a));
+
+        float sh = curfont->maxh*sc, sw = (t->w*sh)/float(t->h), w1 = sw*0.25f, w2 = sw*0.5f, amt = swidth/w2;
+        int count = int(floorf(amt));
+        varray::attrib<float>(sx + ss,     sy    ); varray::attrib<float>(0, 0);
+        varray::attrib<float>(sx + ss + w1, sy    ); varray::attrib<float>(0.25f, 0);
+        varray::attrib<float>(sx + ss + w1, sy + sh); varray::attrib<float>(0.25f, 1);
+        varray::attrib<float>(sx + ss,     sy + sh); varray::attrib<float>(0, 1);
+        sp = (ss += w1);
+        loopi(count)
+        {
+            varray::attrib<float>(sx + ss,     sy    ); varray::attrib<float>(0.25f, 0);
+            varray::attrib<float>(sx + ss + w2, sy    ); varray::attrib<float>(0.75f, 0);
+            varray::attrib<float>(sx + ss + w2, sy + sh); varray::attrib<float>(0.75f, 1);
+            varray::attrib<float>(sx + ss,     sy + sh); varray::attrib<float>(0.25f, 1);
+            ss += w2;
+        }
+        float w3 = amt-float(count), w4 = w1 + w2*w3, w5 = 0.75f - 0.5f*w3;
+        varray::attrib<float>(sx + ss,     sy    ); varray::attrib<float>(w5, 0);
+        varray::attrib<float>(sx + ss + w4, sy    ); varray::attrib<float>(1, 0);
+        varray::attrib<float>(sx + ss + w4, sy + sh); varray::attrib<float>(1, 1);
+        varray::attrib<float>(sx + ss,     sy + sh); varray::attrib<float>(w5, 1);
+        ss += w4;
+    }
+    else ss = swidth;
+    xtraverts += varray::end();
+
+    #define TEXTINDEX(idx)
+    #define TEXTWHITE(idx)
+    #define TEXTLINE(idx) ly += FONTH;
+    #define TEXTCOLOR(idx) text_color(str[idx], colorstack, sizeof(colorstack), colorpos, color, r, g, b, fade);
+    #define TEXTHEXCOLOR(ret) \
+        if(usecolor) \
+        { \
+            int alpha = colorstack[colorpos].a; \
+            color = TVECA(ret, alpha); \
+            colorstack[colorpos] = color; \
+            xtraverts += varray::end(); \
+            glColor4ub((uchar)color.r, (uchar)color.g, (uchar)color.b, (char)color.a); \
+        }
+    #define TEXTICON(ret) x += draw_icon(tex, ret, left+x, top+y, scale);
+    #define TEXTKEY(ret) x += draw_key(tex, ret, left+x, top+y, scale, color, flags);
+    #define TEXTCHAR(idx) { draw_char(tex, c, left+x, top+y, scale); x += cw; }
+    #define TEXTWORD TEXTWORDSKELETON
+    bool usecolor = true;
+    int fade = textkeyfgblend*cl.a, r = (textkeyfgcolour>>16)&0xFF, g = (textkeyfgcolour>>8)&0xFF, b = textkeyfgcolour&0xFF,
+        colorpos = 1, ly = 0, left = sx + sp, top = sy, cursor = -1, maxwidth = -1;
+    cvec colorstack[16], color = TVECX(r, g, b, fade);
+    loopi(16) colorstack[i] = color;
+    glColor4ub((uchar)color.r, (uchar)color.g, (uchar)color.b, (uchar)color.a);
+    TEXTSKELETON
+    TEXTEND(cursor)
+    xtraverts += varray::end();
+
+    glColor4ub((uchar)cl.r, (uchar)cl.g, (uchar)cl.b, (uchar)cl.a);
+
+    #undef TEXTINDEX
+    #undef TEXTWHITE
+    #undef TEXTLINE
+    #undef TEXTCOLOR
+    #undef TEXTHEXCOLOR
+    #undef TEXTCHAR
+    #undef TEXTWORD
+    return ss;
+}
+
 int draw_text(const char *str, int rleft, int rtop, int r, int g, int b, int a, int flags, int cursor, int maxwidth)
 {
     #define TEXTINDEX(idx) \
@@ -491,6 +601,7 @@ int draw_text(const char *str, int rleft, int rtop, int r, int g, int b, int a,
             glColor4ub((uchar)color.r, (uchar)color.g, (uchar)color.b, (char)color.a); \
         }
     #define TEXTICON(ret) x += draw_icon(tex, ret, left+x, top+y, scale);
+    #define TEXTKEY(ret) x += draw_key(tex, ret, left+x, top+y, scale, color, flags);
     #define TEXTCHAR(idx) { draw_char(tex, c, left+x, top+y, scale); x += cw; }
     #define TEXTWORD TEXTWORDSKELETON
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -538,7 +649,7 @@ void reloadfonts()
 
 int draw_textx(const char *fstr, int left, int top, int r, int g, int b, int a, int flags, int cursor, int maxwidth, ...)
 {
-    defvformatstring(str, maxwidth, fstr);
+    defvformatbigstring(str, maxwidth, fstr);
 
     int width = 0, height = 0;
     text_bounds(str, width, height, maxwidth, flags);
@@ -548,7 +659,8 @@ int draw_textx(const char *fstr, int left, int top, int r, int g, int b, int a,
         case TEXT_RIGHT_JUSTIFY: left -= width; break;
         default: break;
     }
-    if(flags&TEXT_UPWARD) top -= height;
+    if(flags&TEXT_BALLOON) top -= height/2;
+    else if(flags&TEXT_UPWARD) top -= height;
     if(flags&TEXT_SHADOW) draw_text(str, left-2, top-2, 0, 0, 0, a, flags, cursor, maxwidth);
     return draw_text(str, left, top, r, g, b, a, flags, cursor, maxwidth);
 }
