@@ -226,14 +226,44 @@ static inline bool insideva(const vtxarray *va, const vec &v, int margin = 2)
 ///////// occlusion queries /////////////
 
 #define MAXQUERY 2048
+#define MAXQUERYFRAMES 2
 
 struct queryframe
 {
     int cur, max;
     occludequery queries[MAXQUERY];
+
+    queryframe() : cur(0), max(0) {}
+
+    void flip() { loopi(cur) queries[i].owner = NULL; cur = 0; }
+
+    occludequery *newquery(void *owner)
+    {
+        if(cur >= max)
+        {
+            if(max >= MAXQUERY) return NULL;
+            glGenQueries_(1, &queries[max++].id);
+        }
+        occludequery *query = &queries[cur++];
+        query->owner = owner;
+        query->fragments = -1;
+        return query;
+    }
+
+    void reset() { loopi(max) queries[i].owner = NULL; }
+
+    void cleanup()
+    {
+        loopi(max)
+        {
+            glDeleteQueries_(1, &queries[i].id);
+            queries[i].owner = NULL;
+        }
+        cur = max = 0;
+    }
 };
 
-static queryframe queryframes[2] = {{0, 0}, {0, 0}};
+static queryframe queryframes[MAXQUERYFRAMES];
 static uint flipquery = 0;
 
 int getnumqueries()
@@ -243,43 +273,23 @@ int getnumqueries()
 
 void flipqueries()
 {
-    flipquery = (flipquery + 1) % 2;
-    queryframe &qf = queryframes[flipquery];
-    loopi(qf.cur) qf.queries[i].owner = NULL;
-    qf.cur = 0;
+    flipquery = (flipquery + 1) % MAXQUERYFRAMES;
+    queryframes[flipquery].flip();
 }
 
 occludequery *newquery(void *owner)
 {
-    queryframe &qf = queryframes[flipquery];
-    if(qf.cur >= qf.max)
-    {
-        if(qf.max >= MAXQUERY) return NULL;
-        glGenQueries_(1, &qf.queries[qf.max++].id);
-    }
-    occludequery *query = &qf.queries[qf.cur++];
-    query->owner = owner;
-    query->fragments = -1;
-    return query;
+    return queryframes[flipquery].newquery(owner);
 }
 
 void resetqueries()
 {
-    loopi(2) loopj(queryframes[i].max) queryframes[i].queries[j].owner = NULL;
+    loopi(MAXQUERYFRAMES) queryframes[i].reset();
 }
 
 void clearqueries()
 {
-    loopi(2)
-    {
-        queryframe &qf = queryframes[i];
-        loopj(qf.max)
-        {
-            glDeleteQueries_(1, &qf.queries[j].id);
-            qf.queries[j].owner = NULL;
-        }
-        qf.cur = qf.max = 0;
-    }
+    loopi(MAXQUERYFRAMES) queryframes[i].cleanup();
 }
 
 VAR(0, oqfrags, 0, 8, 64);
@@ -344,7 +354,7 @@ void findvisiblemms(const vector<extentity *> &ents)
         loopv(va->mapmodels)
         {
             octaentities *oe = va->mapmodels[i];
-            if(isfoggedcube(oe->o, oe->size) || pvsoccluded(oe->bbmin, ivec(oe->bbmax).sub(oe->bbmin))) continue;
+            if(isfoggedcube(oe->o, oe->size) || pvsoccluded(oe->bbmin, oe->bbmax)) continue;
 
             bool occluded = oe->query && oe->query->owner == oe && checkquery(oe->query);
             if(occluded)
@@ -361,8 +371,8 @@ void findvisiblemms(const vector<extentity *> &ents)
                 loopv(oe->mapmodels)
                 {
                     extentity &e = *ents[oe->mapmodels[i]];
-                    if(e.lastemit && e.spawned && e.attrs[6]&MMT_HIDE) continue;
-                    e.visible = true;
+                    if(e.lastemit && e.spawned() && e.attrs[6]&MMT_HIDE) continue;
+                    e.flags |= EF_RENDER;
                     ++visible;
                 }
                 if(!visible) continue;
@@ -386,8 +396,6 @@ void findvisiblemms(const vector<extentity *> &ents)
 
 VAR(0, oqmm, 0, 4, 8);
 
-extern bool getentboundingbox(extentity &e, ivec &o, ivec &r);
-
 VAR(0, mmanimoverride, -1, 0, ANIM_ALL);
 
 void rendermapmodel(extentity &e)
@@ -395,8 +403,8 @@ void rendermapmodel(extentity &e)
     int anim = ANIM_MAPMODEL|ANIM_LOOP, basetime = 0, flags = MDL_CULL_VFC|MDL_CULL_DIST|MDL_DYNLIGHT;
     if(e.lastemit)
     {
-        if(e.attrs[6]&MMT_HIDE && e.spawned) return;
-        anim = e.spawned ? ANIM_TRIGGER_ON : ANIM_TRIGGER_OFF;
+        if(e.attrs[6]&MMT_HIDE && e.spawned()) return;
+        anim = e.spawned() ? ANIM_TRIGGER_ON : ANIM_TRIGGER_OFF;
         if(e.lastemit > 0 && lastmillis-e.lastemit < entities::triggertime(e)) basetime = e.lastemit;
         else anim |= ANIM_END;
     }
@@ -451,8 +459,8 @@ void renderreflectedmapmodels()
         loopv(oe->mapmodels)
         {
            extentity &e = *ents[oe->mapmodels[i]];
-           if(e.visible || (e.lastemit && e.spawned && e.attrs[6]&MMT_HIDE)) continue;
-           e.visible = true;
+           if(e.flags&EF_RENDER || (e.lastemit && e.spawned() && e.attrs[6]&MMT_HIDE)) continue;
+           e.flags |= EF_RENDER;
         }
     }
     if(mms)
@@ -463,9 +471,9 @@ void renderreflectedmapmodels()
             loopv(oe->mapmodels)
             {
                 extentity &e = *ents[oe->mapmodels[i]];
-                if(!e.visible) continue;
+                if(!(e.flags&EF_RENDER)) continue;
                 rendermapmodel(e);
-                e.visible = false;
+                e.flags &= ~EF_RENDER;
             }
         }
         endmodelbatches();
@@ -490,7 +498,7 @@ void rendermapmodels()
         loopv(oe->mapmodels)
         {
             extentity &e = *ents[oe->mapmodels[i]];
-            if(!e.visible) continue;
+            if(!(e.flags&EF_RENDER)) continue;
             if(!rendered)
             {
                 rendered = true;
@@ -498,7 +506,7 @@ void rendermapmodels()
                 if(oe->query) startmodelquery(oe->query);
             }
             rendermapmodel(e);
-            e.visible = false;
+            e.flags &= ~EF_RENDER;
         }
         if(rendered && oe->query) endmodelquery();
     }
@@ -530,7 +538,7 @@ void rendermapmodels()
 static inline bool bbinsideva(const ivec &bo, const ivec &br, vtxarray *va)
 {
     return bo.x >= va->bbmin.x && bo.y >= va->bbmin.y && va->o.z >= va->bbmin.z &&
-        bo.x + br.x <= va->bbmax.x && bo.y + br.y <= va->bbmax.y && bo.z + br.z <= va->bbmax.z;
+        br.x <= va->bbmax.x && br.y <= va->bbmax.y && br.z <= va->bbmax.z;
 }
 
 static inline bool bboccluded(const ivec &bo, const ivec &br, cube *c, const ivec &o, int size)
@@ -551,7 +559,7 @@ static inline bool bboccluded(const ivec &bo, const ivec &br, cube *c, const ive
 
 bool bboccluded(const ivec &bo, const ivec &br)
 {
-    int diff = (bo.x^(bo.x+br.x)) | (bo.y^(bo.y+br.y)) | (bo.z^(bo.z+br.z));
+    int diff = (bo.x^br.x) | (bo.y^br.y) | (bo.z^br.z);
     if(diff&~((1<<worldscale)-1)) return false;
     int scale = worldscale-1;
     if(diff&(1<<scale)) return bboccluded(bo, br, worldroot, ivec(0, 0, 0), 1<<scale);
@@ -2011,8 +2019,8 @@ void cleanupva()
     loopi(NUMCAUSTICS) caustictex[i] = NULL;
 }
 
-VAR(IDF_WORLD, causticscale, 0, 50, 10000);
-VAR(IDF_WORLD, causticmillis, 0, 75, 1000);
+VAR(IDF_WORLD, causticscale, 1, 50, 10000);
+VAR(IDF_WORLD, causticmillis, 1, 75, 1000);
 VARF(IDF_PERSIST, caustics, 0, 1, 1, loadcaustics());
 
 void setupcaustics(int tmu, float blend, GLfloat *color = NULL)
