@@ -6,27 +6,49 @@
 #include <shlobj.h>
 #endif
 
-bool versioning = false;
-VAR(0, version, 1, 0, -1);
-VAR(0, versionmajor, 0, 0, VAR_MAX);
-VAR(0, versionminor, 0, 0, VAR_MAX);
-VAR(0, versionpatch, 0, 0, VAR_MAX);
-SVAR(0, versionstring, "0.0");
-SVAR(0, versionname, "");
-SVAR(0, versionuname, "");
-SVAR(0, versionrelease, "");
-SVAR(0, versionurl, "");
+const char *platnames[MAX_PLATFORMS] = {
+    "win", "osx", "nix"
+}, *platlongnames[MAX_PLATFORMS] = {
+    "windows", "macosx", "linux/bsd"
+};
+
+VAR(0, versioning, 1, 0, -1);
+
+VAR(IDF_READONLY, version, 1, CUR_VERSION, -1);
+VAR(IDF_READONLY, versionmajor, 1, VERSION_MAJOR, -1);
+VAR(IDF_READONLY, versionminor, 1, VERSION_MINOR, -1);
+VAR(IDF_READONLY, versionpatch, 1, VERSION_PATCH, -1);
+SVAR(IDF_READONLY, versionstring, VERSION_STRING);
+SVAR(IDF_READONLY, versionname, VERSION_NAME);
+SVAR(IDF_READONLY, versionuname, VERSION_UNAME);
+SVAR(IDF_READONLY, versionrelease, VERSION_RELEASE);
+SVAR(IDF_READONLY, versionurl, VERSION_URL);
+SVAR(IDF_READONLY, versioncopy, VERSION_COPY);
+SVAR(IDF_READONLY, versiondesc, VERSION_DESC);
+SVAR(IDF_READONLY, versionplatname, plat_name(CUR_PLATFORM));
+SVAR(IDF_READONLY, versionplatlongname, plat_longname(CUR_PLATFORM));
+VAR(IDF_READONLY, versionplatform, 0, CUR_PLATFORM, VAR_MAX);
+VAR(IDF_READONLY, versionarch, 0, CUR_ARCH, VAR_MAX);
+#ifdef STANDALONE
+VAR(IDF_READONLY, versionisserver, 0, 1, 1);
+#else
+VAR(IDF_READONLY, versionisserver, 0, 0, 1);
+#endif
+uint versioncrc = 0;
+ICOMMAND(0, platname, "ii", (int *p, int *g), result(*p >= 0 && *p < MAX_PLATFORMS ? (*g!=0 ? plat_longname(*p) : plat_name(*p)) : ""));
 
 VAR(0, rehashing, 1, 0, -1);
 
-extern const char * const disc_reasons[] = { "normal", "end of packet", "client num", "user was kicked", "message error", "address is banned", "server is in private mode", "server is full", "connection timed out", "packet overflow", "server shutting down" };
+const char * const disc_reasons[] = { "normal", "end of packet", "client num", "user was kicked", "message error", "address is banned", "server is in private mode", "server is password protected", "server requires pure official builds", "server is at maximum capacity", "server and client are incompatible", "connection timed out", "packet overflow", "server shutting down" };
 
-SVAR(IDF_PERSIST, logtimeformat, "%c");
+SVAR(IDF_PERSIST, logtimeformat, "%Y-%m-%d %H:%M.%S");
 SVAR(IDF_PERSIST, filetimeformat, "%Y%m%d%H%M%S");
+VAR(IDF_PERSIST, filetimelocal, 0, 1, 1); // use clockoffset to localise
+
 const char *gettime(time_t ctime, const char *format)
 {
     static string buf;
-    if(!ctime) ctime = clocktime;
+    if(!ctime) ctime = currenttime;
     struct tm *t = localtime(&ctime);
     if(!strftime(buf, sizeof(buf), format && *format ? format : logtimeformat, t)) buf[0] = '\0';
     return buf;
@@ -37,13 +59,12 @@ const char *timestr(int dur, int style)
 {
     static string buf; buf[0] = 0;
     int tm = dur, ms = 0, ss = 0, mn = 0;
-    if(style < 2 && tm > 0)
+    if(tm > 0)
     {
         ms = tm%1000;
         tm = (tm-ms)/1000;
     }
-    if(style < 0 && tm > 0) ss = tm;
-    else if(style < 4 && tm > 0)
+    if(style > 0 && tm > 0)
     {
         ss = tm%60;
         tm = (tm-ss)/60;
@@ -51,11 +72,11 @@ const char *timestr(int dur, int style)
     }
     switch(style)
     {
-        case -1: formatstring(buf)("%d.%d", ss, ms/100); break;
-        case 0: formatstring(buf)("%d:%02d.%03d", mn, ss, ms); break;
-        case 1: formatstring(buf)("%d:%02d.%d", mn, ss, ms/100); break;
-        case 2: formatstring(buf)("%d:%02d", mn, ss); break;
-        case 3:
+        case 0: formatstring(buf)("%d.%d", tm, ms/100); break;
+        case 1: formatstring(buf)("%d:%02d.%03d", mn, ss, ms); break;
+        case 2: formatstring(buf)("%d:%02d.%d", mn, ss, ms/100); break;
+        case 3: formatstring(buf)("%d:%02d", mn, ss); break;
+        case 4:
         {
             if(mn > 0)
             {
@@ -71,7 +92,7 @@ const char *timestr(int dur, int style)
 ICOMMAND(0, timestr, "ii", (int *d, int *s), result(timestr(*d, *s)));
 
 vector<ipinfo> control;
-void addipinfo(vector<ipinfo> &info, int type, const char *name)
+void addipinfo(vector<ipinfo> &info, int type, const char *name, const char *reason)
 {
     union { uchar b[sizeof(enet_uint32)]; enet_uint32 i; } ip, mask;
     ip.i = 0;
@@ -94,12 +115,13 @@ void addipinfo(vector<ipinfo> &info, int type, const char *name)
 #ifdef STANDALONE
     p.version = nextcontrolversion();
 #endif
+    if(reason && *reason) p.reason = newstring(reason);
     server::updatecontrols = true;
 }
-ICOMMAND(0, addallow, "s", (char *name), addipinfo(control, ipinfo::ALLOW, name));
-ICOMMAND(0, addban, "s", (char *name), addipinfo(control, ipinfo::BAN, name));
-ICOMMAND(0, addmute, "s", (char *name), addipinfo(control, ipinfo::MUTE, name));
-ICOMMAND(0, addlimit, "s", (char *name), addipinfo(control, ipinfo::LIMIT, name));
+ICOMMAND(0, addallow, "ss", (char *name, char *reason), addipinfo(control, ipinfo::ALLOW, name, reason));
+ICOMMAND(0, addban, "ss", (char *name, char *reason), addipinfo(control, ipinfo::BAN, name, reason));
+ICOMMAND(0, addmute, "ss", (char *name, char *reason), addipinfo(control, ipinfo::MUTE, name, reason));
+ICOMMAND(0, addlimit, "ss", (char *name, char *reason), addipinfo(control, ipinfo::LIMIT, name, reason));
 
 const char *ipinfotypes[ipinfo::MAXTYPES] = { "allow", "ban", "mute", "limit" };
 char *printipinfo(const ipinfo &info, char *buf)
@@ -120,10 +142,10 @@ char *printipinfo(const ipinfo &info, char *buf)
     return str;
 }
 
-bool checkipinfo(vector<ipinfo> &info, int type, enet_uint32 ip)
+ipinfo *checkipinfo(vector<ipinfo> &info, int type, enet_uint32 ip)
 {
-    loopv(info) if(info[i].type == type && (ip & info[i].mask) == info[i].ip) return true;
-    return false;
+    loopv(info) if(info[i].type == type && (ip & info[i].mask) == info[i].ip) return &info[i];
+    return NULL;
 }
 
 #define LOGSTRLEN 512
@@ -171,10 +193,10 @@ void logoutf(const char *fmt, ...)
 
 void console(int type, const char *s, ...)
 {
-    defvformatstring(sf, s, s);
-    string osf;
-    filtertext(osf, sf);
-    if(*logtimeformat) logoutf("%s %s", gettime(clocktime, logtimeformat), osf);
+    defvformatbigstring(sf, s, s);
+    bigstring osf;
+    filterbigstring(osf, sf);
+    if(*logtimeformat) logoutf("%s %s", gettime(currenttime, logtimeformat), osf);
     else logoutf("%s", osf);
 #ifndef STANDALONE
     conline(type, sf, 0);
@@ -183,14 +205,14 @@ void console(int type, const char *s, ...)
 
 void conoutft(int type, const char *s, ...)
 {
-    defvformatstring(sf, s, s);
+    defvformatbigstring(sf, s, s);
     console(type, "%s", sf);
     ircoutf(5, "%s", sf);
 }
 
 void conoutf(const char *s, ...)
 {
-    defvformatstring(sf, s, s);
+    defvformatbigstring(sf, s, s);
     conoutft(0, "%s", sf);
 }
 
@@ -198,11 +220,11 @@ VAR(0, verbose, 0, 0, 6);
 
 static void writelog(FILE *file, const char *buf)
 {
-    static uchar ubuf[512];
-    int len = strlen(buf), carry = 0;
+    static uchar ubuf[LOGSTRLEN];
+    size_t len = strlen(buf), carry = 0;
     while(carry < len)
     {
-        int numu = encodeutf8(ubuf, sizeof(ubuf)-1, &((const uchar *)buf)[carry], len - carry, &carry);
+        size_t numu = encodeutf8(ubuf, sizeof(ubuf)-1, &((const uchar *)buf)[carry], len - carry, &carry);
         if(carry >= len) ubuf[numu++] = '\n';
         fwrite(ubuf, 1, numu, file);
     }
@@ -218,28 +240,65 @@ static void writelogv(FILE *file, const char *fmt, va_list args)
 #ifdef STANDALONE
 void localservertoclient(int chan, ENetPacket *packet) {}
 VAR(0, servertype, 1, 3, 3); // 1: private, 2: public, 3: dedicated
+VAR(0, standalone, 1, 1, -1);
 #else
 VARF(0, servertype, 0, 0, 3, changeservertype()); // 0: local only, 1: private, 2: public, 3: dedicated
+VAR(0, standalone, 1, 0, -1);
 #endif
 VAR(0, serveruprate, 0, 0, VAR_MAX);
 VAR(0, serverport, 1, SERVER_PORT, VAR_MAX);
+VAR(0, serverlanport, 0, LAN_PORT, VAR_MAX);
 SVAR(0, serverip, "");
 
 int curtime = 0, totalmillis = 1, lastmillis = 1, timescale = 100, paused = 0, timeerr = 0;
-time_t clocktime = 0;
+time_t clocktime = 0, currenttime = 0, clockoffset = 0;
 uint totalsecs = 0;
 const char *load = NULL;
 vector<char *> gameargs;
 
-bool filtertext(char *dst, const char *src, bool newline, bool colour, bool whitespace, int len)
+bool filterword(char *src, const char *list)
 {
     bool filtered = false;
-    for(int c = uchar(*src); c; c = uchar(*++src))
+    int len = listlen(list);
+    loopi(len)
+    {
+        char *word = indexlist(list, i);
+        if(word)
+        {
+            if(*word)
+            {
+                char *t = src;
+                int count = strlen(word);
+                while((t = strstr(t, word)) != NULL)
+                {
+                    loopj(count) if(*t) *t++ = '*';
+                    filtered = true;
+                }
+            }
+            delete[] word;
+        }
+    }
+    return filtered;
+}
+
+ICOMMAND(0, filterword, "ss", (char *s, char *t),
+{
+    char *d = newstring(s);
+    filterstring(d, d, t);
+    stringret(d);
+});
+
+
+bool filterstring(char *dst, const char *src, bool newline, bool colour, bool whitespace, bool wsstrip, size_t len)
+{
+    bool filtered = false;
+    size_t n = 0;
+    for(int c = uchar(*src); c && n < len; c = uchar(*++src))
     {
         if(newline && (c=='\n' || c=='\r')) c = ' ';
         if(c=='\f')
         {
-            if(!colour) *dst++ = c;
+            if(!colour) dst[n++] = c;
             else
             {
                 filtered = true;
@@ -251,29 +310,32 @@ bool filtertext(char *dst, const char *src, bool newline, bool colour, bool whit
                     if(c) c = *++src;
                     if(!c) break;
                 }
-                else if(c == '[' || c == '(')
+                else if(c == '[' || c == '(' || c == '{')
                 {
-                    const char *end = strchr(src, c == '[' ? ']' : ')');
+                    const char *end = strchr(src, c == '[' ? ']' : (c == '(' ? ')' : '}'));
                     src += end ? end-src : strlen(src);
                 }
 
             }
             continue;
         }
-        if(iscubeprint(c) || (iscubespace(c) && whitespace))
-        {
-            *dst++ = c;
-            if(!--len) break;
-        }
+        if(iscubeprint(c) || (iscubespace(c) && whitespace && (!wsstrip || n)))
+            dst[n++] = c;
         else filtered = true;
     }
-    *dst = '\0';
+    if(whitespace && wsstrip && n) while(iscubespace(dst[n-1])) dst[--n] = 0;
+    dst[n < len ? n : len-1] = 0;
     return filtered;
 }
-ICOMMAND(0, filter, "siiiN", (char *s, int *a, int *b, int *c, int *numargs),
+bool filterbigstring(char *dst, const char *src, bool newline, bool colour, bool whitespace, bool wsstrip, size_t len)
 {
-    char *d = newstring(s);
-    filtertext(d, s, *numargs >= 2 ? *a>0 : true, *numargs >= 3 ? *b>0 : true, *numargs >= 4 ? *c>0 : true);
+    return filterstring(dst, src, newline, colour, whitespace, wsstrip, len);
+}
+ICOMMAND(0, filter, "siiiiN", (char *s, int *a, int *b, int *c, int *d, int *numargs),
+{
+    size_t len = strlen(s);
+    char *d = newstring(len);
+    filterstring(d, s, *numargs >= 2 ? *a>0 : true, *numargs >= 3 ? *b>0 : true, *numargs >= 4 ? *c>0 : true, *numargs >= 5 ? *d>0 : false, len);
     stringret(d);
 });
 
@@ -359,15 +421,17 @@ void process(ENetPacket *packet, int sender, int chan);
 int getservermtu() { return serverhost ? serverhost->mtu : -1; }
 void *getinfo(int i)            { return !clients.inrange(i) || clients[i]->type==ST_EMPTY ? NULL : clients[i]->info; }
 const char *gethostname(int i)  { int o = server::peerowner(i); return !clients.inrange(o) || clients[o]->type==ST_EMPTY ? "unknown" : clients[o]->hostname; }
+const char *gethostip(int i)    { int o = server::peerowner(i); return !clients.inrange(o) || clients[o]->type==ST_EMPTY ? "0.0.0.0" : clients[o]->hostip; }
 int getnumclients()             { return clients.length(); }
 uint getclientip(int n)         { int o = server::peerowner(n); return clients.inrange(o) && clients[o]->type==ST_TCPIP ? clients[o]->peer->address.host : 0; }
 
 void sendpacket(int n, int chan, ENetPacket *packet, int exclude)
 {
+    if(n < 0 || chan < 0) server::recordpacket(abs(chan), packet->data, packet->dataLength);
+    if(chan < 0) return; // was just a record packet
     if(n < 0)
     {
-        server::recordpacket(abs(chan), packet->data, packet->dataLength);
-        if(chan >= 0) loopv(clients) if(i != server::peerowner(exclude) && server::allowbroadcast(i)) sendpacket(i, chan, packet, exclude);
+        loopv(clients) if(i != server::peerowner(exclude) && server::allowbroadcast(i)) sendpacket(i, chan, packet, exclude);
         return;
     }
     switch(clients[n]->type)
@@ -376,10 +440,7 @@ void sendpacket(int n, int chan, ENetPacket *packet, int exclude)
         {
             int owner = server::peerowner(n);
             if(owner >= 0 && clients.inrange(owner) && owner != n && owner != server::peerowner(exclude))
-            {
-                //conoutf("redirect %d packet to %d [%d:%d]", n, owner, exclude, server::peerowner(exclude));
                 sendpacket(owner, chan, packet, exclude);
-            }
             break;
         }
         case ST_TCPIP:
@@ -486,10 +547,15 @@ void sendfile(int cn, int chan, stream *file, const char *format, ...)
 
 void disconnect_client(int n, int reason)
 {
-    if(clients[n]->type!=ST_TCPIP) return;
-    enet_peer_disconnect(clients[n]->peer, reason);
-    server::clientdisconnect(n, false, reason);
+    if(clients[n]->type==ST_TCPIP) enet_peer_disconnect(clients[n]->peer, reason);
+    server::clientdisconnect(n, clients[n]->type==ST_LOCAL, reason);
     delclient(n);
+    if(clients[n]->type==ST_LOCAL) loopv(clients) if(i != n && clients[i] && clients[i]->type==ST_LOCAL)
+    {
+        clientdata &c = *clients[i];
+        server::clientdisconnect(c.num, true);
+        delclient(c.num);
+    }
 }
 
 void kicknonlocalclients(int reason)
@@ -524,6 +590,7 @@ void localconnect(bool force)
         clientdata &c = *clients[cn];
         c.peer = NULL;
         copystring(c.hostname, "localhost");
+        copystring(c.hostip, "127.0.0.1");
         conoutf("\fglocal client %d connected", c.num);
         client::gameconnect(false);
         server::clientconnect(c.num, 0, true);
@@ -547,17 +614,16 @@ bool resolverwait(const char *name, ENetAddress *address)
     return enet_address_set_host(address, name) >= 0;
 }
 
-int connectwithtimeout(ENetSocket sock, const char *hostname, ENetAddress &remoteaddress)
+int connectwithtimeout(ENetSocket sock, const char *hostname, const ENetAddress &remoteaddress)
 {
-    int result = enet_socket_connect(sock, &remoteaddress);
-    if(result<0) enet_socket_destroy(sock);
-    return result;
+    return enet_socket_connect(sock, &remoteaddress);
 }
 #endif
 
 static ENetSocket mastersock = ENET_SOCKET_NULL;
 ENetAddress masteraddress = { ENET_HOST_ANY, ENET_PORT_ANY };
 static ENetAddress serveraddress = { ENET_HOST_ANY, ENET_PORT_ANY };
+static int masterconnecting = 0, masterconnected = 0;
 static vector<char> masterout, masterin;
 static int masteroutpos = 0, masterinpos = 0;
 
@@ -565,10 +631,10 @@ void disconnectmaster()
 {
     if(mastersock != ENET_SOCKET_NULL)
     {
-        server::disconnectedmaster();
+        server::masterdisconnected();
         enet_socket_destroy(mastersock);
         mastersock = ENET_SOCKET_NULL;
-        if(servertype >= 2) conoutf("disconnected from master server");
+        if(servertype >= 2 && masterconnected) conoutf("disconnected from master server");
     }
 
     masterout.setsize(0);
@@ -577,6 +643,7 @@ void disconnectmaster()
 
     masteraddress.host = ENET_HOST_ANY;
     masteraddress.port = ENET_PORT_ANY;
+    masterconnecting = masterconnected = 0;
 }
 
 VARF(0, servermasterport, 1, MASTER_PORT, INT_MAX-1, disconnectmaster());
@@ -586,7 +653,6 @@ ENetSocket connectmaster(bool reuse)
 {
     if(reuse && mastersock != ENET_SOCKET_NULL) return mastersock;
     if(!servermaster[0]) return ENET_SOCKET_NULL;
-
     if(masteraddress.host == ENET_HOST_ANY)
     {
         if(servertype >= 2) conoutf("\falooking up %s:[%d]...", servermaster, servermasterport);
@@ -598,20 +664,28 @@ ENetSocket connectmaster(bool reuse)
         }
     }
     ENetSocket sock = enet_socket_create(ENET_SOCKET_TYPE_STREAM);
-    if(sock != ENET_SOCKET_NULL && serveraddress.host != ENET_HOST_ANY && enet_socket_bind(sock, &serveraddress) < 0)
+    if(sock == ENET_SOCKET_NULL)
     {
-        enet_socket_destroy(sock);
-        sock = ENET_SOCKET_NULL;
-    }
-    if(sock == ENET_SOCKET_NULL || connectwithtimeout(sock, servermaster, masteraddress) < 0)
-    {
-        conoutf(sock==ENET_SOCKET_NULL ? "\frcould not open socket to connect to master server" : "\frcould not connect to master server");
+        conoutf("\frcould not open master server socket");
         return ENET_SOCKET_NULL;
     }
-
-    enet_socket_set_option(sock, ENET_SOCKOPT_NONBLOCK, 1);
-    if(reuse) mastersock = sock;
-    return sock;
+    if(!reuse || serveraddress.host == ENET_HOST_ANY || !enet_socket_bind(sock, &serveraddress))
+    {
+        enet_socket_set_option(sock, ENET_SOCKOPT_NONBLOCK, 1);
+        if(!reuse)
+        {
+            if(!connectwithtimeout(sock, servermaster, masteraddress)) return sock;
+        }
+        else if(!enet_socket_connect(sock, &masteraddress))
+        {
+            masterconnecting = totalmillis ? totalmillis : 1;
+            mastersock = sock;
+            return sock;
+        }
+    }
+    enet_socket_destroy(sock);
+    conoutf("\frcould not connect to master server");
+    return ENET_SOCKET_NULL;
 }
 
 bool connectedmaster() { return mastersock != ENET_SOCKET_NULL; }
@@ -624,6 +698,8 @@ bool requestmaster(const char *req)
     //    mastersock = connectmaster();
         if(mastersock == ENET_SOCKET_NULL) return false;
     //}
+    if(masterout.length() >= 4096) return false;
+
     masterout.put(req, strlen(req));
     return true;
 }
@@ -664,7 +740,12 @@ void processmasterinput()
 
 void flushmasteroutput()
 {
-    if(masterout.empty()) return;
+    if(masterconnecting && totalmillis - masterconnecting >= 60000)
+    {
+        conoutf("\frcould not connect to master server");
+        disconnectmaster();
+    }
+    if(masterout.empty() || !masterconnected) return;
 
     ENetBuffer buf;
     buf.data = &masterout[masteroutpos];
@@ -709,25 +790,26 @@ void sendqueryreply(ucharbuf &p)
     enet_socket_send(pongsock, &pongaddr, &buf, 1);
 }
 
+#define MAXPINGDATA 32
+
 void checkserversockets()        // reply all server info requests
 {
-    static ENetSocketSet sockset;
+    static ENetSocketSet readset, writeset;
+    ENET_SOCKETSET_EMPTY(readset);
+    ENET_SOCKETSET_EMPTY(writeset);
     ENetSocket maxsock = ENET_SOCKET_NULL;
-#define CHECKSOCKET(sock) \
+#define ADDSOCKET(sock, write) \
     if(sock != ENET_SOCKET_NULL) \
     { \
-        if(maxsock == ENET_SOCKET_NULL) \
-        { \
-            ENET_SOCKETSET_EMPTY(sockset); \
-            maxsock = sock; \
-        }  \
-        else maxsock = max(maxsock, sock); \
-        ENET_SOCKETSET_ADD(sockset, sock); \
+        maxsock = maxsock == ENET_SOCKET_NULL ? sock : max(maxsock, sock); \
+        ENET_SOCKETSET_ADD(readset, sock); \
+        if(write) ENET_SOCKETSET_ADD(writeset, sock); \
     }
-    CHECKSOCKET(pongsock);
-    CHECKSOCKET(mastersock);
-    CHECKSOCKET(lansock);
-    if(maxsock == ENET_SOCKET_NULL || enet_socketset_select(maxsock, &sockset, NULL, 0) <= 0) return;
+    ADDSOCKET(pongsock, false);
+    ADDSOCKET(mastersock, true);
+    ADDSOCKET(lansock, false);
+    bool ircsocks = ircaddsockets(maxsock, readset, writeset);
+    if(maxsock == ENET_SOCKET_NULL || enet_socketset_select(maxsock, &readset, &writeset, 0) <= 0) return;
 
     if(serverhost)
     {
@@ -736,18 +818,38 @@ void checkserversockets()        // reply all server info requests
         loopi(2)
         {
             ENetSocket sock = i ? lansock : pongsock;
-            if(sock == ENET_SOCKET_NULL || !ENET_SOCKETSET_CHECK(sockset, sock)) continue;
+            if(sock == ENET_SOCKET_NULL || !ENET_SOCKETSET_CHECK(readset, sock)) continue;
 
             buf.data = pong;
             buf.dataLength = sizeof(pong);
             int len = enet_socket_receive(sock, &pongaddr, &buf, 1);
-            if(len < 0) return;
+            if(len < 0 || len > MAXPINGDATA) continue;
             ucharbuf req(pong, len), p(pong, sizeof(pong));
             p.len += len;
             server::queryreply(req, p);
         }
     }
-    if(mastersock != ENET_SOCKET_NULL && ENET_SOCKETSET_CHECK(sockset, mastersock)) flushmasterinput();
+
+    if(mastersock != ENET_SOCKET_NULL)
+    {
+        if(!masterconnected)
+        {
+            if(ENET_SOCKETSET_CHECK(readset, mastersock) || ENET_SOCKETSET_CHECK(writeset, mastersock))
+            {
+                int error = 0;
+                if(enet_socket_get_option(mastersock, ENET_SOCKOPT_ERROR, &error) < 0 || error) disconnectmaster();
+                else
+                {
+                    masterconnecting = 0;
+                    masterconnected = totalmillis ? totalmillis : 1;
+                    server::masterconnected();
+                }
+            }
+        }
+        if(mastersock != ENET_SOCKET_NULL && ENET_SOCKETSET_CHECK(readset, mastersock)) flushmasterinput();
+    }
+
+    if(ircsocks) ircchecksockets(readset, writeset);
 }
 
 void serverslice(uint timeout)  // main server update, called from main loop in sp, or from below in dedicated server
@@ -788,8 +890,22 @@ void serverslice(uint timeout)  // main server update, called from main loop in
                 clientdata &c = *clients[cn];
                 c.peer = event.peer;
                 c.peer->data = &c;
-                static string hostn = "";
-                copystring(c.hostname, (enet_address_get_host_ip(&c.peer->address, hostn, sizeof(hostn))==0) ? hostn : "unknown");
+                if(enet_address_get_host_ip(&c.peer->address, c.hostip, sizeof(c.hostip)) >= 0)
+                {
+                    if(enet_address_get_host(&c.peer->address, c.hostname, sizeof(c.hostname)) >= 0)
+                    {
+                        ENetAddress address;
+                        string hostname;
+                        if(enet_address_set_host(&address, c.hostname) < 0 || enet_address_get_host_ip(&address, hostname, sizeof(hostname)) < 0 || strcmp(hostname, c.hostname))
+                            copystring(c.hostname, c.hostip);
+                    }
+                    else copystring(c.hostname, c.hostip);
+                }
+                else
+                {
+                    copystring(c.hostname, "unknown");
+                    copystring(c.hostip, "0.0.0.0");
+                }
                 int reason = server::clientconnect(c.num, c.peer->address.host);
                 if(reason) disconnect_client(c.num, reason);
                 break;
@@ -842,7 +958,10 @@ int getclockmillis()
 
 int updatetimer(bool limit)
 {
-    clocktime = time(NULL);
+    currenttime = time(NULL);
+    clocktime = mktime(gmtime(&currenttime));
+    clockoffset = currenttime-clocktime;
+
     int millis = getclockmillis();
 #ifndef STANDALONE
     if(limit) limitfps(millis, totalmillis);
@@ -889,7 +1008,7 @@ static HMENU appmenu = NULL;
 static HANDLE outhandle = NULL;
 static const int MAXLOGLINES = 200;
 struct logline { int len; char buf[LOGSTRLEN]; };
-static ringbuf<logline, MAXLOGLINES> loglines;
+static queue<logline, MAXLOGLINES> loglines;
 
 static void cleanupsystemtray()
 {
@@ -963,10 +1082,10 @@ static BOOL WINAPI consolehandler(DWORD dwCtrlType)
 static void writeline(logline &line)
 {
     static uchar ubuf[512];
-    int len = strlen(line.buf), carry = 0;
+    size_t len = strlen(line.buf), carry = 0;
     while(carry < len)
     {
-        int numu = encodeutf8(ubuf, sizeof(ubuf), &((uchar *)line.buf)[carry], len - carry, &carry);
+        size_t numu = encodeutf8(ubuf, sizeof(ubuf), &((uchar *)line.buf)[carry], len - carry, &carry);
         DWORD written = 0;
         WriteConsole(outhandle, ubuf, numu, &written, NULL);
     }
@@ -1082,6 +1201,7 @@ static void setupwindow(const char *title)
     atexit(cleanupwindow);
 
     if(!setupsystemtray(WM_APP)) fatal("failed adding to system tray");
+    conoutf("identity: v%s-%s%d %s (%s) [0x%x]", VERSION_STRING, versionplatname, versionarch, versionisserver ? "server" : "client", VERSION_RELEASE, versioncrc);
 }
 
 static char *parsecommandline(const char *src, vector<char *> &args)
@@ -1151,7 +1271,7 @@ void logoutfv(const char *fmt, va_list args)
 void serverloop()
 {
 #ifdef WIN32
-    defformatstring(cap)("%s server", versionname);
+    defformatstring(cap)("%s server", VERSION_NAME);
     setupwindow(cap);
     SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);
 #endif
@@ -1180,6 +1300,13 @@ void serverloop()
     exit(EXIT_SUCCESS);
 }
 
+void limitdupclients()
+{
+    if(!serverhost) return;
+    int dupclients = server::dupclients();
+    serverhost->duplicatePeers = dupclients ? dupclients : serverhost->peerCount;
+}
+
 int setupserversockets()
 {
     if(!servertype || (serverhost && pongsock != ENET_SOCKET_NULL)) return servertype;
@@ -1187,7 +1314,11 @@ int setupserversockets()
     ENetAddress address = { ENET_HOST_ANY, enet_uint16(serverport) };
     if(*serverip)
     {
-        if(enet_address_set_host(&address, serverip) < 0) conoutf("\frWARNING: server address not resolved");
+        if(enet_address_set_host(&address, serverip) < 0)
+        {
+            setsvar("serverip", "");
+            conoutf("\frWARNING: server address not resolved");
+        }
         else serveraddress.host = address.host;
     }
 
@@ -1196,13 +1327,15 @@ int setupserversockets()
         serverhost = enet_host_create(&address, server::reserveclients(), server::numchannels(), 0, serveruprate);
         if(!serverhost)
         {
-            conoutf("\frcould not create server socket");
-#ifndef STANDALONE
+#ifdef STANDALONE
+            fatal("could not create server socket on port %d", serverport);
+#else
+            conoutf("\frcould not create server socket on port %d", serverport);
             setvar("servertype", 0);
 #endif
             return servertype;
         }
-        loopi(server::reserveclients()) serverhost->peers[i].data = NULL;
+        limitdupclients();
     }
 
     if(pongsock == ENET_SOCKET_NULL)
@@ -1216,23 +1349,27 @@ int setupserversockets()
         }
         if(pongsock == ENET_SOCKET_NULL)
         {
-            conoutf("\frcould not create server info socket, publicity disabled");
-#ifndef STANDALONE
+#ifdef STANDALONE
+            fatal("could not create server info socket on port %d", serverport+1);
+#else
+            conoutf("\frcould not create server info socket on port %d, publicity disabled", serverport+1);
             setvar("servertype", 1);
 #endif
             return servertype;
         }
         enet_socket_set_option(pongsock, ENET_SOCKOPT_NONBLOCK, 1);
-
-        address.port = LAN_PORT;
-        lansock = enet_socket_create(ENET_SOCKET_TYPE_DATAGRAM);
-        if(lansock != ENET_SOCKET_NULL && (enet_socket_set_option(lansock, ENET_SOCKOPT_REUSEADDR, 1) < 0 || enet_socket_bind(lansock, &address) < 0))
+        if(serverlanport)
         {
-            enet_socket_destroy(lansock);
-            lansock = ENET_SOCKET_NULL;
+            address.port = LAN_PORT;
+            lansock = enet_socket_create(ENET_SOCKET_TYPE_DATAGRAM);
+            if(lansock != ENET_SOCKET_NULL && (enet_socket_set_option(lansock, ENET_SOCKOPT_REUSEADDR, 1) < 0 || enet_socket_bind(lansock, &address) < 0))
+            {
+                enet_socket_destroy(lansock);
+                lansock = ENET_SOCKET_NULL;
+            }
+            if(lansock == ENET_SOCKET_NULL) conoutf("\frcould not create LAN server info socket");
+            else enet_socket_set_option(lansock, ENET_SOCKOPT_NONBLOCK, 1);
         }
-        if(lansock == ENET_SOCKET_NULL) conoutf("\frcould not create LAN server info socket");
-        else enet_socket_set_option(lansock, ENET_SOCKOPT_NONBLOCK, 1);
     }
 
     return servertype;
@@ -1255,8 +1392,8 @@ void setupserver()
 #ifdef STANDALONE
     setupmaster();
 #endif
-    conoutf("init: server (%s:%d)", *serverip ? serverip : "*", serverport);
-    if(setupserversockets() && verbose) conoutf("\fggame server started");
+    conoutf("loading server (%s:%d)..", *serverip ? serverip : "*", serverport);
+    if(setupserversockets() && verbose) conoutf("game server started");
 #ifndef STANDALONE
     if(servertype >= 3) serverloop();
 #endif
@@ -1264,6 +1401,7 @@ void setupserver()
 
 void initgame()
 {
+    conoutf("identity: v%s-%s%d %s (%s) [0x%x]", VERSION_STRING, versionplatname, versionarch, versionisserver ? "server" : "client", VERSION_RELEASE, versioncrc);
     server::start();
     loopv(gameargs)
     {
@@ -1334,7 +1472,7 @@ bool serveroption(char *opt)
 
 bool findoctadir(const char *name, bool fallback)
 {
-    mkstring(s);
+    string s = "";
     copystring(s, name);
     path(s);
     defformatstring(octadefs)("%s/data/default_map_settings.cfg", s);
@@ -1365,13 +1503,18 @@ void trytofindocta(bool fallback)
 {
     if(!octadir || !*octadir)
     {
-        const char *dir = getenv("OCTA_DIR");
+        const char *dir = getenv("OCTA_DATA");
         if(dir && *dir) setsvar("octadir", dir, false);
+        else
+        {
+            dir = getenv("OCTA_DIR"); // backward compat
+            if(dir && *dir) setsvar("octadir", dir, false);
+        }
     }
     if((!octadir || !*octadir || !findoctadir(octadir, false)) && fallback)
     { // user hasn't specifically set it, try some common locations alongside our folder
 #if defined(WIN32)
-        mkstring(dir);
+        string dir = "";
         if(SHGetFolderPath(NULL, CSIDL_PROGRAM_FILESX86, NULL, 0, dir) == S_OK)
         {
             defformatstring(s)("%s\\Sauerbraten", dir);
@@ -1401,28 +1544,30 @@ void setlocations(bool wanthome)
 #if defined(__APPLE__)
     extern const char *mac_resourcedir();
     const char *dir = mac_resourcedir(); // ./blah.app/Contents/Resources
-    if(dir && *dir)
-    {
+    if(dir && *dir && !chdir(dir))
         conoutf("attempting to use resources in: %s", dir);
-        chdir(dir);
-    }
 #endif
-    loopi(3) if(!fileexists(findfile("data/config/keymap.cfg", "r"), "r"))
-    {
-        if(i != 2) chdir("..");
-        else fatal("could not find data directory");
+    loopirev(3) if(!fileexists(findfile("config/version.cfg", "r"), "r"))
+    { // standalone solution to this is: pebkac
+        if(!i || chdir("..") < 0) fatal("could not find config directory");
     }
-    addpackagedir("data");
-    defformatstring(gamedata)("game/%s", server::gameid());
-    addpackagedir(gamedata);
-    execfile("version.cfg", false, EXEC_VERSION);
+    if(!execfile("config/version.cfg", false, EXEC_VERSION|EXEC_BUILTIN)) fatal("cannot exec 'config/version.cfg'");
+    // pseudo directory with game content
+    const char *dir = getenv("GAME_DATA");
+    if(dir && *dir) addpackagedir(dir);
+    else addpackagedir("data");
+#ifndef STANDALONE
+    if(!fileexists(findfile("textures/logo.png", "r"), "r")) fatal("could not find game data");
+#endif
+    //defformatstring(gamedata)("game/%s", server::gameid());
+    //addpackagedir(gamedata);
     if(wanthome)
     {
 #if defined(WIN32)
-        mkstring(dir);
+        string dir = "";
         if(SHGetFolderPath(NULL, CSIDL_PERSONAL, NULL, 0, dir) == S_OK)
         {
-            defformatstring(s)("%s\\My Games\\%s", dir, versionname);
+            defformatstring(s)("%s\\My Games\\%s", dir, VERSION_NAME);
             sethomedir(s);
         }
 #elif defined(__APPLE__)
@@ -1430,14 +1575,14 @@ void setlocations(bool wanthome)
         const char *dir = mac_personaldir(); // typically  /Users/<name>/Application Support/
         if(dir && *dir)
         {
-            defformatstring(s)("%s/%s", dir, versionuname);
+            defformatstring(s)("%s/%s", dir, VERSION_UNAME);
             sethomedir(s);
         }
 #else
         const char *dir = getenv("HOME");
         if(dir && *dir)
         {
-            defformatstring(s)("%s/.%s", dir, versionuname);
+            defformatstring(s)("%s/.%s", dir, VERSION_UNAME);
             sethomedir(s);
         }
 #endif
@@ -1457,17 +1602,11 @@ void writecfg()
     loopv(ids)
     {
         ident &id = *ids[i];
-        bool saved = false;
         if(id.flags&IDF_PERSIST) switch(id.type)
         {
-            case ID_VAR: if(*id.storage.i != id.def.i) { found = saved = true; f->printf((id.flags&IDF_HEX && *id.storage.i >= 0 ? (id.maxval==0xFFFFFF ? "%s 0x%.6X" : "%s 0x%X") : "%s %d"), escapeid(id), *id.storage.i); } break;
-            case ID_FVAR: if(*id.storage.f != id.def.f) { found = saved = true; f->printf("%s %s", escapeid(id), floatstr(*id.storage.f)); } break;
-            case ID_SVAR: if(strcmp(*id.storage.s, id.def.s)) { found = saved = true; f->printf("%s %s", escapeid(id), escapestring(*id.storage.s)); } break;
-        }
-        if(saved)
-        {
-            if(!(id.flags&IDF_COMPLETE)) f->printf("; setcomplete %s 0", escapeid(id));
-            f->printf("\n");
+            case ID_VAR: if(*id.storage.i != id.def.i) { found = true; f->printf("%s %s\n", escapeid(id), intstr(&id)); } break;
+            case ID_FVAR: if(*id.storage.f != id.def.f) { found = true; f->printf("%s %s\n", escapeid(id), floatstr(*id.storage.f)); } break;
+            case ID_SVAR: if(strcmp(*id.storage.s, id.def.s)) { found = true; f->printf("%s %s\n", escapeid(id), escapestring(*id.storage.s)); } break;
         }
     }
     if(found) f->printf("\n");
@@ -1475,27 +1614,17 @@ void writecfg()
     loopv(ids)
     {
         ident &id = *ids[i];
-        bool saved = false;
         if(id.flags&IDF_PERSIST) switch(id.type)
         {
             case ID_ALIAS:
             {
                 const char *str = id.getstr();
-                //if(str[0])
-                {
-                    found = saved = true;
-                    if(validateblock(str)) f->printf("%s = [%s]", escapeid(id), str);
-                    else f->printf("%s = %s", escapeid(id), escapestring(str));
-                }
+                found = true;
+                if(validateblock(str)) f->printf("%s = [%s]\n", escapeid(id), str);
+                else f->printf("%s = %s\n", escapeid(id), escapestring(str));
             }
             break;
         }
-        if(saved)
-        {
-            f->printf("; setpersist \"%s\" 1", id.name);
-            if(id.flags&IDF_COMPLETE) f->printf("; setcomplete \"%s\" 1", id.name);
-            f->printf("\n");
-        }
     }
     if(found) f->printf("\n");
     writebinds(f);
@@ -1528,11 +1657,21 @@ void rehash(bool reload)
     interactive = false;
     initing = NOT_INITING;
 #endif
-    conoutf("\fwconfiguration reloaded");
+    conoutf("\fwconfiguration %s", reload ? "reloaded" : "loaded");
     rehashing = 0;
 }
 ICOMMAND(0, rehash, "i", (int *nosave), if(!(identflags&IDF_WORLD)) rehash(*nosave ? false : true));
 
+void setcrc(const char *bin)
+{
+    if(!bin || !*bin) return;
+    size_t len = 0;
+    char *buf = loadfile(bin, &len, false);
+    if(!buf) return;
+    versioncrc = crc32(0, (const Bytef *)buf, len);
+    delete[] buf;
+}
+
 #ifdef STANDALONE
 #include <signal.h>
 volatile int errors = 0;
@@ -1540,7 +1679,7 @@ void fatal(const char *s, ...)    // failure exit
 {
     if(++errors <= 2) // print up to one extra recursive error
     {
-        defvformatstring(msg, s, s);
+        defvformatbigstring(msg, s, s);
         if(logfile) logoutf("%s", msg);
 #ifndef WIN32
         fprintf(stderr, "Exiting: %s\n", msg);
@@ -1553,7 +1692,7 @@ void fatal(const char *s, ...)    // failure exit
 #endif
             enet_deinitialize();
 #ifdef WIN32
-            defformatstring(cap)("%s: Error", versionname);
+            defformatstring(cap)("%s: Error", VERSION_NAME);
             MessageBox(NULL, msg, cap, MB_OK|MB_SYSTEMMODAL);
 #endif
         }
@@ -1578,10 +1717,13 @@ void reloadsignal(int signum)
 
 int main(int argc, char **argv)
 {
-    clocktime = time(NULL); // initialise
+    currenttime = time(NULL); // initialise
+    clocktime = mktime(gmtime(&currenttime));
+    clockoffset = currenttime-clocktime;
 
     setlogfile(NULL);
     setlocations(true);
+    setcrc(argv[0]);
 
     char *initscript = NULL;
     for(int i = 1; i<argc; i++)
