@@ -182,80 +182,37 @@ bool resolverwait(const char *name, ENetAddress *address)
     return resolved;
 }
 
-SDL_Thread *connthread = NULL;
-SDL_mutex *connmutex = NULL;
-SDL_cond *conncond = NULL;
-
-struct connectdata
-{
-    ENetSocket sock;
-    ENetAddress address;
-    int result;
-};
-
-// do this in a thread to prevent timeouts
-// could set timeouts on sockets, but this is more reliable and gives more control
-int connectthread(void *data)
-{
-    SDL_LockMutex(connmutex);
-    if(!connthread || SDL_GetThreadID(connthread) != SDL_ThreadID())
-    {
-        SDL_UnlockMutex(connmutex);
-        return 0;
-    }
-    connectdata cd = *(connectdata *)data;
-    SDL_UnlockMutex(connmutex);
-
-    int result = enet_socket_connect(cd.sock, &cd.address);
-
-    SDL_LockMutex(connmutex);
-    if(!connthread || SDL_GetThreadID(connthread) != SDL_ThreadID())
-    {
-        enet_socket_destroy(cd.sock);
-        SDL_UnlockMutex(connmutex);
-        return 0;
-    }
-    ((connectdata *)data)->result = result;
-    SDL_CondSignal(conncond);
-    SDL_UnlockMutex(connmutex);
-
-    return 0;
-}
-
 #define CONNLIMIT 20000
 
-int connectwithtimeout(ENetSocket sock, const char *hostname, ENetAddress &address)
+int connectwithtimeout(ENetSocket sock, const char *hostname, const ENetAddress &address)
 {
     defformatstring(text)("connecting to %s:[%d]...", hostname != NULL ? hostname : "local server", address.port);
     progress(0, text);
 
-    if(!connmutex) connmutex = SDL_CreateMutex();
-    if(!conncond) conncond = SDL_CreateCond();
-    SDL_LockMutex(connmutex);
-    connectdata cd = { sock, address, -1 };
-    connthread = SDL_CreateThread(connectthread, &cd);
-
-    int starttime = SDL_GetTicks(), timeout = 0;
-    for(;;)
+    ENetSocketSet readset, writeset;
+    if(!enet_socket_connect(sock, &address)) for(int starttime = SDL_GetTicks(), timeout = 0; timeout <= CONNLIMIT;)
     {
-        if(!SDL_CondWaitTimeout(conncond, connmutex, 250))
+        ENET_SOCKETSET_EMPTY(readset);
+        ENET_SOCKETSET_EMPTY(writeset);
+        ENET_SOCKETSET_ADD(readset, sock);
+        ENET_SOCKETSET_ADD(writeset, sock);
+        int result = enet_socketset_select(sock, &readset, &writeset, 250);
+        if(result < 0) break;
+        else if(result > 0)
         {
-            if(cd.result<0) enet_socket_destroy(sock);
-            break;
+            if(ENET_SOCKETSET_CHECK(readset, sock) || ENET_SOCKETSET_CHECK(writeset, sock))
+            {
+                int error = 0;
+                if(enet_socket_get_option(sock, ENET_SOCKOPT_ERROR, &error) < 0 || error) break;
+                return 0;
+            }
         }
         timeout = SDL_GetTicks() - starttime;
         progress(min(float(timeout)/CONNLIMIT, 1.0f), text);
-        if(interceptkey(SDLK_ESCAPE)) timeout = CONNLIMIT + 1;
-        if(timeout > CONNLIMIT) break;
+        if(interceptkey(SDLK_ESCAPE)) break;
     }
 
-    /* thread will actually timeout eventually if its still trying to connect
-     * so just leave it (and let it destroy socket) instead of causing problems on some platforms by killing it
-     */
-    connthread = NULL;
-    SDL_UnlockMutex(connmutex);
-
-    return cd.result;
+    return -1;
 }
 
 vector<serverinfo *> servers;
@@ -263,9 +220,9 @@ bool sortedservers = true;
 ENetSocket pingsock = ENET_SOCKET_NULL;
 int lastinfo = 0;
 
-static serverinfo *newserver(const char *name, int port = SERVER_PORT, const char *desc = NULL, int priority = 0, uint ip = ENET_HOST_ANY)
+static serverinfo *newserver(const char *name, int port = SERVER_PORT, int priority = 0, const char *desc = NULL, uint ip = ENET_HOST_ANY)
 {
-    serverinfo *si = new serverinfo(ip, port);
+    serverinfo *si = new serverinfo(ip, port, priority);
 
     if(name) copystring(si->name, name);
     else if(ip==ENET_HOST_ANY || enet_address_get_host_ip(&si->address, si->name, sizeof(si->name)) < 0)
@@ -281,13 +238,14 @@ static serverinfo *newserver(const char *name, int port = SERVER_PORT, const cha
     return si;
 }
 
-void addserver(const char *name, int port, const char *desc)
+void addserver(const char *name, int port, int priority, const char *desc)
 {
     loopv(servers) if(!strcmp(servers[i]->name, name) && servers[i]->port == port) return;
-    if(newserver(name, port, desc) && verbose >= 2)
+    if(newserver(name, port, priority, desc) && verbose >= 2)
         conoutf("added server %s (%d) [%s]", name, port, desc);
 }
-ICOMMAND(0, addserver, "sis", (char *n, int *p, char *d), addserver(n, *p > 0 ? *p : SERVER_PORT, d));
+ICOMMAND(0, addserver, "siis", (char *n, int *p, int *r, char *d), addserver(n, *p > 0 ? *p : SERVER_PORT, *r >= 0 ? *r : 0, d));
+
 VAR(0, searchlan, 0, 0, 1);
 VAR(IDF_PERSIST, maxservpings, 0, 10, 1000);
 VAR(IDF_PERSIST, serverupdateinterval, 0, 10, VAR_MAX);
@@ -310,7 +268,7 @@ void pingservers()
     ENetBuffer buf;
     uchar ping[MAXTRANS];
     ucharbuf p(ping, sizeof(ping));
-    putint(p, totalmillis);
+    putint(p, totalmillis ? totalmillis : 1);
 
     static int lastping = 0;
     if(lastping >= servers.length()) lastping = 0;
@@ -326,11 +284,11 @@ void pingservers()
         si.checkdecay(serverdecay*1000);
     }
 
-    if(searchlan)
+    if(searchlan && serverlanport)
     {
         ENetAddress address;
         address.host = ENET_HOST_BROADCAST;
-        address.port = LAN_PORT;
+        address.port = serverlanport;
         buf.data = ping;
         buf.dataLength = p.length();
         enet_socket_send(pingsock, &address, &buf, 1);
@@ -389,7 +347,7 @@ void checkpings()
         if(len <= 0) return;
         serverinfo *si = NULL;
         loopv(servers) if(addr.host == servers[i]->address.host && addr.port == servers[i]->address.port) { si = servers[i]; break; }
-        if(!si && searchlan) si = newserver(NULL, addr.port-1, NULL, 1, addr.host);
+        if(!si && searchlan) si = newserver(NULL, addr.port-1, 1, NULL, addr.host);
         if(!si) continue;
         ucharbuf p(ping, len);
         int millis = getint(p), rtt = clamp(totalmillis - millis, 0, min(serverdecay*1000, totalmillis));
@@ -400,14 +358,22 @@ void checkpings()
         si->attr.shrink(0);
         loopj(numattr) si->attr.add(getint(p));
         getstring(text, p);
-        filtertext(si->map, text, false);
+        filterstring(si->map, text, false);
         getstring(text, p);
-        filtertext(si->sdesc, text);
+        filterstring(si->sdesc, text);
         si->players.deletearrays();
+        si->handles.deletearrays();
         loopi(si->numplayers)
         {
+            if(p.overread()) break;
             getstring(text, p);
-            if(text[0]) si->players.add(newstring(text));
+            si->players.add(newstring(text));
+        }
+        loopi(si->numplayers)
+        {
+            if(p.overread()) break;
+            getstring(text, p);
+            si->handles.add(newstring(text));
         }
         sortedservers = false;
     }
@@ -498,8 +464,22 @@ void retrieveservers(vector<char> &data)
     enet_socket_destroy(sock);
 }
 
+void sortservers()
+{
+    if(!sortedservers)
+    {
+        servers.sort(serverinfocompare);
+        sortedservers = true;
+    }
+}
+COMMAND(0, sortservers, "");
+
+VAR(IDF_PERSIST, autosortservers, 0, 1, 1);
+VAR(0, pausesortservers, 0, 0, 1);
+
 void updatefrommaster()
 {
+    pausesortservers = 0;
     vector<char> data;
     retrieveservers(data);
     if(data.length() && data[0])
@@ -515,36 +495,25 @@ void updatefrommaster()
 }
 COMMAND(0, updatefrommaster, "");
 
-void sortservers()
-{
-    if(!sortedservers)
-    {
-        servers.sort(serverinfocompare);
-        sortedservers = true;
-    }
-}
-COMMAND(0, sortservers, "");
-
-VAR(IDF_PERSIST, autosortservers, 0, 1, 1);
-
 void updateservers()
 {
     if(!reqmaster) updatefrommaster();
     refreshservers();
-    if(autosortservers) sortservers();
+    if(autosortservers && !pausesortservers) sortservers();
     intret(servers.length());
 }
 COMMAND(0, updateservers, "");
 
 void writeservercfg()
 {
+    if(servers.empty()) return;
     stream *f = openutf8file("servers.cfg", "w");
     if(!f) return;
-    f->printf("// servers connected to are added here automatically\n\n");
+    f->printf("// servers which are connected to or queried get added here automatically\n\n");
     loopv(servers)
     {
         serverinfo *s = servers[i];
-        f->printf("addserver %s %d %s\n", escapeid(s->name), s->port, escapeid(s->sdesc[0] ? s->sdesc : s->name));
+        f->printf("addserver %s %d %d %s\n", escapeid(s->name), s->port, s->priority, escapeid(s->sdesc[0] ? s->sdesc : s->name));
     }
     delete f;
 }
