@@ -99,7 +99,7 @@ struct skelmodel : animmodel
             else
             {
                 int total = 0;
-                loopk(4) total += (v.weights[k] = uchar(weights[k]*255));
+                loopk(4) total += (v.weights[k] = uchar(0.5f + weights[k]*255));
                 while(total > 255)
                 {
                     loopk(4) if(v.weights[k] > 0 && total > 255) { v.weights[k]--; total--; }
@@ -212,7 +212,7 @@ struct skelmodel : animmodel
             mesh::calctangents(bumpverts, verts, verts, numverts, tris, numtris, areaweight);
         }
 
-        void calcbb(int frame, vec &bbmin, vec &bbmax, const matrix3x4 &m)
+        void calcbb(vec &bbmin, vec &bbmax, const matrix3x4 &m)
         {
             loopj(numverts)
             {
@@ -225,7 +225,7 @@ struct skelmodel : animmodel
             }
         }
 
-        void gentris(int frame, Texture *tex, vector<BIH::tri> *out, const matrix3x4 &m)
+        void gentris(Texture *tex, vector<BIH::tri> *out, const matrix3x4 &m)
         {
             loopj(numtris)
             {
@@ -603,6 +603,8 @@ struct skelmodel : animmodel
     {
         int bone, target, parent;
         float pitchmin, pitchmax, pitchscale, pitchangle, pitchtotal;
+
+        pitchcorrect() : parent(-1), pitchangle(0), pitchtotal(0) {}
     };
 
     struct skeleton
@@ -1193,7 +1195,7 @@ struct skelmodel : animmodel
             loopv(antipodes) sc.bdata[antipodes[i].child].fixantipodal(sc.bdata[antipodes[i].parent]);
         }
 
-        void concattagtransform(part *p, int frame, int i, const matrix3x4 &m, matrix3x4 &n)
+        void concattagtransform(part *p, int i, const matrix3x4 &m, matrix3x4 &n)
         {
             matrix3x4 t;
             t.mul(bones[tags[i].bone].base, tags[i].matrix);
@@ -1639,9 +1641,9 @@ struct skelmodel : animmodel
             }
         }
 
-        void concattagtransform(part *p, int frame, int i, const matrix3x4 &m, matrix3x4 &n)
+        void concattagtransform(part *p, int i, const matrix3x4 &m, matrix3x4 &n)
         {
-            skel->concattagtransform(p, frame, i, m, n);
+            skel->concattagtransform(p, i, m, n);
         }
 
         int addblendcombo(const blendcombo &c)
