@@ -21,7 +21,7 @@ soundslot::~soundslot() { DELETEA(name); }
 
 sound::sound() : hook(NULL) { reset(); }
 sound::~sound() {}
-bool sound::playing() { return chan >= 0 && Mix_Playing(chan); }
+bool sound::playing() { return chan >= 0 && (Mix_Playing(chan) || Mix_Paused(chan)); }
 void sound::reset()
 {
     pos = oldpos = vec(-1, -1, -1);
@@ -45,20 +45,21 @@ Mix_Music *music = NULL;
 SDL_RWops *musicrw = NULL;
 stream *musicstream = NULL;
 char *musicfile = NULL, *musicdonecmd = NULL;
-int soundsatonce = 0, lastsoundmillis = 0, musictime = -1, musicdonetime = -1;
+int musictime = -1, musicdonetime = -1;
 
 VARF(IDF_PERSIST, mastervol, 0, 255, 255, changedvol = true);
 VAR(IDF_PERSIST, soundvol, 0, 255, 255);
 VARF(0, soundmono, 0, 0, 1, initwarning("sound configuration", INIT_RESET, CHANGE_SOUND));
-VARF(0, soundchans, 1, 32, 128, initwarning("sound configuration", INIT_RESET, CHANGE_SOUND));
+VARF(0, soundmixchans, 16, 32, 1024, initwarning("sound configuration", INIT_RESET, CHANGE_SOUND));
 VARF(0, soundfreq, 0, 44100, 48000, initwarning("sound configuration", INIT_RESET, CHANGE_SOUND));
-VARF(0, soundbufferlen, 128, 1024, VAR_MAX, initwarning("sound configuration", INIT_RESET, CHANGE_SOUND));
+VARF(0, soundbuflen, 128, 4096, VAR_MAX, initwarning("sound configuration", INIT_RESET, CHANGE_SOUND));
 VAR(IDF_PERSIST, soundmaxrad, 0, 512, VAR_MAX);
 VAR(IDF_PERSIST, soundminrad, 0, 0, VAR_MAX);
 FVAR(IDF_PERSIST, soundevtvol, 0, 1, FVAR_MAX);
 FVAR(IDF_PERSIST, soundevtscale, 0, 1, FVAR_MAX);
 FVAR(IDF_PERSIST, soundenvvol, 0, 1, FVAR_MAX);
 FVAR(IDF_PERSIST, soundenvscale, 0, 1, FVAR_MAX);
+VAR(IDF_PERSIST, soundcull, 0, 1, 1);
 
 VARF(IDF_PERSIST, musicvol, 0, 64, 255, changedvol = true);
 VAR(IDF_PERSIST, musicfadein, 0, 1000, VAR_MAX);
@@ -69,13 +70,13 @@ void initsound()
 {
     if(nosound)
     {
-        if(Mix_OpenAudio(soundfreq, MIX_DEFAULT_FORMAT, soundmono ? 1 : 2, soundbufferlen) == -1)
+        if(Mix_OpenAudio(soundfreq, MIX_DEFAULT_FORMAT, soundmono ? 1 : 2, soundbuflen) == -1)
         {
             conoutf("\frsound initialisation failed: %s", Mix_GetError());
             return;
         }
-
-        Mix_AllocateChannels(soundchans);
+        int chans = Mix_AllocateChannels(soundmixchans);
+        conoutf("allocated %d of %d sound channels", chans, soundmixchans);
         nosound = false;
     }
     initmumble();
@@ -139,6 +140,58 @@ void clearsound()
     mapsounds.setsize(0);
 }
 
+void getsounds(bool mapsnd, int idx, int prop)
+{
+    vector<soundslot> &soundset = mapsnd ? mapsounds : gamesounds;
+    if(idx < 0) intret(soundset.length());
+    else if(soundset.inrange(idx))
+    {
+        if(prop < 0) intret(4);
+        else switch(prop)
+        {
+            case 0: intret(soundset[idx].vol); break;
+            case 1: intret(soundset[idx].maxrad); break;
+            case 2: intret(soundset[idx].minrad); break;
+            case 3: result(soundset[idx].name); break;
+            default: break;
+        }
+    }
+}
+ICOMMAND(0, getsound, "ibb", (int *n, int *v, int *p), getsounds(*n!=0, *v, *p));
+
+void getcursounds(int idx, int prop)
+{
+    if(idx < 0) intret(sounds.length());
+    else if(sounds.inrange(idx))
+    {
+        if(prop < 0) intret(19);
+        else switch(prop)
+        {
+            case 0: intret(sounds[idx].vol); break;
+            case 1: intret(sounds[idx].curvol); break;
+            case 2: intret(sounds[idx].curpan); break;
+            case 3: intret(sounds[idx].flags); break;
+            case 4: intret(sounds[idx].maxrad); break;
+            case 5: intret(sounds[idx].minrad); break;
+            case 6: intret(sounds[idx].material); break;
+            case 7: intret(sounds[idx].millis); break;
+            case 8: intret(sounds[idx].ends); break;
+            case 9: intret(sounds[idx].slotnum); break;
+            case 10: intret(sounds[idx].chan); break;
+            case 11: defformatstring(pos)("%.f %.f %.f", sounds[idx].pos.x, sounds[idx].pos.y, sounds[idx].pos.z); result(pos); break;
+            case 12: defformatstring(oldpos)("%.f %.f %.f", sounds[idx].oldpos.x, sounds[idx].oldpos.y, sounds[idx].oldpos.z); result(oldpos); break;
+            case 13: intret(sounds[idx].valid() ? 1 : 0); break;
+            case 14: intret(sounds[idx].playing() ? 1 : 0); break;
+            case 15: intret(sounds[idx].flags&SND_MAP ? 1 : 0); break;
+            case 16: intret(sounds[idx].owner!=NULL ? 1 : 0); break;
+            case 17: intret(sounds[idx].owner==camera1 ? 1 : 0); break;
+            case 18: intret(client::getcn(sounds[idx].owner)); break;
+            default: break;
+        }
+    }
+}
+ICOMMAND(0, getcursound, "bb", (int *n, int *p), getcursounds(*n, *p));
+
 Mix_Music *loadmusic(const char *name)
 {
     if(!musicstream) musicstream = openzipfile(name, "rb");
@@ -229,9 +282,9 @@ void smartmusic(bool cond, bool autooff)
 }
 ICOMMAND(0, smartmusic, "ii", (int *a, int *b), smartmusic(*a, *b));
 
-int findsound(const char *name, int vol, vector<soundslot> &sounds)
+int findsound(const char *name, int vol, vector<soundslot> &soundset)
 {
-    loopv(sounds) if(!strcmp(sounds[i].name, name) && (!vol || sounds[i].vol == vol)) return i;
+    loopv(soundset) if(!strcmp(soundset[i].name, name) && (!vol || soundset[i].vol == vol)) return i;
     return -1;
 }
 
@@ -253,20 +306,28 @@ static Mix_Chunk *loadwav(const char *name)
     return c;
 }
 
-int addsound(const char *name, int vol, int maxrad, int minrad, int value, vector<soundslot> &sounds)
+int addsound(const char *name, int vol, int maxrad, int minrad, int value, vector<soundslot> &soundset)
 {
     if(vol <= 0 || vol >= 255) vol = 255;
     if(maxrad <= 0) maxrad = -1;
     if(minrad < 0) minrad = -1;
     if(value == 1)
     {
-        loopv(sounds)
+        loopv(soundset)
         {
-            soundslot &slot = sounds[i];
+            soundslot &slot = soundset[i];
             if(slot.vol == vol && slot.maxrad == maxrad && slot.minrad == minrad && !strcmp(slot.name, name))
                 return i;
         }
     }
+    if(!strcmp(name, "<none>"))
+    {
+        soundslot &slot = soundset.add();
+        slot.name = newstring(name);
+        slot.vol = 0;
+        slot.maxrad = slot.minrad = -1;
+        return soundset.length()-1;
+    }
     soundsample *sample = NULL;
     #define loadsound(req) \
     { \
@@ -306,7 +367,7 @@ int addsound(const char *name, int vol, int maxrad, int minrad, int value, vecto
         }
         else break;
     }
-    soundslot &slot = sounds.add();
+    soundslot &slot = soundset.add();
     slot.name = newstring(name);
     slot.vol = vol;
     slot.maxrad = maxrad; // use these values if none are supplied when playing
@@ -319,7 +380,7 @@ int addsound(const char *name, int vol, int maxrad, int minrad, int value, vecto
         if(!sample->sound) conoutf("\frfailed to load sample: %s", sam);
         else slot.samples.add(sample);
     }
-    return sounds.length()-1;
+    return soundset.length()-1;
 }
 
 ICOMMAND(0, registersound, "sissi", (char *n, int *v, char *w, char *x, int *u), intret(addsound(n, *v, *w ? parseint(w) : -1, *x ? parseint(x) : -1, *u, gamesounds)));
@@ -327,7 +388,9 @@ ICOMMAND(0, mapsound, "sissi", (char *n, int *v, char *w, char *x, int *u), intr
 
 void calcvol(int flags, int vol, int slotvol, int maxrad, int minrad, const vec &pos, int *curvol, int *curpan, bool liquid)
 {
-    int svol = flags&SND_CLAMPED ? 255 : clamp(vol, 0, 255), span = 127; vec v; float dist = pos.dist(camera1->o, v);
+    vec v;
+    float dist = pos.dist(camera1->o, v);
+    int svol = flags&SND_CLAMPED ? 255 : clamp(vol, 0, 255), span = 127;
     if(!(flags&SND_NOATTEN) && dist > 0)
     {
         if(!(flags&SND_NOPAN) && !soundmono && (v.x != 0 || v.y != 0))
@@ -356,7 +419,7 @@ void updatesound(int chan)
 {
     sound &s = sounds[chan];
     bool waiting = (!(s.flags&SND_NODELAY) && Mix_Paused(chan));
-    if((s.flags&SND_NOCULL) || s.curvol > 0 || (s.owner && game::camerapos(s.owner).dist(camera1->o) == 0))
+    if((s.flags&SND_NOCULL) || !soundcull || s.curvol > 0 || s.pos.dist(camera1->o) <= 0)
     {
         if(waiting)
         { // delay the sound based on average physical constants
@@ -419,44 +482,48 @@ void updatesounds()
 
 int playsound(int n, const vec &pos, physent *d, int flags, int vol, int maxrad, int minrad, int *hook, int ends, int *oldhook)
 {
-    if(nosound || !mastervol || !soundvol) return -1;
-
+    if(nosound || !mastervol || !soundvol || ((flags&SND_MAP || n >= S_GAMESPECIFIC) && client::waiting(true)) || (!d && !insideworld(pos))) return -1;
     vector<soundslot> &soundset = flags&SND_MAP ? mapsounds : gamesounds;
 
-    if(soundset.inrange(n) && !soundset[n].samples.empty())
+    if(soundset.inrange(n))
     {
+        if(soundset[n].samples.empty() || !soundset[n].vol)
+        {
+            if(oldhook && issound(*oldhook)) removesound(*oldhook);
+            return -1;
+        }
         soundslot *slot = &soundset[n];
         if(!oldhook || !issound(*oldhook) || (n != sounds[*oldhook].slotnum && strcmp(slot->name, gamesounds[sounds[*oldhook].slotnum].name)))
             oldhook = NULL;
 
-        int cvol = 0, cpan = 0, v = vol > 0 && vol < 256 ? vol : (flags&SND_CLAMPED ? 64 : 255),
+        vec o = d ? game::camerapos(d) : pos;
+        int cvol = 0, cpan = 0, v = clamp(vol >= 0 ? vol : 255, flags&SND_CLAMPED ? 64 : 0, 255),
             x = maxrad > 0 ? maxrad : (flags&SND_CLAMPED ? getworldsize() : (slot->maxrad > 0 ? slot->maxrad : soundmaxrad)),
             y = minrad >= 0 ? minrad : (flags&SND_CLAMPED ? 32 : (slot->minrad >= 0 ? slot->minrad : soundminrad)),
-            mat = lookupmaterial(pos);
+            mat = lookupmaterial(o);
 
         bool liquid = isliquid(lookupmaterial(camera1->o)&MATF_VOLUME);
-        calcvol(flags, v, slot->vol, x, y, pos, &cvol, &cpan, liquid || isliquid(mat&MATF_VOLUME));
-        bool nocull = flags&SND_NOCULL || (d && game::camerapos(d).dist(camera1->o) == 0);
+        calcvol(flags, v, slot->vol, x, y, o, &cvol, &cpan, liquid || isliquid(mat&MATF_VOLUME));
+        bool nocull = flags&SND_NOCULL || o.dist(camera1->o) <= 0;
 
-        if(nocull || cvol > 0)
+        if(nocull || !soundcull || cvol > 0)
         {
             int chan = -1;
             if(oldhook) chan = *oldhook;
             else
             {
                 oldhook = NULL;
-                soundsample *sample = soundset[n].samples[soundset[n].samples.length() > 1 ? rnd(soundset[n].samples.length()) : 0];
+                soundsample *sample = soundset[n].samples[rnd(soundset[n].samples.length())];
                 if((chan = Mix_PlayChannel(-1, sample->sound, flags&SND_LOOP ? -1 : 0)) < 0)
                 {
                     int lowest = -1;
-                    loopv(sounds) if(sounds[i].chan >= 0 && !(sounds[i].flags&SND_NOCULL) && !(sounds[i].flags&SND_MAP) && (sounds[i].owner && game::camerapos(sounds[i].owner).dist(camera1->o) > 0))
-                        if((nocull || sounds[i].curvol < cvol) && (!sounds.inrange(lowest) || sounds[i].curvol < sounds[lowest].curvol))
-                            lowest = i;
+                    loopv(sounds) if(sounds[i].chan >= 0 && (!(sounds[i].flags&SND_MAP) || flags&SND_MAP) && sounds[i].curvol < cvol && (lowest < 0 || sounds[i].curvol < sounds[lowest].curvol) && (nocull || (!(sounds[i].flags&SND_NOCULL) && sounds[i].pos.dist(camera1->o) > 0)))
+                        lowest = i;
                     if(sounds.inrange(lowest))
                     {
+                        if(verbose >= 4) conoutf("culled channel %d (%d)", lowest, sounds[lowest].curvol);
                         removesound(lowest);
                         chan = Mix_PlayChannel(-1, sample->sound, flags&SND_LOOP ? -1 : 0);
-                        if(verbose >= 4) conoutf("culled channel %d (%d)", lowest, sounds[lowest].curvol);
                     }
                 }
             }
@@ -477,7 +544,7 @@ int playsound(int n, const vec &pos, physent *d, int flags, int vol, int maxrad,
                 s.ends = ends;
                 s.slotnum = n;
                 s.owner = d;
-                s.pos = s.oldpos = pos;
+                s.pos = s.oldpos = o;
                 s.curvol = cvol;
                 s.curpan = cpan;
                 s.chan = chan;
@@ -548,6 +615,7 @@ void resetsound()
         soundsamples.clear();
         return;
     }
+    rehash(true);
     if(music && loadmusic(musicfile))
     {
         if(musicfadein) Mix_FadeInMusic(music, musicdonecmd ? 0 : -1, musicfadein);
@@ -618,7 +686,7 @@ void initmumble()
         if(mumblelink)
         {
             mumbleinfo = (MumbleInfo *)MapViewOfFile(mumblelink, FILE_MAP_ALL_ACCESS, 0, 0, sizeof(MumbleInfo));
-            if(mumbleinfo) wcsncpy(mumbleinfo->name, (const wchar_t *)versionuname, 256);
+            if(mumbleinfo) wcsncpy(mumbleinfo->name, (const wchar_t *)VERSION_UNAME, 256);
         }
     #elif defined(_POSIX_SHARED_MEMORY_OBJECTS)
         defformatstring(shmname)("/MumbleLink.%d", getuid());
@@ -626,7 +694,7 @@ void initmumble()
         if(mumblelink >= 0)
         {
             mumbleinfo = (MumbleInfo *)mmap(NULL, sizeof(MumbleInfo), PROT_READ|PROT_WRITE, MAP_SHARED, mumblelink, 0);
-            if(mumbleinfo != (MumbleInfo *)-1) wcsncpy(mumbleinfo->name, (const wchar_t *)versionuname, 256);
+            if(mumbleinfo != (MumbleInfo *)-1) wcsncpy(mumbleinfo->name, (const wchar_t *)VERSION_UNAME, 256);
         }
     #endif
     if(!VALID_MUMBLELINK) closemumble();
