@@ -17,7 +17,7 @@ enum
     SND_LOOP    = 1<<7,
     SND_MAP     = 1<<8,
     SND_IMPORT  = SND_NODELAY|SND_NOCULL|SND_NOQUIET,
-    SND_FORCED  = SND_IMPORT|SND_NOATTEN|SND_CLAMPED,
+    SND_FORCED  = SND_IMPORT|SND_NOATTEN|SND_NODIST,
     SND_DIRECT  = SND_IMPORT|SND_CLAMPED,
     SND_MASKF   = SND_LOOP|SND_MAP,
     SND_LAST    = 7
@@ -70,7 +70,7 @@ extern bool playingmusic(bool check = true);
 extern void smartmusic(bool cond, bool autooff);
 extern void musicdone(bool docmd);
 extern void updatesounds();
-extern int addsound(const char *name, int vol, int maxrad, int minrad, int value, vector<soundslot> &sounds);
+extern int addsound(const char *name, int vol, int maxrad, int minrad, int value, vector<soundslot> &soundset);
 extern void removesound(int c);
 extern void clearsound();
 extern int playsound(int n, const vec &pos, physent *d = NULL, int flags = 0, int vol = -1, int maxrad = -1, int minrad = -1, int *hook = NULL, int ends = 0, int *oldhook = NULL);
