@@ -29,7 +29,7 @@ struct editline
         {
             va_list args;
             va_start(args, fmt);
-            _vsnprintf(newtext, maxlen, fmt, args);
+            vformatstring(newtext, fmt, args, maxlen);
             va_end(args);
         }
         DELETEA(text);
@@ -215,7 +215,7 @@ struct editor
 
     void mark(bool enable)
     {
-        mx = (enable) ? cx : -1;
+        mx = enable ? cx : -1;
         my = cy;
     }
 
@@ -230,13 +230,20 @@ struct editor
     bool region(int &sx, int &sy, int &ex, int &ey)
     {
         int n = lines.length();
-        assert(n != 0);
-        if(cy < 0) cy = 0; else if(cy >= n) cy = n-1;
+        if(!n)
+        {
+            lines.add().set("");
+            n = 1;
+        }
+        if(cy < 0) cy = 0;
+        else if(cy >= n) cy = n-1;
         int len = lines[cy].len;
-        if(cx < 0) cx = 0; else if(cx > len) cx = len;
+        if(cx < 0) cx = 0;
+        else if(cx > len) cx = len;
         if(mx >= 0)
         {
-            if(my < 0) my = 0; else if(my >= n) my = n-1;
+            if(my < 0) my = 0;
+            else if(my >= n) my = n-1;
             len = lines[my].len;
             if(mx > len) mx = len;
         }
@@ -245,7 +252,7 @@ struct editor
         ex = cx;
         ey = cy;
         if(sy > ey) { swap(sy, ey); swap(sx, ex); }
-        else if(sy==ey && sx > ex) swap(sx, ex);
+        else if(sy == ey && sx > ex) swap(sx, ex);
         return (sx != ex) || (sy != ey);
     }
 
@@ -255,15 +262,21 @@ struct editor
     editline &currentline()
     {
         int n = lines.length();
-        assert(n != 0);
-        if(cy < 0) cy = 0; else if(cy >= n) cy = n-1;
-        if(cx < 0) cx = 0; else if(cx > lines[cy].len) cx = lines[cy].len;
+        if(!n)
+        {
+            lines.add().set("");
+            n = 1;
+        }
+        if(cy < 0) cy = 0;
+        else if(cy >= n) cy = n-1;
+        if(cx < 0) cx = 0;
+        else if(cx > lines[cy].len) cx = lines[cy].len;
         return lines[cy];
     }
 
     void copyselectionto(editor *b)
     {
-        if(b==this) return;
+        if(b == this) return;
 
         b->clear(NULL);
         int sx, sy, ex, ey;
@@ -396,7 +409,7 @@ struct editor
 
     void insertallfrom(editor *b)
     {
-        if(b==this) return;
+        if(b == this) return;
 
         del();
 
@@ -521,8 +534,7 @@ struct editor
 
     void hit(int hitx, int hity, bool dragged)
     {
-        int maxwidth = linewrap?pixelwidth:-1;
-        int h = 0;
+        int maxwidth = linewrap ? pixelwidth : -1, h = 0;
         for(int i = scrolly; i < lines.length(); i++)
         {
             int width, height;
@@ -535,14 +547,13 @@ struct editor
                 if(dragged) { mx = x; my = i; } else { cx = x; cy = i; };
                 break;
             }
-           h+=height;
+           h += height;
         }
     }
 
     int limitscrolly()
     {
-        int maxwidth = linewrap?pixelwidth:-1;
-        int slines = lines.length();
+        int maxwidth = linewrap ? pixelwidth : -1, slines = lines.length();
         for(int ph = pixelheight; slines > 0 && ph > 0;)
         {
             int width, height;
@@ -554,16 +565,27 @@ struct editor
         return slines;
     }
 
-    void draw(int x, int y, int color, bool hit)
+    void draw(int x, int y, int color, bool hit, const char *prompt = NULL)
     {
-        int maxwidth = linewrap?pixelwidth:-1;
-
-        int sx, sy, ex, ey;
+        int h = 0, maxwidth = linewrap ? pixelwidth : -1;
+        if(lines.empty() || !lines[0].text[0])
+        {
+            if(!hit && prompt && *prompt)
+            {
+                int width = 0, height = 0;
+                text_bounds(prompt, width, height, maxwidth, TEXT_NO_INDENT);
+                if(h+height <= pixelheight)
+                    draw_text(prompt, x, y+h, color>>16, (color>>8)&0xFF, color&0xFF, 0xFF, TEXT_NO_INDENT, -1, maxwidth);
+            }
+            return;
+        }
+        int starty = scrolly, sx = 0, sy = 0, ex = 0, ey = 0;
         bool selection = region(sx, sy, ex, ey);
-
-        // fix scrolly so that <cx, cy> is always on screen
-        int starty = scrolly;
-        if(starty==SCROLLEND) { cy = lines.length()-1; starty = 0; }
+        if(starty == SCROLLEND) // fix scrolly so that <cx, cy> is always on screen
+        {
+            cy = lines.length()-1;
+            starty = 0;
+        }
         if(cy < starty) starty = cy;
         else
         {
@@ -573,14 +595,13 @@ struct editor
                 int width, height;
                 text_bounds(lines[i].text, width, height, maxwidth, TEXT_NO_INDENT);
                 h += height;
-                if(h > pixelheight) { starty = i + 1; break; }
+                if(h > pixelheight) { starty = i+1; break; }
             }
         }
 
         if(selection)
         {
-            // convert from cursor coords into pixel coords
-            int psx, psy, pex, pey;
+            int psx, psy, pex, pey; // convert from cursor coords into pixel coords
             text_pos(lines[sy].text, sx, psx, psy, maxwidth, TEXT_NO_INDENT);
             text_pos(lines[ey].text, ex, pex, pey, maxwidth, TEXT_NO_INDENT);
             int maxy = lines.length();
@@ -589,7 +610,7 @@ struct editor
             {
                 int width, height;
                 text_bounds(lines[i].text, width, height, maxwidth, TEXT_NO_INDENT);
-                if(h + height > pixelheight) { maxy = i; break; }
+                if(h+height > pixelheight) { maxy = i; break; }
                 if(i == sy) psy += h;
                 if(i == ey) { pey += h; break; }
                 h += height;
@@ -598,10 +619,18 @@ struct editor
 
             if(ey >= starty && sy <= maxy)
             {
-                // crop top/bottom within window
-                if(sy < starty) { sy = starty; psy = 0; psx = 0; }
-                if(ey > maxy) { ey = maxy; pey = pixelheight - FONTH; pex = pixelwidth; }
-
+                if(sy < starty) // crop top/bottom within window
+                {
+                    sy = starty;
+                    psy = 0;
+                    psx = 0;
+                }
+                if(ey > maxy)
+                {
+                    ey = maxy;
+                    pey = pixelheight-FONTH;
+                    pex = pixelwidth;
+                }
                 notextureshader->set();
                 glDisable(GL_TEXTURE_2D);
                 glColor3f(0.25f, 0.25f, 0.75f);
@@ -614,19 +643,19 @@ struct editor
                     glVertex2f(x+psx, y+pey+FONTH);
                 }
                 else
-                {   glVertex2f(x+psx,        y+psy);
-                    glVertex2f(x+psx,        y+psy+FONTH);
+                {   glVertex2f(x+psx, y+psy);
+                    glVertex2f(x+psx, y+psy+FONTH);
                     glVertex2f(x+pixelwidth, y+psy+FONTH);
                     glVertex2f(x+pixelwidth, y+psy);
                     if(pey-psy > FONTH)
                     {
-                        glVertex2f(x,            y+psy+FONTH);
+                        glVertex2f(x, y+psy+FONTH);
                         glVertex2f(x+pixelwidth, y+psy+FONTH);
                         glVertex2f(x+pixelwidth, y+pey);
-                        glVertex2f(x,            y+pey);
+                        glVertex2f(x, y+pey);
                     }
-                    glVertex2f(x,     y+pey);
-                    glVertex2f(x,     y+pey+FONTH);
+                    glVertex2f(x, y+pey);
+                    glVertex2f(x, y+pey+FONTH);
                     glVertex2f(x+pex, y+pey+FONTH);
                     glVertex2f(x+pex, y+pey);
                 }
@@ -636,29 +665,27 @@ struct editor
             }
         }
 
-        int h = 0;
         for(int i = starty; i < lines.length(); i++)
         {
             int width, height;
             text_bounds(lines[i].text, width, height, maxwidth, TEXT_NO_INDENT);
-            if(h + height > pixelheight) break;
-
-            draw_text(lines[i].text, x, y+h, color>>16, (color>>8)&0xFF, color&0xFF, 0xFF, TEXT_NO_INDENT, hit&&(cy==i)?cx:-1, maxwidth);
+            if(h+height > pixelheight) break;
+            draw_text(lines[i].text, x, y+h, color>>16, (color>>8)&0xFF, color&0xFF, 0xFF, TEXT_NO_INDENT, hit && (cy == i) ? cx : -1, maxwidth);
             if(linewrap && height > FONTH) // line wrap indicator
             {
                 notextureshader->set();
                 glDisable(GL_TEXTURE_2D);
-                glColor3f(0.5f, 0.5f, 0.5f);
+                glColor4f((guifieldbordercolour>>16)/255.f, ((guifieldbordercolour>>8)&0xFF)/255.f, (guifieldbordercolour&0xFF)/255.f, guifieldborderblend);
                 glBegin(GL_TRIANGLE_STRIP);
-                glVertex2f(x,         y+h+FONTH);
-                glVertex2f(x,         y+h+height);
-                glVertex2f(x-FONTW/2, y+h+FONTH);
-                glVertex2f(x-FONTW/2, y+h+height);
+                glVertex2f(x, y+h+FONTH);
+                glVertex2f(x, y+h+height);
+                glVertex2f(x-FONTW/4, y+h+FONTH);
+                glVertex2f(x-FONTW/4, y+h+height);
                 glEnd();
                 glEnable(GL_TEXTURE_2D);
                 defaultshader->set();
             }
-            h+=height;
+            h += height;
         }
     }
 };
@@ -692,7 +719,8 @@ static editor *useeditor(const char *name, int mode, bool focus, const char *ini
         return e;
     }
     editor *e = new editor(name, mode, initval, parent);
-    if(focus) editors.add(e); else editors.insert(0, e);
+    if(focus) editors.add(e);
+    else editors.insert(0, e);
     return e;
 }
 
@@ -734,10 +762,12 @@ TEXTCOMMAND(textmode, "i", (int *m), // (1= keep while focused, 2= keep while us
     else intret(top->mode);
 );
 TEXTCOMMAND(textsave, "s", (char *file),  // saves the topmost (filename is optional)
+    if(identflags&IDF_WORLD) return;
     if(*file) top->setfile(copypath(file));
     top->save();
 );
 TEXTCOMMAND(textload, "s", (char *file), // loads into the topmost editor, returns filename if no args
+    if(identflags&IDF_WORLD) return;
     if(*file)
     {
         top->setfile(copypath(file));
@@ -747,11 +777,12 @@ TEXTCOMMAND(textload, "s", (char *file), // loads into the topmost editor, retur
 );
 TEXTCOMMAND(textinit, "sss", (char *name, char *file, char *initval), // loads into named editor if no file assigned and editor has been rendered
 {
+    char *f = identflags&IDF_WORLD ? NULL : file;
     editor *e = NULL;
     loopv(editors) if(!strcmp(editors[i]->name, name)) { e = editors[i]; break; }
-    if(e && e->rendered && !e->filename && *file && (e->lines.empty() || (e->lines.length() == 1 && !strcmp(e->lines[0].text, initval))))
+    if(e && e->rendered && !e->filename && f && *f && (e->lines.empty() || (e->lines.length() == 1 && !strcmp(e->lines[0].text, initval))))
     {
-        e->setfile(copypath(file));
+        e->setfile(copypath(f));
         e->load();
     }
 });
@@ -774,3 +805,9 @@ TEXTCOMMAND(textexec, "i", (int *selected), // execute script commands from the
     delete[] script;
 );
 
+TEXTCOMMAND(textadd, "ss", (char *name, char *str), // loads into named editor if no file assigned and editor has been rendered
+{
+    editor *e = NULL;
+    loopv(editors) if(!strcmp(editors[i]->name, name)) { e = editors[i]; break; }
+    if(e && e->rendered) e->insert(str);
+});
