@@ -5,8 +5,8 @@ VARF(IDF_PERSIST, waterrefract, 0, 1, 1, { cleanreflections(); preloadwatershade
 VARF(IDF_PERSIST, waterenvmap, 0, 1, 1, { cleanreflections(); preloadwatershaders(); });
 VARF(IDF_PERSIST, waterfallrefract, 0, 0, 1, { cleanreflections(); preloadwatershaders(); });
 
-VAR(IDF_WORLD, watersubdiv, 0, 3, 3);
-VAR(IDF_WORLD, waterlod, 0, 1, 3);
+VAR(IDF_WORLD, watersubdiv, 0, 2, 3);
+VAR(IDF_WORLD, waterlod, 0, 2, 3);
 
 static int wx1, wy1, wx2, wy2, wsize;
 static float whscale, whoffset;
@@ -20,7 +20,7 @@ static uchar wcol[4];
     } \
     static inline void vertw(float v1, float v2, float v3) \
     { \
-        float angle = float((v1-wx1)*(v2-wy1))*float((v1-wx2)*(v2-wy2))*whscale+whoffset; \
+        float angle = (v1-wx1)*(v2-wy1)*(v1-wx2)*(v2-wy2)*whscale+whoffset; \
         float s = angle - int(angle) - 0.5f; \
         s *= 8 - fabs(s)*16; \
         float h = WATER_AMPLITUDE*s-WATER_OFFSET; \
@@ -157,7 +157,7 @@ VERTWN(vertln, {
     } \
 }
 
-void rendervertwater(uint subdiv, int xo, int yo, int z, uint size, int mat)
+void rendervertwater(int subdiv, int xo, int yo, int z, int size, int mat)
 {
     wx1 = xo;
     wy1 = yo;
@@ -201,47 +201,40 @@ void rendervertwater(uint subdiv, int xo, int yo, int z, uint size, int mat)
     }
 }
 
-uint calcwatersubdiv(int x, int y, int z, uint size)
+int calcwatersubdiv(int x, int y, int z, int size)
 {
     float dist;
     if(camera1->o.x >= x && camera1->o.x < x + size &&
-        camera1->o.y >= y && camera1->o.y < y + size)
+       camera1->o.y >= y && camera1->o.y < y + size)
         dist = fabs(camera1->o.z - float(z));
     else
-    {
-        vec t(x + size/2, y + size/2, z + size/2);
-        dist = t.dist(camera1->o) - size*1.42f/2;
-    }
-    uint subdiv = watersubdiv + int(dist) / (32 << waterlod);
-    if(subdiv >= 8*sizeof(subdiv))
-        subdiv = ~0;
-    else
-        subdiv = 1 << subdiv;
-    return subdiv;
+        dist = vec(x + size/2, y + size/2, z + size/2).dist(camera1->o) - size*1.42f/2;
+    int subdiv = watersubdiv + int(dist) / (32 << waterlod);
+    return subdiv >= 31 ? INT_MAX : 1<<subdiv;
 }
 
-uint renderwaterlod(int x, int y, int z, uint size, int mat)
+int renderwaterlod(int x, int y, int z, int size, int mat)
 {
-    if(size <= (uint)(32 << waterlod))
+    if(size <= (32 << waterlod))
     {
-        uint subdiv = calcwatersubdiv(x, y, z, size);
+        int subdiv = calcwatersubdiv(x, y, z, size);
         if(subdiv < size * 2) rendervertwater(min(subdiv, size), x, y, z, size, mat);
         return subdiv;
     }
     else
     {
-        uint subdiv = calcwatersubdiv(x, y, z, size);
+        int subdiv = calcwatersubdiv(x, y, z, size);
         if(subdiv >= size)
         {
             if(subdiv < size * 2) rendervertwater(size, x, y, z, size, mat);
             return subdiv;
         }
-        uint childsize = size / 2,
-             subdiv1 = renderwaterlod(x, y, z, childsize, mat),
-             subdiv2 = renderwaterlod(x + childsize, y, z, childsize, mat),
-             subdiv3 = renderwaterlod(x + childsize, y + childsize, z, childsize, mat),
-             subdiv4 = renderwaterlod(x, y + childsize, z, childsize, mat),
-             minsubdiv = subdiv1;
+        int childsize = size / 2,
+            subdiv1 = renderwaterlod(x, y, z, childsize, mat),
+            subdiv2 = renderwaterlod(x + childsize, y, z, childsize, mat),
+            subdiv3 = renderwaterlod(x + childsize, y + childsize, z, childsize, mat),
+            subdiv4 = renderwaterlod(x, y + childsize, z, childsize, mat),
+            minsubdiv = subdiv1;
         minsubdiv = min(minsubdiv, subdiv2);
         minsubdiv = min(minsubdiv, subdiv3);
         minsubdiv = min(minsubdiv, subdiv4);
@@ -270,7 +263,7 @@ uint renderwaterlod(int x, int y, int z, uint size, int mat)
         xtraverts += 4; \
     }
 
-void renderflatwater(int x, int y, int z, uint rsize, uint csize, int mat)
+void renderflatwater(int x, int y, int z, int rsize, int csize, int mat)
 {
     switch(mat)
     {
@@ -303,7 +296,7 @@ VARF(IDF_WORLD, vertwater, 0, 1, 1, if(!(identflags&IDF_WORLD)) allchanged());
 static inline void renderwater(const materialsurface &m, int mat = MAT_WATER)
 {
     if(!vertwater || minimapping) renderflatwater(m.o.x, m.o.y, m.o.z, m.rsize, m.csize, mat);
-    else if(renderwaterlod(m.o.x, m.o.y, m.o.z, m.csize, mat) >= (uint)m.csize * 2)
+    else if(renderwaterlod(m.o.x, m.o.y, m.o.z, m.csize, mat) >= int(m.csize) * 2)
         rendervertwater(m.csize, m.o.x, m.o.y, m.o.z, m.csize, mat);
 }
 
@@ -322,12 +315,13 @@ void renderlava(const materialsurface &m, Texture *tex, float scale)
 struct Reflection
 {
     GLuint tex, refracttex;
-    int material, height, depth, lastupdate, lastused;
+    int material, height, depth, age;
+    bool init;
     glmatrixf projmat;
     occludequery *query, *prevquery;
     vector<materialsurface *> matsurfs;
 
-    Reflection() : tex(0), refracttex(0), material(-1), height(-1), depth(0), lastused(0), query(NULL), prevquery(NULL)
+    Reflection() : tex(0), refracttex(0), material(-1), height(-1), depth(0), age(0), init(false), query(NULL), prevquery(NULL)
     {}
 };
 
@@ -375,9 +369,13 @@ GETMATIDXVAR(lava, colour, int)
 GETMATIDXVAR(lava, col, const bvec &)
 GETMATIDXVAR(lava, fog, int)
 
-void setprojtexmatrix(Reflection &ref, bool init = true)
+void setprojtexmatrix(Reflection &ref)
 {
-    if(init && ref.lastupdate==totalmillis) (ref.projmat = mvpmatrix).projective();
+    if(ref.init)
+    {
+        ref.init = false;
+        (ref.projmat = mvpmatrix).projective();
+    }
 
     glLoadMatrixf(ref.projmat.v);
 }
@@ -474,7 +472,7 @@ void renderwaterff()
     loopi(MAXREFLECTIONS)
     {
         Reflection &ref = reflections[i];
-        if(ref.height<0 || ref.lastused<totalmillis || ref.matsurfs.empty()) continue;
+        if(ref.height<0 || ref.age || ref.matsurfs.empty()) continue;
 
         bool below = camera1->o.z < ref.height + offset;
         if(!nowater && (waterrefract || waterreflect || (waterenvmap && hasCM)) && !minimapping)
@@ -487,7 +485,6 @@ void renderwaterff()
                 }
             }
 
-            bool projtex = false;
             if(waterreflect || (waterenvmap && hasCM))
             {
                 bool tmu1 = waterrefract && (!below || !wasbelow);
@@ -504,9 +501,8 @@ void renderwaterff()
                     {
                         glBindTexture(GL_TEXTURE_2D, ref.tex);
                         setprojtexmatrix(ref);
-                        projtex = true;
                     }
-                    else 
+                    else
                     {
                         glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, lookupenvmap(lookupmaterialslot(ref.material)));
                     }
@@ -522,7 +518,7 @@ void renderwaterff()
             if(waterrefract)
             {
                 glBindTexture(GL_TEXTURE_2D, ref.refracttex);
-                setprojtexmatrix(ref, !projtex);
+                setprojtexmatrix(ref);
             }
         }
 
@@ -688,7 +684,7 @@ void renderwater()
     loopi(MAXREFLECTIONS)
     {
         Reflection &ref = reflections[i];
-        if(ref.height<0 || ref.lastused<totalmillis || ref.matsurfs.empty()) continue;
+        if(ref.height<0 || ref.age || ref.matsurfs.empty()) continue;
         if(!glaring && hasOQ && oqfrags && oqwater && ref.query && ref.query->owner==&ref)
         {
             if(!ref.prevquery || ref.prevquery->owner!=&ref || checkquery(ref.prevquery))
@@ -824,7 +820,7 @@ void cleanreflection(Reflection &ref)
 {
     ref.material = -1;
     ref.height = -1;
-    ref.lastupdate = 0;
+    ref.init = false;
     ref.query = ref.prevquery = NULL;
     ref.matsurfs.setsize(0);
     if(ref.tex)
@@ -935,9 +931,10 @@ void genwatertex(GLuint &tex, GLuint &fb, GLuint &db, bool refract = false)
 void addwaterfallrefraction(materialsurface &m)
 {
     Reflection &ref = waterfallrefraction;
-    if(ref.lastused!=totalmillis)
+    if(ref.age>=0)
     {
-        ref.lastused = totalmillis;
+        ref.age = -1;
+        ref.init = false;
         ref.matsurfs.setsize(0);
         ref.material = MAT_WATER;
         ref.height = INT_MAX;
@@ -962,25 +959,26 @@ void addreflection(materialsurface &m)
         {
             r.matsurfs.add(&m);
             r.depth = max(r.depth, int(m.depth));
-            if(r.lastused==totalmillis) return;
+            if(r.age<0) return;
             ref = &r;
             break;
         }
-        else if(!oldest || r.lastused<oldest->lastused) oldest = &r;
+        else if(!oldest || r.age>oldest->age) oldest = &r;
     }
     if(!ref)
     {
-        if(!oldest || oldest->lastused==totalmillis) return;
+        if(!oldest || oldest->age<0) return;
         ref = oldest;
     }
-    if(ref->height!=height || ref->material!=mat) 
+    if(ref->height!=height || ref->material!=mat)
     {
         ref->material = mat;
         ref->height = height;
         ref->prevquery = NULL;
     }
     rplanes++;
-    ref->lastused = totalmillis;
+    ref->age = -1;
+    ref->init = false;
     ref->matsurfs.setsize(0);
     ref->matsurfs.add(&m);
     ref->depth = m.depth;
@@ -1003,7 +1001,7 @@ static void drawmaterialquery(const materialsurface &m, float offset, float bord
 #define GENFACEORIENT(orient, v0, v1, v2, v3) \
         case orient: v0 v1 v2 v3 break;
 #define GENFACEVERT(orient, vert, mx,my,mz, sx,sy,sz) \
-            varray::attrib<float>(mx sx, my sy, mz sz); 
+            varray::attrib<float>(mx sx, my sy, mz sz);
         GENFACEVERTS(x, x, y, y, z, z, - border, + csize, - border, + rsize, + offset, - offset)
 #undef GENFACEORIENT
 #undef GENFACEVERT
@@ -1015,8 +1013,6 @@ extern void drawreflection(float z, bool refract, int fogdepth = -1, const bvec
 
 int rplanes = 0;
 
-static int lastquery = 0;
-
 void queryreflection(Reflection &ref, bool init)
 {
     if(init)
@@ -1079,7 +1075,8 @@ void queryreflections()
     loopi(MAXREFLECTIONS)
     {
         Reflection &ref = reflections[i];
-        if(ref.height>=0 && ref.lastused>=totalmillis && ref.matsurfs.length())
+        ++ref.age;
+        if(ref.height>=0 && !ref.age && ref.matsurfs.length())
         {
             if(waterpvsoccluded(ref.height)) ref.matsurfs.setsize(0);
         }
@@ -1087,14 +1084,13 @@ void queryreflections()
     if(renderpath!=R_FIXEDFUNCTION && waterfallrefract)
     {
         Reflection &ref = waterfallrefraction;
-        if(ref.height>=0 && ref.lastused>=totalmillis && ref.matsurfs.length())
+        ++ref.age;
+        if(ref.height>=0 && !ref.age && ref.matsurfs.length())
         {
             if(waterpvsoccluded(-1)) ref.matsurfs.setsize(0);
         }
     }
 
-    lastquery = totalmillis;
-
     if((editmode && showmat && !envmapping) || !hasOQ || !oqfrags || !oqwater || nowater || minimapping) return;
 
     varray::enable();
@@ -1104,14 +1100,14 @@ void queryreflections()
     {
         Reflection &ref = reflections[i];
         ref.prevquery = oqwater > 1 ? ref.query : NULL;
-        ref.query = ref.height>=0 && ref.lastused>=totalmillis && ref.matsurfs.length() ? newquery(&ref) : NULL;
+        ref.query = ref.height>=0 && !ref.age && ref.matsurfs.length() ? newquery(&ref) : NULL;
         if(ref.query) queryreflection(ref, !refs++);
     }
     if(renderpath!=R_FIXEDFUNCTION && waterfallrefract)
     {
         Reflection &ref = waterfallrefraction;
         ref.prevquery = oqwater > 1 ? ref.query : NULL;
-        ref.query = ref.height>=0 && ref.lastused>=totalmillis && ref.matsurfs.length() ? newquery(&ref) : NULL;
+        ref.query = ref.height>=0 && !ref.age && ref.matsurfs.length() ? newquery(&ref) : NULL;
         if(ref.query) queryreflection(ref, !refs++);
     }
 
@@ -1128,7 +1124,7 @@ void queryreflections()
     glFlush();
 }
 
-VAR(IDF_PERSIST, maxreflect, 1, 1, 8);
+VAR(IDF_PERSIST, maxreflect, 1, 2, 8);
 
 int refracting = 0, refractfog = 0;
 bvec refractcolor(0, 0, 0);
@@ -1280,9 +1276,9 @@ void drawreflections()
     if(waterreflect || waterrefract) loopi(MAXREFLECTIONS)
     {
         Reflection &ref = reflections[++n%MAXREFLECTIONS];
-        if(ref.height<0 || ref.lastused<lastquery || ref.matsurfs.empty()) continue;
+        if(ref.height<0 || ref.age || ref.matsurfs.empty()) continue;
         if(hasOQ && oqfrags && oqwater && ref.query && ref.query->owner==&ref)
-        { 
+        {
             if(!ref.prevquery || ref.prevquery->owner!=&ref || checkquery(ref.prevquery))
             {
                 if(checkquery(ref.query)) continue;
@@ -1295,7 +1291,7 @@ void drawreflections()
             if(hasFBO) glBindFramebuffer_(GL_FRAMEBUFFER_EXT, reflectionfb);
         }
         refs++;
-        ref.lastupdate = totalmillis;
+        ref.init = true;
         lastdrawn = n;
 
         vec clipmin(-1, -1, -1), clipmax(1, 1, 1);
@@ -1319,7 +1315,7 @@ void drawreflections()
             maskreflection(ref, offset, true);
             if(scissor && nvidia_scissor_bug) glEnable(GL_SCISSOR_TEST);
             savevfcP();
-            setvfcP(ref.height+offset, clipmin, clipmax); 
+            setvfcP(ref.height+offset, clipmin, clipmax);
             drawreflection(ref.height+offset, false);
             restorevfcP();
             if(scissor) glDisable(GL_SCISSOR_TEST);
@@ -1358,7 +1354,7 @@ void drawreflections()
     {
         Reflection &ref = waterfallrefraction;
 
-        if(ref.height<0 || ref.lastused<lastquery || ref.matsurfs.empty()) goto nowaterfall;
+        if(ref.height<0 || ref.age || ref.matsurfs.empty()) goto nowaterfall;
         if(hasOQ && oqfrags && oqwater && ref.query && ref.query->owner==&ref)
         {
             if(!ref.prevquery || ref.prevquery->owner!=&ref || checkquery(ref.prevquery))
@@ -1373,7 +1369,7 @@ void drawreflections()
             if(hasFBO) glBindFramebuffer_(GL_FRAMEBUFFER_EXT, reflectionfb);
         }
         refs++;
-        ref.lastupdate = totalmillis;
+        ref.init = true;
 
         vec clipmin(-1, -1, -1), clipmax(1, 1, 1);
         int sx, sy, sw, sh;
