@@ -5,10 +5,34 @@
 mapz hdr;
 int worldscale;
 
-VAR(0, octaentsize, 0, 128, 1024);
+VAR(0, octaentsize, 0, 64, 1024);
 VAR(0, entselradius, 0, 2, 10);
 
-bool getentboundingbox(extentity &e, ivec &o, ivec &r)
+static inline void mmboundbox(const entity &e, model *m, vec &center, vec &radius)
+{
+    m->boundbox(center, radius);
+    if(e.attrs[5])
+    {
+       float scale = max(e.attrs[5]/100.0f, 1e-3f);
+       center.mul(scale);
+       radius.mul(scale);
+    }
+    rotatebb(center, radius, e.attrs[1], e.attrs[2], e.attrs[3]);
+}
+
+static inline void mmcollisionbox(const entity &e, model *m, vec &center, vec &radius)
+{
+    m->collisionbox(center, radius);
+    if(e.attrs[5])
+    {
+       float scale = max(e.attrs[5]/100.0f, 1e-3f);
+       center.mul(scale);
+       radius.mul(scale);
+    }
+    rotatebb(center, radius, e.attrs[1], e.attrs[2], e.attrs[3]);
+}
+
+bool getentboundingbox(const extentity &e, ivec &o, ivec &r)
 {
     switch(e.type)
     {
@@ -16,30 +40,22 @@ bool getentboundingbox(extentity &e, ivec &o, ivec &r)
             return false;
         case ET_MAPMODEL:
         {
-            model *m = loadmodel(NULL, e.attrs[0]);
+            model *m = loadmapmodel(e.attrs[0]);
             if(m)
             {
                 vec center, radius;
-                m->boundbox(0, center, radius);
-                if(e.attrs[5])
-                {
-                    float scale = max(e.attrs[5]/100.0f, 1e-3f);
-                    center.mul(scale);
-                    radius.mul(scale);
-                }
-                rotatebb(center, radius, e.attrs[1], e.attrs[2], e.attrs[3]);
-                o = vec(center).add(e.o);
-                r = vec(radius).add(1);
-                o.sub(r);
-                r.mul(2);
+                mmboundbox(e, m, center, radius);
+                center.add(e.o);
+                radius.max(entselradius);
+                o = vec(center).sub(radius);
+                r = vec(center).add(radius).add(1);
                 break;
             }
         }
         // invisible mapmodels use entselradius
         default:
-            o = e.o;
-            o.sub(entselradius);
-            r.x = r.y = r.z = entselradius*2;
+            o = vec(e.o).sub(entselradius);
+            r = vec(e.o).add(entselradius+1);
             break;
     }
     return true;
@@ -67,7 +83,7 @@ void modifyoctaentity(int flags, int id, extentity &e, cube *c, const ivec &cor,
             switch(e.type)
             {
                 case ET_MAPMODEL:
-                    if(loadmodel(NULL, e.attrs[0]))
+                    if(loadmapmodel(e.attrs[0]))
                     {
                         if(va)
                         {
@@ -75,11 +91,8 @@ void modifyoctaentity(int flags, int id, extentity &e, cube *c, const ivec &cor,
                             if(oe.mapmodels.empty()) va->mapmodels.add(&oe);
                         }
                         oe.mapmodels.add(id);
-                        loopk(3)
-                        {
-                            oe.bbmin[k] = min(oe.bbmin[k], max(oe.o[k], bo[k]));
-                            oe.bbmax[k] = max(oe.bbmax[k], min(oe.o[k]+size, bo[k]+br[k]));
-                        }
+                        oe.bbmin.min(bo).max(oe.o);
+                        oe.bbmax.max(br).min(ivec(oe.o).add(oe.size));
                         break;
                     }
                     // invisible mapmodel
@@ -95,7 +108,7 @@ void modifyoctaentity(int flags, int id, extentity &e, cube *c, const ivec &cor,
             switch(e.type)
             {
                 case ET_MAPMODEL:
-                    if(loadmodel(NULL, e.attrs[0]))
+                    if(loadmapmodel(e.attrs[0]))
                     {
                         oe.mapmodels.removeobj(id);
                         if(va)
@@ -109,17 +122,14 @@ void modifyoctaentity(int flags, int id, extentity &e, cube *c, const ivec &cor,
                         {
                             extentity &e = *entities::getents()[oe.mapmodels[j]];
                             ivec eo, er;
-                            if(getentboundingbox(e, eo, er)) loopk(3)
+                            if(getentboundingbox(e, eo, er))
                             {
-                                oe.bbmin[k] = min(oe.bbmin[k], eo[k]);
-                                oe.bbmax[k] = max(oe.bbmax[k], eo[k]+er[k]);
+                                oe.bbmin.min(eo);
+                                oe.bbmax.max(er);
                             }
                         }
-                        loopk(3)
-                        {
-                            oe.bbmin[k] = max(oe.bbmin[k], oe.o[k]);
-                            oe.bbmax[k] = min(oe.bbmax[k], oe.o[k]+size);
-                        }
+                        oe.bbmin.max(oe.o);
+                        oe.bbmax.min(ivec(oe.o).add(oe.size));
                         break;
                     }
                     // invisible mapmodel
@@ -146,7 +156,7 @@ vector<int> outsideents;
 
 static bool modifyoctaent(int flags, int id, extentity &e)
 {
-    if(flags&MODOE_ADD ? e.inoctanode : !e.inoctanode) return false;
+    if(flags&MODOE_ADD ? e.flags&EF_OCTA : !(e.flags&EF_OCTA)) return false;
 
     ivec o, r;
     if(!getentboundingbox(e, o, r)) return false;
@@ -162,13 +172,13 @@ static bool modifyoctaent(int flags, int id, extentity &e)
     }
     else
     {
-        int leafsize = octaentsize, limit = max(r.x, max(r.y, r.z));
+        int leafsize = octaentsize, limit = max(r.x - o.x, max(r.y - o.y, r.z - o.z));
         while(leafsize < limit) leafsize *= 2;
-        int diff = ~(leafsize-1) & ((o.x^(o.x+r.x))|(o.y^(o.y+r.y))|(o.z^(o.z+r.z)));
+        int diff = ~(leafsize-1) & ((o.x^r.x)|(o.y^r.y)|(o.z^r.z));
         if(diff && (limit > octaentsize/2 || diff < leafsize*2)) leafsize *= 2;
         modifyoctaentity(flags, id, e, worldroot, ivec(0, 0, 0), hdr.worldsize>>1, o, r, leafsize);
     }
-    e.inoctanode = flags&MODOE_ADD ? 1 : 0;
+    e.flags ^= EF_OCTA;
     if(e.type == ET_LIGHT || e.type == ET_SUNLIGHT) clearlightcache(id);
     else if(flags&MODOE_LIGHTENT) lightent(e);
     return true;
@@ -204,26 +214,26 @@ void entitiesinoctanodes()
     loopv(ents) modifyoctaent(MODOE_ADD, i, *ents[i]);
 }
 
-static inline void findents(octaentities &oe, int low, int high, bool notspawned, const vec &pos, const vec &radius, vector<int> &found)
+static inline void findents(octaentities &oe, int low, int high, bool notspawned, const vec &pos, const vec &invradius, vector<int> &found)
 {
     vector<extentity *> &ents = entities::getents();
     loopv(oe.other)
     {
         int id = oe.other[i];
         extentity &e = *ents[id];
-        if(e.type >= low && e.type <= high && (e.spawned || notspawned) && vec(e.o).mul(radius).squaredlen() <= 1) found.add(id);
+        if(e.type >= low && e.type <= high && (e.spawned() || notspawned) && vec(e.o).sub(pos).mul(invradius).squaredlen() <= 1) found.add(id);
     }
 }
 
-static inline void findents(cube *c, const ivec &o, int size, const ivec &bo, const ivec &br, int low, int high, bool notspawned, const vec &pos, const vec &radius, vector<int> &found)
+static inline void findents(cube *c, const ivec &o, int size, const ivec &bo, const ivec &br, int low, int high, bool notspawned, const vec &pos, const vec &invradius, vector<int> &found)
 {
     loopoctabox(o, size, bo, br)
     {
-        if(c[i].ext && c[i].ext->ents) findents(*c[i].ext->ents, low, high, notspawned, pos, radius, found);
+        if(c[i].ext && c[i].ext->ents) findents(*c[i].ext->ents, low, high, notspawned, pos, invradius, found);
         if(c[i].children && size > octaentsize)
         {
             ivec co(i, o.x, o.y, o.z, size);
-            findents(c[i].children, co, size>>1, bo, br, low, high, notspawned, pos, radius, found);
+            findents(c[i].children, co, size>>1, bo, br, low, high, notspawned, pos, invradius, found);
         }
     }
 }
@@ -232,10 +242,10 @@ void findents(int low, int high, bool notspawned, const vec &pos, const vec &rad
 {
     vec invradius(1/radius.x, 1/radius.y, 1/radius.z);
     ivec bo = vec(pos).sub(radius).sub(1),
-         br = vec(radius).add(1).mul(2);
-    int diff = (bo.x^(bo.x+br.x)) | (bo.y^(bo.y+br.y)) | (bo.z^(bo.z+br.z)) | octaentsize,
+         br = vec(pos).add(radius).add(1);
+    int diff = (bo.x^br.x) | (bo.y^br.y) | (bo.z^br.z) | octaentsize,
         scale = worldscale-1;
-    if(diff&~((1<<scale)-1) || uint(bo.x|bo.y|bo.z|(bo.x+br.x)|(bo.y+br.y)|(bo.z+br.z)) >= uint(hdr.worldsize))
+    if(diff&~((1<<scale)-1) || uint(bo.x|bo.y|bo.z|br.x|br.y|br.z) >= uint(hdr.worldsize))
     {
         findents(worldroot, ivec(0, 0, 0), 1<<scale, bo, br, low, high, notspawned, pos, invradius, found);
         return;
@@ -265,7 +275,7 @@ bool noentedit()
     return !entediting;
 }
 
-bool pointinsel(selinfo &sel, vec &o)
+bool pointinsel(const selinfo &sel, const vec &o)
 {
     return(o.x <= sel.o.x+sel.s.x*sel.grid
         && o.x >= sel.o.x
@@ -331,17 +341,20 @@ void makeundoent()
 // e         entity, currently edited ent
 // n         int,   index to currently edited ent
 #define addimplicit(f)  { if(entgroup.empty() && enthover>=0) { entadd(enthover); undonext = (enthover != oldhover); f; entgroup.drop(); } else f; }
-#define entedit(i, f) \
+#define enteditv(i, f, v) \
 { \
-    entfocus(i, \
-    removeentity(n);  \
-    f; \
-    if(e.type!=ET_EMPTY) { addentity(n); } \
-    entities::editent(n)); \
+    entfocusv(i, \
+    { \
+        removeentity(n);  \
+        f; \
+        if(e.type!=ET_EMPTY) { addentity(n); } \
+        entities::editent(n); \
+    }, v); \
 }
-#define addgroup(exp)   { loopv(entities::getents()) entfocus(i, if(exp) entadd(n)); }
+#define entedit(i, f)   enteditv(i, f, entities::getents())
+#define addgroup(exp)   { vector<extentity *> &ents = entities::getents(); loopv(ents) entfocusv(i, if(exp) entadd(n), ents); }
 #define setgroup(exp)   { entcancel(); addgroup(exp); }
-#define groupeditloop(f){ entlooplevel++; int _ = efocus; loopv(entgroup) entedit(entgroup[i], f); efocus = _; entlooplevel--; }
+#define groupeditloop(f){ vector<extentity *> &ents = entities::getents(); entlooplevel++; int _ = efocus; loopv(entgroup) enteditv(entgroup[i], f, ents); efocus = _; entlooplevel--; }
 #define groupeditpure(f){ if(entlooplevel>0) { entedit(efocus, f); } else groupeditloop(f); }
 #define groupeditundo(f){ makeundoent(); groupeditpure(f); }
 #define groupedit(f)    { addimplicit(groupeditundo(f)); }
@@ -399,15 +412,10 @@ void entrotate(int *cw)
 void entselectionbox(const extentity &e, vec &eo, vec &es)
 {
     model *m = NULL;
-    if(e.type == ET_MAPMODEL && (m = loadmodel(NULL, e.attrs[0])))
+    if(e.type == ET_MAPMODEL && (m = loadmapmodel(e.attrs[0])))
     {
-        m->collisionbox(0, eo, es);
-        if(e.attrs[5]) { float scale = max(e.attrs[5]/100.f, 1e-3f); eo.mul(scale); es.mul(scale); }
-        rotatebb(eo, es, e.attrs[1], e.attrs[2], e.attrs[3]);
-        if(m->collide)
-            eo.z -= camera1->aboveeye; // wacky but true. see physics collide
-        else
-            es.div(2);  // cause the usual bb is too big...
+        mmcollisionbox(e, m, eo, es);
+        es.max(entselradius);
         eo.add(e.o);
     }
     else
@@ -424,9 +432,9 @@ VAR(0, entmovingshadow, 0, 1, 1);
 
 extern void boxs(int orient, vec o, const vec &s);
 extern void boxs3D(const vec &o, vec s, int g);
-extern void editmoveplane(const vec &o, const vec &ray, int d, float off, vec &handle, vec &dest, bool first);
+extern bool editmoveplane(const vec &o, const vec &ray, int d, float off, vec &handle, vec &dest, bool first);
 
-bool initentdragging = true;
+int entmoving = 0;
 
 void entdrag(const vec &ray)
 {
@@ -441,7 +449,8 @@ void entdrag(const vec &ray)
     entfocus(entgroup.last(),
         entselectionbox(e, eo, es);
 
-        editmoveplane(e.o, ray, d, eo[d] + (dc ? es[d] : 0), handle, v, initentdragging);
+        if(!editmoveplane(e.o, ray, d, eo[d] + (dc ? es[d] : 0), handle, v, entmoving==1))
+            return;
 
         ivec g(v);
         int z = g[d]&(~(sel.grid-1));
@@ -452,9 +461,9 @@ void entdrag(const vec &ray)
         c = (entselsnap ? g[C[d]] : v[C[d]]) - e.o[C[d]];
     );
 
-    if(initentdragging) makeundoent();
+    if(entmoving==1) makeundoent();
     groupeditpure(e.o[R[d]] += r; e.o[C[d]] += c);
-    initentdragging = false;
+    entmoving = 2;
 }
 
 void renderentselection(const vec &o, const vec &ray, bool entmoving)
@@ -510,16 +519,35 @@ bool hoveringonent(int ent, int orient)
 }
 
 VAR(0, entitysurf, 0, 0, 1);
-VARF(0, entmoving, 0, 0, 2,
-    if(enthover < 0 || noentedit())
-        entmoving = 0;
-    else if(entmoving == 1)
-        entmoving = enttoggle(enthover);
-    else if(entmoving == 2 && entgroup.find(enthover) < 0)
-        entadd(enthover);
-    if(entmoving > 0)
-        initentdragging = true;
-);
+
+ICOMMAND(0, entadd, "", (),
+{
+    if(enthover >= 0 && !noentedit())
+    {
+        if(entgroup.find(enthover) < 0) entadd(enthover);
+        if(entmoving > 1) entmoving = 1;
+    }
+});
+
+ICOMMAND(0, enttoggle, "", (),
+{
+    if(enthover < 0 || noentedit() || !enttoggle(enthover)) { entmoving = 0; intret(0); }
+    else { if(entmoving > 1) entmoving = 1; intret(1); }
+});
+
+ICOMMAND(0, entmoving, "b", (int *n),
+{
+    if(*n >= 0)
+    {
+        if(!*n || enthover < 0 || noentedit()) entmoving = 0;
+        else
+        {
+            if(entgroup.find(enthover) < 0) { entadd(enthover); entmoving = 1; }
+            else if(!entmoving) entmoving = 1;
+        }
+    }
+    intret(entmoving);
+});
 
 void entpush(int *dir)
 {
@@ -599,13 +627,11 @@ bool dropentity(extentity &e, int drop = -1)
     if(drop<0) drop = entdrop;
     if(e.type == ET_MAPMODEL)
     {
-        model *m = loadmodel(NULL, e.attrs[0]);
+        model *m = loadmapmodel(e.attrs[0]);
         if(m)
         {
             vec center;
-            m->boundbox(0, center, radius);
-            if(e.attrs[5]) { float scale = max(e.attrs[5]/100.f, 1e-3f); center.mul(scale); radius.mul(scale); }
-            rotatebb(center, radius, e.attrs[1], e.attrs[2], e.attrs[3]);
+            mmboundbox(e, m, center, radius);
             radius.x += fabs(center.x);
             radius.y += fabs(center.y);
         }
@@ -648,7 +674,7 @@ void dropent()
 
 static int keepents = 0;
 
-extentity *newentity(bool local, const vec &o, int type, const attrvector &attrs, int &idx)
+extentity *newentity(bool local, const vec &o, int type, const attrvector &attrs, int &idx, bool fix = true)
 {
     vector<extentity *> &ents = entities::getents();
     if(local)
@@ -660,40 +686,40 @@ extentity *newentity(bool local, const vec &o, int type, const attrvector &attrs
     else while(ents.length() < idx) ents.add(entities::newent())->type = ET_EMPTY;
     extentity &e = *entities::newent();
     e.o = o;
-    e.attrs.add(0, clamp(attrs.length(), entities::numattrs(type), MAXENTATTRS) - e.attrs.length());
+    e.attrs.add(0, min(attrs.length(), MAXENTATTRS) - e.attrs.length());
     loopi(min(attrs.length(), e.attrs.length())) e.attrs[i] = attrs[i];
     e.type = type;
-    e.spawned = false;
-    e.inoctanode = false;
     e.light.color = vec(1, 1, 1);
     e.light.dir = vec(0, 0, 1);
     if(ents.inrange(idx)) { entities::deleteent(ents[idx]); ents[idx] = &e; }
     else { idx = ents.length(); ents.add(&e); }
-    if(local) entities::fixentity(idx, true, true);
+    if(local && fix) entities::fixentity(idx, true, true);
     return &e;
 }
 
-void newentity(const vec &v, int type, const attrvector &attrs)
+int newentity(const vec &v, int type, const attrvector &attrs)
 {
-    int idx;
+    int idx = -1;
     extentity *t = newentity(true, v, type, attrs, idx);
-    if(!t) return;
+    if(!t) return -1;
     t->type = ET_EMPTY;
     enttoggle(idx);
     makeundoent();
     entedit(idx, e.type = type);
+    return idx;
 }
 
-void newentity(int type, const attrvector &attrs)
+int newentity(int type, const attrvector &attrs)
 {
-    int idx;
+    int idx = -1;
     extentity *t = newentity(true, camera1->o, type, attrs, idx);
-    if(!t) return;
+    if(!t) return -1;
     dropentity(*t);
     t->type = ET_EMPTY;
     enttoggle(idx);
     makeundoent();
     entedit(idx, e.type = type);
+    return idx;
 }
 
 void entattrs(const char *str, attrvector &attrs)
@@ -739,7 +765,7 @@ void entpaste()
         vec o(c.o);
         o.mul(m).add(sel.o.tovec());
         int idx;
-        extentity *e = newentity(true, o, ET_EMPTY, c.attrs, idx);
+        extentity *e = newentity(true, o, ET_EMPTY, c.attrs, idx, false);
         if(!e) continue;
         loopvk(c.links) e->links.add(c.links[k]);
         entadd(idx);
@@ -967,7 +993,7 @@ bool emptymap(int scale, bool force, char *mname, bool nocfg)   // main empty wo
     clearworldvars();
     resetmap(nocfg);
     setnames(mname, MAP_MAPZ);
-    strncpy(hdr.head, "MAPZ", 4);
+    memcpy(hdr.head, "MAPZ", 4);
 
     hdr.version = MAPVERSION;
     hdr.gamever = server::getver(1);
@@ -997,7 +1023,7 @@ bool emptymap(int scale, bool force, char *mname, bool nocfg)   // main empty wo
 
     initlights();
     allchanged(true);
-
+    entities::initents(MAP_MAPZ, hdr.version, hdr.gameid, hdr.gamever);
     game::startmap(nocfg ? "" : "maps/untitled", NULL, true);
     return true;
 }
@@ -1070,7 +1096,7 @@ void shrinkmap()
     conoutf("shrunk map to size %d", worldscale);
 }
 
-ICOMMAND(0, newmap, "is", (int *i), if(emptymap(*i, false)) game::newmap(::max(*i, 0)));
+ICOMMAND(0, newmap, "is", (int *i, char *n), if(emptymap(*i, false, n)) game::newmap(::max(*i, 0)));
 ICOMMAND(0, mapenlarge, "", (), if(enlargemap(false)) game::newmap(-1));
 COMMAND(0, shrinkmap, "");
 ICOMMAND(0, mapsize, "", (void),
