@@ -134,8 +134,8 @@ enum
 
 #define TEX_SCALE 8.0f
 
-struct vertexff { vec pos; bvec norm; uchar reserved; float u, v; float lmu, lmv; };
-struct vertex { vec pos; bvec norm; uchar reserved; float u, v; short lmu, lmv; bvec tangent; uchar bitangent; };
+struct vertexff { vec pos; bvec4 norm; float u, v; float lmu, lmv; };
+struct vertex { vec pos; bvec4 norm; float u, v; short lmu, lmv; bvec4 tangent; };
 
 #define VTXSIZE (renderpath==R_FIXEDFUNCTION ? sizeof(vertexff) : sizeof(vertex))
 
