@@ -24,7 +24,7 @@ VAR(IDF_PERSIST, autosavemapshot, 0, 1, 1);
 void fixmaptitle()
 {
     string s; // remove colour from these things in RE
-    if(filtertext(s, maptitle)) setsvar("maptitle", s, false);
+    if(filterstring(s, maptitle)) setsvar("maptitle", s, false);
     const char *title = maptitle, *author = strstr(title, " by ");
     if(author && *author)
     {
@@ -33,7 +33,7 @@ void fixmaptitle()
         {
             if(*t)
             {
-                loopi(4) if(*author) author += 1;
+                loopi(4) if(*author) author++;
                 if(*author) setsvar("mapauthor", author, true);
                 setsvar("maptitle", t, false);
             }
@@ -43,7 +43,7 @@ void fixmaptitle()
 }
 
 SVARF(IDF_WORLD, maptitle, "", fixmaptitle());
-SVARF(IDF_WORLD, mapauthor, "", { string s; if(filtertext(s, mapauthor)) setsvar("mapauthor", s, false); });
+SVARF(IDF_WORLD, mapauthor, "", { string s; if(filterstring(s, mapauthor)) setsvar("mapauthor", s, false); });
 
 void setnames(const char *fname, int type)
 {
@@ -81,26 +81,28 @@ void savec(cube *c, const ivec &o, int size, stream *f, bool nolms)
         {
             int oflags = 0, surfmask = 0, totalverts = 0;
             if(c[i].material!=MAT_AIR) oflags |= 0x40;
-            if(!nolms)
-            {
-                if(c[i].merged) oflags |= 0x80;
-                if(c[i].ext) loopj(6)
-                {
-                    const surfaceinfo &surf = c[i].ext->surfaces[j];
-                    if(!surf.used()) continue;
-                    oflags |= 0x20;
-                    surfmask |= 1<<j;
-                    totalverts += surf.totalverts();
-                }
-            }
-
-            if(c[i].children) f->putchar(oflags | OCTSAV_LODCUBE);
-            else if(isempty(c[i])) f->putchar(oflags | OCTSAV_EMPTY);
-            else if(isentirelysolid(c[i])) f->putchar(oflags | OCTSAV_SOLID);
+            if(isempty(c[i])) f->putchar(oflags | OCTSAV_EMPTY);
             else
             {
-                f->putchar(oflags | OCTSAV_NORMAL);
-                f->write(c[i].edges, 12);
+                if(!nolms)
+                {
+                    if(c[i].merged) oflags |= 0x80;
+                    if(c[i].ext) loopj(6)
+                    {
+                        const surfaceinfo &surf = c[i].ext->surfaces[j];
+                        if(!surf.used()) continue;
+                        oflags |= 0x20;
+                        surfmask |= 1<<j;
+                        totalverts += surf.totalverts();
+                    }
+                }
+
+                if(isentirelysolid(c[i])) f->putchar(oflags | OCTSAV_SOLID);
+                else
+                {
+                    f->putchar(oflags | OCTSAV_NORMAL);
+                    f->write(c[i].edges, 12);
+                }
             }
 
             loopj(6) f->putlil<ushort>(c[i].texture[j]);
@@ -217,8 +219,6 @@ void savec(cube *c, const ivec &o, int size, stream *f, bool nolms)
                     }
                 }
             }
-
-            if(c[i].children) savec(c[i].children, co, size>>1, f, nolms);
         }
     }
 }
@@ -289,7 +289,7 @@ void convertoldsurfaces(cube &c, const ivec &co, int size, surfacecompat *srcsur
                     const ivec &coords = facecoords[i][k];
                     int cc = coords[vc] ? m.u2 : m.u1,
                         rc = coords[vr] ? m.v2 : m.v1,
-                        dc = -(offset + n[vc]*cc + n[vr]*rc)/n[dim];
+                        dc = n[dim] ? -(offset + n[vc]*cc + n[vr]*rc)/n[dim] : vo[dim];
                     ivec &mv = pos[k];
                     mv[vc] = cc;
                     mv[vr] = rc;
@@ -489,7 +489,7 @@ void loadc(stream *f, cube &c, const ivec &co, int size, bool &failed)
                 surf.verts = offset;
                 vertinfo *verts = c.ext->verts() + offset;
                 offset += numverts;
-                ivec v[4], n;
+                ivec v[4], n, vo = ivec(co).mask(0xFFF).shl(3);
                 int layerverts = surf.numverts&MAXFACEVERTS, dim = dimension(i), vc = C[dim], vr = R[dim], bias = 0;
                 genfaceverts(c, i, v);
                 bool hasxyz = (vertmask&0x04)!=0, hasuv = (vertmask&0x40)!=0, hasnorm = (vertmask&0x80)!=0;
@@ -498,12 +498,11 @@ void loadc(stream *f, cube &c, const ivec &co, int size, bool &failed)
                     ivec e1, e2, e3;
                     n.cross((e1 = v[1]).sub(v[0]), (e2 = v[2]).sub(v[0]));
                     if(n.iszero()) n.cross(e2, (e3 = v[3]).sub(v[0]));
-                    bias = -n.dot(ivec(v[0]).mul(size).add(ivec(co).mask(0xFFF).shl(3)));
+                    bias = -n.dot(ivec(v[0]).mul(size).add(vo));
                 }
                 else
                 {
                     int vis = layerverts < 4 ? (vertmask&0x02 ? 2 : 1) : 3, order = vertmask&0x01 ? 1 : 0, k = 0;
-                    ivec vo = ivec(co).mask(0xFFF).shl(3);
                     verts[k++].setxyz(v[order].mul(size).add(vo));
                     if(vis&1) verts[k++].setxyz(v[order+1].mul(size).add(vo));
                     verts[k++].setxyz(v[order+2].mul(size).add(vo));
@@ -515,13 +514,13 @@ void loadc(stream *f, cube &c, const ivec &co, int size, bool &failed)
                     {
                         ushort c1 = f->getlil<ushort>(), r1 = f->getlil<ushort>(), c2 = f->getlil<ushort>(), r2 = f->getlil<ushort>();
                         ivec xyz;
-                        xyz[vc] = c1; xyz[vr] = r1; xyz[dim] = -(bias + n[vc]*xyz[vc] + n[vr]*xyz[vr])/n[dim];
+                        xyz[vc] = c1; xyz[vr] = r1; xyz[dim] = n[dim] ? -(bias + n[vc]*xyz[vc] + n[vr]*xyz[vr])/n[dim] : vo[dim];
                         verts[0].setxyz(xyz);
-                        xyz[vc] = c1; xyz[vr] = r2; xyz[dim] = -(bias + n[vc]*xyz[vc] + n[vr]*xyz[vr])/n[dim];
+                        xyz[vc] = c1; xyz[vr] = r2; xyz[dim] = n[dim] ? -(bias + n[vc]*xyz[vc] + n[vr]*xyz[vr])/n[dim] : vo[dim];
                         verts[1].setxyz(xyz);
-                        xyz[vc] = c2; xyz[vr] = r2; xyz[dim] = -(bias + n[vc]*xyz[vc] + n[vr]*xyz[vr])/n[dim];
+                        xyz[vc] = c2; xyz[vr] = r2; xyz[dim] = n[dim] ? -(bias + n[vc]*xyz[vc] + n[vr]*xyz[vr])/n[dim] : vo[dim];
                         verts[2].setxyz(xyz);
-                        xyz[vc] = c2; xyz[vr] = r1; xyz[dim] = -(bias + n[vc]*xyz[vc] + n[vr]*xyz[vr])/n[dim];
+                        xyz[vc] = c2; xyz[vr] = r1; xyz[dim] = n[dim] ? -(bias + n[vc]*xyz[vc] + n[vr]*xyz[vr])/n[dim] : vo[dim];
                         verts[3].setxyz(xyz);
                         hasxyz = false;
                     }
@@ -557,7 +556,7 @@ void loadc(stream *f, cube &c, const ivec &co, int size, bool &failed)
                     {
                         ivec xyz;
                         xyz[vc] = f->getlil<ushort>(); xyz[vr] = f->getlil<ushort>();
-                        xyz[dim] = -(bias + n[vc]*xyz[vc] + n[vr]*xyz[vr])/n[dim];
+                        xyz[dim] = n[dim] ? -(bias + n[vc]*xyz[vc] + n[vr]*xyz[vr])/n[dim] : vo[dim];
                         v.setxyz(xyz);
                     }
                     if(hasuv) { v.u = f->getlil<ushort>(); v.v = f->getlil<ushort>(); }
@@ -812,12 +811,12 @@ void save_config(char *mname)
     if(!h) { conoutf("\frcould not write config to %s", fname); return; }
 
     string title, author;
-    if(*maptitle) filtertext(title, maptitle);
+    if(*maptitle) filterstring(title, maptitle);
     else copystring(title, maptitle);
-    if(*mapauthor) filtertext(author, mapauthor);
+    if(*mapauthor) filterstring(author, mapauthor);
     else copystring(author, mapauthor);
     // config
-    h->printf("// %s by %s (%s)\n// Config generated by %s\n\n", title, author, mapname, versionname);
+    h->printf("// %s by %s (%s)\n// Config generated by %s\n\n", title, author, mapname, VERSION_NAME);
 
     int vars = 0;
     h->printf("// Variables stored in map file, may be uncommented here, or changed from editmode.\n");
@@ -829,14 +828,14 @@ void save_config(char *mname)
         ident &id = *ids[i];
         if(id.flags&IDF_WORLD && !(id.flags&IDF_SERVER)) switch(id.type)
         {
-            case ID_VAR: h->printf((id.flags&IDF_HEX && *id.storage.i >= 0 ? (id.maxval==0xFFFFFF ? "// %s 0x%.6X\n" : "// %s 0x%X\n") : "// %s %d\n"), escapeid(id), *id.storage.i); vars++;break;
+            case ID_VAR: h->printf("// %s %s\n", escapeid(id), intstr(&id)); vars++;break;
             case ID_FVAR: h->printf("// %s %s\n", escapeid(id), floatstr(*id.storage.f)); vars++; break;
             case ID_SVAR: h->printf("// %s %s\n", escapeid(id), escapestring(*id.storage.s)); vars++; break;
             default: break;
         }
     }
     if(vars) h->printf("\n");
-    if(verbose >= 2) conoutf("\fawrote %d variable values", vars);
+    if(verbose >= 2) conoutf("wrote %d variable values", vars);
 
     int aliases = 0;
     loopv(ids)
@@ -854,7 +853,7 @@ void save_config(char *mname)
         }
     }
     if(aliases) h->printf("\n");
-    if(verbose >= 2) conoutf("\fasaved %d aliases", aliases);
+    if(verbose >= 2) conoutf("saved %d aliases", aliases);
 
     // texture slots
     int nummats = sizeof(materialslots)/sizeof(materialslots[0]);
@@ -869,14 +868,14 @@ void save_config(char *mname)
                 break;
         }
     }
-    if(verbose) conoutf("\fasaved %d material slots", nummats);
+    if(verbose) conoutf("saved %d material slots", nummats);
 
     loopv(slots)
     {
         progress(i/float(slots.length()), "saving texture slots...");
         saveslotconfig(h, *slots[i], i);
     }
-    if(verbose) conoutf("\fasaved %d texture slots", slots.length());
+    if(verbose) conoutf("saved %d texture slots", slots.length());
 
     loopv(mapmodels)
     {
@@ -884,7 +883,7 @@ void save_config(char *mname)
         h->printf("mmodel %s\n", escapestring(mapmodels[i].name));
     }
     if(mapmodels.length()) h->printf("\n");
-    if(verbose) conoutf("\fasaved %d mapmodel slots", mapmodels.length());
+    if(verbose) conoutf("saved %d mapmodel slots", mapmodels.length());
 
     loopv(mapsounds)
     {
@@ -897,14 +896,14 @@ void save_config(char *mname)
         h->printf("\n");
     }
     if(mapsounds.length()) h->printf("\n");
-    if(verbose) conoutf("\fasaved %d mapsound slots", mapsounds.length());
+    if(verbose) conoutf("saved %d mapsound slots", mapsounds.length());
 
     delete h;
-    if(verbose) conoutf("\fasaved config %s", fname);
+    if(verbose) conoutf("saved config %s", fname);
 }
-ICOMMAND(0, savemapconfig, "s", (char *mname), save_config(*mname ? mname : mapname));
+ICOMMAND(0, savemapconfig, "s", (char *mname), if(!(identflags&IDF_WORLD)) save_config(*mname ? mname : mapname));
 
-VARF(IDF_PERSIST, mapshotsize, 0, 256, INT_MAX-1, mapshotsize -= mapshotsize%2);
+VARF(IDF_PERSIST, mapshotsize, 0, 512, INT_MAX-1, mapshotsize -= mapshotsize%2);
 
 void save_mapshot(char *mname)
 {
@@ -926,7 +925,7 @@ void save_mapshot(char *mname)
     defformatstring(texname)("%s", mname);
     reloadtexture(texname);
 }
-ICOMMAND(0, savemapshot, "s", (char *mname), save_mapshot(*mname ? mname : mapname));
+ICOMMAND(0, savemapshot, "s", (char *mname), if(!(identflags&IDF_WORLD)) save_mapshot(*mname ? mname : mapname));
 
 #define istempname(n) (!strncmp(n, "temp/", 5) || !strncmp(n, "temp\\", 5))
 
@@ -952,14 +951,16 @@ void save_world(const char *mname, bool nodata, bool forcesave)
 
     savemapprogress = 0;
     progress(0, "saving map..");
-    strncpy(hdr.head, "MAPZ", 4);
+    memcpy(hdr.head, "MAPZ", 4);
     hdr.version = MAPVERSION;
     hdr.headersize = sizeof(mapz);
     hdr.gamever = server::getver(1);
     hdr.numents = 0;
     hdr.numvslots = numvslots;
     hdr.revision++;
-    strncpy(hdr.gameid, server::gameid(), 4);
+    string gameid;
+    copystring(gameid, server::gameid());
+    memcpy(hdr.gameid, gameid, 4);
 
     const vector<extentity *> &ents = entities::getents();
     loopv(ents)
@@ -1009,7 +1010,7 @@ void save_world(const char *mname, bool nodata, bool forcesave)
         }
     });
 
-    if(verbose) conoutf("\fasaved %d variables", vars);
+    if(verbose) conoutf("saved %d variables", vars);
 
     // texture slots
     f->putlil<ushort>(texmru.length());
@@ -1050,15 +1051,15 @@ void save_world(const char *mname, bool nodata, bool forcesave)
 
                 f->putlil<int>(links.length());
                 loopvj(links) f->putlil<int>(links[j]); // aligned index
-                if(verbose >= 2) conoutf("\faentity %s (%d) saved %d links", entities::findname(e.type), i, links.length());
+                if(verbose >= 2) conoutf("entity %s (%d) saved %d links", entities::findname(e.type), i, links.length());
             }
             count++;
         }
     }
-    if(verbose) conoutf("\fasaved %d entities", count);
+    if(verbose) conoutf("saved %d entities", count);
 
     savevslots(f, numvslots);
-    if(verbose) conoutf("\fasaved %d vslots", numvslots);
+    if(verbose) conoutf("saved %d vslots", numvslots);
 
     progress(0, "saving octree...");
     savec(worldroot, ivec(0, 0, 0), hdr.worldsize>>1, f, nodata);
@@ -1077,26 +1078,26 @@ void save_world(const char *mname, bool nodata, bool forcesave)
             }
             f->write(lm.data, lm.bpp*LM_PACKW*LM_PACKH);
         }
-        if(verbose) conoutf("\fasaved %d lightmaps", lightmaps.length());
+        if(verbose) conoutf("saved %d lightmaps", lightmaps.length());
         if(getnumviewcells()>0)
         {
             progress(0, "saving PVS...");
             savepvs(f);
-            if(verbose) conoutf("\fasaved %d PVS view cells", getnumviewcells());
+            if(verbose) conoutf("saved %d PVS view cells", getnumviewcells());
         }
         if(shouldsaveblendmap())
         {
             progress(0, "saving blendmap...");
             saveblendmap(f);
-            if(verbose) conoutf("\fasaved blendmap");
+            if(verbose) conoutf("saved blendmap");
         }
     }
     delete f;
-    conoutf("\fasaved map %s v.%d:%d (r%d) in %.1f secs", mapname, hdr.version, hdr.gamever, hdr.revision, (SDL_GetTicks()-savingstart)/1000.0f);
+    conoutf("saved %s (\fs%s\fS by \fs%s\fS) v.%d:%d (r%d) in %.1f secs", mapname, *maptitle ? maptitle : "Untitled", *mapauthor ? mapauthor : "Unknown", hdr.version, hdr.gamever, hdr.revision, (SDL_GetTicks()-savingstart)/1000.0f);
     if(istempname(mapname)) setnames(&mapname[5], MAP_MAPZ);
 }
 
-ICOMMAND(0, savemap, "s", (char *mname), save_world(*mname ? (istempname(mname) ? &mname[5] : mname) : (istempname(mapname) ? &mapname[5] : mapname)));
+ICOMMAND(0, savemap, "s", (char *mname), if(!(identflags&IDF_WORLD)) save_world(*mname ? (istempname(mname) ? &mname[5] : mname) : (istempname(mapname) ? &mapname[5] : mapname)));
 
 static uint mapcrc = 0;
 
@@ -1133,6 +1134,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
             stream *f = opengzfile(mapfile, "rb");
             if(!f)
             {
+                conoutf("\frnot found: %s", mapfile);
                 maskpackagedirs(mask);
                 continue;
             }
@@ -1153,11 +1155,11 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
             clearworldvars();
             if(memcmp(newhdr.head, "MAPZ", 4) == 0 || memcmp(newhdr.head, "BFGZ", 4) == 0)
             {
-                // this check removed below due to breakage: (size_t)newhdr.headersize > sizeof(chdr) || f->read(&chdr.worldsize, newhdr.headersize-sizeof(binary))!=newhdr.headersize-(int)sizeof(binary)
+                // this check removed below due to breakage: size_t(newhdr.headersize) > sizeof(chdr) || f->read(&chdr.worldsize, newhdr.headersize-sizeof(binary))!=size_t(newhdr.headersize)-sizeof(binary)
                 #define MAPZCOMPAT(ver) \
                     mapzcompat##ver chdr; \
                     memcpy(&chdr, &newhdr, sizeof(binary)); \
-                    if(f->read(&chdr.worldsize, sizeof(chdr)-sizeof(binary))!=int(sizeof(chdr)-sizeof(binary))) \
+                    if(f->read(&chdr.worldsize, sizeof(chdr)-sizeof(binary))!=sizeof(chdr)-sizeof(binary)) \
                     { \
                         conoutf("\frerror loading %s: malformatted mapz v%d[%d] header", mapname, newhdr.version, ver); \
                         delete f; \
@@ -1210,7 +1212,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                 }
                 else
                 {
-                    if((size_t)newhdr.headersize > sizeof(newhdr) || f->read(&newhdr.worldsize, newhdr.headersize-sizeof(binary))!=newhdr.headersize-(int)sizeof(binary))
+                    if(size_t(newhdr.headersize) > sizeof(newhdr) || f->read(&newhdr.worldsize, newhdr.headersize-sizeof(binary))!=size_t(newhdr.headersize)-sizeof(binary))
                     {
                         conoutf("\frerror loading %s: malformatted mapz v%d header", mapname, newhdr.version);
                         delete f;
@@ -1223,7 +1225,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
 
                 if(newhdr.version > MAPVERSION)
                 {
-                    conoutf("\frerror loading %s: requires a newer version of %s", mapname, versionname);
+                    conoutf("\frerror loading %s: requires a newer version of %s", mapname, VERSION_NAME);
                     delete f;
                     maploading = 0;
                     maskpackagedirs(mask);
@@ -1236,7 +1238,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                 maptype = MAP_MAPZ;
 
                 if(hdr.version <= 24) copystring(hdr.gameid, "bfa", 4); // all previous maps were bfa-fps
-                if(verbose) conoutf("\faloading v%d map from %s game v%d", hdr.version, hdr.gameid, hdr.gamever);
+                if(verbose) conoutf("loading v%d map from %s game v%d", hdr.version, hdr.gameid, hdr.gamever);
 
                 if(hdr.version >= 25 || (hdr.version == 24 && hdr.gamever >= 44))
                 {
@@ -1330,7 +1332,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                         }
                     }
                     identflags &= ~IDF_WORLD;
-                    if(verbose) conoutf("\faloaded %d variables", vars);
+                    if(verbose) conoutf("loaded %d variables", vars);
                 }
                 sanevars();
 
@@ -1349,7 +1351,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                 #define OCTACOMPAT(ver) \
                     octacompat##ver chdr; \
                     memcpy(&chdr, &ohdr, sizeof(binary)); \
-                    if(f->read(&chdr.worldsize, sizeof(chdr)-sizeof(binary))!=int(sizeof(chdr)-sizeof(binary))) \
+                    if(f->read(&chdr.worldsize, sizeof(chdr)-sizeof(binary))!=sizeof(chdr)-sizeof(binary)) \
                     { \
                         conoutf("\frerror loading %s: malformatted octa v%d[%d] header", mapname, ver, ohdr.version); \
                         delete f; \
@@ -1431,7 +1433,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                 progress(0, "please wait..");
                 maptype = MAP_OCTA;
 
-                strncpy(hdr.head, ohdr.head, 4);
+                memcpy(hdr.head, ohdr.head, 4);
                 hdr.gamever = 0; // sauer has no gamever
                 hdr.worldsize = ohdr.worldsize;
                 if(hdr.worldsize > 1<<18) hdr.worldsize = 1<<18;
@@ -1494,14 +1496,14 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                     f->read(gameid, len+1);
                 }
                 else copystring(gameid, "fps");
-                strncpy(hdr.gameid, gameid, 4);
+                memcpy(hdr.gameid, gameid, 4);
 
                 if(!server::canload(hdr.gameid))
                 {
                     if(verbose) conoutf("\frWARNING: loading OCTA v%d map from %s game, ignoring game specific data", hdr.version, hdr.gameid);
                     samegame = false;
                 }
-                else if(verbose) conoutf("\faloading OCTA v%d map from %s game", hdr.version, hdr.gameid);
+                else if(verbose) conoutf("loading OCTA v%d map from %s game", hdr.version, hdr.gameid);
 
                 if(hdr.version>=16)
                 {
@@ -1601,7 +1603,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                     int links = f->getlil<int>();
                     e.links.add(0, links);
                     loopk(links) e.links[k] = f->getlil<int>();
-                    if(verbose >= 2) conoutf("\faentity %s (%d) loaded %d link(s)", entities::findname(e.type), i, links);
+                    if(verbose >= 2) conoutf("entity %s (%d) loaded %d link(s)", entities::findname(e.type), i, links);
                 }
 
                 if(maptype == MAP_OCTA && e.type == ET_PARTICLES && e.attrs[0] >= 11)
@@ -1646,7 +1648,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                 if(verbose && !insideworld(e.o) && e.type != ET_LIGHT && e.type != ET_LIGHTFX && e.type != ET_SUNLIGHT)
                     conoutf("\frWARNING: ent outside of world: enttype[%s] index %d (%f, %f, %f)", entities::findname(e.type), i, e.o.x, e.o.y, e.o.z);
             }
-            if(verbose) conoutf("\faloaded %d entities", hdr.numents);
+            if(verbose) conoutf("loaded %d entities", hdr.numents);
             if(maptype == MAP_OCTA && sunlight)
             {
                 extentity &e = *ents.add(entities::newent());
@@ -1700,7 +1702,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                 if(hdr.numpvs > 0) loadpvs(f);
                 if(hdr.blendmap) loadblendmap(f);
 
-                if(verbose) conoutf("\faloaded %d lightmaps", hdr.lightmaps);
+                if(verbose) conoutf("loaded %d lightmaps", hdr.lightmaps);
             }
 
             progress(0, "initialising entities...");
@@ -1731,7 +1733,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                     }
                 }
             }
-            entities::initents(f, maptype, hdr.version, hdr.gameid, hdr.gamever);
+            entities::initents(maptype, hdr.version, hdr.gameid, hdr.gamever);
 
             progress(0, "initialising config...");
             mapcrc = f->getcrc();
@@ -1749,7 +1751,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
             preloadusedmapmodels(true);
 
             delete f;
-            conoutf("\faloaded map %s v.%d:%d (r%d) in %.1f secs", mapname, hdr.version, hdr.gamever, hdr.revision, (SDL_GetTicks()-loadingstart)/1000.0f);
+            conoutf("loaded %s (\fs%s\fS by \fs%s\fS) v.%d:%d(r%d) [%.1fs]", mapname, *maptitle ? maptitle : "Untitled", *mapauthor ? mapauthor : "Unknown", hdr.version, hdr.gamever, hdr.revision, (SDL_GetTicks()-loadingstart)/1000.0f);
 
             progress(0, "checking world...");
             if((maptype == MAP_OCTA && hdr.version <= 25) || (maptype == MAP_MAPZ && hdr.version <= 26))
@@ -1878,7 +1880,7 @@ void writeobj(char *name)
     delete f;
 }
 
-COMMAND(0, writeobj, "s");
+ICOMMAND(0, writeobj, "s", (char *s), if(!(identflags&IDF_WORLD)) writeobj(s));
 
 int getworldsize() { return hdr.worldsize; }
 int getmapversion() { return hdr.version; }
