@@ -1,49 +1,5 @@
-struct aistyles
-{
-    int type,           weap,           health;
-    float   xradius,    yradius,    height,     weight,     speed,      scale;
-    bool    canmove,    canstrafe,  canjump,    cancrouch,  useweap,    living,     hitbox;
-    const char  *name,      *playermodel[4];
-};
-
-enum { AI_NONE = 0, AI_BOT, AI_TURRET, AI_GRUNT, AI_DRONE, AI_MAX, AI_START = AI_TURRET, AI_TOTAL = AI_MAX-AI_START };
-#ifdef GAMESERVER
-aistyles aistyle[] = {
-    {
-        AI_NONE,         -1,             0,
-            3,          3,          14,         200,        50,         1,
-            true,       true,       true,       true,       true,       true,       true,
-                "player",   { "actors/player/male/hwep",      "actors/player/male",     "actors/player/male/body",      "actors/player/male/headless" }
-    },
-    {
-        AI_BOT,         -1,             0,
-            3,          3,          14,         200,        50,         1,
-            true,       true,       true,       true,       true,       true,       true,
-                "bot",      { "actors/player/male/hwep",      "actors/player/male",     "actors/player/male/body",      "actors/player/male/headless" }
-    },
-    {
-        AI_TURRET,      W_SMG,       100,
-            4.75,       4.75,       8.75,       150,        1,          1,
-            false,      false,      false,      false,      false,      false,      false,
-                "turret",   { "actors/player/male/hwep",      "actors/turret",          "actors/player/male/body",      "actors/turret" }
-    },
-    {
-        AI_GRUNT,       W_PISTOL,   50,
-            3,          3,          14,         200,        50,         1,
-            true,       true,       true,       true,       true,       true,       true,
-                "grunt",   { "actors/player/male/hwep",      "actors/player/male",      "actors/player/male/body",      "actors/player/male/headless" }
-    },
-    {
-        AI_DRONE,       W_MELEE,     50,
-            3,          3,          14,         150,        40,         1,
-            true,       true,       true,       true,       true,       true,       true,
-                "drone",    { "actors/player/male/hwep",      "actors/drone",           "actors/player/male/body",      "actors/drone" }
-    },
-};
-#endif
-#ifndef GAMESERVER
 struct gameent;
-extern aistyles aistyle[];
+extern actors actor[];
 
 namespace ai
 {
@@ -54,7 +10,6 @@ namespace ai
     const float MINWPDIST       = 4.f;     // is on top of
     const float CLOSEDIST       = 32.f;    // is close
     const float RETRYDIST       = 64.f;    // is close when retrying
-    const float JETDIST         = 128.f;   // close when jetting
     const float FARDIST         = 256.f;   // too far to remap close
     const float JUMPMIN         = 2.f;     // decides to jump
     const float JUMPMAX         = 32.f;    // max jump
@@ -102,9 +57,10 @@ namespace ai
     extern vector<oldwaypoint> oldwaypoints;
 
     extern int showwaypoints, dropwaypoints;
-    extern int closestwaypoint(const vec &pos, float mindist, bool links);
+    extern int closestwaypoint(const vec &pos, float mindist = CLOSEDIST, bool links = true);
     extern void findwaypointswithin(const vec &pos, float mindist, float maxdist, vector<int> &results);
 	extern void inferwaypoints(gameent *d, const vec &o, const vec &v, float mindist = ai::CLOSEDIST);
+	extern void delwaypoint(int n = -1);
 
     struct avoidset
     {
@@ -208,21 +164,31 @@ namespace ai
         AI_T_MAX
     };
 
+    enum
+    {
+        AI_A_NORMAL = 0,
+        AI_A_IDLE,
+        AI_A_LOCKON,
+        AI_A_PROTECT,
+        AI_A_HASTE,
+        AI_A_MAX
+    };
+
     struct interest
     {
-        int state, node, target, targtype;
+        int state, node, target, targtype, acttype;
         float score, tolerance;
         bool team;
-        interest() : state(-1), node(-1), target(-1), targtype(-1), score(0.f), tolerance(1.f), team(false) {}
+        interest() : state(-1), node(-1), target(-1), targtype(-1), acttype(AI_A_NORMAL), score(0.f), tolerance(1.f), team(false) {}
         ~interest() {}
     };
 
     struct aistate
     {
-        int type, millis, targtype, target, idle;
+        int type, millis, targtype, target, acttype, owner;
         bool override;
 
-        aistate(int m, int t, int r = -1, int v = -1) : type(t), millis(m), targtype(r), target(v)
+        aistate(int m, int t, int r = -1, int v = -1, int a = AI_A_NORMAL, int o = -1) : type(t), millis(m), targtype(r), target(v), acttype(a), owner(o)
         {
             reset();
         }
@@ -230,7 +196,7 @@ namespace ai
 
         void reset()
         {
-            idle = 0;
+            acttype = AI_A_NORMAL;
             override = false;
         }
     };
@@ -241,10 +207,10 @@ namespace ai
     {
         vector<aistate> state;
         vector<int> route;
-        vec target, spot;
-        int weappref, enemy, enemyseen, enemymillis, prevnodes[NUMPREVNODES], targnode, targlast, targtime, targseq,
+        vec target, spot, views, aimrnd;
+        int enemy, enemyseen, enemymillis, prevnodes[NUMPREVNODES], targnode, targlast, targtime, targseq,
             lastrun, lastaction, lastcheck, jumpseed, jumprand, blocktime, blockseq, lastaimrnd, lastmelee, lastturn;
-        float targyaw, targpitch, views[3], aimrnd[3];
+        float targyaw, targpitch;
         bool dontmove, tryreset;
 
         aiinfo()
@@ -261,7 +227,7 @@ namespace ai
             lastaction = lastcheck = enemyseen = enemymillis = blocktime = blockseq = targtime = targseq = lastaimrnd = lastmelee = lastturn = 0;
             lastrun = jumpseed = lastmillis;
             jumprand = lastmillis+5000;
-            weappref = targnode = targlast = enemy = -1;
+            targnode = targlast = enemy = -1;
             targyaw = targpitch = 0;
         }
 
@@ -272,11 +238,14 @@ namespace ai
             lastcheck = 0;
         }
 
-        void reset(bool tryit = false)
+        void reset(bool tryit = false, bool keepowner = true)
         {
             clear(tryit);
+            vector<aistate> keep;
+            if(keepowner) loopv(state) if(state[i].owner >= 0) keep.add(state[i]);
             state.setsize(0);
             addstate(AI_S_WAIT);
+            if(keepowner) loopv(keep) state.add(keep[i]);
             tryreset = false;
         }
 
@@ -296,9 +265,9 @@ namespace ai
             }
         }
 
-        aistate &addstate(int t, int r = -1, int v = -1)
+        aistate &addstate(int t, int r = -1, int v = -1, int a = AI_A_NORMAL, int o = -1)
         {
-            return state.add(aistate(lastmillis, t, r, v));
+            return state.add(aistate(lastmillis, t, r, v, a, o));
         }
 
         void removestate(int index = -1)
@@ -314,16 +283,18 @@ namespace ai
             return state.last();
         }
 
-        aistate &switchstate(aistate &b, int t, int r = -1, int v = -1)
+        aistate &switchstate(aistate &b, int t, int r = -1, int v = -1, int a = AI_A_NORMAL, int o = -1)
         {
-            if((b.type == t && b.targtype == r) || (b.type == AI_S_INTEREST && b.targtype == AI_T_NODE))
+            if(((b.type == t && b.targtype == r) || (b.type == AI_S_INTEREST && b.targtype == AI_T_NODE)) && b.owner == o)
             {
                 b.millis = lastmillis;
                 b.target = v;
+                b.acttype = a;
+                b.owner = o;
                 b.reset();
                 return b;
             }
-            return addstate(t, r, v);
+            return addstate(t, r, v, a);
         }
     };
 
@@ -340,9 +311,9 @@ namespace ai
     extern bool targetable(gameent *d, gameent *e, bool solid = false);
     extern bool cansee(gameent *d, vec &x, vec &y, bool force = false, vec &targ = aitarget);
     extern bool altfire(gameent *d, gameent *e);
-    extern int owner(gameent *d);
+    extern int weappref(gameent *d);
 
-    extern void init(gameent *d, int at, int et, int on, int sk, int bn, char *name, int tm, int cl, int md, const char *vn = "");
+    extern void init(gameent *d, int at, int et, int on, int sk, int bn, char *name, int tm, int cl, int md, const char *vn, vector<int> &lweaps);
 
     extern bool badhealth(gameent *d);
     extern int checkothers(vector<int> &targets, gameent *d = NULL, int state = -1, int targtype = -1, int target = -1, bool teams = false, int *members = NULL);
@@ -351,8 +322,8 @@ namespace ai
     extern bool randomnode(gameent *d, aistate &b, const vec &pos, float guard = ALERTMIN, float wander = ALERTMAX);
     extern bool randomnode(gameent *d, aistate &b, float guard = ALERTMIN, float wander = ALERTMAX);
     extern bool violence(gameent *d, aistate &b, gameent *e, int pursue = 0);
-    extern bool patrol(gameent *d, aistate &b, const vec &pos, float guard = ALERTMIN, float wander = ALERTMAX, int walk = 1, bool retry = false);
-    extern bool defense(gameent *d, aistate &b, const vec &pos, float guard = ALERTMIN, float wander = ALERTMAX, int walk = 0);
+    extern bool patrol(gameent *d, aistate &b, const vec &pos, float guard = CLOSEDIST, float wander = FARDIST, int walk = 1, bool retry = false);
+    extern bool defense(gameent *d, aistate &b, const vec &pos, float guard = CLOSEDIST, float wander = FARDIST, int walk = 0);
 
     extern void respawned(gameent *d, bool local, int ent = -1);
     extern void damaged(gameent *d, gameent *e, int weap, int flags, int damage);
@@ -363,5 +334,6 @@ namespace ai
     extern void think(gameent *d, bool run);
     extern void render();
     extern void preload();
+
+    extern void scanchat(gameent *d, gameent *t, int flags, const char *text);
 };
-#endif
