@@ -12,9 +12,10 @@ struct bomberstate
 {
     struct flag
     {
-        vec droploc, inertia, spawnloc;
-        int team, ent, droptime, taketime;
+        vec droploc, droppos, inertia, spawnloc;
+        int team, yaw, pitch, droptime, taketime;
         bool enabled;
+        float distance;
 #ifdef GAMESERVER
         int owner, lastowner;
         vector<int> votes;
@@ -22,31 +23,29 @@ struct bomberstate
         gameent *owner, *lastowner;
         projent *proj;
         int displaytime, pickuptime, movetime, inittime, viewtime, rendertime, interptime;
-        vec viewpos, renderpos, interppos;
+        vec viewpos, renderpos, interppos, render, above;
+        entitylight light, baselight;
 #endif
 
-        flag()
-#ifndef GAMESERVER
-          : ent(-1)
-#endif
-        { reset(); }
+        flag() { reset(); }
 
         void reset()
         {
             inertia = vec(0, 0, 0);
-            droploc = spawnloc = vec(-1, -1, -1);
+            droploc = droppos = spawnloc = vec(-1, -1, -1);
 #ifdef GAMESERVER
             owner = lastowner = -1;
             votes.shrink(0);
 #else
             owner = lastowner = NULL;
             proj = NULL;
-            displaytime = pickuptime = movetime = inittime = viewtime = rendertime = 0;
+            displaytime = pickuptime = movetime = inittime = viewtime = rendertime = interptime = 0;
             viewpos = renderpos = vec(-1, -1, -1);
 #endif
             team = T_NEUTRAL;
-            taketime = droptime = 0;
+            yaw = pitch = taketime = droptime = 0;
             enabled = false;
+            distance = 0;
         }
 
 #ifndef GAMESERVER
@@ -61,8 +60,7 @@ struct bomberstate
                         if(totalmillis != rendertime)
                         {
                             float yaw = 360-((lastmillis/2)%360), off = (lastmillis%1000)/500.f;
-                            vecfromyawpitch(yaw, 0, 1, 0, renderpos);
-                            renderpos.normalize().mul(owner->radius+4).add(owner->center());
+                            renderpos = vec(yaw*RAD, 0.f).mul(owner->radius+4).add(owner->center());
                             renderpos.z += owner->height*(off > 1 ?  2-off : off);
                             rendertime = totalmillis;
                         }
@@ -72,7 +70,7 @@ struct bomberstate
                 }
                 if(droptime) return proj ? proj->o : droploc;
             }
-            return spawnloc;
+            return above;
         }
 
         vec &pos(bool view = false, bool render = false)
@@ -93,6 +91,12 @@ struct bomberstate
             return position(render);
         }
 #endif
+        bool travel(const vec &o, float dist)
+        {
+            if(!droptime) return false;
+            if(distance > 0 && distance >= dist) return true;
+            return droppos.dist(o) >= dist;
+        }
     };
     vector<flag> flags;
 
@@ -101,13 +105,20 @@ struct bomberstate
         flags.shrink(0);
     }
 
-    void addaffinity(const vec &o, int team, int ent)
+    void addaffinity(const vec &o, int team, int yaw, int pitch)
     {
         flag &f = flags.add();
         f.reset();
         f.team = team;
         f.spawnloc = o;
-        f.ent = ent;
+        f.yaw = yaw;
+        f.pitch = pitch;
+#ifndef GAMESERVER
+        f.render = f.above = o;
+        f.render.z += 2;
+        physics::droptofloor(f.render);
+        if(f.render.z >= f.above.z-1) f.above.z += f.render.z-(f.above.z-1);
+#endif
     }
 
 #ifndef GAMESERVER
@@ -155,7 +166,8 @@ struct bomberstate
 #else
         f.pickuptime = f.movetime = 0;
         if(!f.inittime) f.inittime = t;
-        (f.lastowner = owner)->addicon(eventicon::AFFINITY, t, game::eventiconfade, f.team);
+        owner->addicon(eventicon::AFFINITY, t, game::eventiconfade, f.team);
+        f.lastowner = owner;
         destroy(i);
 #endif
     }
@@ -166,16 +178,16 @@ struct bomberstate
 #ifndef GAMESERVER
         interp(i, t);
 #endif
-        f.droploc = o;
+        f.droploc = f.droppos = o;
         f.inertia = p;
         f.droptime = t;
         f.taketime = 0;
+        f.distance = 0;
 #ifdef GAMESERVER
         f.owner = -1;
         f.votes.shrink(0);
 #else
-        f.pickuptime = 0;
-        f.movetime = t;
+        f.pickuptime = f.movetime = 0;
         if(!f.inittime) f.inittime = t;
         f.owner = NULL;
         destroy(i);
@@ -183,12 +195,11 @@ struct bomberstate
 #endif
     }
 
-    void returnaffinity(int i, int t, bool enabled, bool isreset = false)
+    void returnaffinity(int i, int t, bool enabled)
     {
         flag &f = flags[i];
 #ifndef GAMESERVER
         interp(i, t);
-        if(!isreset) f.interptime = 0;
 #endif
         f.droptime = f.taketime = 0;
         f.enabled = enabled;
@@ -214,12 +225,12 @@ namespace bomber
     extern void dropaffinity(gameent *d, int i, const vec &droploc, const vec &inertia, int target = -1);
     extern void scoreaffinity(gameent *d, int relay, int goal, int score);
     extern void takeaffinity(gameent *d, int i);
-    extern void resetaffinity(int i, int value);
+    extern void resetaffinity(int i, int enabled);
     extern void reset();
     extern void setup();
     extern void setscore(int team, int total);
-    extern void checkaffinity(dynent *e);
-    extern void killed(gameent *d, gameent *actor);
+    extern void update();
+    extern void killed(gameent *d, gameent *v);
     extern void drawnotices(int w, int h, int &tx, int &ty, float blend);
     extern void drawblips(int w, int h, float blend);
     extern int drawinventory(int x, int y, int s, int m, float blend);
@@ -227,6 +238,7 @@ namespace bomber
     extern void render();
     extern void adddynlights();
     extern int aiowner(gameent *d);
+    extern bool aihomerun(gameent *d, ai::aistate &b);
     extern void aifind(gameent *d, ai::aistate &b, vector<ai::interest> &interests);
     extern bool aicheck(gameent *d, ai::aistate &b);
     extern bool aidefense(gameent *d, ai::aistate &b);
