@@ -5,7 +5,7 @@ struct bomberservmode : bomberstate, servmode
 
     bomberservmode() : hasflaginfo(false), bombertime(-1) {}
 
-    void reset(bool empty)
+    void reset()
     {
         bomberstate::reset();
         hasflaginfo = false;
@@ -14,25 +14,25 @@ struct bomberservmode : bomberstate, servmode
 
     void dropaffinity(clientinfo *ci, const vec &o, const vec &inertia = vec(0, 0, 0), int target = -1)
     {
-        if(!hasflaginfo || ci->state.aitype >= AI_START) return;
-        vec n = inertia.iszero() ? vec(0, 0, G(bomberspeed)/10.f) : inertia, v = o; v.z += 1;
+        if(!canplay(hasflaginfo) || ci->state.actortype >= A_ENEMY) return;
+        vec n = inertia.iszero() ? vec(0, 0, G(bomberspeed)/10.f) : inertia;
         loopv(flags) if(flags[i].owner == ci->clientnum)
         {
-            ivec p(vec(v).mul(DMF)), q(vec(n).mul(DMF));
+            ivec p(vec(o).mul(DMF)), q(vec(n).mul(DMF));
             sendf(-1, 1, "ri3i7", N_DROPAFFIN, ci->clientnum, target, i, p.x, p.y, p.z, q.x, q.y, q.z);
-            bomberstate::dropaffinity(i, v, n, gamemillis);
+            bomberstate::dropaffinity(i, o, n, gamemillis);
         }
     }
 
     void leavegame(clientinfo *ci, bool disconnecting = false)
     {
-        if(!hasflaginfo) return;
-        dropaffinity(ci, ci->state.o, vec(ci->state.vel).add(ci->state.falling));
+        if(!canplay(hasflaginfo)) return;
+        dropaffinity(ci, ci->state.feetpos(G(bomberdropheight)), vec(ci->state.vel).add(ci->state.falling));
     }
 
-    void dodamage(clientinfo *target, clientinfo *actor, int &damage, int &hurt, int &weap, int &flags, int &material, const ivec &hitpush)
+    void dodamage(clientinfo *m, clientinfo *v, int &damage, int &hurt, int &weap, int &flags, int &material, const ivec &hitpush, const ivec &hitvel, float dist)
     {
-        //if(weaptype[weap].melee) dropaffinity(target, target->state.o);
+        //if(weaptype[weap].melee) dropaffinity(m, m->state.o);
     }
 
     void spawned(clientinfo *ci)
@@ -54,13 +54,13 @@ struct bomberservmode : bomberstate, servmode
         loopvj(sents) if(enttype[sents[j].type].usetype == EU_ITEM) setspawn(j, hasitem(j), true, true);
     }
 
-    void died(clientinfo *ci, clientinfo *actor)
+    void died(clientinfo *ci, clientinfo *v)
     {
-        if(!hasflaginfo) return;
-        dropaffinity(ci, ci->state.o, vec(ci->state.vel).add(ci->state.falling));
-        if(actor && m_gsp1(gamemode, mutators) && (!m_team(gamemode, mutators) || ci->team != actor->team))
+        if(!canplay(hasflaginfo)) return;
+        dropaffinity(ci, ci->state.feetpos(G(bomberdropheight)), vec(ci->state.vel).add(ci->state.falling));
+        if(v && m_gsp1(gamemode, mutators) && (!m_team(gamemode, mutators) || ci->team != v->team))
         {
-            loopv(flags) if(isbomberaffinity(flags[i]) && flags[i].owner == actor->clientnum)
+            loopv(flags) if(isbomberaffinity(flags[i]) && flags[i].owner == v->clientnum)
                 flags[i].taketime = gamemillis;
         }
     }
@@ -74,21 +74,22 @@ struct bomberservmode : bomberstate, servmode
 
     void scorebomb(clientinfo *ci, int relay, int goal)
     {
+        if(!canplay(hasflaginfo)) return;
         flag g = flags[goal];
         if(!g.enabled) return;
-        int score = 0;
+        int total = 0;
         if(g.team != ci->team)
         {
-            givepoints(ci, G(bomberpoints));
-            score = addscore(ci->team, 1);
+            if(!m_nopoints(gamemode, mutators)) givepoints(ci, G(bomberpoints));
+            total = addscore(ci->team, 1);
         }
         else
         {
-            givepoints(ci, -G(bomberpenalty));
-            score = addscore(ci->team, -1);
+            if(!m_nopoints(gamemode, mutators)) givepoints(ci, -G(bomberpenalty));
+            total = addscore(ci->team, -1);
         }
         bomberstate::returnaffinity(relay, gamemillis, false);
-        sendf(-1, 1, "ri5", N_SCOREAFFIN, ci->clientnum, relay, goal, score);
+        sendf(-1, 1, "ri5", N_SCOREAFFIN, ci->clientnum, relay, goal, total);
         mutate(smuts, mut->scoreaffinity(ci, g.team != ci->team));
         bombertime = m_duke(gamemode, mutators) ? -1 : gamemillis+G(bomberdelay);
         loopvj(flags) if(flags[j].enabled)
@@ -96,87 +97,108 @@ struct bomberservmode : bomberstate, servmode
             bomberstate::returnaffinity(j, gamemillis, false);
             sendf(-1, 1, "ri3", N_RESETAFFIN, j, 0);
         }
-        if(!m_balance(gamemode, mutators) && G(bomberlimit) && score >= G(bomberlimit))
+        if(!m_balance(gamemode, mutators, teamspawns) && G(bomberlimit) && total >= G(bomberlimit))
         {
             ancmsgft(-1, S_V_NOTIFY, CON_EVENT, "\fyscore limit has been reached");
             startintermission();
         }
-    }
-
-    void scoreaffinity(clientinfo *ci, int relay, int goal)
-    {
-        if(!m_team(gamemode, mutators) || m_gsp2(gamemode, mutators) || !flags.inrange(relay) || !flags.inrange(goal) || flags[relay].lastowner != ci->clientnum || !flags[relay].droptime) return;
-        scorebomb(ci, relay, goal);
+        if(m_gsp3(gamemode, mutators) && G(bomberattackwinner) && !m_multi(gamemode, mutators))
+        {
+            int numt = numteams(gamemode, mutators);
+            if(curbalance == numt-1)
+            {
+                bool found = false;
+                loopi(numt)
+                {
+                    int t = i+T_FIRST, s = teamscore(t).total;
+                    if(t != ci->team && s >= total)
+                    {
+                        found = true;
+                        break;
+                    }
+                }
+                if(!found)
+                {
+                    ancmsgft(-1, S_V_NOTIFY, CON_EVENT, "\fybest score has been reached");
+                    startintermission();
+                }
+            }
+        }
     }
 
     void moved(clientinfo *ci, const vec &oldpos, const vec &newpos)
     {
-        if(!hasflaginfo || ci->state.aitype >= AI_START) return;
+        if(!canplay(hasflaginfo) || ci->state.actortype >= A_ENEMY) return;
         if(G(bomberthreshold) > 0 && oldpos.dist(newpos) >= G(bomberthreshold))
             dropaffinity(ci, oldpos, vec(ci->state.vel).add(ci->state.falling));
-        if(!m_team(gamemode, mutators) || m_gsp1(gamemode, mutators)) return;
-        loopv(flags) if(isbomberaffinity(flags[i]) && flags[i].owner == ci->clientnum)
+        if(m_gsp1(gamemode, mutators) || (G(bomberbasketonly) && m_gsp2(gamemode, mutators))) return;
+        loopv(flags) if(isbomberaffinity(flags[i]) && flags[i].owner == ci->clientnum) loopvk(flags)
+            if(isbombertarg(flags[k], ci->team) && newpos.dist(flags[k].spawnloc) <= enttype[AFFINITY].radius/2) scorebomb(ci, i, k);
+    }
+
+    void returnaffinity(int i, bool enabled)
+    {
+        flag &f = flags[i];
+        bool wasenabled = isbomberaffinity(f) && f.enabled;
+        bomberstate::returnaffinity(i, gamemillis, enabled);
+        sendf(-1, 1, "ri3", N_RESETAFFIN, i, f.enabled ? 1 : 0);
+        if(wasenabled && !f.enabled)
         {
-            loopvk(flags)
-            {
-                flag &f = flags[k];
-                if(isbombertarg(f, ci->team) && newpos.dist(f.spawnloc) <= enttype[AFFINITY].radius/2) scorebomb(ci, i, k);
-            }
+            loopvj(flags) if(i != j && flags[j].enabled) returnaffinity(j, false);
+            if(bombertime >= 0) bombertime = gamemillis+G(bomberdelay);
         }
     }
 
     void takeaffinity(clientinfo *ci, int i)
     {
-        if(!hasflaginfo || !flags.inrange(i) || ci->state.state!=CS_ALIVE || ci->state.aitype >= AI_START) return;
+        if(!canplay(hasflaginfo) || !flags.inrange(i) || ci->state.state!=CS_ALIVE || ci->state.actortype >= A_ENEMY) return;
         flag &f = flags[i];
         if(!isbomberaffinity(f) || f.owner >= 0 || !f.enabled) return;
-        if(f.lastowner == ci->clientnum && f.droptime && (G(bomberpickupdelay) < 0 || lastmillis-f.droptime <= G(bomberpickupdelay))) return;
-        bomberstate::takeaffinity(i, ci->clientnum, gamemillis);
-        if(!m_nopoints(gamemode, mutators) && (!f.droptime || f.lastowner != ci->clientnum)) givepoints(ci, G(bomberpickuppoints));
-        sendf(-1, 1, "ri3", N_TAKEAFFIN, ci->clientnum, i);
-    }
-
-    void returnaffinity(int i, int v) // 0 = disable, 1 = return and wait, 2 = return instantly
-    {
-        bomberstate::returnaffinity(i, gamemillis, v == 2);
-        sendf(-1, 1, "ri3", N_RESETAFFIN, i, flags[i].enabled ? v : 0);
-        if(v && !flags[i].enabled)
+        if(f.lastowner == ci->clientnum && f.droptime && gamemillis-f.droptime <= G(bomberpickupdelay)) return;
+        if(m_gsp3(gamemode, mutators) && ci->team == T_ALPHA && G(bomberattackreset))
         {
-            loopvj(flags) if(flags[j].enabled) returnaffinity(j, 0);
-            if(bombertime >= 0) bombertime = gamemillis+G(bomberdelay);
+            if(!f.droptime) return;
+            if(!m_nopoints(gamemode, mutators) && (!f.droptime || f.lastowner != ci->clientnum)) givepoints(ci, G(bomberpickuppoints));
+            returnaffinity(i, false);
+        }
+        else
+        {
+            bomberstate::takeaffinity(i, ci->clientnum, gamemillis);
+            if(!m_nopoints(gamemode, mutators) && (!f.droptime || f.lastowner != ci->clientnum)) givepoints(ci, G(bomberpickuppoints));
+            sendf(-1, 1, "ri3", N_TAKEAFFIN, ci->clientnum, i);
         }
     }
 
-    void resetaffinity(clientinfo *ci, int i, bool force = false)
+    void resetaffinity(clientinfo *ci, int i)
     {
-        if(!hasflaginfo || !flags.inrange(i) || ci->state.ownernum >= 0) return;
+        if(!canplay(hasflaginfo) || !flags.inrange(i) || ci->state.ownernum >= 0) return;
         flag &f = flags[i];
         if(!isbomberaffinity(f) || f.owner >= 0 || !f.droptime || f.votes.find(ci->clientnum) >= 0 || !f.enabled) return;
         f.votes.add(ci->clientnum);
-        if(f.votes.length() >= numclients()/2) returnaffinity(i, 2);
+        if(f.votes.length() >= int(floorf(numclients()*0.5f))) returnaffinity(i, false);
     }
 
     void layout()
     {
-        if(!hasflaginfo) return;
+        if(!canplay(hasflaginfo)) return;
         bombertime = -1;
-        loopv(flags) if(flags[i].owner >= 0 || flags[i].droptime) returnaffinity(i, 0);
+        loopv(flags) if(flags[i].owner >= 0 || flags[i].droptime) returnaffinity(i, false);
         bombertime = gamemillis+G(bomberdelay);
     }
 
     void update()
     {
-        if(!hasflaginfo || bombertime < 0) return;
+        if(!canplay(hasflaginfo) || bombertime < 0) return;
         if(bombertime)
         {
             if(gamemillis < bombertime) return;
             int hasaffinity = 0;
             vector<int> candidates[T_MAX];
             loopv(flags) candidates[flags[i].team].add(i);
-            int wants = m_gsp1(gamemode, mutators) ? 1 : teamcount(gamemode, mutators);
-            loopi(wants)
+            int wants = m_gsp1(gamemode, mutators) ? 1 : (m_gsp3(gamemode, mutators) ? 2 : teamcount(gamemode, mutators));
+            loopi(wants) if(!candidates[i].empty())
             {
-                int c = candidates[i].length(), r = c > 1 ? rnd(c) : 0;
+                int c = candidates[i].length(), r = rnd(c);
                 if(candidates[i].inrange(r) && flags.inrange(candidates[i][r]) && isteam(gamemode, mutators, flags[candidates[i][r]].team, T_NEUTRAL))
                 {
                     bomberstate::returnaffinity(candidates[i][r], gamemillis, true);
@@ -188,16 +210,19 @@ struct bomberservmode : bomberstate, servmode
             {
                 if(!candidates[T_NEUTRAL].empty() && !m_gsp1(gamemode, mutators))
                 {
+                    int muts = mutators;
+                    if(muts&(1<<G_M_GSP2)) muts &= ~(1<<G_M_GSP2);
+                    muts |= (1<<G_M_GSP1);
                     srvmsgf(-1, "\fzoythis map does have enough goals, switching on hold mutator");
-                    sendf(-1, 1, "risi3", N_MAPCHANGE, smapname, 0, gamemode, mutators|(1<<G_M_GSP1));
-                    changemap(smapname, gamemode, mutators|(1<<G_M_GSP1));
+                    sendf(-1, 1, "risi3", N_MAPCHANGE, smapname, 0, gamemode, muts);
+                    changemap(smapname, gamemode, muts);
                     return;
                 }
                 hasflaginfo = false;
                 loopv(flags) sendf(-1, 1, "ri3", N_RESETAFFIN, i, 0);
                 srvmsgf(-1, "\fs\fzoythis map is not playable in:\fS %s", gamename(gamemode, mutators));
             }
-            else ancmsgft(-1, m_duke(gamemode, mutators) ? S_V_BOMBDUEL : S_V_BOMBSTART, CON_INFO, "\fathe \fs\fwbomb\fS has been spawned");
+            else ancmsgft(-1, m_duke(gamemode, mutators) ? S_V_BOMBDUEL : S_V_BOMBSTART, CON_INFO, "\fathe \fs\fzwvbomb\fS has been spawned");
             bombertime = 0;
         }
         int t = (gamemillis/G(bomberholdinterval))-((gamemillis-(curtime+scoresec))/G(bomberholdinterval));
@@ -209,7 +234,7 @@ struct bomberservmode : bomberstate, servmode
             if(f.owner >= 0)
             {
                 clientinfo *ci = (clientinfo *)getinfo(f.owner);
-                if((!m_team(gamemode, mutators) || m_gsp1(gamemode, mutators)) && t > 0)
+                if(m_gsp1(gamemode, mutators) && t > 0)
                 {
                     int score = G(bomberholdpoints)*t;
                     if(score)
@@ -221,7 +246,7 @@ struct bomberservmode : bomberstate, servmode
                             total = addscore(ci->team, score);
                             sendf(-1, 1, "ri3", N_SCORE, ci->team, total);
                         }
-                        if(!m_balance(gamemode, mutators) && G(bomberholdlimit) && total >= G(bomberholdlimit))
+                        if(!m_balance(gamemode, mutators, teamspawns) && G(bomberholdlimit) && total >= G(bomberholdlimit))
                         {
                             ancmsgft(-1, S_V_NOTIFY, CON_EVENT, "\fyscore limit has been reached");
                             startintermission();
@@ -231,12 +256,12 @@ struct bomberservmode : bomberstate, servmode
                 if(ci && carrytime && gamemillis-f.taketime >= carrytime)
                 {
                     ci->state.weapshots[W_GRENADE][0].add(1);
-                    sendf(-1, 1, "ri8", N_DROP, ci->clientnum, -1, 1, W_GRENADE, -1, -1, -1);
-                    dropaffinity(ci, ci->state.o, vec(ci->state.vel).add(ci->state.falling));
-                    if((!m_team(gamemode, mutators) || m_gsp1(gamemode, mutators)) && G(bomberholdpenalty))
+                    sendf(-1, 1, "ri7", N_DROP, ci->clientnum, -1, 1, W_GRENADE, -1, -1);
+                    dropaffinity(ci, ci->state.feetpos(G(bomberdropheight)), vec(ci->state.vel).add(ci->state.falling));
+                    if(m_gsp1(gamemode, mutators) && G(bomberholdpenalty))
                     {
                         givepoints(ci, -G(bomberholdpenalty));
-                        if(m_team(gamemode, mutators) && m_gsp1(gamemode, mutators))
+                        if(m_team(gamemode, mutators))
                         {
                             int total = addscore(ci->team, -G(bomberholdpenalty));
                             sendf(-1, 1, "ri3", N_SCORE, ci->team, total);
@@ -245,7 +270,7 @@ struct bomberservmode : bomberstate, servmode
                 }
                 continue;
             }
-            if(f.droptime && gamemillis-f.droptime >= G(bomberresetdelay)) returnaffinity(i, 2);
+            if(f.droptime && gamemillis-f.droptime >= G(bomberresetdelay)) returnaffinity(i, false);
         }
     }
 
@@ -265,7 +290,8 @@ struct bomberservmode : bomberstate, servmode
         {
             flag &f = flags[i];
             putint(p, f.team);
-            putint(p, f.ent);
+            putint(p, f.yaw);
+            putint(p, f.pitch);
             putint(p, f.enabled ? 1 : 0);
             putint(p, f.owner);
             loopj(3) putint(p, int(f.spawnloc[j]*DMF));
@@ -292,7 +318,7 @@ struct bomberservmode : bomberstate, servmode
 
     void regen(clientinfo *ci, int &total, int &amt, int &delay)
     {
-        if(!hasflaginfo || !G(bomberregenbuff) || !ci->state.lastbuff) return;
+        if(!canplay(hasflaginfo) || !G(bomberregenbuff) || !ci->state.lastbuff) return;
         if(G(maxhealth)) total = max(m_maxhealth(gamemode, mutators, ci->state.model), total);
         if(ci->state.lastregen && G(bomberregendelay)) delay = G(bomberregendelay);
         if(G(bomberregenextra)) amt += G(bomberregenextra);
@@ -300,9 +326,9 @@ struct bomberservmode : bomberstate, servmode
 
     void checkclient(clientinfo *ci)
     {
-        if(!hasflaginfo || ci->state.state != CS_ALIVE || m_insta(gamemode, mutators)) return;
-        #define bomberbuff1 (G(bomberbuffing)&1 && isbomberhome(f, ci->team) && ci->state.o.dist(f.spawnloc) <= G(bomberbuffarea))
-        #define bomberbuff2 (G(bomberbuffing)&2 && isbomberaffinity(f) && f.owner == ci->clientnum)
+        if(!canplay(hasflaginfo) || ci->state.state != CS_ALIVE || m_insta(gamemode, mutators)) return;
+        #define bomberbuff1 (G(bomberbuffing)&1 && isbomberhome(f, ci->team) && (G(bomberbuffarea) > 0 ? ci->state.o.dist(f.spawnloc) <= G(bomberbuffarea) : true))
+        #define bomberbuff2 ((G(bomberbuffing)&2 || (G(bomberbuffing)&4 && m_gsp3(gamemode, mutators) && ci->team == T_ALPHA)) && isbomberaffinity(f) && f.owner == ci->clientnum)
         if(G(bomberbuffing)) loopv(flags)
         {
             flag &f = flags[i];
@@ -313,7 +339,7 @@ struct bomberservmode : bomberstate, servmode
                 return;
             }
         }
-        if(ci->state.lastbuff && (!G(bomberbuffing) || gamemillis-ci->state.lastbuff > G(bomberbuffdelay)))
+        if(ci->state.lastbuff && (!G(bomberbuffing) || gamemillis-ci->state.lastbuff >= G(bomberbuffdelay)))
         {
             ci->state.lastbuff = 0;
             sendf(-1, 1, "ri4", N_SPHY, ci->clientnum, SPHY_BUFF, 0);
@@ -322,13 +348,19 @@ struct bomberservmode : bomberstate, servmode
 
     void moveaffinity(clientinfo *ci, int cn, int id, const vec &o, const vec &inertia = vec(0, 0, 0))
     {
-        if(!flags.inrange(id)) return;
+        if(!canplay(hasflaginfo) || !flags.inrange(id)) return;
         flag &f = flags[id];
-        if(!f.droptime || f.owner >= 0 || !isbomberaffinity(f)) return;
-        clientinfo *co = f.lastowner >= 0 ? (clientinfo *)getinfo(f.lastowner) : NULL;
-        if(!co || co->clientnum != ci->clientnum) return;
+        if(!f.droptime || f.owner >= 0 || !isbomberaffinity(f) || f.lastowner != ci->clientnum) return;
+        f.distance += f.droploc.dist(o);
         f.droploc = o;
         f.inertia = inertia;
+        if(!m_gsp1(gamemode, mutators) && m_gsp2(gamemode, mutators)) loopv(flags)
+        {
+            if(isbomberaffinity(flags[i]) || f.droploc.dist(flags[i].spawnloc) > enttype[AFFINITY].radius/2) continue;
+            if(G(bomberbasketmindist) > 0 && !flags[id].travel(flags[i].spawnloc, G(bomberbasketmindist))) continue;
+            scorebomb(ci, id, i);
+            break;
+        }
         //sendf(-1, 1, "ri9", N_MOVEAFFIN, ci->clientnum, id, int(f.droploc.x*DMF), int(f.droploc.y*DMF), int(f.droploc.z*DMF), int(f.inertia.x*DMF), int(f.inertia.y*DMF), int(f.inertia.z*DMF));
     }
 
@@ -339,10 +371,11 @@ struct bomberservmode : bomberstate, servmode
         {
             loopi(numflags)
             {
-                int team = getint(p), ent = getint(p);
+                int team = getint(p), yaw = getint(p), pitch = getint(p);
                 vec o;
                 loopj(3) o[j] = getint(p)/DMF;
-                if(!hasflaginfo) addaffinity(o, team, ent);
+                if(p.overread()) break;
+                if(!hasflaginfo && i < MAXPARAMS) addaffinity(o, team, yaw, pitch);
             }
             if(!hasflaginfo)
             {
@@ -353,11 +386,11 @@ struct bomberservmode : bomberstate, servmode
         }
     }
 
-    int points(clientinfo *victim, clientinfo *actor)
+    int points(clientinfo *m, clientinfo *v)
     {
-        bool isteam = victim==actor || (m_team(gamemode, mutators) && victim->team == actor->team);
-        int p = isteam ? -1 : (m_team(gamemode, mutators) ? 1 : 0), v = p;
-        if(p) { loopv(flags) if(flags[i].owner == victim->clientnum) p += v; }
+        bool isteam = m==v || (m_team(gamemode, mutators) && m->team == v->team);
+        int p = isteam ? -1 : (m_team(gamemode, mutators) ? 1 : 0), q = p;
+        if(p) { loopv(flags) if(flags[i].owner == m->clientnum) p += q; }
         return p;
     }
 } bombermode;
