@@ -1,45 +1,45 @@
 #ifdef GAMESERVER
 #define capturedelay (m_gsp2(gamemode, mutators) ? G(capturedefenddelay) : G(captureresetdelay))
+#define capturestore (G(captureresetstore)&((m_gsp1(gamemode, mutators) ? 1 : 0)|(m_gsp2(gamemode, mutators) ? 2 : 0)|(m_gsp3(gamemode, mutators) ? 4 : 0)|(!m_gsp1(gamemode, mutators) && !m_gsp2(gamemode, mutators) && !m_gsp3(gamemode, mutators) ? 8 : 0)))
 #define capturestate captureservstate
 #else
 #define capturedelay (m_gsp2(game::gamemode, game::mutators) ? G(capturedefenddelay) : G(captureresetdelay))
+#define capturestore (G(captureresetstore)&((m_gsp1(game::gamemode, game::mutators) ? 1 : 0)|(m_gsp2(game::gamemode, game::mutators) ? 2 : 0)|(m_gsp3(game::gamemode, game::mutators) ? 4 : 0)|(!m_gsp1(game::gamemode, game::mutators) && !m_gsp2(game::gamemode, game::mutators) && !m_gsp3(game::gamemode, game::mutators) ? 8 : 0)))
 #endif
 struct capturestate
 {
     struct flag
     {
         vec droploc, inertia, spawnloc;
-        int team, ent, droptime, taketime;
+        int team, yaw, pitch, droptime, taketime, dropoffset;
 #ifdef GAMESERVER
-        int owner, lastowner;
+        int owner, lastowner, nextreset;
         vector<int> votes;
 #else
         gameent *owner, *lastowner;
         projent *proj;
         int displaytime, pickuptime, movetime, viewtime, interptime;
-        vec viewpos, interppos;
+        vec viewpos, interppos, render, above;
+        entitylight light, baselight;
 #endif
 
-        flag()
-#ifndef GAMESERVER
-          : ent(-1)
-#endif
-        { reset(); }
+        flag() { reset(); }
 
         void reset()
         {
             inertia = vec(0, 0, 0);
             droploc = spawnloc = vec(-1, -1, -1);
 #ifdef GAMESERVER
+            nextreset = 0;
             owner = lastowner = -1;
             votes.shrink(0);
 #else
             owner = lastowner = NULL;
             proj = NULL;
-            displaytime = pickuptime = movetime = 0;
+            displaytime = pickuptime = movetime = viewtime = interptime = 0;
 #endif
             team = T_NEUTRAL;
-            taketime = droptime = 0;
+            yaw = pitch = taketime = droptime = dropoffset = 0;
         }
 
 #ifndef GAMESERVER
@@ -47,7 +47,7 @@ struct capturestate
         {
             if(owner) return owner->waist;
             if(droptime) return proj ? proj->o : droploc;
-            return spawnloc;
+            return above;
         }
 
         vec &pos(bool view = false)
@@ -68,6 +68,11 @@ struct capturestate
             return position();
         }
 #endif
+
+        int dropleft(int t, bool b)
+        {
+            return (t-droptime)+(b ? dropoffset : 0);
+        }
     };
     vector<flag> flags;
 
@@ -76,13 +81,20 @@ struct capturestate
         flags.shrink(0);
     }
 
-    void addaffinity(const vec &o, int team, int ent)
+    void addaffinity(const vec &o, int team, int yaw, int pitch)
     {
         flag &f = flags.add();
         f.reset();
         f.team = team;
         f.spawnloc = o;
-        f.ent = ent;
+        f.yaw = yaw;
+        f.pitch = pitch;
+#ifndef GAMESERVER
+        f.render = f.above = o;
+        f.render.z += 2;
+        physics::droptofloor(f.render);
+        if(f.render.z >= f.above.z-1) f.above.z += f.render.z-(f.above.z-1);
+#endif
     }
 
 #ifndef GAMESERVER
@@ -118,6 +130,7 @@ struct capturestate
 #endif
     {
         flag &f = flags[i];
+        if(f.droptime) f.dropoffset += t-f.droptime;
 #ifndef GAMESERVER
         interp(i, t);
 #endif
@@ -139,9 +152,10 @@ struct capturestate
 #endif
     }
 
-    void dropaffinity(int i, const vec &o, const vec &p, int t)
+    void dropaffinity(int i, const vec &o, const vec &p, int t, int offset = -1)
     {
         flag &f = flags[i];
+        if(offset >= 0) f.dropoffset = offset;
 #ifndef GAMESERVER
         interp(i, t);
 #endif
@@ -153,8 +167,7 @@ struct capturestate
         f.owner = -1;
         f.votes.shrink(0);
 #else
-        f.pickuptime = 0;
-        f.movetime = t;
+        f.pickuptime =  f.movetime = 0;
         f.owner = NULL;
         destroy(i);
         create(i);
@@ -167,7 +180,7 @@ struct capturestate
 #ifndef GAMESERVER
         interp(i, t);
 #endif
-        f.droptime = f.taketime = 0;
+        f.droptime = f.taketime = f.dropoffset = 0;
 #ifdef GAMESERVER
         f.owner = -1;
         f.votes.shrink(0);
@@ -195,7 +208,7 @@ namespace capture
     extern void reset();
     extern void setup();
     extern void setscore(int team, int total);
-    extern void checkaffinity(dynent *e);
+    extern void update();
     extern void drawnotices(int w, int h, int &tx, int &ty, float blend);
     extern void drawblips(int w, int h, float blend);
     extern int drawinventory(int x, int y, int s, int m, float blend);
@@ -203,6 +216,7 @@ namespace capture
     extern void render();
     extern void adddynlights();
     extern int aiowner(gameent *d);
+    extern bool aihomerun(gameent *d, ai::aistate &b);
     extern void aifind(gameent *d, ai::aistate &b, vector<ai::interest> &interests);
     extern bool aicheck(gameent *d, ai::aistate &b);
     extern bool aidefense(gameent *d, ai::aistate &b);
