@@ -4,7 +4,7 @@ struct captureservmode : capturestate, servmode
 
     captureservmode() : hasflaginfo(false) {}
 
-    void reset(bool empty)
+    void reset()
     {
         capturestate::reset();
         hasflaginfo = false;
@@ -12,10 +12,10 @@ struct captureservmode : capturestate, servmode
 
     void dropaffinity(clientinfo *ci, const vec &o, const vec &inertia = vec(0, 0, 0), int target = -1)
     {
-        if(!hasflaginfo || ci->state.aitype >= AI_START) return;
+        if(!canplay(hasflaginfo) || ci->state.actortype >= A_ENEMY) return;
         int numflags = 0, iterflags = 0;
         loopv(flags) if(flags[i].owner == ci->clientnum) numflags++;
-        vec dir = inertia, olddir = dir, v = o; v.z += 1;
+        vec dir = inertia, olddir = dir;
         if(numflags > 1 && dir.iszero())
         {
             dir.x = -sinf(RAD*ci->state.yaw);
@@ -31,27 +31,27 @@ struct captureservmode : capturestate, servmode
                 dir = vec(olddir).rotate_around_z(yaw*RAD);
                 iterflags++;
             }
-            ivec p(vec(v).mul(DMF)), q(vec(dir).mul(DMF));
-            sendf(-1, 1, "ri3i7", N_DROPAFFIN, ci->clientnum, -1, i, p.x, p.y, p.z, q.x, q.y, q.z);
-            capturestate::dropaffinity(i, v, dir, gamemillis);
+            ivec p(vec(o).mul(DMF)), q(vec(dir).mul(DMF));
+            sendf(-1, 1, "ri3i7", N_DROPAFFIN, ci->clientnum, flags[i].dropoffset, i, p.x, p.y, p.z, q.x, q.y, q.z);
+            capturestate::dropaffinity(i, o, dir, gamemillis);
         }
     }
 
     void leavegame(clientinfo *ci, bool disconnecting = false)
     {
-        if(!hasflaginfo) return;
-        dropaffinity(ci, ci->state.o, vec(ci->state.vel).add(ci->state.falling));
+        if(!canplay(hasflaginfo)) return;
+        dropaffinity(ci, ci->state.feetpos(G(capturedropheight)), vec(ci->state.vel).add(ci->state.falling));
     }
 
-    void dodamage(clientinfo *target, clientinfo *actor, int &damage, int &hurt, int &weap, int &flags, int &material, const ivec &hitpush)
+    void dodamage(clientinfo *m, clientinfo *v, int &damage, int &hurt, int &weap, int &flags, int &material, const ivec &hitpush, const ivec &hitvel, float dist)
     {
-        //if(weaptype[weap].melee) dropaffinity(target, target->state.o, vec(ci->state.vel).add(ci->state.falling));
+        //if(weaptype[weap].melee) dropaffinity(m, m->state.o, vec(ci->state.vel).add(ci->state.falling));
     }
 
-    void died(clientinfo *ci, clientinfo *actor)
+    void died(clientinfo *ci, clientinfo *v)
     {
-        if(!hasflaginfo) return;
-        dropaffinity(ci, ci->state.o, vec(ci->state.vel).add(ci->state.falling));
+        if(!canplay(hasflaginfo)) return;
+        dropaffinity(ci, ci->state.feetpos(G(capturedropheight)), vec(ci->state.vel).add(ci->state.falling));
     }
 
     int addscore(int team, int points = 1)
@@ -63,24 +63,25 @@ struct captureservmode : capturestate, servmode
 
     void moved(clientinfo *ci, const vec &oldpos, const vec &newpos)
     {
-        if(!hasflaginfo || ci->state.aitype >= AI_START) return;
+        if(!canplay(hasflaginfo) || ci->state.actortype >= A_ENEMY) return;
         if(G(capturethreshold) > 0 && oldpos.dist(newpos) >= G(capturethreshold))
             dropaffinity(ci, oldpos, vec(ci->state.vel).add(ci->state.falling));
         if(!m_gsp3(gamemode, mutators)) loopv(flags) if(flags[i].owner == ci->clientnum)
         {
+            flag &r = flags[i];
             loopvk(flags)
             {
                 flag &f = flags[k];
-                if(f.team == ci->team && (f.owner < 0 || (f.owner == ci->clientnum && i == k)) && !f.droptime && newpos.dist(f.spawnloc) <= enttype[AFFINITY].radius*2/3)
+                if(f.team == ci->team && (f.owner < 0 || (f.owner == ci->clientnum && i == k && gamemillis-f.taketime >= G(capturepickupdelay))) && !f.droptime && (!f.nextreset || r.team != ci->team) && newpos.dist(f.spawnloc) <= enttype[AFFINITY].radius*2/3)
                 {
                     capturestate::returnaffinity(i, gamemillis);
-                    givepoints(ci, G(capturepoints));
-                    if(flags[i].team != ci->team)
+                    if(r.team != ci->team)
                     {
                         int score = addscore(ci->team);
                         sendf(-1, 1, "ri5", N_SCOREAFFIN, ci->clientnum, i, k, score);
                         mutate(smuts, mut->scoreaffinity(ci));
-                        if(!m_balance(gamemode, mutators) && G(capturelimit) && score >= G(capturelimit))
+                        if(!m_nopoints(gamemode, mutators)) givepoints(ci, G(capturepoints));
+                        if(!m_balance(gamemode, mutators, teamspawns) && G(capturelimit) && score >= G(capturelimit))
                         {
                             ancmsgft(-1, S_V_NOTIFY, CON_EVENT, "\fyscore limit has been reached");
                             startintermission();
@@ -94,32 +95,36 @@ struct captureservmode : capturestate, servmode
 
     void takeaffinity(clientinfo *ci, int i)
     {
-        if(!hasflaginfo || !flags.inrange(i) || ci->state.state!=CS_ALIVE || !ci->team || ci->state.aitype >= AI_START) return;
+        if(!canplay(hasflaginfo) || !flags.inrange(i) || ci->state.state != CS_ALIVE || !ci->team || ci->state.actortype >= A_ENEMY) return;
         flag &f = flags[i];
-        if(f.owner >= 0 || (f.team == ci->team && !m_gsp3(gamemode, mutators) && (m_gsp2(gamemode, mutators) || !f.droptime))) return;
-        if(f.lastowner == ci->clientnum && f.droptime && (G(capturepickupdelay) < 0 || lastmillis-f.droptime <= G(capturepickupdelay))) return;
+        if(f.owner >= 0 || (f.team == ci->team && (f.nextreset || m_gsp2(gamemode, mutators) || (m_gsp1(gamemode, mutators) && !f.droptime)))) return;
+        if(f.lastowner == ci->clientnum && f.droptime && gamemillis-f.droptime <= G(capturepickupdelay)) return;
         if(m_gsp1(gamemode, mutators) && f.team == ci->team)
         {
             capturestate::returnaffinity(i, gamemillis);
-            givepoints(ci, G(capturepoints));
+            if(!m_nopoints(gamemode, mutators)) givepoints(ci, G(capturepoints));
             sendf(-1, 1, "ri3", N_RETURNAFFIN, ci->clientnum, i);
         }
         else
         {
             capturestate::takeaffinity(i, ci->clientnum, gamemillis);
-            if(f.team != ci->team && (!f.droptime || f.lastowner != ci->clientnum)) givepoints(ci, G(capturepickuppoints));
+            if(!m_nopoints(gamemode, mutators) && ((f.team != ci->team && !f.droptime) || f.lastowner != ci->clientnum)) givepoints(ci, G(capturepickuppoints));
             sendf(-1, 1, "ri3", N_TAKEAFFIN, ci->clientnum, i);
         }
     }
 
     void resetaffinity(clientinfo *ci, int i)
     {
-        if(!hasflaginfo || !flags.inrange(i) || ci->state.ownernum >= 0) return;
+        if(!canplay(hasflaginfo) || !flags.inrange(i) || ci->state.ownernum >= 0) return;
         flag &f = flags[i];
         if(f.owner >= 0 || !f.droptime || f.votes.find(ci->clientnum) >= 0) return;
         f.votes.add(ci->clientnum);
-        if(f.votes.length() >= numclients()/2)
+        if(f.votes.length() >= int(floorf(numclients()*0.5f)))
         {
+            f.nextreset = gamemillis;
+            clientinfo *last = (clientinfo *)getinfo(f.lastowner);
+            if(last && last->team == f.team) f.nextreset += G(captureteampenalty);
+            else f.nextreset += G(captureresetpenalty);
             capturestate::returnaffinity(i, gamemillis);
             sendf(-1, 1, "ri3", N_RESETAFFIN, i, 2);
         }
@@ -127,7 +132,7 @@ struct captureservmode : capturestate, servmode
 
     void layout()
     {
-        if(!hasflaginfo) return;
+        if(!canplay(hasflaginfo)) return;
         loopv(flags) if(flags[i].owner >= 0 || flags[i].droptime)
         {
             capturestate::returnaffinity(i, gamemillis);
@@ -137,28 +142,29 @@ struct captureservmode : capturestate, servmode
 
     void update()
     {
-        if(!hasflaginfo) return;
+        if(!canplay(hasflaginfo)) return;
         loopv(flags)
         {
             flag &f = flags[i];
-            if(m_gsp3(gamemode, mutators) && f.owner >= 0 && f.taketime && gamemillis-f.taketime >= G(captureprotectdelay))
+            if(f.nextreset && gamemillis > f.nextreset) f.nextreset = 0;
+            if(m_gsp3(gamemode, mutators) && f.owner >= 0)
             {
                 clientinfo *ci = (clientinfo *)getinfo(f.owner);
-                if(f.team != ci->team)
+                if(f.team != ci->team && f.taketime && gamemillis-f.taketime >= G(captureprotectdelay))
                 {
                     capturestate::returnaffinity(i, gamemillis);
-                    givepoints(ci, G(capturepoints));
+                    if(!m_nopoints(gamemode, mutators)) givepoints(ci, G(capturepoints));
                     int score = addscore(ci->team);
                     sendf(-1, 1, "ri5", N_SCOREAFFIN, ci->clientnum, i, -1, score);
                     mutate(smuts, mut->scoreaffinity(ci));
-                    if(!m_balance(gamemode, mutators) && G(capturelimit) && score >= G(capturelimit))
+                    if(!m_balance(gamemode, mutators, teamspawns) && G(capturelimit) && score >= G(capturelimit))
                     {
                         ancmsgft(-1, S_V_NOTIFY, CON_EVENT, "\fyscore limit has been reached");
                         startintermission();
                     }
                 }
             }
-            else if(f.owner < 0 && f.droptime && gamemillis-f.droptime >= capturedelay)
+            else if(f.owner < 0 && f.droptime && f.dropleft(gamemillis, capturestore) >= capturedelay)
             {
                 capturestate::returnaffinity(i, gamemillis);
                 sendf(-1, 1, "ri3", N_RESETAFFIN, i, 2);
@@ -182,7 +188,8 @@ struct captureservmode : capturestate, servmode
         {
             flag &f = flags[i];
             putint(p, f.team);
-            putint(p, f.ent);
+            putint(p, f.yaw);
+            putint(p, f.pitch);
             putint(p, f.owner);
             loopj(3) putint(p, int(f.spawnloc[j]*DMF));
             if(f.owner<0)
@@ -190,6 +197,7 @@ struct captureservmode : capturestate, servmode
                 putint(p, f.droptime);
                 if(f.droptime)
                 {
+                    putint(p, f.dropoffset);
                     loopj(3) putint(p, int(f.droploc[j]*DMF));
                     loopj(3) putint(p, int(f.inertia[j]*DMF));
                 }
@@ -208,7 +216,7 @@ struct captureservmode : capturestate, servmode
 
     void regen(clientinfo *ci, int &total, int &amt, int &delay)
     {
-        if(!hasflaginfo || !G(captureregenbuff) || !ci->state.lastbuff) return;
+        if(!canplay(hasflaginfo) || !G(captureregenbuff) || !ci->state.lastbuff) return;
         if(G(maxhealth)) total = max(m_maxhealth(gamemode, mutators, ci->state.model), total);
         if(ci->state.lastregen && G(captureregendelay)) delay = G(captureregendelay);
         if(G(captureregenextra)) amt += G(captureregenextra);
@@ -216,22 +224,30 @@ struct captureservmode : capturestate, servmode
 
     void checkclient(clientinfo *ci)
     {
-        if(!hasflaginfo || ci->state.state != CS_ALIVE || m_insta(gamemode, mutators)) return;
-        #define capturebuffx(a) (G(capturebuffing)&a && owner && owner->team == ci->team && ci->state.o.dist(owner->state.o) <= G(capturebuffarea))
-        #define capturebuff1    (G(capturebuffing)&1 && f.team == ci->team && (owner ? (owner->clientnum == ci->clientnum || capturebuffx(4)): (!f.droptime || m_gsp2(gamemode, mutators)) && ci->state.o.dist(f.droptime ? f.droploc : f.spawnloc) <= G(capturebuffarea)))
-        #define capturebuff2    (G(capturebuffing)&2 && f.team != ci->team && owner && (owner->clientnum == ci->clientnum || capturebuffx(8)))
+        if(!canplay(hasflaginfo) || ci->state.state != CS_ALIVE || m_insta(gamemode, mutators)) return;
+        bool buff = false;
         if(G(capturebuffing)) loopv(flags)
         {
             flag &f = flags[i];
             clientinfo *owner = f.owner >= 0 ? (clientinfo *)getinfo(f.owner) : NULL;
-            if(capturebuff1 || capturebuff2)
+            if(f.team == ci->team)
             {
-                if(!ci->state.lastbuff) sendf(-1, 1, "ri4", N_SPHY, ci->clientnum, SPHY_BUFF, 1);
-                ci->state.lastbuff = gamemillis;
-                return;
+                if((G(capturebuffing)&1 || G(capturebuffing)&2) && !owner && (!f.droptime || m_gsp2(gamemode, mutators) || G(capturebuffing)&2) && ci->state.o.dist(f.droptime ? f.droploc : f.spawnloc) <= G(capturebuffarea)) { buff = true; break; }
+                if(G(capturebuffing)&4 && owner && ci == owner) { buff = true; break; }
+                if(G(capturebuffing)&8 && owner && ci != owner && owner->team == ci->team && (G(capturebuffarea) > 0 ? ci->state.o.dist(owner->state.o) <= G(capturebuffarea) : true)) { buff = true; break; }
+            }
+            else
+            {
+                if(G(capturebuffing)&16 && ci == owner) { buff = true; break; }
+                if(G(capturebuffing)&32 && owner && ci != owner && owner->team == ci->team && (G(capturebuffarea) > 0 ? ci->state.o.dist(owner->state.o) <= G(capturebuffarea) : true)) { buff = true; break; }
             }
         }
-        if(ci->state.lastbuff && (!G(capturebuffing) || gamemillis-ci->state.lastbuff > G(capturebuffdelay)))
+        if(buff)
+        {
+            if(!ci->state.lastbuff) sendf(-1, 1, "ri4", N_SPHY, ci->clientnum, SPHY_BUFF, 1);
+            ci->state.lastbuff = gamemillis;
+        }
+        else if(ci->state.lastbuff && (!G(capturebuffing) || gamemillis-ci->state.lastbuff >= G(capturebuffdelay)))
         {
             ci->state.lastbuff = 0;
             sendf(-1, 1, "ri4", N_SPHY, ci->clientnum, SPHY_BUFF, 0);
@@ -240,11 +256,9 @@ struct captureservmode : capturestate, servmode
 
     void moveaffinity(clientinfo *ci, int cn, int id, const vec &o, const vec &inertia = vec(0, 0, 0))
     {
-        if(!flags.inrange(id)) return;
+        if(!canplay(hasflaginfo) || !flags.inrange(id)) return;
         flag &f = flags[id];
-        if(!f.droptime || f.owner >= 0) return;
-        clientinfo *co = f.lastowner >= 0 ? (clientinfo *)getinfo(f.lastowner) : NULL;
-        if(!co || co->clientnum != ci->clientnum) return;
+        if(!f.droptime || f.owner >= 0 || f.lastowner != ci->clientnum) return;
         f.droploc = o;
         f.inertia = inertia;
         //sendf(-1, 1, "ri9", N_MOVEAFFIN, ci->clientnum, id, int(f.droploc.x*DMF), int(f.droploc.y*DMF), int(f.droploc.z*DMF), int(f.inertia.x*DMF), int(f.inertia.y*DMF), int(f.inertia.z*DMF));
@@ -257,10 +271,11 @@ struct captureservmode : capturestate, servmode
         {
             loopi(numflags)
             {
-                int team = getint(p), ent = getint(p);
+                int team = getint(p), yaw = getint(p), pitch = getint(p);
                 vec o;
                 loopj(3) o[j] = getint(p)/DMF;
-                if(!hasflaginfo) addaffinity(o, team, ent);
+                if(p.overread()) break;
+                if(!hasflaginfo && i < MAXPARAMS) addaffinity(o, team, yaw, pitch);
             }
             if(!hasflaginfo)
             {
@@ -271,11 +286,11 @@ struct captureservmode : capturestate, servmode
         }
     }
 
-    int points(clientinfo *victim, clientinfo *actor)
+    int points(clientinfo *m, clientinfo *v)
     {
-        bool isteam = victim==actor || victim->team == actor->team;
-        int p = isteam ? -1 : 1, v = p;
-        loopv(flags) if(flags[i].owner == victim->clientnum) p += v;
+        bool isteam = m==v || m->team == v->team;
+        int p = isteam ? -1 : 1, q = p;
+        loopv(flags) if(flags[i].owner == m->clientnum) p += q;
         return p;
     }
 } capturemode;
