@@ -10,7 +10,7 @@ namespace defend
 
     void preload()
     {
-        preloadmodel("props/flag");
+        preloadmodel("props/point");
     }
 
     static vec skewcolour(int owner, int enemy, float occupy)
@@ -18,7 +18,7 @@ namespace defend
         vec colour = vec::hexcolor(TEAM(owner, colour));
         if(enemy)
         {
-            int team = owner && enemy && !defendinstant ? T_NEUTRAL : enemy;
+            int team = owner && enemy && !m_gsp1(game::gamemode, game::mutators) ? T_NEUTRAL : enemy;
             int timestep = totalmillis%1000;
             float amt = clamp((timestep <= 500 ? timestep/500.f : (1000-timestep)/500.f)*occupy, 0.f, 1.f);
             colour.lerp(vec::hexcolor(TEAM(team, colour)), amt);
@@ -31,7 +31,6 @@ namespace defend
         loopv(st.flags) // flags/bases
         {
             defendstate::flag &f = st.flags[i];
-            if(!entities::ents.inrange(f.ent)) continue;
             cament *c = cameras.add(new cament);
             c->o = f.o;
             c->o.z += enttype[AFFINITY].radius*2/3;
@@ -62,41 +61,34 @@ namespace defend
         loopv(st.flags)
         {
             defendstate::flag &b = st.flags[i];
-            if(!entities::ents.inrange(b.ent)) continue;
-            float occupy = b.occupied(defendinstant, defendcount);
-            entitylight *light = &entities::ents[b.ent]->light;
-            light->material[0] = bvec::fromcolor(skewcolour(b.owner, b.enemy, occupy));
-            rendermodel(light, "props/flag", ANIM_MAPMODEL|ANIM_LOOP, b.o, entities::ents[b.ent]->attrs[1], entities::ents[b.ent]->attrs[2], 0, MDL_DYNSHADOW|MDL_CULL_VFC|MDL_CULL_OCCLUDED);
+            float occupy = b.occupied(m_gsp1(game::gamemode, game::mutators), defendcount);
+            vec effect = skewcolour(b.owner, b.enemy, occupy);
+            int colour = effect.tohexcolor();
+            b.baselight.material[0] = bvec::fromcolor(effect);
+            rendermodel(&b.baselight, "props/point", ANIM_MAPMODEL|ANIM_LOOP, b.render, b.yaw, 0, 0, MDL_DYNSHADOW|MDL_CULL_VFC|MDL_CULL_OCCLUDED, NULL, NULL, 0, 0, 1);
             if(b.enemy && b.owner)
             {
                 defformatstring(bowner)("%s", game::colourteam(b.owner));
-                formatstring(b.info)("%s vs. %s", bowner, game::colourteam(b.enemy));
+                formatstring(b.info)("%s - %s v %s", b.name, bowner, game::colourteam(b.enemy));
             }
             else
             {
                 int defend = b.owner ? b.owner : b.enemy;
-                formatstring(b.info)("%s", game::colourteam(defend));
+                formatstring(b.info)("%s - %s", b.name, game::colourteam(defend));
             }
-            vec above = b.o;
-            above.z += enttype[AFFINITY].radius*2/3;
-            defformatstring(name)("<huge>%s", b.name);
-            part_textcopy(above, name);
-            above.z += 2.f;
-            part_text(above, b.info);
-            above.z += 3.f;
+            vec above = b.above;
+            float blend = camera1->o.distrange(above, enttype[AFFINITY].radius, enttype[AFFINITY].radius/8);
+            part_explosion(above, 3, PART_SHOCKBALL, 1, colour, 1, 0.5f*blend);
+            above.z += 4;
+            part_text(above, b.info, PART_TEXT, 1, 0xFFFFFF, 2, blend);
+            above.z += 3;
             if(b.enemy)
             {
-                part_icon(above, textureload(hud::progresstex, 3), 3, 1, 0, 0, 1, (int(light->material[0].x)<<16)|(int(light->material[0].y)<<8)|int(light->material[0].z), (lastmillis%1000)/1000.f, 0.1f);
-                part_icon(above, textureload(hud::progresstex, 3), 2, 1, 0, 0, 1, TEAM(b.enemy, colour), 0, occupy);
-                part_icon(above, textureload(hud::progresstex, 3), 2, 0.25f, 0, 0, 1, TEAM(b.owner, colour), occupy, 1-occupy);
+                part_icon(above, textureload(hud::progringtex, 3), 4, blend, 0, 0, 1, colour, (lastmillis%1000)/1000.f, 0.1f);
+                part_icon(above, textureload(hud::progresstex, 3), 4, blend, 0, 0, 1, TEAM(b.enemy, colour), 0, occupy);
+                part_icon(above, textureload(hud::progresstex, 3), 4, 0.25f*blend, 0, 0, 1, TEAM(b.owner, colour), occupy, 1-occupy);
             }
-            else
-            {
-                part_icon(above, textureload(hud::progresstex, 3), 3, 1, 0, 0, 1, TEAM(b.owner, colour));
-                part_icon(above, textureload(hud::progresstex, 3), 2, 1, 0, 0, 1, TEAM(b.owner, colour));
-            }
-            above.z += 1.f;
-            defformatstring(str)("<huge>%d%%", int(occupy*100.f)); part_textcopy(above, str);
+            else part_icon(above, textureload(hud::teamtexname(b.owner), 3), 3, blend, 0, 0, 1, TEAM(b.owner, colour));
         }
     }
 
@@ -106,8 +98,7 @@ namespace defend
         loopv(st.flags)
         {
             defendstate::flag &f = st.flags[i];
-            if(!entities::ents.inrange(f.ent)) continue;
-            float occupy = f.occupied(defendinstant, defendcount);
+            float occupy = f.occupied(m_gsp1(game::gamemode, game::mutators), defendcount);
             adddynlight(vec(f.o).add(vec(0, 0, enttype[AFFINITY].radius)), enttype[AFFINITY].radius*2, skewcolour(f.owner, f.enemy, occupy), 0, 0, DL_KEEP);
         }
     }
@@ -117,17 +108,18 @@ namespace defend
         loopv(st.flags)
         {
             defendstate::flag &f = st.flags[i];
-            float occupy = f.occupied(defendinstant, defendcount);
-            vec colour = skewcolour(f.owner, f.enemy, occupy), dir = vec(f.o).sub(camera1->o);
-            const char *tex = f.hasflag ? hud::arrowtex : (f.owner == game::focus->team && f.enemy ? hud::alerttex : hud::flagtex);
-            float size = hud::radaraffinitysize*(f.hasflag ? 1.25f : 1);
+            float occupy = f.occupied(m_gsp1(game::gamemode, game::mutators), defendcount);
+            vec colour = skewcolour(f.owner, f.enemy, occupy);
+            bool attack = f.owner == game::focus->team && f.enemy;
+            const char *tex = f.hasflag ? hud::arrowtex : (attack ? hud::attacktex : hud::pointtex);
+            float size = hud::radaraffinitysize*(f.hasflag ? 1.5f : (attack ? 0.65f : 1.f));
             if(hud::radaraffinitynames >= (f.hasflag ? 1 : 2))
             {
                 bool overthrow = f.owner && f.enemy == game::focus->team;
-                if(occupy < 1.f) hud::drawblip(tex, f.hasflag ? 3 : 2, w, h, size, blend*hud::radaraffinityblend, f.hasflag ? -1-hud::radarstyle : hud::radarstyle, f.hasflag ? dir : f.o, colour, "little", "\f[%d]%d%%", f.hasflag ? (overthrow ? 0xFF8800 : (occupy < 1.f ? 0xFFFF00 : 0x00FF00)) : TEAM(f.owner, colour), int(occupy*100.f));
-                else hud::drawblip(tex, f.hasflag ? 3 : 2, w, h, size, blend*hud::radaraffinityblend, f.hasflag ? -1-hud::radarstyle : hud::radarstyle, f.hasflag ? dir : f.o, colour, "little", "\f[%d]%s", f.hasflag ? (overthrow ? 0xFF8800 : (occupy < 1.f ? 0xFFFF00 : 0x00FF00)) : TEAM(f.owner, colour), TEAM(f.owner, name));
+                if(occupy < 1.f) hud::drawblip(tex, f.hasflag ? 3 : 2, w, h, size, blend*hud::radaraffinityblend, f.hasflag ? 0 : -1, f.o, colour, "little", "\f[%d]%d%%", f.hasflag ? (overthrow ? 0xFF8800 : (occupy < 1.f ? 0xFFFF00 : 0x00FF00)) : TEAM(f.owner, colour), int(occupy*100.f));
+                else hud::drawblip(tex, f.hasflag ? 3 : 2, w, h, size, blend*hud::radaraffinityblend, f.hasflag ? 0 : -1, f.o, colour, "little", "\f[%d]%s", f.hasflag ? (overthrow ? 0xFF8800 : (occupy < 1.f ? 0xFFFF00 : 0x00FF00)) : TEAM(f.owner, colour), TEAM(f.owner, name));
             }
-            else hud::drawblip(tex, f.hasflag ? 3 : 2, w, h, size, blend*hud::radaraffinityblend, f.hasflag ? -1 : hud::radarstyle, f.hasflag ? dir : f.o, colour);
+            else hud::drawblip(tex, f.hasflag ? 3 : 2, w, h, size, blend*hud::radaraffinityblend, f.hasflag ? 0 : -1, f.o, colour);
         }
     }
 
@@ -147,9 +139,9 @@ namespace defend
             {
                 defendstate::flag &f = st.flags[i];
                 pushfont("emphasis");
-                float occupy = !f.owner || f.enemy ? clamp(f.converted/float((!defendinstant && f.owner ? 2 : 1) * defendcount), 0.f, 1.f) : 1.f;
+                float occupy = !f.owner || f.enemy ? clamp(f.converted/float(defendcount), 0.f, 1.f) : 1.f;
                 bool overthrow = f.owner && f.enemy == game::focus->team;
-                ty += draw_textx("%s \fs\f[%d]\f(%s)\f(%s)\fS \fs%s%d%%\fS", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, overthrow ? "Overthrow" : "Secure", TEAM(f.owner, colour), hud::teamtexname(f.owner), hud::flagtex, overthrow ? "\fy" : (occupy < 1.f ? "\fc" : "\fg"), int(occupy*100.f))*hud::noticescale;
+                ty += draw_textx("%s %s \fs\f[%d]\f(%s)\f(%s)\fS \fs%s%d%%\fS", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, overthrow ? "Overthrow" : "Secure", f.name, TEAM(f.owner, colour), hud::teamtexname(f.owner), hud::pointtex, overthrow ? "\fy" : (occupy < 1.f ? "\fc" : "\fg"), int(occupy*100.f))*hud::noticescale;
                 popfont();
                 break;
             }
@@ -158,10 +150,11 @@ namespace defend
 
     int drawinventory(int x, int y, int s, int m, float blend)
     {
-        int sy = 0;
+        int sy = 0, numflags = st.flags.length(), estsize = ((numflags-1)*s*hud::inventoryskew)+s, fitsize = y-m, size = s;
+        if(estsize > fitsize) size = int((fitsize/float(estsize))*s);
         loopv(st.flags)
         {
-            if(y-sy-s < m) break;
+            if(y-sy-size < m) break;
             defendstate::flag &f = st.flags[i];
             bool hasflag = game::focus->state == CS_ALIVE && insideaffinity(f, game::focus);
             if(f.hasflag != hasflag) { f.hasflag = hasflag; f.lasthad = lastmillis-max(1000-(lastmillis-f.lasthad), 0); }
@@ -170,7 +163,7 @@ namespace defend
             if(headsup || f.hasflag || millis <= 1000)
             {
                 float skew = headsup ? hud::inventoryskew : 0.f,
-                    occupy = f.enemy ? clamp(f.converted/float((!defendinstant && f.owner ? 2 : 1)*defendcount), 0.f, 1.f) : (f.owner ? 1.f : 0.f);
+                    occupy = f.enemy ? clamp(f.converted/float(defendcount), 0.f, 1.f) : (f.owner ? 1.f : 0.f);
                 vec c = vec::hexcolor(TEAM(f.owner, colour)), c1 = c;
                 if(f.enemy)
                 {
@@ -182,16 +175,17 @@ namespace defend
                 else if(millis <= 1000) skew += (1.f-skew)-(clamp(float(millis)/1000.f, 0.f, 1.f)*(1.f-skew));
                 int oldy = y-sy;
                 if(hasflag || f.enemy)
-                    sy += hud::drawitem(hud::flagtex, x, oldy, s, 0, true, false, c.r, c.g, c.b, blend, skew, "super", "%s%d%%", hasflag ? (f.owner && f.enemy == game::focus->team ? "\fy" : (occupy < 1.f ? "\fc" : "\fg")) : "\fw", int(occupy*100.f));
-                else sy += hud::drawitem(hud::flagtex, x, oldy, s, 0, true, false, c.r, c.g, c.b, blend, skew);
+                    sy += hud::drawitem(hud::pointtex, x, oldy, size, 0, true, false, c.r, c.g, c.b, blend, skew, "super", "%s%d%%", hasflag ? (f.owner && f.enemy == game::focus->team ? "\fy" : (occupy < 1.f ? "\fc" : "\fg")) : "\fw", int(occupy*100.f));
+                else sy += hud::drawitem(hud::pointtex, x, oldy, size, 0, true, false, c.r, c.g, c.b, blend, skew);
                 if(f.enemy)
                 {
                     vec c2 = vec::hexcolor(TEAM(f.enemy, colour));
-                    hud::drawitem(hud::attacktex, x, oldy, s, 0.5f, true, false, c2.r, c2.g, c2.b, blend, skew);
-                    hud::drawitembar(x, oldy, s, false, c.r, c.g, c.b, blend, skew, occupy);
+                    hud::drawitem(hud::attacktex, x, oldy, size, 0.5f, true, false, c2.r, c2.g, c2.b, blend, skew);
+                    hud::drawitembar(x, oldy, size, false, c.r, c.g, c.b, blend, skew, occupy);
                 }
                 else if(f.owner)
-                    hud::drawitem(hud::teamtexname(f.owner), x, oldy, s, 0.5f, true, false, c1.r, c1.g, c1.b, blend, skew);
+                    hud::drawitem(hud::teamtexname(f.owner), x, oldy, size, 0.5f, true, false, c1.r, c1.g, c1.b, blend, skew);
+                hud::drawitemtext(x, oldy, size, false, skew, "default", blend, "%s", f.name);
             }
         }
         return sy;
@@ -204,60 +198,57 @@ namespace defend
 
     void setup()
     {
+        int df = m_gsp2(game::gamemode, game::mutators) ? 0 : defendflags;
         loopv(entities::ents)
         {
             extentity *e = entities::ents[i];
             if(e->type != AFFINITY || !m_check(e->attrs[3], e->attrs[4], game::gamemode, game::mutators)) continue;
             int team = e->attrs[0];
-            switch(defendflags)
+            switch(df)
             {
                 case 3:
-                {
                     if(team && !isteam(game::gamemode, game::mutators, team, T_NEUTRAL)) team = T_NEUTRAL;
                     break;
-                }
                 case 2:
-                {
                     if(!isteam(game::gamemode, game::mutators, team, T_FIRST)) continue;
                     break;
-                }
                 case 1:
-                {
                     if(team && !isteam(game::gamemode, game::mutators, team, T_NEUTRAL)) continue;
                     break;
-                }
                 case 0: team = T_NEUTRAL; break;
             }
-            defendstate::flag &b = st.flags.add();
-            b.o = e->o;
-            defformatstring(alias)("flag_%d", e->attrs[4]);
+            defformatstring(alias)("point_%d", e->attrs[5]);
             const char *name = getalias(alias);
-            if(name[0]) copystring(b.name, name);
-            else formatstring(b.name)("flag %d", st.flags.length());
-            b.ent = i;
-            b.kinship = team;
-            b.reset();
+            if(!name || !*name)
+            {
+                formatstring(alias)("point #%d", st.flags.length()+1);
+                name = alias;
+            }
+            st.addaffinity(e->o, team, e->attrs[1], e->attrs[2], name);
         }
         if(!st.flags.length()) return; // map doesn't seem to support this mode at all..
-        int bases[T_ALL] = {0};
-        bool hasteams = true;
-        loopv(st.flags) bases[st.flags[i].kinship]++;
-        loopi(numteams(game::gamemode, game::mutators)-1) if(!bases[i+1] || (bases[i+1] != bases[i+2]))
+        bool hasteams = df != 0;
+        if(hasteams)
         {
-            loopvk(st.flags) st.flags[k].kinship = T_NEUTRAL;
-            hasteams = false;
-            break;
+            int bases[T_ALL] = {0};
+            loopv(st.flags) bases[st.flags[i].kinship]++;
+            loopi(numteams(game::gamemode, game::mutators)-1) if(!bases[i+1] || (bases[i+1] != bases[i+2]))
+            {
+                loopvk(st.flags) st.flags[k].kinship = T_NEUTRAL;
+                hasteams = false;
+                break;
+            }
         }
         if(m_gsp2(game::gamemode, game::mutators))
         {
             vec average(0, 0, 0);
             int count = 0;
-            loopv(st.flags) if(!hasteams || st.flags[i].kinship != T_NEUTRAL)
+            loopv(st.flags)
             {
                 average.add(st.flags[i].o);
                 count++;
             }
-            int smallest = -1;
+            int smallest = rnd(st.flags.length());
             if(count)
             {
                 average.div(count);
@@ -268,11 +259,13 @@ namespace defend
                     dist = tdist;
                 }
             }
-            if(!st.flags.inrange(smallest)) smallest = rnd(st.flags.length());
-            int ent = st.flags[smallest].ent;
-            copystring(st.flags[smallest].name, "the flag");
-            st.flags[smallest].kinship = T_NEUTRAL;
-            loopv(st.flags) if(st.flags[i].ent != ent) st.flags.remove(i--);
+            if(st.flags.inrange(smallest))
+            {
+                copystring(st.flags[smallest].name, "center");
+                st.flags[smallest].kinship = T_NEUTRAL;
+                loopi(smallest) st.flags.remove(0);
+                while(st.flags.length() > 1) st.flags.remove(1);
+            }
         }
     }
 
@@ -284,7 +277,8 @@ namespace defend
         {
             defendstate::flag &b = st.flags[i];
             putint(p, b.kinship);
-            putint(p, b.ent);
+            putint(p, b.yaw);
+            putint(p, b.pitch);
             loopj(3) putint(p, int(b.o[j]*DMF));
             sendstring(b.name, p);
         }
@@ -296,14 +290,15 @@ namespace defend
         while(st.flags.length() > numflags) st.flags.pop();
         loopi(numflags)
         {
-            int kin = getint(p), ent = getint(p), converted = getint(p), owner = getint(p), enemy = getint(p);
+            int kin = getint(p), yaw = getint(p), pitch = getint(p), converted = getint(p), owner = getint(p), enemy = getint(p);
             vec o;
             loopj(3) o[j] = getint(p)/DMF;
             string name;
             getstring(name, p);
             if(p.overread()) break;
+            if(i >= MAXPARAMS) continue;
             while(!st.flags.inrange(i)) st.flags.add();
-            st.initaffinity(i, kin, ent, o, owner, enemy, converted, name);
+            st.initaffinity(i, kin, yaw, pitch, o, owner, enemy, converted, name);
         }
     }
 
@@ -319,24 +314,22 @@ namespace defend
                 {
                     gameent *d = NULL, *e = NULL;
                     int numdyns = game::numdynents();
-                    loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && e->type == ENT_PLAYER && insideaffinity(b, e))
+                    loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && e->actortype < A_ENEMY && insideaffinity(b, e))
                         if((d = e) == game::focus) break;
-                    game::announcef(S_V_FLAGSECURED, CON_INFO, d, true, "\fateam %s secured \fw%s", game::colourteam(owner), b.name);
+                    game::announcef(S_V_FLAGSECURED, CON_SELF, d, true, "\fateam %s secured \fw%s", game::colourteam(owner), b.name);
                     part_textcopy(vec(b.o).add(vec(0, 0, enttype[AFFINITY].radius)), "<super>\fzZeSECURED", PART_TEXT, game::eventiconfade, TEAM(owner, colour), 3, 1, -10);
-                    if(game::dynlighteffects) adddynlight(vec(b.o).add(vec(0, 0, enttype[AFFINITY].radius)), enttype[AFFINITY].radius*2, vec::hexcolor(TEAM(owner, colour)).mul(2.f), 500, 250);
-                    entities::execlink(NULL, b.ent, false);
+                    if(game::dynlighteffects) adddynlight(b.o, enttype[AFFINITY].radius*2, vec::hexcolor(TEAM(owner, colour)).mul(2.f), 500, 250);
                 }
             }
             else if(b.owner)
             {
                 gameent *d = NULL, *e = NULL;
                 int numdyns = game::numdynents();
-                loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && e->type == ENT_PLAYER && insideaffinity(b, e))
+                loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && e->actortype < A_ENEMY && insideaffinity(b, e))
                     if((d = e) == game::focus) break;
-                game::announcef(S_V_FLAGOVERTHROWN, CON_INFO, d, true, "\fateam %s overthrew \fw%s", game::colourteam(enemy), b.name);
+                game::announcef(S_V_FLAGOVERTHROWN, CON_SELF, d, true, "\fateam %s overthrew \fw%s", game::colourteam(enemy), b.name);
                 part_textcopy(vec(b.o).add(vec(0, 0, enttype[AFFINITY].radius)), "<super>\fzZeOVERTHROWN", PART_TEXT, game::eventiconfade, TEAM(enemy, colour), 3, 1, -10);
-                if(game::dynlighteffects) adddynlight(vec(b.o).add(vec(0, 0, enttype[AFFINITY].radius)), enttype[AFFINITY].radius*2, vec::hexcolor(TEAM(enemy, colour)).mul(2.f), 500, 250);
-                entities::execlink(NULL, b.ent, false);
+                if(game::dynlighteffects) adddynlight(b.o, enttype[AFFINITY].radius*2, vec::hexcolor(TEAM(enemy, colour)).mul(2.f), 500, 250);
             }
             b.converted = converted;
         }
@@ -349,13 +342,6 @@ namespace defend
         hud::teamscore(team).total = total;
     }
 
-    int aiowner(gameent *d)
-    {
-        loopv(st.flags) if(entities::ents.inrange(st.flags[i].ent) && entities::ents[d->aientity]->links.find(st.flags[i].ent) >= 0)
-            return st.flags[i].owner ? st.flags[i].owner : st.flags[i].enemy;
-        return d->team;
-    }
-
     bool aicheck(gameent *d, ai::aistate &b)
     {
         return false;
@@ -363,7 +349,7 @@ namespace defend
 
     void aifind(gameent *d, ai::aistate &b, vector<ai::interest> &interests)
     {
-        if(d->aitype == AI_BOT)
+        if(d->actortype == A_BOT)
         {
             vec pos = d->feetpos();
             loopvj(st.flags)
@@ -375,13 +361,13 @@ namespace defend
                 gameent *e = NULL;
                 bool regen = !m_regen(game::gamemode, game::mutators) || d->health >= m_health(game::gamemode, game::mutators, d->model);
                 int numdyns = game::numdynents();
-                loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && !e->ai && e->state == CS_ALIVE && ai::owner(d) == ai::owner(e))
+                loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && !e->ai && e->state == CS_ALIVE && d->team == e->team)
                 {
                     vec ep = e->feetpos();
                     if(targets.find(e->clientnum) < 0 && ep.squaredist(f.o) <= (enttype[AFFINITY].radius*enttype[AFFINITY].radius))
                         targets.add(e->clientnum);
                 }
-                if((!regen && f.owner == ai::owner(d)) || (targets.empty() && (f.owner != ai::owner(d) || f.enemy)))
+                if((!regen && f.owner == d->team) || (targets.empty() && (f.owner != d->team || f.enemy)))
                 {
                     ai::interest &n = interests.add();
                     n.state = ai::AI_S_DEFEND;
@@ -391,6 +377,7 @@ namespace defend
                     n.score = pos.squaredist(f.o)/(!regen ? 100.f : 1.f);
                     n.tolerance = 0.25f;
                     n.team = true;
+                    n.acttype = ai::AI_A_PROTECT;
                 }
             }
         }
@@ -401,19 +388,19 @@ namespace defend
         if(st.flags.inrange(b.target))
         {
             defendstate::flag &f = st.flags[b.target];
-            bool regen = d->aitype != AI_BOT || !m_regen(game::gamemode, game::mutators) || d->health >= m_health(game::gamemode, game::mutators, d->model);
-            int walk = f.enemy && f.enemy != ai::owner(d) ? 1 : 0;
-            if(regen && (!f.enemy && ai::owner(d) == f.owner))
+            bool regen = d->actortype != A_BOT || !m_regen(game::gamemode, game::mutators) || d->health >= m_health(game::gamemode, game::mutators, d->model);
+            int walk = f.enemy && f.enemy != d->team ? 1 : 0;
+            if(regen && (!f.enemy && d->team == f.owner))
             {
                 static vector<int> targets; // build a list of others who are interested in this
                 targets.setsize(0);
                 ai::checkothers(targets, d, ai::AI_S_DEFEND, ai::AI_T_AFFINITY, b.target, true);
-                if(d->aitype == AI_BOT)
+                if(d->actortype == A_BOT)
                 {
                     gameent *e = NULL;
                     int numdyns = game::numdynents();
                     float mindist = enttype[AFFINITY].radius*4; mindist *= mindist;
-                    loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && !e->ai && e->state == CS_ALIVE && ai::owner(d) == ai::owner(e))
+                    loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && !e->ai && e->state == CS_ALIVE && d->team == e->team)
                     {
                         vec ep = e->feetpos();
                         if(targets.find(e->clientnum) < 0 && ep.squaredist(f.o) <= mindist)
@@ -431,7 +418,7 @@ namespace defend
                 }
                 else walk = 1;
             }
-            return ai::defense(d, b, f.o, !f.enemy || m_gsp2(game::gamemode, game::mutators) ? ai::CLOSEDIST : float(enttype[AFFINITY].radius), !f.enemy || m_gsp2(game::gamemode, game::mutators) ? ai::CLOSEDIST : float(enttype[AFFINITY].radius*walk*8), walk);
+            return ai::defense(d, b, f.o, enttype[AFFINITY].radius, enttype[AFFINITY].radius*walk*8, m_gsp2(game::gamemode, game::mutators) ? 0 : walk);
         }
         return false;
     }
