@@ -1,9 +1,7 @@
 #ifdef GAMESERVER
     #define defendstate stfservstate
-    #define defendinstant (m_gsp1(gamemode, mutators) || m_gsp2(gamemode, mutators))
     #define defendcount (m_gsp2(gamemode, mutators) ? G(defendking) : G(defendoccupy))
 #else
-    #define defendinstant (m_gsp1(game::gamemode, game::mutators) || m_gsp2(game::gamemode, game::mutators))
     #define defendcount (m_gsp2(game::gamemode, game::mutators) ? G(defendking) : G(defendoccupy))
 #endif
 
@@ -12,21 +10,20 @@ struct defendstate
     struct flag
     {
         vec o;
-        int kinship, ent, owner, enemy;
+        int kinship, yaw, pitch, owner, enemy;
         string name;
 #ifndef GAMESERVER
         string info;
         bool hasflag;
         int lasthad;
+        vec render, above;
+        entitylight baselight;
 #endif
-        int owners, enemies, converted, securetime;
+        int owners, enemies, converted, points;
 
         flag()
         {
             kinship = T_NEUTRAL;
-#ifndef GAMESERVER
-            ent = -1;
-#endif
             reset();
         }
 
@@ -41,8 +38,7 @@ struct defendstate
         {
             noenemy();
             owner = kinship;
-            securetime = -1;
-            owners = 0;
+            yaw = pitch = owners = points = 0;
 #ifndef GAMESERVER
             hasflag = false;
             lasthad = 0;
@@ -92,51 +88,78 @@ struct defendstate
         {
             if(enemy != team) return -1;
             converted += units;
-            if(units<0)
+
+            if(units < 0)
             {
-                if(converted<=0) noenemy();
+                if(converted <= 0) noenemy();
                 return -1;
             }
-            else if(converted<(!instant && owner ? 2 : 1)*occupy) return -1;
-            if(!instant && owner) { owner = T_NEUTRAL; converted = 0; enemy = team; return 0; }
-            else { owner = team; securetime = 0; owners = enemies; noenemy(); return 1; }
+            else if(converted < occupy) return -1;
+
+            if(!instant && owner)
+            {
+                owner = T_NEUTRAL;
+                converted = points = 0;
+                enemy = team;
+                return 0;
+            }
+            else
+            {
+                owner = team;
+                points = 0;
+                owners = enemies;
+                noenemy();
+                return 1;
+            }
         }
 
         float occupied(bool instant, float amt)
         {
-            return (enemy ? enemy : owner) ? (!owner || enemy ? clamp(converted/float((!instant && owner ? 2 : 1)*amt), 0.f, 1.f) : 1.f) : 0.f;
+            return (enemy ? enemy : owner) ? (!owner || enemy ? clamp(converted/amt, 0.f, 1.f) : 1.f) : 0.f;
         }
     };
 
     vector<flag> flags;
-    int secured;
 
-    defendstate() : secured(0) {}
+    defendstate() { reset(); }
 
     void reset()
     {
         flags.shrink(0);
-        secured = 0;
     }
 
-    void addaffinity(const vec &o, int team, int ent, const char *name)
+    void addaffinity(const vec &o, int team, int yaw, int pitch, const char *name)
     {
         flag &b = flags.add();
-        b.o = o;
         b.kinship = team;
         b.reset();
-        b.ent = ent;
+        b.o = o;
+#ifndef GAMESERVER
+        b.render = b.above = o;
+        b.render.z += 2;
+        physics::droptofloor(b.render);
+        if(b.render.z >= b.above.z-1) b.above.z += (b.render.z-(b.above.z-1))+2;
+#endif
+        b.yaw = yaw;
+        b.pitch = pitch;
         copystring(b.name, name);
     }
 
-    void initaffinity(int i, int kin, int ent, vec &o, int owner, int enemy, int converted, const char *name)
+    void initaffinity(int i, int kin, int yaw, int pitch, vec &o, int owner, int enemy, int converted, const char *name)
     {
         if(!flags.inrange(i)) return;
         flag &b = flags[i];
         b.kinship = kin;
         b.reset();
-        b.ent = ent;
+        b.yaw = yaw;
+        b.pitch = pitch;
         b.o = o;
+#ifndef GAMESERVER
+        b.render = b.above = o;
+        b.render.z += 2;
+        physics::droptofloor(b.render);
+        if(b.render.z >= b.above.z-1) b.above.z += (b.render.z-(b.above.z-1))+2;
+#endif
         b.owner = owner;
         b.enemy = enemy;
         b.converted = converted;
@@ -189,7 +212,6 @@ namespace defend
     extern void preload();
     extern void render();
     extern void adddynlights();
-    extern int aiowner(gameent *d);
     extern void aifind(gameent *d, ai::aistate &b, vector<ai::interest> &interests);
     extern bool aicheck(gameent *d, ai::aistate &b);
     extern bool aidefense(gameent *d, ai::aistate &b);
