@@ -5,7 +5,7 @@ struct defendservmode : defendstate, servmode
 
     defendservmode() : scoresec(0), hasflaginfo(false) {}
 
-    void reset(bool empty)
+    void reset()
     {
         defendstate::reset();
         scoresec = 0;
@@ -15,7 +15,7 @@ struct defendservmode : defendstate, servmode
     void stealaffinity(int n, int team)
     {
         flag &b = flags[n];
-        loopv(clients) if(clients[i]->state.aitype < AI_START)
+        loopv(clients) if(clients[i]->state.actortype < A_ENEMY)
         {
             server::clientinfo *ci = clients[i];
             if(ci->state.state==CS_ALIVE && ci->team && ci->team == team && insideaffinity(b, ci->state.o))
@@ -52,7 +52,7 @@ struct defendservmode : defendstate, servmode
     {
         if(!points) return;
         flag &b = flags[i];
-        loopvk(clients) if(clients[k]->state.aitype < AI_START && team == clients[k]->team && insideaffinity(b, clients[k]->state.o)) givepoints(clients[k], points);
+        if(!m_nopoints(gamemode, mutators)) loopvk(clients) if(clients[k]->state.actortype < A_ENEMY && team == clients[k]->team && insideaffinity(b, clients[k]->state.o)) givepoints(clients[k], points);
         score &cs = teamscore(team);
         cs.total += points;
         sendf(-1, 1, "ri3", N_SCORE, team, cs.total);
@@ -71,8 +71,8 @@ struct defendservmode : defendstate, servmode
             {
                 if(!b.owners || !b.enemies)
                 {
-                    int pts = b.occupy(b.enemy, G(defendpoints)*(b.enemies ? b.enemies : -(1+b.owners))*t, defendcount, defendinstant);
-                    if(pts > 0) loopvk(clients) if(clients[k]->state.aitype < AI_START && b.owner == clients[k]->team && insideaffinity(b, clients[k]->state.o)) givepoints(clients[k], G(defendpoints));
+                    int pts = b.occupy(b.enemy, G(defendpoints)*(b.enemies ? b.enemies : -(1+b.owners))*t, defendcount, m_gsp1(gamemode, mutators));
+                    if(!m_nopoints(gamemode, mutators) && pts > 0) loopvk(clients) if(clients[k]->state.actortype < A_ENEMY && b.owner == clients[k]->team && insideaffinity(b, clients[k]->state.o)) givepoints(clients[k], G(defendpoints));
                 }
                 sendaffinity(i);
             }
@@ -80,8 +80,13 @@ struct defendservmode : defendstate, servmode
             {
                 if(!m_gsp2(gamemode, mutators) || b.owners)
                 {
-                    b.securetime += t;
-                    int score = b.securetime/G(defendinterval) - (b.securetime-t)/G(defendinterval);
+                    b.points += G(defendpoints)*t;
+                    int score = 0;
+                    while(b.points >= G(defendhold))
+                    {
+                        b.points -= G(defendhold);
+                        score++;
+                    }
                     if(score) addscore(i, b.owner, score);
                 }
                 sendaffinity(i);
@@ -111,7 +116,8 @@ struct defendservmode : defendstate, servmode
         {
             flag &b = flags[i];
             putint(p, b.kinship);
-            putint(p, b.ent);
+            putint(p, b.yaw);
+            putint(p, b.pitch);
             putint(p, b.converted);
             putint(p, b.owner);
             putint(p, b.enemy);
@@ -137,7 +143,7 @@ struct defendservmode : defendstate, servmode
 
     void endcheck()
     {
-        if(m_balance(gamemode, mutators)) return;
+        if(m_balance(gamemode, mutators, teamspawns)) return;
         int maxscore = G(defendlimit) ? G(defendlimit) : INT_MAX-1;
         loopi(numteams(gamemode, mutators))
         {
@@ -154,37 +160,37 @@ struct defendservmode : defendstate, servmode
 
     void entergame(clientinfo *ci)
     {
-        if(!hasflaginfo || ci->state.state!=CS_ALIVE || ci->state.aitype >= AI_START) return;
+        if(!canplay(hasflaginfo) || ci->state.state!=CS_ALIVE || ci->state.actortype >= A_ENEMY) return;
         enteraffinity(ci->team, ci->state.o);
     }
 
     void spawned(clientinfo *ci)
     {
-        if(!hasflaginfo || ci->state.aitype >= AI_START) return;
+        if(!canplay(hasflaginfo) || ci->state.actortype >= A_ENEMY) return;
         enteraffinity(ci->team, ci->state.o);
     }
 
     void leavegame(clientinfo *ci, bool disconnecting = false)
     {
-        if(!hasflaginfo || ci->state.state!=CS_ALIVE || ci->state.aitype >= AI_START) return;
+        if(!canplay(hasflaginfo) || ci->state.state!=CS_ALIVE || ci->state.actortype >= A_ENEMY) return;
         leaveaffinity(ci->team, ci->state.o);
     }
 
-    void died(clientinfo *ci, clientinfo *actor)
+    void died(clientinfo *ci, clientinfo *v)
     {
-        if(!hasflaginfo || ci->state.aitype >= AI_START) return;
+        if(!canplay(hasflaginfo) || ci->state.actortype >= A_ENEMY) return;
         leaveaffinity(ci->team, ci->state.o);
     }
 
     void moved(clientinfo *ci, const vec &oldpos, const vec &newpos)
     {
-        if(!hasflaginfo || ci->state.aitype >= AI_START) return;
+        if(!canplay(hasflaginfo) || ci->state.actortype >= A_ENEMY) return;
         moveaffinity(ci->team, oldpos, newpos);
     }
 
     void regen(clientinfo *ci, int &total, int &amt, int &delay)
     {
-        if(!hasflaginfo || !G(defendregenbuff) || !ci->state.lastbuff) return;
+        if(!canplay(hasflaginfo) || !G(defendregenbuff) || !ci->state.lastbuff) return;
         if(G(maxhealth)) total = max(m_maxhealth(gamemode, mutators, ci->state.model), total);
         if(ci->state.lastregen && G(defendregendelay)) delay = G(defendregendelay);
         if(G(defendregenextra)) amt += G(defendregenextra);
@@ -192,21 +198,21 @@ struct defendservmode : defendstate, servmode
 
     void checkclient(clientinfo *ci)
     {
-        if(!hasflaginfo || ci->state.state != CS_ALIVE || m_insta(gamemode, mutators)) return;
-        #define defendbuff4 (G(defendbuffing)&4 && b.occupied(defendinstant, defendcount) >= G(defendbuffoccupy))
+        if(!canplay(hasflaginfo) || ci->state.state != CS_ALIVE || m_insta(gamemode, mutators)) return;
+        #define defendbuff4 (G(defendbuffing)&4 && b.occupied(m_gsp1(gamemode, mutators), defendcount) >= G(defendbuffoccupy))
         #define defendbuff1 (G(defendbuffing)&1 && b.owner == ci->team && (!b.enemy || defendbuff4))
         #define defendbuff2 (G(defendbuffing)&2 && b.owner == T_NEUTRAL && (b.enemy == ci->team || defendbuff4))
         if(G(defendbuffing)) loopv(flags)
         {
             flag &b = flags[i];
-            if((defendbuff1 || defendbuff2) && insideaffinity(b, ci->state.o, G(defendbuffarea)))
+            if((defendbuff1 || defendbuff2) && (G(defendbuffarea) ? insideaffinity(b, ci->state.o, G(defendbuffarea)) : true))
             {
                 if(!ci->state.lastbuff) sendf(-1, 1, "ri4", N_SPHY, ci->clientnum, SPHY_BUFF, 1);
                 ci->state.lastbuff = gamemillis;
                 return;
             }
         }
-        if(ci->state.lastbuff && (!G(defendbuffing) || gamemillis-ci->state.lastbuff > G(defendbuffdelay)))
+        if(ci->state.lastbuff && (!G(defendbuffing) || gamemillis-ci->state.lastbuff >= G(defendbuffdelay)))
         {
             ci->state.lastbuff = 0;
             sendf(-1, 1, "ri4", N_SPHY, ci->clientnum, SPHY_BUFF, 0);
@@ -220,12 +226,13 @@ struct defendservmode : defendstate, servmode
         {
             loopi(numflags)
             {
-                int kin = getint(p), ent = getint(p);
+                int kin = getint(p), yaw = getint(p), pitch = getint(p);
                 vec o;
                 loopj(3) o[j] = getint(p)/DMF;
                 string name;
                 getstring(name, p);
-                if(!hasflaginfo) addaffinity(o, kin, ent, name);
+                if(p.overread()) break;
+                if(!hasflaginfo && i < MAXPARAMS) addaffinity(o, kin, yaw, pitch, name);
             }
             if(!hasflaginfo)
             {
@@ -236,11 +243,11 @@ struct defendservmode : defendstate, servmode
         }
     }
 
-    int points(clientinfo *victim, clientinfo *actor)
+    int points(clientinfo *m, clientinfo *v)
     {
-        bool isteam = victim==actor || victim->team == actor->team;
-        int p = isteam ? -1 : 1, v = p;
-        loopv(flags) if(insideaffinity(flags[i], victim->state.o)) p += v;
+        bool isteam = m==v || m->team == v->team;
+        int p = isteam ? -1 : 1, q = p;
+        loopv(flags) if(insideaffinity(flags[i], m->state.o)) p += q;
         return p;
     }
 
