@@ -2,17 +2,32 @@
 struct duelservmode : servmode
 {
     int duelround, dueltime, duelcheck, dueldeath, duelwinner, duelwins;
+    bool waitforhumans;
     vector<clientinfo *> duelqueue, allowed, playing, respawns;
 
     duelservmode() {}
 
-    bool checkready(clientinfo *ci)
+    void shrink()
     {
-        if(ci->state.aitype < AI_START)
-        {
-            if(m_loadout(gamemode, mutators) && !chkloadweap(ci, false)) return false;
-        }
-        return true;
+        allowed.shrink(0);
+        playing.shrink(0);
+        respawns.shrink(0);
+    }
+
+    void doreset()
+    {
+        dueltime = G(duelcooloff);
+        duelcheck = dueldeath = -1;
+    }
+
+    void reset()
+    {
+        waitforhumans = true;
+        shrink();
+        duelqueue.shrink(0);
+        doreset();
+        duelround = duelwins = 0;
+        duelwinner = -1;
     }
 
     void position(clientinfo *ci, int n)
@@ -28,28 +43,36 @@ struct duelservmode : servmode
 
     void queue(clientinfo *ci, bool pos = true, bool top = false, bool wait = true)
     {
-        if(ci->state.state != CS_SPECTATOR && ci->state.state != CS_EDITING && ci->state.aitype < AI_START)
+        if(ci->state.actortype >= A_ENEMY || ci->state.state == CS_SPECTATOR) return;
+        if(ci->state.actortype == A_PLAYER && waitforhumans) waitforhumans = false;
+        int n = duelqueue.find(ci);
+        if(top)
         {
-            int n = duelqueue.find(ci);
-            if(top)
-            {
-                if(n >= 0) duelqueue.remove(n);
-                duelqueue.insert(0, ci);
-                n = 0;
-            }
-            else if(n < 0)
-            {
-                n = duelqueue.length();
-                duelqueue.add(ci);
-            }
-            if(allowed.find(ci) >= 0) allowed.removeobj(ci);
-            if(respawns.find(ci) >= 0) respawns.removeobj(ci);
-            if(wait && ci->state.state != CS_WAITING) waiting(ci, DROP_RESET);
-            if(pos) position(ci, n);
+            if(n >= 0) duelqueue.remove(n);
+            duelqueue.insert(0, ci);
+            n = 0;
         }
+        else if(n < 0)
+        {
+            n = duelqueue.length();
+            duelqueue.add(ci);
+        }
+        if(allowed.find(ci) >= 0) allowed.removeobj(ci);
+        if(respawns.find(ci) >= 0) respawns.removeobj(ci);
+        if(wait && ci->state.state != CS_WAITING) waiting(ci, DROP_RESET);
+        if(pos) position(ci, n);
     }
 
-    void entergame(clientinfo *ci) { queue(ci); }
+    void entergame(clientinfo *ci)
+    {
+        queue(ci);
+        if(dueltime < 0 && dueldeath < 0 && m_affinity(gamemode)) switch(G(duelaffinity))
+        {
+            case 2: allowed.add(ci); // fall through because they need to be in respawns too
+            case 1: respawns.add(ci); break;
+            case 0: default: break;
+        }
+    }
     void leavegame(clientinfo *ci, bool disconnecting = false)
     {
         if(duelwinner == ci->clientnum)
@@ -63,46 +86,45 @@ struct duelservmode : servmode
         respawns.removeobj(ci);
     }
 
-    bool damage(clientinfo *target, clientinfo *actor, int damage, int weap, int flags, int material, const ivec &hitpush)
+    bool damage(clientinfo *m, clientinfo *v, int damage, int weap, int flags, int material, const ivec &hitpush, const ivec &hitvel, float dist)
     {
-        if(dueltime >= 0 && target->state.aitype < AI_START) return false;
+        if(dueltime >= 0 && m->state.actortype < A_ENEMY) return false;
         return true;
     }
 
     bool canspawn(clientinfo *ci, bool tryspawn = false)
     {
-        if(!checkready(ci)) return false;
-        if(allowed.find(ci) >= 0 || ci->state.aitype >= AI_START) return true;
+        if(allowed.find(ci) >= 0 || ci->state.actortype >= A_ENEMY) return true;
+        else if(tryspawn)
+        {
+            if(!m_affinity(gamemode)) queue(ci);
+            return true;
+        }
         if(m_affinity(gamemode) && respawns.find(ci) >= 0)
         {
+            int delay = m_xdelay(gamemode, mutators, ci->team);
+            if(delay && ci->state.respawnwait(gamemillis, delay)) return false;
             if(m_survivor(gamemode, mutators))
             {
                 int alive = 0;
                 vector<clientinfo *> mates;
-                loopv(clients) if(clients[i]->state.aitype < AI_START && clients[i]->team == ci->team)
+                loopv(clients) if(clients[i]->state.actortype < A_ENEMY && clients[i]->team == ci->team)
                 { // includes ci
-                    if(clients[i]->state.state == CS_ALIVE) alive++;
                     mates.add(clients[i]);
+                    if(clients[i]->state.state == CS_ALIVE && (!G(duelbotcheck) || clients[i]->state.actortype != A_BOT)) alive++;
                 }
                 if(!alive)
                 {
-                    loopv(mates)
-                    {
-                        if(allowed.find(mates[i]) < 0) allowed.add(mates[i]);
-                        if(mates[i]->state.state == CS_DEAD) waiting(mates[i], DROP_RESET);
-                    }
+                    loopv(mates) if(allowed.find(mates[i]) < 0) allowed.add(mates[i]);
                     duelcheck = gamemillis+1000;
                     return true;
                 }
                 return false; // only respawn when all dead
             }
-            int delay = m_xdelay(gamemode, mutators);
-            if(delay && ci->state.respawnwait(gamemillis, delay)) return false;
             if(allowed.find(ci) < 0) allowed.add(ci);
             duelcheck = gamemillis+1000;
             return true; // overrides it
         }
-        if(tryspawn && dueltime < 0 && dueldeath < 0) queue(ci);
         return false; // you spawn when we want you to buddy
     }
 
@@ -113,7 +135,7 @@ struct duelservmode : servmode
 
     void layout()
     {
-        loopvj(clients) if(clients[j]->state.aitype < AI_START)
+        loopvj(clients) if(clients[j]->state.actortype < A_ENEMY)
         {
             vector<int> shots;
             loop(a, W_MAX) loop(b, 2)
@@ -131,15 +153,16 @@ struct duelservmode : servmode
 
     void scoreaffinity(clientinfo *ci, bool win)
     {
-        if(!m_affinity(gamemode) || dueltime >= 0 || duelround <= 0 || !win) return;
+        if(!m_affinity(gamemode) || dueltime >= 0 || duelround <= 0) return;
         respawns.shrink(0);
-        loopv(clients) if(clients[i]->state.aitype < AI_START) switch(clients[i]->state.state)
+        #define scoredteam(x,y) (win ? x != y : x == y)
+        loopv(clients) if(clients[i]->state.actortype < A_ENEMY) switch(clients[i]->state.state)
         {
             case CS_ALIVE:
-                if(playing.find(clients[i]) < 0 || clients[i]->team != ci->team) queue(clients[i], false);
+                if(playing.find(clients[i]) < 0 || scoredteam(clients[i]->team, ci->team)) queue(clients[i], false);
                 break;
             case CS_DEAD:
-                if(playing.find(clients[i]) < 0 || clients[i]->team != ci->team)
+                if(playing.find(clients[i]) < 0 || scoredteam(clients[i]->team, ci->team))
                 {
                     queue(clients[i], false);
                     break;
@@ -149,7 +172,7 @@ struct duelservmode : servmode
                 duelcheck = gamemillis+1000;
                 break;
             case CS_WAITING:
-                if(playing.find(clients[i]) < 0 || clients[i]->team != ci->team)
+                if(playing.find(clients[i]) < 0 || scoredteam(clients[i]->team, ci->team))
                 {
                     queue(clients[i], false);
                     break;
@@ -163,8 +186,7 @@ struct duelservmode : servmode
 
     void clear()
     {
-        duelcheck = dueldeath = -1;
-        dueltime = gamemillis+G(duellimit);
+        doreset();
         bool reset = false;
         if(m_duel(gamemode, mutators) && G(duelcycle)&(m_team(gamemode, mutators) ? 2 : 1) && duelwinner >= 0 && duelwins > 0)
         {
@@ -173,7 +195,7 @@ struct duelservmode : servmode
             {
                 int numwins = G(duelcycles), numplrs = 0;
                 loopv(clients)
-                    if(clients[i]->state.aitype < AI_START && clients[i]->state.state != CS_SPECTATOR && clients[i]->team == ci->team)
+                    if(clients[i]->state.actortype < A_ENEMY && clients[i]->state.state != CS_SPECTATOR && clients[i]->team == ci->team)
                         numplrs++;
                 if(numplrs > (m_team(gamemode, mutators) ? 1 : 2))
                 {
@@ -188,74 +210,80 @@ struct duelservmode : servmode
             }
         }
         loopv(clients) queue(clients[i], false, !reset && clients[i]->state.state == CS_ALIVE, reset || G(duelreset) || clients[i]->state.state != CS_ALIVE);
-        allowed.shrink(0);
-        playing.shrink(0);
-        respawns.shrink(0);
+        shrink();
     }
 
     void update()
     {
-        if(interm || !hasgameinfo || numclients(-1, true, AI_BOT) <= 1) return;
+        if(!canplay() || waitforhumans) return;
         if(dueltime >= 0)
         {
-            if(gamemillis >= dueltime && !duelqueue.empty())
+            if(dueltime && ((dueltime -= curtime) <= 0)) dueltime = 0;
+            if(!dueltime)
             {
+                shrink();
                 int wants = max(numteams(gamemode, mutators), 2);
+                loopv(clients) if(clients[i]->state.state != CS_SPECTATOR && clients[i]->state.state != CS_ALIVE) queue(clients[i], false);
                 loopv(duelqueue)
                 {
                     if(m_duel(gamemode, mutators) && playing.length() >= wants) break;
-                    clientinfo *ci = duelqueue[i];
-                    if(!checkready(ci)) continue; // they are not ready yet.
-                    if(ci->state.state != CS_ALIVE)
+                    if(duelqueue[i]->state.state != CS_ALIVE)
                     {
-                        if(ci->state.state != CS_WAITING) waiting(ci, DROP_RESET);
+                        if(duelqueue[i]->state.state != CS_WAITING) waiting(duelqueue[i], DROP_RESET);
                         if(m_duel(gamemode, mutators) && m_team(gamemode, mutators))
                         {
                             bool skip = false;
-                            loopvj(playing) if(ci->team == playing[j]->team) { skip = true; break; }
+                            loopvj(playing) if(duelqueue[i]->team == playing[j]->team) { skip = true; break; }
                             if(skip) continue;
                         }
-                        if(allowed.find(ci) < 0) allowed.add(ci);
                     }
-                    else
-                    {
-                        ci->state.lastregen = gamemillis;
-                        ci->state.resetresidual();
-                        ci->state.health = m_health(gamemode, mutators, ci->state.model);
-                        ci->state.armour = m_armour(gamemode, mutators, ci->state.model);
-                        sendf(-1, 1, "ri5", N_REGEN, ci->clientnum, ci->state.health, ci->state.armour, 0); // amt = 0 regens impulse
-                    }
-                    playing.add(ci);
-                    if(m_affinity(gamemode)) respawns.add(ci);
+                    playing.add(duelqueue[i]);
                 }
-                loopv(playing) duelqueue.removeobj(playing[i]);
                 if(playing.length() >= wants)
                 {
                     if(smode) smode->layout();
                     mutate(smuts, mut->layout());
-                    loopv(duelqueue) position(duelqueue[i], i);
                     duelround++;
-                    string fight;
+                    string fight = "";
                     if(m_duel(gamemode, mutators))
                     {
-                        defformatstring(namea)("%s", colourname(playing[0]));
-                        defformatstring(nameb)("%s", colourname(playing[1]));
-                        formatstring(fight)("\fyduel between %s and %s, round \fs\fr#%d\fS", namea, nameb, duelround);
+                        string names = "";
+                        loopv(playing)
+                        {
+                            concatstring(names, colourname(playing[i]));
+                            if(i == wants-1) break;
+                            else if(i == wants-2) concatstring(names, " and ");
+                            else concatstring(names, ", ");
+                        }
+                        formatstring(fight)("\fyduel between %s, round \fs\fr#%d\fS", names, duelround);
                     }
-                    else if(m_survivor(gamemode, mutators))
-                        formatstring(fight)("\fysurvivor, round \fs\fr#%d\fS", duelround);
+                    else if(m_survivor(gamemode, mutators)) formatstring(fight)("\fysurvivor, round \fs\fr#%d\fS", duelround);
+                    loopv(playing)
+                    {
+                        if(playing[i]->state.state == CS_ALIVE)
+                        {
+                            playing[i]->state.lastregen = gamemillis;
+                            playing[i]->state.resetresidual();
+                            playing[i]->state.health = m_health(gamemode, mutators, playing[i]->state.model);
+                            sendf(-1, 1, "ri4", N_REGEN, playing[i]->clientnum, playing[i]->state.health, 0); // amt = 0 regens impulse
+                        }
+                        else if(allowed.find(playing[i]) < 0) allowed.add(playing[i]);
+                        duelqueue.removeobj(playing[i]);
+                        if(m_affinity(gamemode)) respawns.add(playing[i]);
+                    }
+                    loopv(duelqueue) position(duelqueue[i], i);
                     ancmsgft(-1, S_V_FIGHT, CON_INFO, fight);
                     dueltime = dueldeath = -1;
                     duelcheck = gamemillis+5000;
                 }
-                else loopv(clients) if(playing.find(clients[i]) < 0) queue(clients[i], false);
+                else shrink();
             }
         }
         else if(duelround > 0)
         {
             bool cleanup = false;
             vector<clientinfo *> alive;
-            loopv(clients) if(clients[i]->state.aitype < AI_START && clients[i]->state.state == CS_ALIVE)
+            loopv(clients) if(clients[i]->state.actortype < A_ENEMY && clients[i]->state.state == CS_ALIVE)
             {
                 if(playing.find(clients[i]) < 0) queue(clients[i]);
                 else alive.add(clients[i]);
@@ -274,7 +302,7 @@ struct duelservmode : servmode
                     loopv(alive) if(i && alive[i]->team != alive[i-1]->team) { found = true; break; }
                     if(!found)
                     {
-                        if(dueldeath < 0) dueldeath = gamemillis+DEATHMILLIS;
+                        if(dueldeath < 0) dueldeath = gamemillis+G(dueldelay);
                         else if(gamemillis >= dueldeath)
                         {
                             if(!cleanup)
@@ -294,7 +322,7 @@ struct duelservmode : servmode
                                     }
                                     else ancmsgft(clients[i]->clientnum, S_V_YOULOSE, CON_INFO, end);
                                 }
-                                else ancmsgft(clients[i]->clientnum, S_V_NOTIFY, CON_INFO, end);
+                                else ancmsgft(clients[i]->clientnum, S_V_BOMBSCORE, CON_INFO, end);
                             }
                             clear();
                         }
@@ -307,10 +335,7 @@ struct duelservmode : servmode
                         if(m_affinity(gamemode) && !playing.empty()) break; // this should not happen
                         if(!cleanup)
                         {
-                            defformatstring(end)("\fyeveryone died, \fzoyepic fail");
-                            loopv(clients) if(playing.find(clients[i]) >= 0)
-                                ancmsgft(clients[i]->clientnum, S_V_DRAW, CON_INFO, end);
-                            else ancmsgft(clients[i]->clientnum, S_V_NOTIFY, CON_INFO, end);
+                            ancmsgft(-1, S_V_DRAW, CON_INFO, "\fyeveryone died, \fzoyepic fail");
                             duelwinner = -1;
                             duelwins = 0;
                         }
@@ -323,7 +348,7 @@ struct duelservmode : servmode
                         {
                             if(dueldeath < 0)
                             {
-                                dueldeath = gamemillis+DEATHMILLIS;
+                                dueldeath = gamemillis+G(dueldelay);
                                 break;
                             }
                             else if(gamemillis < dueldeath) break;
@@ -331,9 +356,9 @@ struct duelservmode : servmode
                         if(!cleanup)
                         {
                             string end, hp; hp[0] = 0;
-                            if(!m_affinity(gamemode))
+                            if(!m_insta(gamemode, mutators) && !m_affinity(gamemode))
                             {
-                                if(!m_insta(gamemode, mutators) && !m_vampire(gamemode, mutators) && alive[0]->state.health == m_health(gamemode, mutators, alive[0]->state.model))
+                                if(alive[0]->state.health >= m_health(gamemode, mutators, alive[0]->state.model))
                                     formatstring(hp)(" with a \fs\fcflawless victory\fS");
                                 else formatstring(hp)(" with \fs\fc%d\fS health left", alive[0]->state.health);
                             }
@@ -348,16 +373,19 @@ struct duelservmode : servmode
                                 duelwins++;
                                 formatstring(end)("\fy%s was the winner%s (\fs\fc%d\fS in a row)", colourname(alive[0]), hp, duelwins);
                             }
-                            loopv(clients) if(playing.find(clients[i]) >= 0)
+                            loopv(clients)
                             {
-                                if(clients[i] == alive[0])
+                                if(playing.find(clients[i]) >= 0)
                                 {
-                                    ancmsgft(clients[i]->clientnum, S_V_YOUWIN, CON_INFO, end);
-                                    if(!m_affinity(gamemode)) givepoints(clients[i], 1);
+                                    if(clients[i] == alive[0])
+                                    {
+                                        ancmsgft(clients[i]->clientnum, S_V_YOUWIN, CON_INFO, end);
+                                        if(!m_affinity(gamemode)) givepoints(clients[i], 1);
+                                    }
+                                    else ancmsgft(clients[i]->clientnum, S_V_YOULOSE, CON_INFO, end);
                                 }
-                                else ancmsgft(clients[i]->clientnum, S_V_YOULOSE, CON_INFO, end);
+                                else ancmsgft(clients[i]->clientnum, S_V_BOMBSCORE, CON_INFO, end);
                             }
-                            else ancmsgft(clients[i]->clientnum, S_V_NOTIFY, CON_INFO, end);
                         }
                         clear();
                         break;
@@ -370,20 +398,19 @@ struct duelservmode : servmode
 
     bool wantsovertime()
     {
-        if((dueltime < 0 || dueltime > gamemillis) && duelround > 0) return true;
-        return 0;
+        if(dueltime < 0 && duelround > 0) return true;
+        return false;
     }
 
-    void reset(bool empty)
+    bool canbalance()
     {
-        duelround = duelwins = 0;
-        duelwinner = -1;
-        duelcheck = dueldeath = -1;
-        dueltime = gamemillis+G(duelwarmup);
-        allowed.shrink(0);
-        playing.shrink(0);
-        duelqueue.shrink(0);
-        respawns.shrink(0);
+        if(dueltime < 0 && duelround > 0) return false;
+        return true;
+    }
+
+    void balance(int oldbalance)
+    {
+        doreset();
     }
 } duelmutator;
 #endif
