@@ -40,20 +40,20 @@ namespace ai
         NUMWPCACHES
     };
 
-    struct wpcachenode
-    {
-        float split[2];
-        uint child[2];
-
-        int axis() const { return child[0]>>30; }
-        int childindex(int which) const { return child[which]&0x3FFFFFFF; }
-        bool isleaf(int which) const { return (child[1]&(1<<(30+which)))!=0; }
-    };
-
     struct wpcache
     {
-        vector<wpcachenode> nodes;
-        int firstwp, lastwp, maxdepth;
+        struct node
+        {
+            float split[2];
+            uint child[2];
+
+            int axis() const { return child[0]>>30; }
+            int childindex(int which) const { return child[which]&0x3FFFFFFF; }
+            bool isleaf(int which) const { return (child[1]&(1<<(30+which)))!=0; }
+        };
+
+        vector<node> nodes;
+        int firstwp, lastwp;
         vec bbmin, bbmax;
 
         wpcache() { clear(); }
@@ -62,12 +62,11 @@ namespace ai
         {
             nodes.setsize(0);
             firstwp = lastwp = -1;
-            maxdepth = -1;
             bbmin = vec(1e16f, 1e16f, 1e16f);
             bbmax = vec(-1e16f, -1e16f, -1e16f);
         }
 
-        void build(int first = -1, int last = -1)
+        void build(int first = 0, int last = -1)
         {
             if(last < 0) last = waypoints.length();
             vector<int> indices;
@@ -81,10 +80,14 @@ namespace ai
                 bbmax.max(vec(w.o).add(radius));
             }
             if(first < last) lastwp = max(lastwp, last-1);
-            build(indices.getbuf(), indices.length(), bbmin, bbmax);
+            if(indices.length())
+            {
+                nodes.reserve(indices.length());
+                build(indices.getbuf(), indices.length(), bbmin, bbmax);
+            }
         }
 
-        void build(int *indices, int numindices, const vec &vmin, const vec &vmax, int depth = 1)
+        void build(int *indices, int numindices, const vec &vmin, const vec &vmax)
         {
             int axis = 2;
             loopk(2) if(vmax[k] - vmin[k] > vmax[axis] - vmin[axis]) axis = k;
@@ -139,26 +142,24 @@ namespace ai
                 }
             }
 
-            int node = nodes.length();
-            nodes.add();
-            nodes[node].split[0] = splitleft;
-            nodes[node].split[1] = splitright;
+            int offset = nodes.length();
+            node &curnode = nodes.add();
+            curnode.split[0] = splitleft;
+            curnode.split[1] = splitright;
 
-            if(left<=1) nodes[node].child[0] = (axis<<30) | (left>0 ? indices[0] : 0x3FFFFFFF);
+            if(left<=1) curnode.child[0] = (axis<<30) | (left>0 ? indices[0] : 0x3FFFFFFF);
             else
             {
-                nodes[node].child[0] = (axis<<30) | (nodes.length()-node);
-                if(left) build(indices, left, leftmin, leftmax, depth+1);
+                curnode.child[0] = (axis<<30) | (nodes.length()-offset);
+                if(left) build(indices, left, leftmin, leftmax);
             }
 
-            if(numindices-right<=1) nodes[node].child[1] = (1<<31) | (left<=1 ? 1<<30 : 0) | (numindices-right>0 ? indices[right] : 0x3FFFFFFF);
+            if(numindices-right<=1) curnode.child[1] = (1<<31) | (left<=1 ? 1<<30 : 0) | (numindices-right>0 ? indices[right] : 0x3FFFFFFF);
             else
             {
-                nodes[node].child[1] = (left<=1 ? 1<<30 : 0) | (nodes.length()-node);
-                if(numindices-right) build(&indices[right], numindices-right, rightmin, rightmax, depth+1);
+                curnode.child[1] = (left<=1 ? 1<<30 : 0) | (nodes.length()-offset);
+                if(numindices-right) build(&indices[right], numindices-right, rightmin, rightmax);
             }
-
-            maxdepth = max(maxdepth, depth);
         }
     } wpcaches[NUMWPCACHES];
 
@@ -167,7 +168,11 @@ namespace ai
     static inline void invalidatewpcache(int wp)
     {
         if(++numinvalidatewpcaches >= 1000) { numinvalidatewpcaches = 0; invalidatedwpcaches = (1<<NUMWPCACHES)-1; }
-        else loopi(NUMWPCACHES) if((wp >= wpcaches[i].firstwp && wp <= wpcaches[i].lastwp) || i+1 >= NUMWPCACHES) { invalidatedwpcaches |= 1<<i; break; }
+        else
+        {
+            loopi(WPCACHE_DYNAMIC) if(wp >= wpcaches[i].firstwp && wp <= wpcaches[i].lastwp) { invalidatedwpcaches |= 1<<i; return; }
+            invalidatedwpcaches |= 1<<WPCACHE_DYNAMIC;
+        }
     }
 
     void clearwpcache(bool full = true)
@@ -184,8 +189,8 @@ namespace ai
 
     void buildwpcache()
     {
-        loopi(NUMWPCACHES) if(wpcaches[i].maxdepth < 0)
-            wpcaches[i].build(i > 0 ? wpcaches[i-1].lastwp+1 : 1, i+1 >= NUMWPCACHES || wpcaches[i+1].maxdepth < 0 ? -1 : wpcaches[i+1].firstwp);
+        loopi(NUMWPCACHES) if(wpcaches[i].firstwp < 0)
+            wpcaches[i].build(i > 0 ? wpcaches[i-1].lastwp+1 : 1, i+1 >= NUMWPCACHES || wpcaches[i+1].firstwp < 0 ? -1 : wpcaches[i+1].firstwp);
         clearedwpcaches = 0;
         lastwpcache = waypoints.length();
 
@@ -195,11 +200,11 @@ namespace ai
 
     struct wpcachestack
     {
-        wpcachenode *node;
+        wpcache::node *node;
         float tmin, tmax;
     };
 
-    vector<wpcachenode *> wpcachestack;
+    vector<wpcache::node *> wpcachestack;
 
     int closestwaypoint(const vec &pos, float mindist, bool links)
     {
@@ -208,7 +213,7 @@ namespace ai
 
         #define CHECKCLOSEST(index) do { \
             int n = (index); \
-            if(waypoints.inrange(n)) \
+            if(n < waypoints.length()) \
             { \
                 waypoint &w = waypoints[n]; \
                 if(!links || w.haslinks()) \
@@ -219,10 +224,9 @@ namespace ai
             } \
         } while(0)
         int closest = -1;
-        wpcachenode *curnode;
-        loop(which, NUMWPCACHES) for(curnode = &wpcaches[which].nodes[0], wpcachestack.setsize(0);;)
+        wpcache::node *curnode;
+        loop(which, NUMWPCACHES) if(wpcaches[which].firstwp >= 0) for(curnode = &wpcaches[which].nodes[0], wpcachestack.setsize(0);;)
         {
-            if(!curnode) continue;
             int axis = curnode->axis();
             float dist1 = pos[axis] - curnode->split[0], dist2 = curnode->split[1] - pos[axis];
             if(dist1 >= mindist)
@@ -267,17 +271,16 @@ namespace ai
         float mindist2 = mindist*mindist, maxdist2 = maxdist*maxdist;
         #define CHECKWITHIN(index) do { \
             int n = (index); \
-            if(waypoints.inrange(n)) \
+            if(n < waypoints.length()) \
             { \
                 const waypoint &w = waypoints[n]; \
                 float dist = w.o.squaredist(pos); \
                 if(dist > mindist2 && dist < maxdist2) results.add(n); \
             } \
         } while(0)
-        wpcachenode *curnode;
-        loop(which, NUMWPCACHES) for(curnode = &wpcaches[which].nodes[0], wpcachestack.setsize(0);;)
+        wpcache::node *curnode;
+        loop(which, NUMWPCACHES) if(wpcaches[which].firstwp >= 0) for(curnode = &wpcaches[which].nodes[0], wpcachestack.setsize(0);;)
         {
-            if(!curnode) continue;
             int axis = curnode->axis();
             float dist1 = pos[axis] - curnode->split[0], dist2 = curnode->split[1] - pos[axis];
             if(dist1 >= maxdist)
@@ -321,16 +324,15 @@ namespace ai
         float limit2 = limit*limit;
         #define CHECKNEAR(index) do { \
             int n = (index); \
-            if(ai::waypoints.inrange(n)) \
+            if(n < ai::waypoints.length()) \
             { \
                 const waypoint &w = ai::waypoints[n]; \
                 if(w.o.squaredist(pos) < limit2) add(owner, above, n); \
             } \
         } while(0)
-        wpcachenode *curnode;
-        loop(which, NUMWPCACHES) for(curnode = &wpcaches[which].nodes[0], wpcachestack.setsize(0);;)
+        wpcache::node *curnode;
+        loop(which, NUMWPCACHES) if(wpcaches[which].firstwp >= 0) for(curnode = &wpcaches[which].nodes[0], wpcachestack.setsize(0);;)
         {
-            if(!curnode) continue;
             int axis = curnode->axis();
             float dist1 = pos[axis] - curnode->split[0], dist2 = curnode->split[1] - pos[axis];
             if(dist1 >= limit)
@@ -399,7 +401,7 @@ namespace ai
                             d->o = vec(above).add(vec(0, 0, d->height));
                             bool col = collide(d, vec(0, 0, 1));
                             d->o = old;
-                            if(col)
+                            if(!col)
                             {
                                 pos = above;
                                 return n;
@@ -511,6 +513,7 @@ namespace ai
         if(waypoints.length() > MAXWAYPOINTS) return -1;
         int n = waypoints.length();
         waypoints.add(waypoint(o, weight >= 0 ? weight : getweight(o)));
+        invalidatewpcache(n);
         return n;
     }
 
@@ -527,7 +530,7 @@ namespace ai
     static inline bool shouldnavigate()
     {
         if(dropwaypoints) return true;
-        loopvrev(players) if(players[i] && players[i]->aitype != AI_NONE) return true;
+        loopvrev(players) if(players[i] && players[i]->actortype != A_PLAYER) return true;
         return false;
     }
 
@@ -576,7 +579,7 @@ namespace ai
         }
         else if(!iswaypoint(d->lastnode) || waypoints[d->lastnode].o.squaredist(v) > dist*dist)
         {
-            dist = physics::jetpack(d) ? JETDIST : RETRYDIST; // workaround
+            dist = RETRYDIST; // workaround
 			d->lastnode = closestwaypoint(v, dist, false);
         }
     }
@@ -597,7 +600,7 @@ namespace ai
         clearwpcache();
         if(full) loadedwaypoints[0] = '\0';
     }
-    ICOMMAND(0, clearwaypoints, "", (), clearwaypoints());
+    ICOMMAND(0, clearwaypoints, "", (), clearwaypoints(true));
 
     void remapwaypoints()
     {
@@ -627,7 +630,7 @@ namespace ai
     {
         if(o.dist(v) > CLOSEDIST)
         {
-            loopi(entities::lastenttype[TELEPORT]) if(entities::ents[i]->type == TELEPORT)
+            loopi(entities::lastent(TELEPORT)) if(entities::ents[i]->type == TELEPORT)
             {
                 gameentity &e = *(gameentity *)entities::ents[i];
                 if(o.dist(e.o) < (e.attrs[3] ? e.attrs[3] : enttype[e.type].radius)+CLOSEDIST)
@@ -728,7 +731,7 @@ namespace ai
         if(!cleanwaypoints()) clearwpcache();
         return true;
     }
-    ICOMMAND(0, loadwaypoints, "s", (char *mname), getwaypoints(true, mname));
+    ICOMMAND(0, loadwaypoints, "s", (char *mname), if(!(identflags&IDF_WORLD)) getwaypoints(true, mname));
 
     void savewaypoints(bool force, const char *mname)
     {
@@ -757,7 +760,7 @@ namespace ai
         conoutf("saved %d waypoints to %s", waypoints.length()-1, wptname);
     }
 
-    ICOMMAND(0, savewaypoints, "s", (char *mname), savewaypoints(true, mname));
+    ICOMMAND(0, savewaypoints, "s", (char *mname), if(!(identflags&IDF_WORLD)) savewaypoints(true, mname));
 
     bool importwaypoints()
     {
@@ -791,6 +794,22 @@ namespace ai
         return loadwaypoints(force, mname) || importwaypoints();
     }
 
+    void delwaypoint(int n)
+    {
+        if(n < 0)
+        {
+            if(noedit(true)) return;
+            n = closestwaypoint(camera1->o);
+        }
+        if(!iswaypoint(n)) return;
+        waypoints[n].links[0] = 0;
+        waypoints[n].links[1] = 0xFFFF;
+        remapwaypoints();
+        clearwpcache();
+    }
+    ICOMMAND(0, delwaypoint, "b", (int *n), delwaypoint(*n));
+
+
     void delselwaypoints()
     {
         if(noedit(true)) return;
@@ -814,5 +833,27 @@ namespace ai
         }
     }
     COMMAND(0, delselwaypoints, "");
+
+    void moveselwaypoints(vec &offset)
+    {
+        if(noedit(true)) return;
+        vec o = sel.o.tovec().sub(0.1f), s = sel.s.tovec().mul(sel.grid).add(o).add(0.1f);
+        int moved = 0;
+        for(int i = 1; i < waypoints.length(); i++)
+        {
+            waypoint &w = waypoints[i];
+            if(w.o.x >= o.x && w.o.x <= s.x && w.o.y >= o.y && w.o.y <= s.y && w.o.z >= o.z && w.o.z <= s.z)
+            {
+                w.o.add(offset);
+                moved++;
+            }
+        }
+        if(moved)
+        {
+            player1->lastnode = -1;
+            clearwpcache();
+        }
+    }
+    ICOMMAND(0, moveselwaypoints, "fff", (float *x, float *y, float *z), vec v(*x, *y, *z); moveselwaypoints(v));
 }
 
