@@ -2,7 +2,7 @@
 namespace weapons
 {
     VAR(IDF_PERSIST, autoreloading, 0, 2, 4); // 0 = never, 1 = when empty, 2 = weapons that don't add a full clip, 3 = always (+1 zooming weaps too)
-    VAR(IDF_PERSIST, autoreloaddelay, 0, 100, VAR_MAX);
+    VAR(IDF_PERSIST, autodelayreload, 0, 0, VAR_MAX);
 
     VAR(IDF_PERSIST, skipspawnweapon, 0, 0, 6); // skip spawnweapon; 0 = never, 1 = if numweaps > 1 (+1), 3 = if carry > 0 (+2), 6 = always
     VAR(IDF_PERSIST, skipmelee, 0, 7, 10); // skip melee; 0 = never, 1 = if numweaps > 1 (+2), 4 = if carry > 0 (+2), 7 = if carry > 0 and is offset (+2), 10 = always
@@ -11,7 +11,7 @@ namespace weapons
     VAR(IDF_PERSIST, skipmine, 0, 0, 10); // skip mine; 0 = never, 1 = if numweaps > 1 (+2), 4 = if carry > 0 (+2), 7 = if carry > 0 and is offset (+2), 10 = always
 
     int lastweapselect = 0;
-    VAR(IDF_PERSIST, weapselectdelay, 0, 150, VAR_MAX);
+    VAR(IDF_PERSIST, weapselectdelay, 0, 250, VAR_MAX);
     VARF(IDF_PERSIST, weapselectslot, 0, 1, 1, changedkeys = lastmillis); // 0 = by id, 1 = by slot
 
     int slot(gameent *d, int n, bool back)
@@ -32,7 +32,7 @@ namespace weapons
     ICOMMAND(0, weapslot, "i", (int *o), intret(slot(game::player1, *o >= 0 ? *o : game::player1->weapselect))); // -1 = weapselect slot
     ICOMMAND(0, weapselect, "", (), intret(game::player1->weapselect));
     ICOMMAND(0, ammo, "i", (int *n), intret(isweap(*n) ? game::player1->ammo[*n] : -1));
-    ICOMMAND(0, reloads, "i", (int *n), intret(isweap(*n) ? game::player1->reloads[*n] : -1));
+    ICOMMAND(0, reloadweap, "i", (int *n), intret(isweap(*n) && w_reload(*n, m_weapon(game::gamemode, game::mutators)) ? 1 : 0));
     ICOMMAND(0, hasweap, "ii", (int *n, int *o), intret(isweap(*n) && game::player1->hasweap(*n, *o) ? 1 : 0));
     ICOMMAND(0, getweap, "ii", (int *n, int *o), {
         if(isweap(*n)) switch(*o)
@@ -46,20 +46,19 @@ namespace weapons
 
     bool weapselect(gameent *d, int weap, int filter, bool local)
     {
-        if(game::intermission || (!local && (d == game::player1 || d->ai))) return false;
+        if(!gs_playing(game::gamestate)) return false;
         if(local)
         {
             int interrupts = filter;
             interrupts &= ~(1<<W_S_RELOAD);
             if(!d->canswitch(weap, m_weapon(game::gamemode, game::mutators), lastmillis, interrupts))
             {
-                if(!d->canswitch(weap, m_weapon(game::gamemode, game::mutators), lastmillis, (1<<W_S_RELOAD))) return false;
+                if(!d->canswitch(weap, m_weapon(game::gamemode, game::mutators), lastmillis, filter)) return false;
                 else if(!isweap(d->weapselect) || d->weapload[d->weapselect] <= 0) return false;
                 else
                 {
                     int offset = d->weapload[d->weapselect];
                     d->ammo[d->weapselect] = max(d->ammo[d->weapselect]-offset, 0);
-                    d->reloads[d->weapselect] = max(d->reloads[d->weapselect]-1, 0);
                     d->weapload[d->weapselect] = -d->weapload[d->weapselect];
                 }
             }
@@ -70,67 +69,71 @@ namespace weapons
         return true;
     }
 
-    bool weapreload(gameent *d, int weap, int load, int ammo, int reloads, bool local)
+    bool weapreload(gameent *d, int weap, int load, int ammo, bool local)
     {
-        if(game::intermission || (!local && (d == game::player1 || d->ai))) return false;
+        if(!gs_playing(game::gamestate) || (!local && (d == game::player1 || d->ai))) return false; // this can't be fixed until 1.5
         if(local)
         {
-            if(!d->canreload(weap, m_weapon(game::gamemode, game::mutators), false, lastmillis)) return false;
+            if(!d->canreload(weap, m_weapon(game::gamemode, game::mutators), false, lastmillis))
+            {
+                if(d->weapstate[weap] == W_S_POWER) d->setweapstate(weap, W_S_WAIT, 100, lastmillis);
+                return false;
+            }
             client::addmsg(N_RELOAD, "ri3", d->clientnum, lastmillis-game::maptime, weap);
             int oldammo = d->ammo[weap];
-            ammo = min(max(d->ammo[weap], 0) + W(weap, add), W(weap, max));
-            reloads = max(d->reloads[weap], 0) + 1;
+            ammo = min(max(d->ammo[weap], 0) + W(weap, ammoadd), W(weap, ammomax));
             load = ammo-oldammo;
         }
         d->weapload[weap] = load;
-        d->ammo[weap] = min(ammo, W(weap, max));
-        d->reloads[weap] = max(reloads, 0);
+        d->ammo[weap] = min(ammo, W(weap, ammomax));
         playsound(WSND(weap, S_W_RELOAD), d->o, d, 0, -1, -1, -1, &d->wschan);
-        d->setweapstate(weap, W_S_RELOAD, W(weap, reloaddelay), lastmillis);
+        d->setweapstate(weap, W_S_RELOAD, W(weap, delayreload), lastmillis);
         return true;
     }
 
     void weaponswitch(gameent *d, int a = -1, int b = -1)
     {
-        if(game::intermission || a < -1 || b < -1 || a >= W_MAX || b >= W_MAX) return;
+        if(!gs_playing(game::gamestate) || a < -1 || b < -1 || a >= W_MAX || b >= W_MAX) return;
         if(weapselectdelay && lastweapselect && totalmillis-lastweapselect < weapselectdelay) return;
-        if(!d->weapwaited(d->weapselect, lastmillis, G(weaponinterrupts))) return;
-        int s = slot(d, d->weapselect);
-        loopi(W_MAX) // only loop the amount of times we have weaps for
+        if(d->weapwaited(d->weapselect, lastmillis, (1<<W_S_SWITCH)|(1<<W_S_RELOAD)))
         {
-            if(a >= 0) s = a;
-            else s += b;
-            while(s > W_MAX-1) s -= W_MAX;
-            while(s < 0) s += W_MAX;
-
-            int n = slot(d, s, true);
-            if(a < 0)
-            { // weapon skipping when scrolling
-                int p = m_weapon(game::gamemode, game::mutators);
-                #define skipweap(q,w) \
-                { \
-                    if(q && n == w && (d->aitype >= AI_START || w != W_MELEE || p == W_MELEE || d->weapselect == W_MELEE)) switch(q) \
-                    { \
-                        case 10: continue; break; \
-                        case 7: case 8: case 9: if(d->carry(p, 5, w) > (q-7)) continue; break; \
-                        case 4: case 5: case 6: if(d->carry(p, 1, w) > (q-3)) continue; break; \
-                        case 1: case 2: case 3: if(d->carry(p, 0, w) > q) continue; break; \
-                        case 0: default: break; \
-                    } \
-                }
-                skipweap(skipspawnweapon, p);
-                skipweap(skipmelee, W_MELEE);
-                skipweap(skippistol, W_PISTOL);
-                skipweap(skipgrenade, W_GRENADE);
-                skipweap(skipmine, W_MINE);
-            }
-
-            if(weapselect(d, n, G(weaponinterrupts)))
+            int s = slot(d, d->weapselect);
+            loopi(W_MAX) // only loop the amount of times we have weaps for
             {
-                lastweapselect = totalmillis;
-                return;
+                if(a >= 0) s = a;
+                else s += b;
+                while(s > W_MAX-1) s -= W_MAX;
+                while(s < 0) s += W_MAX;
+
+                int n = slot(d, s, true);
+                if(a < 0)
+                { // weapon skipping when scrolling
+                    int p = m_weapon(game::gamemode, game::mutators);
+                    #define skipweap(q,w) \
+                    { \
+                        if(q && n == w && (d->actortype >= A_ENEMY || w != W_MELEE || p == W_MELEE || d->weapselect == W_MELEE)) switch(q) \
+                        { \
+                            case 10: continue; break; \
+                            case 7: case 8: case 9: if(d->carry(p, 5, w) > (q-7)) continue; break; \
+                            case 4: case 5: case 6: if(d->carry(p, 1, w) > (q-3)) continue; break; \
+                            case 1: case 2: case 3: if(d->carry(p, 0, w) > q) continue; break; \
+                            case 0: default: break; \
+                        } \
+                    }
+                    skipweap(skipspawnweapon, p);
+                    skipweap(skipmelee, W_MELEE);
+                    skipweap(skippistol, W_PISTOL);
+                    skipweap(skipgrenade, W_GRENADE);
+                    skipweap(skipmine, W_MINE);
+                }
+
+                if(weapselect(d, n, (1<<W_S_SWITCH)|(1<<W_S_RELOAD)))
+                {
+                    lastweapselect = totalmillis;
+                    return;
+                }
+                else if(a >= 0) break;
             }
-            else if(a >= 0) break;
         }
         game::errorsnd(d);
     }
@@ -138,26 +141,35 @@ namespace weapons
 
     void weapdrop(gameent *d, int w)
     {
-        if(game::intermission) return;
+        if(!gs_playing(game::gamestate)) return;
         int weap = isweap(w) ? w : d->weapselect, sweap = m_weapon(game::gamemode, game::mutators);
         d->action[AC_DROP] = false;
-        if(d->candrop(weap, sweap, lastmillis, G(weaponinterrupts)))
+        if(!d->candrop(weap, sweap, lastmillis, m_loadout(game::gamemode, game::mutators), (1<<W_S_SWITCH)))
         {
-            client::addmsg(N_DROP, "ri3", d->clientnum, lastmillis-game::maptime, weap);
-            d->setweapstate(weap, W_S_WAIT, weaponswitchdelay, lastmillis);
-            return;
+            if(!d->candrop(weap, sweap, lastmillis, m_loadout(game::gamemode, game::mutators), (1<<W_S_SWITCH)|(1<<W_S_RELOAD)) || !isweap(d->weapselect) || d->weapload[d->weapselect] <= 0)
+            {
+                game::errorsnd(d);
+                return;
+            }
+            else
+            {
+                int offset = d->weapload[d->weapselect];
+                d->ammo[d->weapselect] = max(d->ammo[d->weapselect]-offset, 0);
+                d->weapload[d->weapselect] = -d->weapload[d->weapselect];
+            }
         }
-        game::errorsnd(d);
+        client::addmsg(N_DROP, "ri3", d->clientnum, lastmillis-game::maptime, weap);
+        d->setweapstate(weap, W_S_WAIT, weaponswitchdelay, lastmillis);
     }
 
     bool autoreload(gameent *d, int flags = 0)
     {
-        if(!game::intermission && d == game::player1 && W2(d->weapselect, sub, WS(flags)) && d->canreload(d->weapselect, m_weapon(game::gamemode, game::mutators), false, lastmillis))
+        if(gs_playing(game::gamestate) && d == game::player1 && W2(d->weapselect, ammosub, WS(flags)) && d->canreload(d->weapselect, m_weapon(game::gamemode, game::mutators), false, lastmillis))
         {
-            bool noammo = d->ammo[d->weapselect] < W2(d->weapselect, sub, WS(flags)),
+            bool noammo = d->ammo[d->weapselect] < W2(d->weapselect, ammosub, WS(flags)),
                  noattack = !d->action[AC_PRIMARY] && !d->action[AC_SECONDARY];
-            if((noammo || noattack) && !d->action[AC_USE] && d->weapstate[d->weapselect] == W_S_IDLE && (noammo || lastmillis-d->weaplast[d->weapselect] >= autoreloaddelay))
-                return autoreloading >= (noammo ? 1 : (W(d->weapselect, add) < W(d->weapselect, max) ? 2 : (W(d->weapselect, zooms) ? 4 : 3)));
+            if((noammo || noattack) && !d->action[AC_USE] && d->weapstate[d->weapselect] == W_S_IDLE && (noammo || lastmillis-d->weaplast[d->weapselect] >= autodelayreload))
+                return autoreloading >= (noammo ? 1 : (W(d->weapselect, ammoadd) < W(d->weapselect, ammomax) ? 2 : (W2(d->weapselect, cooked, true)&W_C_ZOOM ? 4 : 3)));
         }
         return false;
     }
@@ -190,12 +202,23 @@ namespace weapons
 
     float accmod(gameent *d, bool zooming, int *x)
     {
-        float r = 1; int y = 0;
-        if(inairspread && (physics::jetpack(d) || d->airmillis) && !d->onladder) { r += inairspread; y++; }
-        if(impulsespread > 0 && physics::pacing(d)) { r += impulsespread; y++; }
-        else if(movespread > 0 && (d->move || d->strafe)) { r += movespread; y++; }
-        else if(stillspread > 0 && !physics::sliding(d) && !physics::iscrouching(d) && !zooming) { r += stillspread; y++; }
-        if(x) *x = y;
+        float r = 1;
+        if(spreadinair > 0 && d->airmillis && !d->onladder)
+        {
+            if(x) (*x)++;
+            r += spreadinair;
+        }
+        bool running = d->running(moveslow) || d->sliding();
+        if((running && spreadrunning > 0 ? spreadrunning : spreadmoving) > 0 && (d->move || d->strafe || running))
+        {
+            if(x) (*x)++;
+            r += running ? spreadrunning : spreadmoving;
+        }
+        else if(spreadstill > 0 && !d->crouching() && !zooming)
+        {
+            if(x) (*x)++;
+            r += spreadstill;
+        }
         return r;
     }
 
@@ -214,33 +237,47 @@ namespace weapons
             else offset = d->weapload[weap];
         }
         float scale = 1;
-        int sub = W2(weap, sub, secondary), cooked = force;
-        if(W2(weap, power, secondary) && !W(weap, zooms))
+        bool zooming = (pressed && secondary && W2(weap, cooked, true)&W_C_ZOOM) || d->weapstate[weap] == W_S_ZOOM, wassecond = secondary;
+        if(zooming)
+        {
+            if(!pressed)
+            {
+                client::addmsg(N_SPHY, "ri4", d->clientnum, SPHY_COOK, W_S_IDLE, 0);
+                d->setweapstate(weap, W_S_IDLE, 0, lastmillis);
+                return false;
+            }
+            else secondary = zooming;
+        }
+        int sub = W2(weap, ammosub, secondary), cooked = force;
+        if(W2(weap, cooktime, secondary) || zooming)
         {
             float maxscale = 1;
             if(sub > 1 && d->ammo[weap] < sub) maxscale = d->ammo[weap]/float(sub);
-            int len = int(W2(weap, power, secondary)*maxscale);
+            int len = int(W2(weap, cooktime, secondary)*maxscale), type = zooming ? W_S_ZOOM : W_S_POWER;
             if(!cooked)
             {
-                if(d->weapstate[weap] != W_S_POWER)
+                if(d->weapstate[weap] != type)
                 {
                     if(pressed)
                     {
                         if(offset > 0)
                         {
                             d->ammo[weap] = max(d->ammo[weap]-offset, 0);
-                            d->reloads[weap] = max(d->reloads[weap]-1, 0);
                             d->weapload[weap] = -offset;
                         }
-                        client::addmsg(N_SPHY, "ri3", d->clientnum, SPHY_POWER, len);
-                        d->setweapstate(weap, W_S_POWER, len, lastmillis);
+                        client::addmsg(N_SPHY, "ri4", d->clientnum, SPHY_COOK, type, len);
+                        d->setweapstate(weap, type, len, lastmillis);
                     }
                     else return false;
                 }
-                cooked = clamp(lastmillis-d->weaplast[weap], 1, len);
-                if(pressed && cooked < len) return false;
+                cooked = len ? clamp(lastmillis-d->weaplast[weap], 1, len) : 1;
+                if(zooming)
+                {
+                    if(pressed && wassecond) return false;
+                }
+                else if(pressed && cooked < len) return false;
             }
-            scale = cooked/float(W2(weap, power, secondary));
+            scale = len ? cooked/float(W2(weap, cooktime, secondary)) : 1;
             if(sub > 1 && scale < 1) sub = int(ceilf(sub*scale));
         }
         else if(!pressed) return false;
@@ -253,52 +290,22 @@ namespace weapons
             s.id = d->getprojid(); \
             s.pos = ivec(int(p.x*DMF), int(p.y*DMF), int(p.z*DMF)); \
         }
+        int rays = W2(weap, rays, secondary);
+        if(rays > 1 && W2(weap, cooked, secondary)&W_C_RAYS && W2(weap, cooktime, secondary) && scale < 1)
+            rays = max(1, int(ceilf(rays*scale)));
         if(weaptype[weap].traced)
         {
             from = d->originpos(weap == W_MELEE, secondary);
             if(weap == W_MELEE) to = vec(targ).sub(from).normalize().mul(d->radius).add(from);
             else to = d->muzzlepos(weap, secondary);
-            int rays = W2(weap, rays, secondary);
-            if(rays > 1 && W2(weap, power, secondary) && scale < 1) rays = int(ceilf(rays*scale));
             loopi(rays) addshot(to);
         }
         else
         {
             from = d->muzzlepos(weap, secondary);
             to = targ;
-
-            vec unitv;
-            float dist = to.dist(from, unitv);
-            if(dist > 0) unitv.div(dist);
-            else vecfromyawpitch(d->yaw, d->pitch, 1, 0, unitv);
-
-            // move along the eye ray towards the weap origin, stopping when something is hit
-            // nudge the target a tiny bit forward in the direction of the target for stability
-
-            vec eyedir(from);
-            eyedir.sub(d->o);
-            float eyedist = eyedir.magnitude();
-            if(eyedist > 0) eyedir.div(eyedist);
-            float barrier = eyedist > 0 ? raycube(d->o, eyedir, eyedist, RAY_CLIPMAT) : eyedist;
-            if(barrier < eyedist)
-            {
-                (from = eyedir).mul(barrier).add(d->o);
-                (to = targ).sub(from).rescale(1e-3f).add(from);
-            }
-            else
-            {
-                barrier = raycube(from, unitv, dist, RAY_CLIPMAT|RAY_ALPHAPOLY);
-                if(barrier < dist)
-                {
-                    to = unitv;
-                    to.mul(barrier);
-                    to.add(from);
-                }
-            }
-
-            int rays = W2(weap, rays, secondary), x = 0;
-            if(rays > 1 && W2(weap, power, secondary) && scale < 1) rays = int(ceilf(rays*scale));
-            float m = accmod(d, W(d->weapselect, zooms) && secondary, &x);
+            int x = 0;
+            float m = accmod(d, W2(d->weapselect, cooked, true)&W_C_ZOOM && secondary, &x);
             float spread = WSP(weap, secondary, game::gamemode, game::mutators, m, x);
             loopi(rays)
             {
@@ -319,9 +326,9 @@ namespace weapons
     void shoot(gameent *d, vec &targ, int force)
     {
         if(!game::allowmove(d)) return;
-        bool secondary = physics::secondaryweap(d), alt = secondary && !W(d->weapselect, zooms);
-        if(doshot(d, targ, d->weapselect, d->action[alt ? AC_SECONDARY : AC_PRIMARY], secondary, force))
-            if(!W2(d->weapselect, fullauto, secondary)) d->action[alt ? AC_SECONDARY : AC_PRIMARY] = false;
+        bool secondary = physics::secondaryweap(d);
+        if(doshot(d, targ, d->weapselect, d->action[secondary ? AC_SECONDARY : AC_PRIMARY], secondary, force))
+            if(!W2(d->weapselect, fullauto, secondary)) d->action[secondary ? AC_SECONDARY : AC_PRIMARY] = false;
     }
 
     void preload()
