@@ -99,69 +99,76 @@ typedef void (__cdecl *identfun)();
 
 struct ident
 {
-    int type;           // one of ID_* above
+    uchar type; // one of ID_* above
+    union
+    {
+        uchar valtype; // ID_ALIAS
+        uchar numargs; // ID_COMMAND
+    };
+    ushort flags;
+    int index;
     const char *name;
     union
     {
-        int minval;    // ID_VAR
-        float minvalf; // ID_FVAR
-        int valtype;   // ID_ALIAS
-    };
-    union
-    {
-        int maxval;    // ID_VAR
-        float maxvalf; // ID_FVAR
-        uint *code;    // ID_ALIAS
-    };
-    union
-    {
-        const char *args;    // ID_COMMAND
-        identval val;        // ID_ALIAS
-        identvalptr storage; // ID_VAR, ID_FVAR, ID_SVAR
-    };
-    union
-    {
-        identval overrideval; // ID_VAR, ID_FVAR, ID_SVAR
-        identstack *stack;    // ID_ALIAS
-        uint argmask;         // ID_COMMAND
+        struct // ID_VAR, ID_FVAR, ID_SVAR
+        {
+            union
+            {
+                struct { int minval, maxval; };     // ID_VAR
+                struct { float minvalf, maxvalf; }; // ID_FVAR
+            };
+            identvalptr storage;
+            identval overrideval;
+            identval def; // declared-default (by *init.cfg)
+            identval bin; // builtin-default (hard coded or version.cfg)
+        };
+        struct // ID_ALIAS
+        {
+            uint *code;
+            identval val;
+            identstack *stack;
+        };
+        struct // ID_COMMAND
+        {
+            const char *args;
+            uint argmask;
+        };
     };
     identfun fun; // ID_VAR, ID_FVAR, ID_SVAR, ID_COMMAND
-    identval def;
-    int flags, index;
     char *desc, *usage;
 
     ident() {}
     // ID_VAR
     ident(int t, const char *n, int m, int c, int x, int *s, void *f = NULL, int flags = IDF_COMPLETE)
-        : type(t), name(n), minval(m), maxval(x), fun((identfun)f), flags(flags | (m > x ? IDF_READONLY : 0)), desc(NULL), usage(NULL)
-    { def.i = c; storage.i = s; }
+        : type(t), flags(flags | (m > x ? IDF_READONLY : 0)), name(n), minval(m), maxval(x), fun((identfun)f), desc(NULL), usage(NULL)
+    { def.i = c; bin.i = c; storage.i = s; }
     // ID_FVAR
     ident(int t, const char *n, float m, float c, float x, float *s, void *f = NULL, int flags = IDF_COMPLETE)
-        : type(t), name(n), minvalf(m), maxvalf(x), fun((identfun)f), flags(flags | (m > x ? IDF_READONLY : 0)), desc(NULL), usage(NULL)
-    { def.f = c; storage.f = s; }
+        : type(t), flags(flags | (m > x ? IDF_READONLY : 0)), name(n), minvalf(m), maxvalf(x), fun((identfun)f), desc(NULL), usage(NULL)
+    { def.f = c; bin.f = c; storage.f = s; }
     // ID_SVAR
     ident(int t, const char *n, char *c, char **s, void *f = NULL, int flags = IDF_COMPLETE)
-        : type(t), name(n), fun((identfun)f), flags(flags), desc(NULL), usage(NULL)
-    { def.s = c; storage.s = s; }
+        : type(t), flags(flags), name(n), fun((identfun)f), desc(NULL), usage(NULL)
+    { def.s = c; bin.s = newstring(c); storage.s = s; }
     // ID_ALIAS
     ident(int t, const char *n, char *a, int flags)
-        : type(t), name(n), valtype(VAL_STR), code(NULL), stack(NULL), flags(flags), desc(NULL), usage(NULL)
+        : type(t), valtype(VAL_STR), flags(flags), name(n), code(NULL), stack(NULL), desc(NULL), usage(NULL)
     { val.s = a; }
     ident(int t, const char *n, int a, int flags)
-        : type(t), name(n), valtype(VAL_INT), code(NULL), stack(NULL), flags(flags), desc(NULL), usage(NULL)
+        : type(t), valtype(VAL_INT), flags(flags), name(n), code(NULL), stack(NULL), desc(NULL), usage(NULL)
     { val.i = a; }
     ident(int t, const char *n, float a, int flags)
-        : type(t), name(n), valtype(VAL_FLOAT), code(NULL), stack(NULL), flags(flags), desc(NULL), usage(NULL)
+        : type(t), valtype(VAL_FLOAT), flags(flags), name(n), code(NULL), stack(NULL), desc(NULL), usage(NULL)
     { val.f = a; }
     ident(int t, const char *n, int flags)
-        : type(t), name(n), valtype(VAL_NULL), code(NULL), stack(NULL), flags(flags), desc(NULL), usage(NULL)
+        : type(t), valtype(VAL_NULL), flags(flags), name(n), code(NULL), stack(NULL), desc(NULL), usage(NULL)
     {}
     ident(int t, const char *n, const tagval &v, int flags)
-        : type(t), name(n), valtype(v.type), code(NULL), stack(NULL), flags(flags), desc(NULL), usage(NULL)
+        : type(t), valtype(v.type), flags(flags), name(n), code(NULL), stack(NULL), desc(NULL), usage(NULL)
     { val = v; }
     // ID_COMMAND
-    ident(int t, const char *n, const char *args, uint argmask, void *f = NULL, int flags = IDF_COMPLETE)
-        : type(t), name(n), args(args), argmask(argmask), fun((identfun)f), flags(flags), desc(NULL), usage(NULL)
+    ident(int t, const char *n, const char *args, uint argmask, int numargs, void *f = NULL, int flags = IDF_COMPLETE)
+        : type(t), numargs(numargs), flags(flags), name(n), args(args), argmask(argmask), fun((identfun)f), desc(NULL), usage(NULL)
     {}
 
     void changed() { if(fun) fun(); }
@@ -206,11 +213,21 @@ extern void floatret(float v);
 extern void stringret(char *s);
 extern void result(tagval &v);
 extern void result(const char *s);
-extern char *conc(tagval *v, int n, bool space, const char *prefix = NULL);
+extern char *conc(tagval *v, int n, bool space, const char *prefix, int prefixlen);
+
+static inline char *conc(tagval *v, int n, bool space)
+{
+    return conc(v, n, space, NULL, 0);
+}
+
+static inline char *conc(tagval *v, int n, bool space, const char *prefix)
+{
+    return conc(v, n, space, prefix, strlen(prefix));
+}
 
 static inline int parseint(const char *s)
 {
-    return int(strtol(s, NULL, 0));
+    return int(strtoul(s, NULL, 0));
 }
 
 static inline float parsefloat(const char *s)
@@ -221,6 +238,9 @@ static inline float parsefloat(const char *s)
     return val || end==s || (*end!='x' && *end!='X') ? float(val) : float(parseint(s));
 }
 
+static inline void intformat(char *buf, int v) { formatstring(buf)("%d", v); }
+static inline void floatformat(char *buf, float v) { formatstring(buf)(v==int(v) ? "%.1f" : "%.7g", v); }
+
 static inline const char *getstr(const identval &v, int type)
 {
     switch(type)
@@ -283,7 +303,7 @@ extern void touchvar(const char *name);
 extern int getvar(const char *name);
 extern int getvarmin(const char *name);
 extern int getvarmax(const char *name);
-extern int getvardef(const char *name);
+extern int getvardef(const char *name, bool rb = false);
 extern float getfvarmin(const char *name);
 extern float getfvarmax(const char *name);
 extern bool identexists(const char *name);
@@ -306,12 +326,19 @@ extern int execute(const char *p);
 extern int execute(const char *p, bool nonworld);
 extern bool executebool(const uint *code);
 extern bool executebool(const char *p);
-enum { EXEC_NOWORLD = 1<<0, EXEC_VERSION = 1<<1 };
+enum { EXEC_NOWORLD = 1<<0, EXEC_VERSION = 1<<1, EXEC_BUILTIN = 1<<2 };
 extern bool execfile(const char *cfgfile, bool msg = true, int flags = 0);
 extern void alias(const char *name, const char *action);
 extern void alias(const char *name, tagval &v);
 extern void worldalias(const char *name, const char *action);
 extern const char *getalias(const char *name);
+extern void loopiter(ident *id, identstack &stack, const tagval &v);
+extern void loopend(ident *id, identstack &stack);
+
+#define loopstart(id, stack) if((id)->type != ID_ALIAS) return; identstack stack;
+static inline void loopiter(ident *id, identstack &stack, int i) { tagval v; v.setint(i); loopiter(id, stack, v); }
+static inline void loopiter(ident *id, identstack &stack, float f) { tagval v; v.setfloat(f); loopiter(id, stack, v); }
+static inline void loopiter(ident *id, identstack &stack, const char *s) { tagval v; v.setstr(newstring(s)); loopiter(id, stack, v); }
 
 extern int identflags;
 extern bool interactive;
@@ -332,11 +359,12 @@ extern void checksleep(int millis);
 extern void clearsleep(bool clearworlds = true);
 
 extern char *logtimeformat, *filetimeformat;
+extern int filetimelocal;
 extern const char *gettime(time_t ctime = 0, const char *format = NULL);
 
 // nasty macros for registering script functions, abuses globals to avoid excessive infrastructure
-#define KEYWORD(flags, name, type) static bool __dummy_##name = addkeyword(type, #name, flags)
-#define COMMANDN(flags, name, fun, nargs) static bool __dummy_##fun = addcommand(#name, (identfun)fun, nargs, flags|IDF_COMPLETE)
+#define KEYWORD(flags, name, type) UNUSED static bool __dummy_##name = addkeyword(type, #name, flags)
+#define COMMANDN(flags, name, fun, nargs) UNUSED static bool __dummy_##fun = addcommand(#name, (identfun)fun, nargs, flags|IDF_COMPLETE)
 #define COMMAND(flags, name, nargs) COMMANDN(flags, name, name, nargs)
 
 // anonymous inline commands, uses nasty template trick with line numbers to keep names unique
