@@ -22,6 +22,7 @@
 #include <ctype.h>
 #include <stdarg.h>
 #include <limits.h>
+#include <float.h>
 #include <assert.h>
 #include <time.h>
 
@@ -48,22 +49,12 @@
 
 #ifndef STANDALONE
 #include <SDL.h>
-#include <SDL_image.h>
 #include <SDL_syswm.h>
 #include <SDL_opengl.h>
 #endif
 
 #include <enet/enet.h>
 
-#ifdef __sun__
-#undef sun
-#undef MAXNAMELEN
-#ifdef queue
-  #undef queue
-#endif
-#define queue __squeue
-#endif
-
 #include "tools.h"
 #include "command.h"
 #include "geom.h"
