@@ -41,30 +41,27 @@ struct entity : entbase
 {
     attrvector attrs;
     linkvector links;
+};
 
-    entity() { reset(); }
-
-    void reset()
-    {
-        attrs.setsize(0);
-        links.setsize(0);
-    }
+enum
+{
+    EF_OCTA      = 1<<0,
+    EF_RENDER    = 1<<1,
+    EF_SPAWNED   = 1<<2
 };
 
 struct extentity : entity                       // part of the entity that doesn't get saved to disk
 {
-    uchar spawned, inoctanode, visible;        // the only dynamic state of a map entity
+    int flags;        // the only dynamic state of a map entity
     entitylight light;
     int lastemit, emit[3];
 
-    extentity() { reset(); }
+    extentity() : flags(0), lastemit(0) { emit[0] = emit[1] = emit[2] = 0; }
 
-    void reset()
-    {
-        entity::reset();
-        spawned = inoctanode = visible = false;
-        lastemit = emit[0] = emit[1] = emit[2] = 0;
-    }
+    bool spawned() const { return (flags&EF_SPAWNED) != 0; }
+    void setspawned(bool val) { if(val) flags |= EF_SPAWNED; else flags &= ~EF_SPAWNED; }
+    void setspawned() { flags |= EF_SPAWNED; }
+    void clearspawned() { flags &= ~EF_SPAWNED; }
 };
 
 #define MAXENTS 30000
@@ -72,12 +69,13 @@ struct extentity : entity                       // part of the entity that doesn
 #define MAXENTKIN 100
 
 extern int efocus, enthover, entorient;
-#define entfocus(i, f)  { int n = efocus = (i); if(n>=0) { extentity &e = *entities::getents()[n]; f; } }
+#define entfocusv(i, f, v){ int n = efocus = (i); if(n>=0) { extentity &e = *v[n]; f; } }
+#define entfocus(i, f)    entfocusv(i, f, entities::getents())
 
 enum { CS_ALIVE = 0, CS_DEAD, CS_EDITING, CS_SPECTATOR, CS_WAITING }; // beware, some stuff uses >= CS_SPECTATOR
 enum { PHYS_FLOAT = 0, PHYS_FALL, PHYS_SLIDE, PHYS_SLOPE, PHYS_FLOOR, PHYS_STEP_UP, PHYS_STEP_DOWN };
 enum { ENT_PLAYER = 0, ENT_AI, ENT_INANIMATE, ENT_CAMERA, ENT_PROJ, ENT_RAGDOLL, ENT_DUMMY };
-enum { COLLIDE_AABB = 0, COLLIDE_OBB, COLLIDE_ELLIPSE };
+enum { COLLIDE_NONE = 0, COLLIDE_ELLIPSE, COLLIDE_OBB, COLLIDE_ELLIPSE_PRECISE };
 
 struct physent                                  // base entity type, can be affected by physics
 {
@@ -150,7 +148,7 @@ struct physent                                  // base entity type, can be affe
     vec abovehead(float offset = 1) const { return vec(o).add(vec(0, 0, aboveeye+offset)); }
     vec feetpos(float offset = 0) const { return vec(o).add(vec(0, 0, offset-height)); }
     vec headpos(float offset = 0) const { return vec(o).add(vec(0, 0, offset)); }
-    vec center() const { return vec(o).sub(vec(0, 0, height/2)); }
+    vec center() const { return vec(o).sub(vec(0, 0, height*0.5f)); }
 };
 
 enum
