@@ -2,8 +2,8 @@
 
 extern int verbose, curtime, lastmillis, totalmillis, timescale, paused;
 extern uint totalsecs;
-extern time_t clocktime;
-extern int servertype, serverport, servermasterport;
+extern time_t clocktime, currenttime, clockoffset;
+extern int servertype, serverport, serverlanport, servermasterport;
 extern char *servermaster, *serverip;
 extern ENetAddress masteraddress;
 extern void fatal(const char *s, ...) PRINTFARGS(1, 2);
@@ -109,7 +109,12 @@ extern char *getcurcommand();
 extern void resetcomplete();
 extern void complete(char *s, const char *cmdprefix);
 extern const char *searchbind(const char *action, int type);
-extern void searchbindlist(const char *action, int type, int limit, const char *sep, const char *pretty, vector<char> &names);
+extern void searchbindlist(const char *action, int type, int limit, const char *s1, const char *s2, const char *sep1, const char *sep2, vector<char> &names, bool force = true);
+extern int textkeybg, textkeyseps;
+
+extern bool capslockon, numlockon;
+extern bool capslocked();
+extern bool numlocked();
 
 struct bindlist
 {
@@ -118,12 +123,12 @@ struct bindlist
 
     bindlist() : lastsearch(-1) {}
 
-    const char *search(const char *action, int type = 0, int limit = 5, const char *sep = "or", const char *pretty = "\fw")
+    const char *search(const char *action, int type = 0, const char *s1 = "\f{", const char *s2 = "}", int limit = 5)
     {
         if(names.empty() || lastsearch != changedkeys)
         {
             names.shrink(0);
-            searchbindlist(action, type, limit, sep, pretty, names);
+            searchbindlist(action, type, limit, s1, s2, textkeyseps ? (textkeybg ? "|" : ", ") : (textkeybg ? "" : " "), textkeyseps ? (textkeybg ? "|" : " or ") : (textkeybg ? "" : " "), names);
             lastsearch = changedkeys;
         }
         return names.getbuf();
@@ -134,7 +139,7 @@ struct bindlist
 
 // menus
 extern void newgui(char *name, char *contents, char *initscript = NULL);
-extern void showgui(const char *name, int tab = 0);
+extern void showgui(const char *name, int tab = 0, bool *keep = NULL);
 
 // main
 struct igame;
@@ -148,6 +153,7 @@ enum
     TEXT_SHADOW         = 1<<0,
     TEXT_NO_INDENT      = 1<<1,
     TEXT_UPWARD         = 1<<2,
+    TEXT_BALLOON        = 1<<3,
 
     TEXT_ALIGN          = 3<<8,
     TEXT_LEFT_JUSTIFY   = 0<<8,
@@ -156,7 +162,11 @@ enum
 
     TEXT_LEFT_UP        = TEXT_UPWARD|TEXT_LEFT_JUSTIFY,
     TEXT_CENTER_UP      = TEXT_UPWARD|TEXT_CENTERED,
-    TEXT_RIGHT_UP       = TEXT_UPWARD|TEXT_RIGHT_JUSTIFY
+    TEXT_RIGHT_UP       = TEXT_UPWARD|TEXT_RIGHT_JUSTIFY,
+
+    TEXT_LEFT_BAL       = TEXT_BALLOON|TEXT_LEFT_JUSTIFY,
+    TEXT_CENTER_BAL     = TEXT_BALLOON|TEXT_CENTERED,
+    TEXT_RIGHT_BAL      = TEXT_BALLOON|TEXT_RIGHT_JUSTIFY
 };
 
 extern bool setfont(const char *name);
@@ -259,13 +269,14 @@ enum
     PART_FIREBALL_SOFT, PART_FIREBALL,
     PART_PLASMA_SOFT, PART_PLASMA,
     PART_ELECTRIC_SOFT, PART_ELECTRIC,
+    PART_ELECZAP_SOFT, PART_ELECZAP,
     PART_FLAME,
-    PART_FLARE, PART_MUZZLE_FLARE, PART_LIGHTNING_FLARE,
+    PART_FLARE, PART_MUZZLE_FLARE, PART_LIGHTNING_FLARE, PART_LIGHTZAP_FLARE,
     PART_MUZZLE_FLASH,
     PART_SNOW,
     PART_TEXT, PART_TEXT_ONTOP,
     PART_EXPLOSION, PART_SHOCKWAVE, PART_SHOCKBALL,
-    PART_LIGHTNING,
+    PART_LIGHTNING, PART_LIGHTZAP,
     PART_LENS_FLARE,
     PART_MAX
 };
@@ -346,7 +357,6 @@ extern void clearmapcrc();
 
 // physics
 extern bool ellipsecollide(physent *d, const vec &dir, const vec &o, const vec &center, float yaw, float xr, float yr, float hi, float lo);
-extern bool rectcollide(physent *d, const vec &dir, const vec &o, float xr, float yr,  float hi, float lo, uchar visible = 0xFF);
 extern bool collide(physent *d, const vec &dir = vec(0, 0, 0), float cutoff = 0, bool playercol = true);
 extern bool plcollide(physent *d, const vec &dir = vec(0, 0, 0));
 extern bool plcollide(physent *d, const vec &dir, physent *o);
@@ -388,15 +398,17 @@ extern bool validragdoll(dynent *d, int millis);
 extern void moveragdoll(dynent *d, bool smooth);
 extern void cleanragdoll(dynent *d);
 extern void warpragdoll(dynent *d, const vec &vel, const vec &offset = vec(0, 0, 0));
+extern void twitchragdoll(dynent *d, float vel);
 
 // server
 #define MAXCLIENTS 256                  // in a multiplayer game, can be arbitrarily changed
 #define MAXTRANS 5000                  // max amount of data to swallow in 1 go
 
-enum { DISC_NONE = 0, DISC_EOP, DISC_CN, DISC_KICK, DISC_MSGERR, DISC_IPBAN, DISC_PRIVATE, DISC_MAXCLIENTS, DISC_TIMEOUT, DISC_OVERFLOW, DISC_SHUTDOWN, DISC_NUM };
+enum { DISC_NONE = 0, DISC_EOP, DISC_CN, DISC_KICK, DISC_MSGERR, DISC_IPBAN, DISC_PRIVATE, DISC_PASSWORD, DISC_PURE, DISC_MAXCLIENTS, DISC_INCOMPATIBLE, DISC_TIMEOUT, DISC_OVERFLOW, DISC_SHUTDOWN, DISC_NUM };
 
 extern void *getinfo(int i);
 extern const char *gethostname(int i);
+extern const char *gethostip(int i);
 extern void sendf(int cn, int chan, const char *format, ...);
 extern void sendfile(int cn, int chan, stream *file, const char *format = "", ...);
 extern void sendpacket(int cn, int chan, ENetPacket *packet, int exclude = -1);
@@ -404,14 +416,17 @@ extern void flushserver(bool force);
 extern int getservermtu();
 extern int getnumclients();
 extern uint getclientip(int n);
-extern bool filtertext(char *dst, const char *src, bool newline = true, bool colour = true, bool whitespace = true, int len = sizeof(string)-1);
+extern bool filterword(char *src, const char *list);
+extern bool filterstring(char *dst, const char *src, bool newline = true, bool colour = true, bool whitespace = true, bool wsstrip = false, size_t len = MAXSTRLEN-1);
+extern bool filterbigstring(char *dst, const char *src, bool newline = true, bool colour = true, bool whitespace = true, bool wsstrip = false, size_t len = BIGSTRLEN-1);
 extern void disconnect_client(int n, int reason);
 extern void kicknonlocalclients(int reason);
 extern bool hasnonlocalclients();
 extern bool haslocalclients();
+extern void limitdupclients();
 extern void sendqueryreply(ucharbuf &p);
 extern bool resolverwait(const char *name, ENetAddress *address);
-extern int connectwithtimeout(ENetSocket sock, const char *hostname, ENetAddress &address);
+extern int connectwithtimeout(ENetSocket sock, const char *hostname, const ENetAddress &address);
 extern bool connectedmaster();
 extern ENetSocket connectmaster(bool reuse = true);
 extern void disconnectmaster();
@@ -439,7 +454,7 @@ struct serverinfo
     int numplayers, lastping, lastinfo, nextping, ping, resolved, port, priority;
     int pings[MAXPINGS];
     vector<int> attr;
-    vector<char *> players;
+    vector<char *> players, handles;
     ENetAddress address;
 
     serverinfo(uint ip, int port, int priority = 0)
@@ -465,6 +480,7 @@ struct serverinfo
         clearpings();
         attr.setsize(0);
         players.deletearrays();
+        handles.deletearrays();
         numplayers = 0;
     }
 
@@ -526,35 +542,39 @@ struct guient
 {
     virtual ~guient() {}
 
-    virtual void start(int starttime, float basescale, int *tab = NULL, bool allowinput = true, bool wantstitle = true) = 0;
+    virtual void start(int starttime, float basescale, int *tab = NULL, bool allowinput = true, bool wantstitle = true, bool wantsbgfx = true) = 0;
     virtual void end() = 0;
 
-    virtual int text(const char *text, int color, const char *icon = NULL, int icolor = 0xFFFFFF) = 0;
-    int textf(const char *fmt, int color, const char *icon = NULL, int icolor = 0xFFFFFF, ...) PRINTFARGS(2, 6)
+    virtual int text(const char *text, int color, const char *icon = NULL, int icolor = 0xFFFFFF, int wrap = -1) = 0;
+    int textf(const char *fmt, int color, const char *icon = NULL, int icolor = 0xFFFFFF, int wrap = -1, ...) PRINTFARGS(2, 7)
     {
-        defvformatstring(str, icolor, fmt);
-        return text(str, color, icon, icolor);
+        defvformatstring(str, wrap, fmt);
+        return text(str, color, icon, icolor, wrap);
     }
-    virtual int button(const char *text, int color, const char *icon = NULL, int icolor = 0xFFFFFF, bool faded = true) = 0;
-    int buttonf(const char *fmt, int color, const char *icon = NULL, int icolor = 0xFFFFFF, bool faded = true, ...) PRINTFARGS(2, 7)
+    virtual int button(const char *text, int color, const char *icon = NULL, int icolor = 0xFFFFFF, int wrap = -1, bool faded = true) = 0;
+    int buttonf(const char *fmt, int color, const char *icon = NULL, int icolor = 0xFFFFFF, int wrap = -1, bool faded = true, ...) PRINTFARGS(2, 8)
     {
         defvformatstring(str, faded, fmt);
-        return button(str, color, icon, icolor, faded);
+        return button(str, color, icon, icolor, wrap, faded);
     }
-    virtual void background(int color, int parentw = 0, int parenth = 0) = 0;
+    virtual void fill(int color, int parentw = 0, int parenth = 0) = 0;
+    virtual void outline(int color, int parentw = 0, int parenth = 0, int offsetx = 0, int offsety = 0) = 0;
+    virtual void background(int colour1 = -1, float blend1 = -1, int colour2 = -1, float blend2 = -1, bool skinborder = false, int parentw = 0, int parenth = 0) = 0;
 
     virtual void pushlist(bool merge = false) {}
     virtual int poplist() { return 0; }
 
     virtual void allowhitfx(bool on) = 0;
+    virtual bool visibletab() = 0;
     virtual bool visible() = 0;
     virtual bool shouldtab() { return false; }
     virtual void tab(const char *name = NULL, int color = 0xFFFFFF, bool front = false) = 0;
-    virtual int title(const char *text, int color = 0xFFFFFF, const char *icon = NULL, int icolor = 0xFFFFFF) = 0;
+    virtual void setstatus(const char *fmt, int width, ...) = 0;
+    virtual void settooltip(const char *fmt, int width, ...) = 0;
     virtual int image(Texture *t, float scale, bool overlaid = false, int icolor = 0xFFFFFF) = 0;
     virtual int texture(VSlot &vslot, float scale, bool overlaid = true) = 0;
     virtual int slice(Texture *t, float scale, float start = 0, float end = 1, const char *text = NULL) = 0;
-    virtual void slider(int &val, int vmin, int vmax, int color, const char *label = NULL, bool reverse = false, bool scroll = false) = 0;
+    virtual int slider(int &val, int vmin, int vmax, int colour, const char *label = NULL, bool reverse = false, bool scroll = false, int style = 0, int scolour = -1) = 0;
     virtual void separator() = 0;
     virtual void progress(float percent, float scale) = 0;
     virtual void pushfont(const char *font) = 0;
@@ -562,8 +582,8 @@ struct guient
     virtual void strut(float size = 1) = 0;
     virtual void space(float size = 1) = 0;
     virtual void spring(int weight = 1) = 0;
-    virtual char *field(const char *name, int color, int length, int height = 0, const char *initval = NULL, int initmode = EDITORFOCUSED, bool focus = false, const char *parent = NULL) = 0;
-    virtual char *keyfield(const char *name, int color, int length, int height = 0, const char *initval = NULL, int initmode = EDITORFOCUSED, bool focus = false, const char *parent = NULL) = 0;
+    virtual char *field(const char *name, int color, int length, int height = 0, const char *initval = NULL, int initmode = EDITORFOCUSED, bool focus = false, const char *parent = NULL, const char *prompt = NULL, bool immediate = false) = 0;
+    virtual char *keyfield(const char *name, int color, int length, int height = 0, const char *initval = NULL, int initmode = EDITORFOCUSED, bool focus = false, const char *parent = NULL, const char *prompt = NULL, bool immediate = false) = 0;
     virtual int playerpreview(int model, int color, int team, int weap, const char *vanity, float sizescale, bool overlaid = false, float scale = 1, float blend = 1) { return 0; }
     virtual int modelpreview(const char *name, int anim, float sizescale, bool overlaid = false, float scale = 1, float blend = 1) { return 0; }
 
@@ -576,15 +596,14 @@ struct guicb
     virtual void gui(guient &g, bool firstpass) = 0;
 };
 
-extern char *guioverlaytex, *guislidertex;
+extern char *guiskintex, *guiskinbordertex, *guioverlaytex, *guiexittex, *guihovertex;
 
 struct editor;
 
 namespace UI
 {
-    extern bool isopen, ready;
+    extern bool isopen;
     extern bool keypress(int code, bool isdown, int cooked);
-    extern void setup();
     extern void update();
     extern void render();
     extern bool active(bool pass = true);
@@ -605,7 +624,7 @@ struct clientdata
     int type;
     int num;
     ENetPeer *peer;
-    string hostname;
+    string hostname, hostip;
     void *info;
 };
 
@@ -617,11 +636,10 @@ extern ENetHost *serverhost;
 
 // world
 
-extern bool inside;
+extern bool collideinside;
 extern physent *hitplayer;
 extern int hitflags;
-extern vec wall, hitsurface;
-extern float walldistance;
+extern vec collidewall, hitsurface;
 
 enum { HITFLAG_NONE = 0, HITFLAG_LEGS = 1<<0, HITFLAG_TORSO = 1<<1, HITFLAG_HEAD = 1<<2, HITFLAG_FULL = 1<<4 };
 
