@@ -8,7 +8,7 @@ namespace entities
     extern void readent(stream *g, int mtype, int mver, char *gid, int gver, int id);
     extern void writeent(stream *g, int id);
     extern void remapents(vector<int> &idxs);
-    extern void initents(stream *g, int mtype, int mver, char *gid, int gver);
+    extern void initents(int mtype, int mver, char *gid, int gver);
     extern float dropheight(extentity &e);
     extern void fixentity(int n, bool recurse = true, bool create = false);
     extern bool cansee(int n);
@@ -27,13 +27,14 @@ namespace entities
 
 namespace client
 {
+    extern int getcn(physent *d);
     extern int maxmsglen();
     extern void gamedisconnect(int clean);
     extern void parsepacketclient(int chan, packetbuf &p);
     extern void gameconnect(bool _remote);
     extern bool allowedittoggle(bool edit);
     extern void edittoggled(bool edit);
-    extern void toserver(int flags, char *text);
+    extern void toserver(int flags, const char *text, const char *target = NULL);
     extern void editvar(ident *id, bool local);
     extern void edittrigger(const selinfo &sel, int op, int arg1 = 0, int arg2 = 0, int arg3 = 0);
     extern void changemap(const char *name);
@@ -41,13 +42,16 @@ namespace client
     extern void connectattempt(const char *name, int port, const char *password, const ENetAddress &address);
     extern void connectfail();
     extern int state();
-    extern int otherclients(bool nospec = false);
+    extern int otherclients(bool self = false, bool nospec = false);
+    extern int numplayers();
     extern int servercompare(serverinfo *a, serverinfo *b);
+    extern const char *getname();
+    extern bool sendcmd(int nargs, const char *cmd, const char *arg);
 }
 
 namespace hud
 {
-    extern char *progresstex;
+    extern char *progresstex, *progringtex;
     extern bool hasinput(bool pass = false, bool focus = true);
     extern bool keypress(int code, bool isdown, int cooked);
     extern void drawhud(bool noview = false);
@@ -79,8 +83,7 @@ namespace physics
     extern void move(physent *d, int moveres = 10, bool local = true);
     extern bool entinmap(physent *d, bool avoidplayers);
     extern void updatephysstate(physent *d);
-    extern bool droptofloor(vec &o, float radius = 1, float height = 1);
-    extern bool iscrouching(physent *d);
+    extern bool droptofloor(vec &o, int type = ENT_CAMERA, float radius = 1, float height = 1);
     extern bool moveplayer(physent *pl, int moveres, bool local, int millis);
     extern void interppos(physent *d);
     extern void updatematerial(physent *pl, const vec &center, const vec &bottom, bool local = false);
@@ -91,6 +94,7 @@ namespace physics
 
 namespace game
 {
+    extern void start();
     extern bool clientoption(char *arg);
     extern void preload();
     extern void updateworld();
@@ -134,9 +138,10 @@ namespace server
     extern bool serveroption(char *arg);
     extern void *newinfo();
     extern void deleteinfo(void *ci);
-    extern int numclients(int exclude = -1, bool nospec = false, int aitype = -1);
+    extern int numclients(int exclude = -1, bool nospec = false, int actortype = -1);
     extern int reserveclients();
     extern int numchannels();
+    extern int dupclients();
     extern void clientdisconnect(int n, bool local = false, int reason = DISC_NONE);
     extern int clientconnect(int n, uint ip, bool local = false);
     extern bool allowbroadcast(int n);
@@ -148,11 +153,12 @@ namespace server
     extern void serverupdate();
     extern const char *gameid();
     extern int getver(int n = 0);
-    extern const char *pickmap(const char *suggest = NULL);
-    extern const char *choosemap(const char *suggest = NULL, int mode = -1, int muts = -1, int force = 0);
+    extern const char *pickmap(const char *suggest = NULL, int mode = -1, int muts = -1, bool notry = false);
+    extern const char *choosemap(const char *suggest = NULL, int mode = -1, int muts = -1, int force = 0, bool notry = false);
     extern void changemap(const char *name = NULL, int mode = -1, int muts = -1);
     extern bool canload(const char *type);
     extern bool rewritecommand(ident *id, tagval *args, int numargs);
-    extern void disconnectedmaster();
     extern void processmasterinput(const char *cmd, int cmdlen, const char *args);
+    extern void masterconnected();
+    extern void masterdisconnected();
 }
