@@ -84,8 +84,46 @@ extern const uchar uni2cubechars[878] =
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
 };
+extern const uchar cubelowerchars[256] =
+{
+    0, 130, 131, 132, 133, 134, 135, 136, 137, 9, 10, 11, 12, 13, 138, 139,
+    140, 141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155,
+    32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47,
+    48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63,
+    64, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111,
+    112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 91, 92, 93, 94, 95,
+    96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111,
+    112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 156,
+    157, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143,
+    144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 160,
+    160, 162, 162, 164, 164, 166, 166, 168, 168, 170, 170, 172, 172, 105, 174, 176,
+    176, 178, 178, 180, 180, 182, 182, 184, 184, 186, 186, 188, 188, 190, 190, 192,
+    192, 194, 194, 196, 196, 198, 198, 158, 201, 201, 203, 203, 205, 205, 206, 207,
+    208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223,
+    224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239,
+    240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255
+};
+extern const uchar cubeupperchars[256] =
+{
+    0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15,
+    16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
+    32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47,
+    48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63,
+    64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
+    80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95,
+    96, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
+    80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 123, 124, 125, 126, 127,
+    128, 129, 1, 2, 3, 4, 5, 6, 7, 8, 14, 15, 16, 17, 18, 19,
+    20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 127, 128, 199, 159,
+    159, 161, 161, 163, 163, 165, 165, 167, 167, 169, 169, 171, 171, 173, 73, 175,
+    175, 177, 177, 179, 179, 181, 181, 183, 183, 185, 185, 187, 187, 189, 189, 191,
+    191, 193, 193, 195, 195, 197, 197, 199, 200, 200, 202, 202, 204, 204, 206, 207,
+    208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223,
+    224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239,
+    240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255
+};
 
-int decodeutf8(uchar *dstbuf, int dstlen, const uchar *srcbuf, int srclen, int *carry)
+size_t decodeutf8(uchar *dstbuf, size_t dstlen, const uchar *srcbuf, size_t srclen, size_t *carry)
 {
     uchar *dst = dstbuf, *dstend = &dstbuf[dstlen];
     const uchar *src = srcbuf, *srcend = &srcbuf[srclen];
@@ -139,7 +177,7 @@ decode:
     return dst - dstbuf;
 }
 
-int encodeutf8(uchar *dstbuf, int dstlen, const uchar *srcbuf, int srclen, int *carry)
+size_t encodeutf8(uchar *dstbuf, size_t dstlen, const uchar *srcbuf, size_t srclen, size_t *carry)
 {
     uchar *dst = dstbuf, *dstend = &dstbuf[dstlen];
     const uchar *src = srcbuf, *srcend = &srcbuf[srclen];
@@ -303,6 +341,11 @@ char *path(char *s, bool simple)
             {
                 if(prevdir+2==curdir && prevdir[0]=='.' && prevdir[1]=='.') continue;
                 memmove(prevdir, curdir+4, strlen(curdir+4)+1);
+                if(prevdir-2 >= curpart && prevdir[-1]==PATHDIV)
+                {
+                    prevdir -= 2;
+                    while(prevdir-1 >= curpart && prevdir[-1] != PATHDIV) --prevdir;
+                }
                 curdir = prevdir;
             }
         }
@@ -338,6 +381,7 @@ bool fileexists(const char *path, const char *mode)
 {
     bool exists = true;
     if(mode[0]=='w' || mode[0]=='a') path = parentdir(path);
+    else if(findzipfile(path)) return true;
 #ifdef WIN32
     if(GetFileAttributes(path[0] ? path : ".\\") == INVALID_FILE_ATTRIBUTES) exists = false;
 #else
@@ -423,14 +467,15 @@ const char *findfile(const char *filename, const char *mode)
         formatstring(s)("%s%s", packagedirs[i].name, filename); path(s);
         if(fileexists(s, mode)) return s;
     }
+    if(mode[0]=='e') return NULL;
     copystring(s, filename); path(s);
     return s;
 }
 
 bool listdir(const char *dirname, bool rel, const char *ext, vector<char *> &files)
 {
-    int extsize = ext ? (int)strlen(ext)+1 : 0;
-    #ifdef WIN32
+    size_t extsize = ext ? strlen(ext)+1 : 0;
+#ifdef WIN32
     defformatstring(pathname)(rel ? ".\\%s\\*.%s" : "%s\\*.%s", dirname, ext ? ext : "*");
     WIN32_FIND_DATA FindFileData;
     HANDLE Find = FindFirstFile(pathname, &FindFileData);
@@ -440,15 +485,19 @@ bool listdir(const char *dirname, bool rel, const char *ext, vector<char *> &fil
             if(!ext) files.add(newstring(FindFileData.cFileName));
             else
             {
-                int namelength = (int)strlen(FindFileData.cFileName) - extsize;
-                if(namelength > 0 && FindFileData.cFileName[namelength] == '.' && strncmp(FindFileData.cFileName+namelength+1, ext, extsize-1)==0)
-                    files.add(newstring(FindFileData.cFileName, namelength));
+                size_t namelen = strlen(FindFileData.cFileName);
+                if(namelen > extsize)
+                {
+                    namelen -= extsize;
+                    if(FindFileData.cFileName[namelen] == '.' && strncmp(FindFileData.cFileName+namelen+1, ext, extsize-1)==0)
+                        files.add(newstring(FindFileData.cFileName, namelen));
+                }
             }
         } while(FindNextFile(Find, &FindFileData));
         FindClose(Find);
         return true;
     }
-    #else
+#else
     defformatstring(pathname)(rel ? "./%s" : "%s", dirname);
     DIR *d = opendir(pathname);
     if(d)
@@ -459,15 +508,19 @@ bool listdir(const char *dirname, bool rel, const char *ext, vector<char *> &fil
             if(!ext) files.add(newstring(de->d_name));
             else
             {
-                int namelength = (int)strlen(de->d_name) - extsize;
-                if(namelength > 0 && de->d_name[namelength] == '.' && strncmp(de->d_name+namelength+1, ext, extsize-1)==0)
-                    files.add(newstring(de->d_name, namelength));
+                size_t namelen = strlen(de->d_name);
+                if(namelen > extsize)
+                {
+                    namelen -= extsize;
+                    if(de->d_name[namelen] == '.' && strncmp(de->d_name+namelen+1, ext, extsize-1)==0)
+                        files.add(newstring(de->d_name, namelen));
+                }
             }
         }
         closedir(d);
         return true;
     }
-    #endif
+#endif
     else return false;
 }
 
@@ -476,7 +529,7 @@ int listfiles(const char *dir, const char *ext, vector<char *> &files)
     string dirname;
     copystring(dirname, dir);
     path(dirname);
-    int dirlen = (int)strlen(dirname);
+    size_t dirlen = strlen(dirname);
     while(dirlen > 1 && dirname[dirlen-1] == PATHDIV) dirname[--dirlen] = '\0';
     int dirs = 0;
     if(listdir(dirname, true, ext, files)) dirs++;
@@ -491,9 +544,7 @@ int listfiles(const char *dir, const char *ext, vector<char *> &files)
         formatstring(s)("%s%s", packagedirs[i].name, dirname);
         if(listdir(s, false, ext, files)) dirs++;
     }
-#ifndef STANDALONE
     dirs += listzipfiles(dirname, ext, files);
-#endif
     return dirs;
 }
 
@@ -543,7 +594,7 @@ stream::offset stream::size()
     return pos == endpos || seek(pos, SEEK_SET) ? endpos : -1;
 }
 
-bool stream::getline(char *str, int len)
+bool stream::getline(char *str, size_t len)
 {
     loopi(len-1)
     {
@@ -554,22 +605,24 @@ bool stream::getline(char *str, int len)
     return true;
 }
 
-int stream::printf(const char *fmt, ...)
+size_t stream::printf(const char *fmt, ...)
 {
     char buf[512];
     char *str = buf;
     va_list args;
 #if defined(WIN32) && !defined(__GNUC__)
     va_start(args, fmt);
-    size_t len = _vscprintf(fmt, args);
-    if(len >= sizeof(buf)) str = new char[len+1];
+    int len = _vscprintf(fmt, args);
+    if(len <= 0) { va_end(args); return 0; }
+    if(len >= (int)sizeof(buf)) str = new char[len+1];
     _vsnprintf(str, len+1, fmt, args);
     va_end(args);
 #else
     va_start(args, fmt);
-    size_t len = vsnprintf(buf, sizeof(buf), fmt, args);
+    int len = vsnprintf(buf, sizeof(buf), fmt, args);
     va_end(args);
-    if(len >= sizeof(buf))
+    if(len <= 0) return 0;
+    if(len >= (int)sizeof(buf))
     {
         str = new char[len+1];
         va_start(args, fmt);
@@ -577,7 +630,7 @@ int stream::printf(const char *fmt, ...)
         va_end(args);
     }
 #endif
-    int n = write(str, len);
+    size_t n = write(str, len);
     if(str != buf) delete[] str;
     return n;
 }
@@ -638,20 +691,21 @@ struct filestream : stream
 #endif
     }
 
-    int read(void *buf, int len) { return (int)fread(buf, 1, len, file); }
-    int write(const void *buf, int len) { return (int)fwrite(buf, 1, len, file); }
+    size_t read(void *buf, size_t len) { return fread(buf, 1, len, file); }
+    size_t write(const void *buf, size_t len) { return fwrite(buf, 1, len, file); }
+    bool flush() { return !fflush(file); }
     int getchar() { return fgetc(file); }
     bool putchar(int c) { return fputc(c, file)!=EOF; }
-    bool getline(char *str, int len) { return fgets(str, len, file)!=NULL; }
+    bool getline(char *str, size_t len) { return fgets(str, len, file)!=NULL; }
     bool putstring(const char *str) { return fputs(str, file)!=EOF; }
 
-    int printf(const char *fmt, ...)
+    size_t printf(const char *fmt, ...)
     {
         va_list v;
         va_start(v, fmt);
         int result = vfprintf(file, fmt, v);
         va_end(v);
-        return result;
+        return max(result, 0);
     }
 };
 
@@ -684,7 +738,7 @@ struct gzstream : stream
     uchar *buf;
     bool reading, writing, autoclose;
     uint crc;
-    int headersize;
+    size_t headersize;
 
     gzstream() : file(NULL), buf(NULL), reading(false), writing(false), autoclose(false), crc(0), headersize(0)
     {
@@ -706,15 +760,15 @@ struct gzstream : stream
         file->write(header, sizeof(header));
     }
 
-    void readbuf(int size = BUFSIZE)
+    void readbuf(size_t size = BUFSIZE)
     {
         if(!zfile.avail_in) zfile.next_in = (Bytef *)buf;
-        size = min(size, int(&buf[BUFSIZE] - &zfile.next_in[zfile.avail_in]));
-        int n = file->read(zfile.next_in + zfile.avail_in, size);
+        size = min(size, size_t(&buf[BUFSIZE] - &zfile.next_in[zfile.avail_in]));
+        size_t n = file->read(zfile.next_in + zfile.avail_in, size);
         if(n > 0) zfile.avail_in += n;
     }
 
-    int readbyte(int size = BUFSIZE)
+    uchar readbyte(size_t size = BUFSIZE)
     {
         if(!zfile.avail_in) readbuf(size);
         if(!zfile.avail_in) return 0;
@@ -722,11 +776,11 @@ struct gzstream : stream
         return *(uchar *)zfile.next_in++;
     }
 
-    void skipbytes(int n)
+    void skipbytes(size_t n)
     {
         while(n > 0 && zfile.avail_in > 0)
         {
-            int skipped = min(n, (int)zfile.avail_in);
+            size_t skipped = min(n, size_t(zfile.avail_in));
             zfile.avail_in -= skipped;
             zfile.next_in += skipped;
             n -= skipped;
@@ -739,19 +793,19 @@ struct gzstream : stream
     {
         readbuf(10);
         if(readbyte() != MAGIC1 || readbyte() != MAGIC2 || readbyte() != Z_DEFLATED) return false;
-        int flags = readbyte();
+        uchar flags = readbyte();
         if(flags & F_RESERVED) return false;
         skipbytes(6);
         if(flags & F_EXTRA)
         {
-            int len = readbyte(512);
-            len |= readbyte(512)<<8;
+            size_t len = readbyte(512);
+            len |= size_t(readbyte(512))<<8;
             skipbytes(len);
         }
         if(flags & F_NAME) while(readbyte(512));
         if(flags & F_COMMENT) while(readbyte(512));
         if(flags & F_CRC) skipbytes(2);
-        headersize = int(file->tell() - zfile.avail_in);
+        headersize = size_t(file->tell() - zfile.avail_in);
         return zfile.avail_in > 0 || !file->end();
     }
 
@@ -770,7 +824,6 @@ struct gzstream : stream
         else if(writing && deflateInit2(&zfile, level, Z_DEFLATED, -MAX_WBITS, min(MAX_MEM_LEVEL, 8), Z_DEFAULT_STRATEGY) != Z_OK) writing = false;
         if(!reading && !writing) return false;
 
-        autoclose = needclose;
         file = f;
         crc = crc32(0, NULL, 0);
         buf = new uchar[BUFSIZE];
@@ -780,6 +833,8 @@ struct gzstream : stream
             if(!checkheader()) { stopreading(); return false; }
         }
         else if(writing) writeheader();
+
+        autoclose = needclose;
         return true;
     }
 
@@ -816,7 +871,7 @@ struct gzstream : stream
         {
             int err = zfile.avail_out > 0 ? deflate(&zfile, Z_FINISH) : Z_OK;
             if(err != Z_OK && err != Z_STREAM_END) break;
-            flush();
+            flushbuf();
             if(err == Z_STREAM_END) break;
         }
         uchar trailer[8] =
@@ -845,8 +900,8 @@ struct gzstream : stream
     }
 
     bool end() { return !reading && !writing; }
-    offset tell() { return reading ? zfile.total_out : (writing ? zfile.total_in : -1); }
-    offset rawtell() { return file ? file->tell() : -1; }
+    offset tell() { return reading ? zfile.total_out : (writing ? zfile.total_in : offset(-1)); }
+    offset rawtell() { return file ? file->tell() : offset(-1); }
 
     offset size()
     {
@@ -854,10 +909,10 @@ struct gzstream : stream
         offset pos = tell();
         if(!file->seek(-4, SEEK_END)) return -1;
         uint isize = file->getlil<uint>();
-        return file->seek(pos, SEEK_SET) ? isize : -1;
+        return file->seek(pos, SEEK_SET) ? isize : offset(-1);
     }
 
-    offset rawsize() { return file ? file->size() : -1; }
+    offset rawsize() { return file ? file->size() : offset(-1); }
 
     bool seek(offset pos, int whence)
     {
@@ -892,7 +947,7 @@ struct gzstream : stream
         uchar skip[512];
         while(pos > 0)
         {
-            int skipped = (int)min(pos, (offset)sizeof(skip));
+            size_t skipped = (size_t)min(pos, (offset)sizeof(skip));
             if(read(skip, skipped) != skipped) { stopreading(); return false; }
             pos -= skipped;
         }
@@ -900,7 +955,7 @@ struct gzstream : stream
         return true;
     }
 
-    int read(void *buf, int len)
+    size_t read(void *buf, size_t len)
     {
         if(!reading || !buf || !len) return 0;
         zfile.next_out = (Bytef *)buf;
@@ -920,11 +975,12 @@ struct gzstream : stream
         return len - zfile.avail_out;
     }
 
-    bool flush()
+    bool flushbuf(bool full = false)
     {
+        if(full) deflate(&zfile, Z_SYNC_FLUSH);
         if(zfile.next_out && zfile.avail_out < BUFSIZE)
         {
-            if(file->write(buf, BUFSIZE - zfile.avail_out) != int(BUFSIZE - zfile.avail_out))
+            if(file->write(buf, BUFSIZE - zfile.avail_out) != BUFSIZE - zfile.avail_out || (full && !file->flush()))
                 return false;
         }
         zfile.next_out = buf;
@@ -932,14 +988,16 @@ struct gzstream : stream
         return true;
     }
 
-    int write(const void *buf, int len)
+    bool flush() { return flushbuf(true); }
+
+    size_t write(const void *buf, size_t len)
     {
         if(!writing || !buf || !len) return 0;
         zfile.next_in = (Bytef *)buf;
         zfile.avail_in = len;
         while(zfile.avail_in > 0)
         {
-            if(!zfile.avail_out && !flush()) { stopwriting(); break; }
+            if(!zfile.avail_out && !flushbuf()) { stopwriting(); break; }
             int err = deflate(&zfile, Z_NO_FLUSH);
             if(err != Z_OK) { stopwriting(); break; }
         }
@@ -956,7 +1014,7 @@ struct utf8stream : stream
     };
     stream *file;
     offset pos;
-    int bufread, bufcarry, buflen;
+    size_t bufread, bufcarry, buflen;
     bool reading, writing, autoclose;
     uchar buf[BUFSIZE];
 
@@ -969,13 +1027,13 @@ struct utf8stream : stream
         close();
     }
 
-    bool readbuf(int size = BUFSIZE)
+    bool readbuf(size_t size = BUFSIZE)
     {
         if(bufread >= bufcarry) { if(bufcarry > 0 && bufcarry < buflen) memmove(buf, &buf[bufcarry], buflen - bufcarry); buflen -= bufcarry; bufread = bufcarry = 0; }
-        int n = file->read(&buf[buflen], min(size, BUFSIZE - buflen));
+        size_t n = file->read(&buf[buflen], min(size, BUFSIZE - buflen));
         if(n <= 0) return false;
         buflen += n;
-        int carry = bufcarry;
+        size_t carry = bufcarry;
         bufcarry += decodeutf8(&buf[bufcarry], BUFSIZE-bufcarry, &buf[bufcarry], buflen-bufcarry, &carry);
         if(carry > bufcarry && carry < buflen) { memmove(&buf[bufcarry], &buf[carry], buflen - carry); buflen -= carry - bufcarry; }
         return true;
@@ -983,7 +1041,7 @@ struct utf8stream : stream
 
     bool checkheader()
     {
-        int n = file->read(buf, 3);
+        size_t n = file->read(buf, 3);
         if(n == 3 && buf[0] == 0xEF && buf[1] == 0xBB && buf[2] == 0xBF) return true;
         buflen = n;
         return false;
@@ -999,11 +1057,11 @@ struct utf8stream : stream
         }
         if(!reading && !writing) return false;
 
-        autoclose = needclose;
         file = f;
 
         if(reading) checkheader();
 
+        autoclose = needclose;
         return true;
     }
 
@@ -1032,7 +1090,7 @@ struct utf8stream : stream
     }
 
     bool end() { return !reading && !writing; }
-    offset tell() { return reading || writing ? pos : -1; }
+    offset tell() { return reading || writing ? pos : offset(-1); }
 
     bool seek(offset off, int whence)
     {
@@ -1058,7 +1116,7 @@ struct utf8stream : stream
         uchar skip[512];
         while(off > 0)
         {
-            int skipped = (int)min(off, (offset)sizeof(skip));
+            size_t skipped = (size_t)min(off, (offset)sizeof(skip));
             if(read(skip, skipped) != skipped) { stopreading(); return false; }
             off -= skipped;
         }
@@ -1066,14 +1124,14 @@ struct utf8stream : stream
         return true;
     }
 
-    int read(void *dst, int len)
+    size_t read(void *dst, size_t len)
     {
         if(!reading || !dst || !len) return 0;
-        int next = 0;
+        size_t next = 0;
         while(next < len)
         {
             if(bufread >= bufcarry) { if(readbuf(BUFSIZE)) continue; stopreading(); break; }
-            int n = min(len - next, bufcarry - bufread);
+            size_t n = min(len - next, bufcarry - bufread);
             memcpy(&((uchar *)dst)[next], &buf[bufread], n);
             next += n;
             bufread += n;
@@ -1082,15 +1140,15 @@ struct utf8stream : stream
         return next;
     }
 
-    bool getline(char *dst, int len)
+    bool getline(char *dst, size_t len)
     {
         if(!reading || !dst || !len) return false;
         --len;
-        int next = 0;
+        size_t next = 0;
         while(next < len)
         {
             if(bufread >= bufcarry) { if(readbuf(BUFSIZE)) continue; stopreading(); if(!next) return false; break; }
-            int n = min(len - next, bufcarry - bufread);
+            size_t n = min(len - next, bufcarry - bufread);
             uchar *endline = (uchar *)memchr(&buf[bufread], '\n', n);
             if(endline) { n = endline+1 - &buf[bufread]; len = next + n; }
             memcpy(&((uchar *)dst)[next], &buf[bufread], n);
@@ -1102,20 +1160,22 @@ struct utf8stream : stream
         return true;
     }
 
-    int write(const void *src, int len)
+    size_t write(const void *src, size_t len)
     {
         if(!writing || !src || !len) return 0;
         uchar dst[512];
-        int next = 0;
+        size_t next = 0;
         while(next < len)
         {
-            int carry = 0, n = encodeutf8(dst, sizeof(dst), &((uchar *)src)[next], len - next, &carry);
+            size_t carry = 0, n = encodeutf8(dst, sizeof(dst), &((uchar *)src)[next], len - next, &carry);
             if(n > 0 && file->write(dst, n) != n) { stopwriting(); break; }
             next += carry;
         }
         pos += next;
         return next;
     }
+
+    bool flush() { return file->flush(); }
 };
 
 stream *openrawfile(const char *filename, const char *mode)
@@ -1129,10 +1189,8 @@ stream *openrawfile(const char *filename, const char *mode)
 
 stream *openfile(const char *filename, const char *mode)
 {
-#ifndef STANDALONE
     stream *s = openzipfile(filename, mode);
     if(s) return s;
-#endif
     return openrawfile(filename, mode);
 }
 
@@ -1149,7 +1207,7 @@ stream *opengzfile(const char *filename, const char *mode, stream *file, int lev
     stream *source = file ? file : openfile(filename, mode);
     if(!source) return NULL;
     gzstream *gz = new gzstream;
-    if(!gz->open(source, mode, !file, level)) { if(!file) delete source; return NULL; }
+    if(!gz->open(source, mode, !file, level)) { if(!file) delete source; delete gz; return NULL; }
     return gz;
 }
 
@@ -1158,26 +1216,26 @@ stream *openutf8file(const char *filename, const char *mode, stream *file)
     stream *source = file ? file : openfile(filename, mode);
     if(!source) return NULL;
     utf8stream *utf8 = new utf8stream;
-    if(!utf8->open(source, mode, !file)) { if(!file) delete source; return NULL; }
+    if(!utf8->open(source, mode, !file)) { if(!file) delete source; delete utf8; return NULL; }
     return utf8;
 }
 
-char *loadfile(const char *fn, int *size, bool utf8)
+char *loadfile(const char *fn, size_t *size, bool utf8)
 {
     stream *f = openfile(fn, "rb");
     if(!f) return NULL;
-    int len = (int)f->size();
+    size_t len = f->size();
     if(len <= 0) { delete f; return NULL; }
     char *buf = new char[len+1];
     if(!buf) { delete f; return NULL; }
-    int offset = 0;
+    size_t offset = 0;
     if(utf8 && len >= 3)
     {
         if(f->read(buf, 3) != 3) { delete f; delete[] buf; return NULL; }
         if(((uchar *)buf)[0] == 0xEF && ((uchar *)buf)[1] == 0xBB && ((uchar *)buf)[2] == 0xBF) len -= 3;
         else offset += 3;
     }
-    int rlen = f->read(&buf[offset], len-offset);
+    size_t rlen = f->read(&buf[offset], len-offset);
     delete f;
     if(rlen != len-offset) { delete[] buf; return NULL; }
     if(utf8) len = decodeutf8((uchar *)buf, len, (uchar *)buf, len);
