@@ -74,23 +74,27 @@ static bool findzipdirectory(FILE *f, zipdirectoryheader &hdr)
 {
     if(fseek(f, 0, SEEK_END) < 0) return false;
 
+    long offset = ftell(f);
+    if(offset < 0) return false;
+
     uchar buf[1024], *src = NULL;
-    int len = 0, offset = ftell(f), end = max(offset - 0xFFFF - ZIP_DIRECTORY_SIZE, 0);
+    long end = max(offset - 0xFFFFL - ZIP_DIRECTORY_SIZE, 0L);
+    size_t len = 0;
     const uint signature = lilswap<uint>(ZIP_DIRECTORY_SIGNATURE);
 
     while(offset > end)
     {
-        int carry = min(len, ZIP_DIRECTORY_SIZE-1), next = min((int)sizeof(buf) - carry, offset - end);
+        size_t carry = min(len, size_t(ZIP_DIRECTORY_SIZE-1)), next = min(sizeof(buf) - carry, size_t(offset - end));
         offset -= next;
         memmove(&buf[next], buf, carry);
-        if(next + carry < ZIP_DIRECTORY_SIZE || fseek(f, offset, SEEK_SET) < 0 || (int)fread(buf, 1, next, f) != next) return false;
+        if(next + carry < ZIP_DIRECTORY_SIZE || fseek(f, offset, SEEK_SET) < 0 || fread(buf, 1, next, f) != next) return false;
         len = next + carry;
         uchar *search = &buf[next-1];
         for(; search >= buf; search--) if(*(uint *)search == signature) break;
         if(search >= buf) { src = search; break; }
     }
 
-    if(&buf[len] - src < ZIP_DIRECTORY_SIZE) return false;
+    if(!src || &buf[len] - src < ZIP_DIRECTORY_SIZE) return false;
 
     hdr.signature = lilswap(*(uint *)src); src += 4;
     hdr.disknumber = lilswap(*(ushort *)src); src += 2;
@@ -110,10 +114,10 @@ static bool findzipdirectory(FILE *f, zipdirectoryheader &hdr)
 VAR(0, dbgzip, 0, 0, 1);
 #endif
 
-static bool readzipdirectory(const char *archname, FILE *f, int entries, int offset, int size, vector<zipfile> &files)
+static bool readzipdirectory(const char *archname, FILE *f, int entries, int offset, uint size, vector<zipfile> &files)
 {
     uchar *buf = new uchar[size], *src = buf;
-    if(fseek(f, offset, SEEK_SET) < 0 || (int)fread(buf, 1, size, f) != size) { delete[] buf; return false; }
+    if(fseek(f, offset, SEEK_SET) < 0 || fread(buf, 1, size, f) != size) { delete[] buf; return false; }
     loopi(entries)
     {
         if(src + ZIP_FILE_SIZE > &buf[size]) break;
@@ -169,9 +173,8 @@ static bool readzipdirectory(const char *archname, FILE *f, int entries, int off
 
 static bool readlocalfileheader(FILE *f, ziplocalfileheader &h, uint offset)
 {
-    fseek(f, offset, SEEK_SET);
     uchar buf[ZIP_LOCAL_FILE_SIZE];
-    if(fread(buf, 1, ZIP_LOCAL_FILE_SIZE, f) != ZIP_LOCAL_FILE_SIZE)
+    if(fseek(f, offset, SEEK_SET) < 0 || fread(buf, 1, ZIP_LOCAL_FILE_SIZE, f) != ZIP_LOCAL_FILE_SIZE)
         return false;
     uchar *src = buf;
     h.signature = lilswap(*(uint *)src); src += 4;
@@ -209,9 +212,9 @@ static bool checkprefix(vector<zipfile> &files, const char *prefix, int prefixle
 
 static void mountzip(ziparchive &arch, vector<zipfile> &files, const char *mountdir, const char *stripdir)
 {
-    string packagesdir = "packages/";
+    string packagesdir = "data/";
     path(packagesdir);
-    int striplen = stripdir ? (int)strlen(stripdir) : 0;
+    size_t striplen = stripdir ? strlen(stripdir) : 0;
     if(!mountdir && !stripdir) loopv(files)
     {
         zipfile &f = files[i];
@@ -225,19 +228,19 @@ static void mountzip(ziparchive &arch, vector<zipfile> &files, const char *mount
             }
             break;
         }
-        const char *foundogz = strstr(f.name, ".ogz");
-        if(foundogz)
+        const char *foundmpz = strstr(f.name, ".mpz");
+        if(foundmpz)
         {
-            const char *ogzdir = foundogz;
-            while(--ogzdir >= f.name && *ogzdir != PATHDIV);
-            if(ogzdir < f.name || checkprefix(files, f.name, ogzdir + 1 - f.name))
+            const char *mpzdir = foundmpz;
+            while(--mpzdir >= f.name && *mpzdir != PATHDIV);
+            if(mpzdir < f.name || checkprefix(files, f.name, mpzdir + 1 - f.name))
             {
-                if(ogzdir >= f.name)
+                if(mpzdir >= f.name)
                 {
                     stripdir = f.name;
-                    striplen = ogzdir + 1 - f.name;
+                    striplen = mpzdir + 1 - f.name;
                 }
-                if(!mountdir) mountdir = "packages/base/";
+                if(!mountdir) mountdir = "maps/";
                 break;
             }
         }
@@ -265,7 +268,7 @@ bool addzip(const char *name, const char *mount = NULL, const char *strip = NULL
     string pname;
     copystring(pname, name);
     path(pname);
-    int plen = (int)strlen(pname);
+    size_t plen = strlen(pname);
     if(plen < 4 || !strchr(&pname[plen-4], '.')) concatstring(pname, ".zip");
 
     ziparchive *exists = findzip(pname);
@@ -335,10 +338,10 @@ struct zipstream : stream
     zipfile *info;
     z_stream zfile;
     uchar *buf;
-    int reading;
+    uint reading;
     bool ended;
 
-    zipstream() : arch(NULL), info(NULL), buf(NULL), reading(-1), ended(false)
+    zipstream() : arch(NULL), info(NULL), buf(NULL), reading(~0U), ended(false)
     {
         zfile.zalloc = NULL;
         zfile.zfree = NULL;
@@ -362,8 +365,8 @@ struct zipstream : stream
             if(fseek(arch->data, reading, SEEK_SET) >= 0) arch->owner = this;
             else return;
         }
-        uint remaining = info->offset + info->compressedsize - reading;
-        int n = arch->owner == this ? (int)fread(zfile.next_in + zfile.avail_in, 1, min(size, remaining), arch->data) : 0;
+        uint remaining = info->offset + info->compressedsize - reading,
+             n = arch->owner == this ? fread(zfile.next_in + zfile.avail_in, 1, min(size, remaining), arch->data) : 0U;
         zfile.avail_in += n;
         reading += n;
     }
@@ -391,12 +394,12 @@ struct zipstream : stream
 
     void stopreading()
     {
-        if(reading < 0) return;
+        if(reading == ~0U) return;
 #ifndef STANDALONE
         if(dbgzip) conoutf(info->compressedsize ? "%s: zfile.total_out %u, info->size %u" : "%s: reading %u, info->size %u", info->name, info->compressedsize ? uint(zfile.total_out) : reading - info->offset, info->size);
 #endif
         if(info->compressedsize) inflateEnd(&zfile);
-        reading = -1;
+        reading = ~0U;
     }
 
     void close()
@@ -407,12 +410,12 @@ struct zipstream : stream
     }
 
     offset size() { return info->size; }
-    bool end() { return reading < 0 || ended; }
-    offset tell() { return reading >= 0 ? (info->compressedsize ? zfile.total_out : reading - info->offset) : -1; }
+    bool end() { return reading == ~0U || ended; }
+    offset tell() { return reading != ~0U ? (info->compressedsize ? zfile.total_out : reading - info->offset) : offset(-1); }
 
     bool seek(offset pos, int whence)
     {
-        if(reading < 0) return false;
+        if(reading == ~0U) return false;
         if(!info->compressedsize)
         {
             switch(whence)
@@ -472,7 +475,7 @@ struct zipstream : stream
         uchar skip[512];
         while(pos > 0)
         {
-            int skipped = (int)min(pos, (offset)sizeof(skip));
+            size_t skipped = (size_t)min(pos, (offset)sizeof(skip));
             if(read(skip, skipped) != skipped) return false;
             pos -= skipped;
         }
@@ -481,9 +484,9 @@ struct zipstream : stream
         return true;
     }
 
-    int read(void *buf, int len)
+    size_t read(void *buf, size_t len)
     {
-        if(reading < 0 || !buf || !len) return 0;
+        if(reading == ~0U || !buf || !len) return 0;
         if(!info->compressedsize)
         {
             if(arch->owner != this)
@@ -493,7 +496,7 @@ struct zipstream : stream
                 arch->owner = this;
             }
 
-            int n = (int)fread(buf, 1, min(len, int(info->size + info->offset - reading)), arch->data);
+            size_t n = fread(buf, 1, min(len, size_t(info->size + info->offset - reading)), arch->data);
             reading += n;
             if(n < len) ended = true;
             return n;
@@ -525,6 +528,7 @@ struct zipstream : stream
 stream *openzipfile(const char *name, const char *mode)
 {
     for(; *mode; mode++) if(*mode=='w' || *mode=='a') return NULL;
+    name = copypath(name, true);
     loopvrev(archives)
     {
         ziparchive *arch = archives[i];
@@ -537,9 +541,22 @@ stream *openzipfile(const char *name, const char *mode)
     return NULL;
 }
 
+bool findzipfile(const char *name)
+{
+    name = copypath(name, true);
+    loopvrev(archives)
+    {
+        ziparchive *arch = archives[i];
+        if(arch->files.access(name)) return true;
+    }
+    return false;
+}
+
 int listzipfiles(const char *dir, const char *ext, vector<char *> &files)
 {
-    int extsize = ext ? (int)strlen(ext)+1 : 0, dirsize = (int)strlen(dir), dirs = 0;
+    dir = copypath(dir, true);
+    size_t extsize = ext ? strlen(ext)+1 : 0, dirsize = strlen(dir);
+    int dirs = 0;
     loopvrev(archives)
     {
         ziparchive *arch = archives[i];
@@ -553,9 +570,13 @@ int listzipfiles(const char *dir, const char *ext, vector<char *> &files)
             if(!ext) files.add(newstring(name));
             else
             {
-                int namelength = (int)strlen(name) - extsize;
-                if(namelength > 0 && name[namelength] == '.' && strncmp(name+namelength+1, ext, extsize-1)==0)
-                    files.add(newstring(name, namelength));
+                size_t namelen = strlen(name);
+                if(namelen > extsize)
+                {
+                    namelen -= extsize;
+                    if(name[namelen] == '.' && strncmp(name+namelen+1, ext, extsize-1)==0)
+                        files.add(newstring(name, namelen));
+                }
             }
         });
         if(files.length() > oldsize) dirs++;
@@ -563,8 +584,6 @@ int listzipfiles(const char *dir, const char *ext, vector<char *> &files)
     return dirs;
 }
 
-#ifndef STANDALONE
 ICOMMAND(0, addzip, "sss", (const char *name, const char *mount, const char *strip), addzip(name, mount[0] ? mount : NULL, strip[0] ? strip : NULL));
 ICOMMAND(0, removezip, "s", (const char *name), removezip(name));
-#endif
 
